@@ -21,7 +21,13 @@ InputListType = Union[list, "np.ndarray", "pa.Array", "pa.ChunkedArray"]
 def from_pylist(data: List[Dict[str, Any]]) -> "DataFrame":
     """Creates a DataFrame from a list of dictionaries.
 
-    Example:
+    Args:
+        data: List of dictionaries, where each key is a column name.
+
+    Returns:
+        DataFrame: DataFrame created from list of dictionaries.
+
+    Examples:
         >>> import daft
         >>> df = daft.from_pylist([{"foo": 1}, {"foo": 2}])
         >>> df.show()
@@ -36,12 +42,6 @@ def from_pylist(data: List[Dict[str, Any]]) -> "DataFrame":
         ╰───────╯
         <BLANKLINE>
         (Showing first 2 of 2 rows)
-
-    Args:
-        data: List of dictionaries, where each key is a column name.
-
-    Returns:
-        DataFrame: DataFrame created from list of dictionaries.
     """
     from daft import DataFrame
 
@@ -52,7 +52,14 @@ def from_pylist(data: List[Dict[str, Any]]) -> "DataFrame":
 def from_pydict(data: Dict[str, InputListType]) -> "DataFrame":
     """Creates a DataFrame from a Python dictionary.
 
-    Example:
+    Args:
+        data: Key -> Sequence[item] of data. Each Key is created as a column, and must have a value that is
+            a Python list, Numpy array or PyArrow array. Values must be equal in length across all keys.
+
+    Returns:
+        DataFrame: DataFrame created from dictionary of columns
+
+    Examples:
         >>> import daft
         >>> df = daft.from_pydict({"foo": [1, 2]})
         >>> df.show()
@@ -67,13 +74,6 @@ def from_pydict(data: Dict[str, InputListType]) -> "DataFrame":
         ╰───────╯
         <BLANKLINE>
         (Showing first 2 of 2 rows)
-
-    Args:
-        data: Key -> Sequence[item] of data. Each Key is created as a column, and must have a value that is
-            a Python list, Numpy array or PyArrow array. Values must be equal in length across all keys.
-
-    Returns:
-        DataFrame: DataFrame created from dictionary of columns
     """
     from daft import DataFrame
 
@@ -84,7 +84,13 @@ def from_pydict(data: Dict[str, InputListType]) -> "DataFrame":
 def from_arrow(data: Union["pa.Table", List["pa.Table"], Iterable["pa.Table"]]) -> "DataFrame":
     """Creates a DataFrame from a pyarrow Table.
 
-    Example:
+    Args:
+        data: pyarrow Table(s) that we wish to convert into a Daft DataFrame.
+
+    Returns:
+        DataFrame: DataFrame created from the provided pyarrow Table.
+
+    Examples:
         >>> import pyarrow as pa
         >>> import daft
         >>> t = pa.table({"a": [1, 2, 3], "b": ["foo", "bar", "baz"]})
@@ -103,12 +109,6 @@ def from_arrow(data: Union["pa.Table", List["pa.Table"], Iterable["pa.Table"]])
         ╰───────┴──────╯
         <BLANKLINE>
         (Showing first 3 of 3 rows)
-
-    Args:
-        data: pyarrow Table(s) that we wish to convert into a Daft DataFrame.
-
-    Returns:
-        DataFrame: DataFrame created from the provided pyarrow Table.
     """
     from daft import DataFrame
 
@@ -119,7 +119,13 @@ def from_arrow(data: Union["pa.Table", List["pa.Table"], Iterable["pa.Table"]])
 def from_pandas(data: Union["pd.DataFrame", List["pd.DataFrame"]]) -> "DataFrame":
     """Creates a Daft DataFrame from a pandas DataFrame.
 
-    Example:
+    Args:
+        data: pandas DataFrame(s) that we wish to convert into a Daft DataFrame.
+
+    Returns:
+        DataFrame: Daft DataFrame created from the provided pandas DataFrame.
+
+    Examples:
         >>> import pandas as pd
         >>> import daft
         >>> pd_df = pd.DataFrame({"a": [1, 2, 3], "b": ["foo", "bar", "baz"]})
@@ -138,12 +144,6 @@ def from_pandas(data: Union["pd.DataFrame", List["pd.DataFrame"]]) -> "DataFrame
         ╰───────┴──────╯
         <BLANKLINE>
         (Showing first 3 of 3 rows)
-
-    Args:
-        data: pandas DataFrame(s) that we wish to convert into a Daft DataFrame.
-
-    Returns:
-        DataFrame: Daft DataFrame created from the provided pandas DataFrame.
     """
     from daft import DataFrame
 
@@ -154,11 +154,15 @@ def from_pandas(data: Union["pd.DataFrame", List["pd.DataFrame"]]) -> "DataFrame
 def from_ray_dataset(ds: "RayDataset") -> "DataFrame":
     """Creates a DataFrame from a Ray Dataset.
 
-    .. NOTE::
-        This function can only work if Daft is running using the RayRunner.
-
     Args:
         ds: The Ray Dataset to create a Daft DataFrame from.
+
+    Returns:
+        DataFrame: Daft DataFrame created from the provided Ray dataset.
+
+    Note:
+        This function can only work if Daft is running using the RayRunner
+
     """
     from daft import DataFrame
 
@@ -169,14 +173,14 @@ def from_ray_dataset(ds: "RayDataset") -> "DataFrame":
 def from_dask_dataframe(ddf: "dask.DataFrame") -> "DataFrame":
     """Creates a Daft DataFrame from a Dask DataFrame.
 
-    The provided Dask DataFrame must have been created using
-    `Dask-on-Ray <https://docs.ray.io/en/latest/ray-more-libs/dask-on-ray.html>`__.
-
-    .. NOTE::
-        This function can only work if Daft is running using the RayRunner
+    The provided Dask DataFrame must have been created using [Dask-on-Ray](https://docs.ray.io/en/latest/ray-more-libs/dask-on-ray.html).
 
     Args:
         ddf: The Dask DataFrame to create a Daft DataFrame from.
+
+    Note:
+        This function can only work if Daft is running using the RayRunner
+
     """
     from daft import DataFrame
 
