@@ -5,8 +5,10 @@ from dataclasses import dataclass, field
 from typing import TYPE_CHECKING, Generic, Protocol
 
 from daft.context import get_context
-from daft.daft import JoinSide, ResourceRequest
+from daft.daft import JoinSide, PyRecordBatch, ResourceRequest
 from daft.expressions import Expression, ExpressionsProjection, col
+from daft.filesystem import overwrite_files
+from daft.recordbatch import MicroPartition, RecordBatch, recordbatch_io
 from daft.runners.partitioning import (
     Boundaries,
     MaterializedResult,
@@ -14,7 +16,7 @@ from daft.runners.partitioning import (
     PartitionMetadata,
     PartitionT,
 )
-from daft.table import MicroPartition, table_io
+from daft.series import Series
 
 if TYPE_CHECKING:
     import pathlib
@@ -23,6 +25,7 @@ if TYPE_CHECKING:
     from pyiceberg.table import TableProperties as IcebergTableProperties
 
     from daft.daft import FileFormat, IOConfig, JoinType, ScanTask
+    from daft.io import DataSink
     from daft.logical.map_partition_ops import MapPartitionOp
     from daft.logical.schema import Schema
 
@@ -51,6 +54,9 @@ class PartitionTask(Generic[PartitionT]):
     # This is used when a specific executor (e.g. an Actor pool) must be provisioned and used for the task
     actor_pool_id: str | None
 
+    # Indicates that the metadata of the result partition should be cached when the task is done
+    cache_metadata_on_done: bool = True
+
     # Indicates if the PartitionTask is "done" or not
     is_done: bool = False
 
@@ -70,11 +76,17 @@ class PartitionTask(Generic[PartitionT]):
         """Sets the PartitionTask as done."""
         assert not self.is_done, "Cannot set PartitionTask as done more than once"
         self.is_done = True
+        if self.cache_metadata_on_done:
+            self.cache_metadata()
 
     def cancel(self) -> None:
         """If possible, cancel the execution of this PartitionTask."""
         raise NotImplementedError()
 
+    def cache_metadata(self) -> None:
+        """Cache the metadata of the result partition."""
+        raise NotImplementedError()
+
     def set_result(self, result: list[MaterializedResult[PartitionT]]) -> None:
         """Set the result of this Task. For use by the Task executor.
 
@@ -140,7 +152,9 @@ class PartitionTaskBuilder(Generic[PartitionT]):
         """Whether this partition task is guaranteed to result in an empty partition."""
         return len(self.partial_metadatas) > 0 and all(meta.num_rows == 0 for meta in self.partial_metadatas)
 
-    def finalize_partition_task_single_output(self, stage_id: int) -> SingleOutputPartitionTask[PartitionT]:
+    def finalize_partition_task_single_output(
+        self, stage_id: int, cache_metadata_on_done: bool = True
+    ) -> SingleOutputPartitionTask[PartitionT]:
         """Create a SingleOutputPartitionTask from this PartitionTaskBuilder.
 
         Returns a "frozen" version of this PartitionTask that cannot have instructions added.
@@ -162,9 +176,12 @@ class PartitionTaskBuilder(Generic[PartitionT]):
             partial_metadatas=self.partial_metadatas,
             actor_pool_id=self.actor_pool_id,
             node_id=self.node_id,
+            cache_metadata_on_done=cache_metadata_on_done,
         )
 
-    def finalize_partition_task_multi_output(self, stage_id: int) -> MultiOutputPartitionTask[PartitionT]:
+    def finalize_partition_task_multi_output(
+        self, stage_id: int, cache_metadata_on_done: bool = True
+    ) -> MultiOutputPartitionTask[PartitionT]:
         """Create a MultiOutputPartitionTask from this PartitionTaskBuilder.
 
         Same as finalize_partition_task_single_output, except the output of this PartitionTask is a list of partitions.
@@ -184,6 +201,7 @@ class PartitionTaskBuilder(Generic[PartitionT]):
             partial_metadatas=self.partial_metadatas,
             actor_pool_id=self.actor_pool_id,
             node_id=self.node_id,
+            cache_metadata_on_done=cache_metadata_on_done,
         )
 
     def __str__(self) -> str:
@@ -201,6 +219,7 @@ class SingleOutputPartitionTask(PartitionTask[PartitionT]):
 
     # When available, the partition created from running the PartitionTask.
     _result: None | MaterializedResult[PartitionT] = None
+    _partition_metadata: None | PartitionMetadata = None
 
     def set_result(self, result: list[MaterializedResult[PartitionT]]) -> None:
         assert self._result is None, f"Cannot set result twice. Result is already {self._result}"
@@ -220,13 +239,22 @@ class SingleOutputPartitionTask(PartitionTask[PartitionT]):
         """Get the PartitionT resulting from running this PartitionTask."""
         return self.result().partition()
 
+    def cache_metadata(self) -> None:
+        assert self._result is not None, "Cannot cache metadata without a result"
+        if self._partition_metadata is not None:
+            return
+
+        [partial_metadata] = self.partial_metadatas
+        self._partition_metadata = self.result().metadata().merge_with_partial(partial_metadata)
+
     def partition_metadata(self) -> PartitionMetadata:
         """Get the metadata of the result partition.
 
         (Avoids retrieving the actual partition itself if possible.)
         """
-        [partial_metadata] = self.partial_metadatas
-        return self.result().metadata().merge_with_partial(partial_metadata)
+        self.cache_metadata()
+        assert self._partition_metadata is not None
+        return self._partition_metadata
 
     def micropartition(self) -> MicroPartition:
         """Get the raw vPartition of the result."""
@@ -249,6 +277,7 @@ class MultiOutputPartitionTask(PartitionTask[PartitionT]):
 
     # When available, the partitions created from running the PartitionTask.
     _results: None | list[MaterializedResult[PartitionT]] = None
+    _partition_metadatas: None | list[PartitionMetadata] = None
 
     def set_result(self, result: list[MaterializedResult[PartitionT]]) -> None:
         assert self._results is None, f"Cannot set result twice. Result is already {self._results}"
@@ -264,16 +293,24 @@ class MultiOutputPartitionTask(PartitionTask[PartitionT]):
         assert self._results is not None
         return [result.partition() for result in self._results]
 
+    def cache_metadata(self) -> None:
+        assert self._results is not None, "Cannot cache metadata without a result"
+        if self._partition_metadatas is not None:
+            return
+
+        self._partition_metadatas = [
+            result.metadata().merge_with_partial(partial_metadata)
+            for result, partial_metadata in zip(self._results, self.partial_metadatas)
+        ]
+
     def partition_metadatas(self) -> list[PartitionMetadata]:
         """Get the metadata of the result partitions.
 
         (Avoids retrieving the actual partition itself if possible.)
         """
-        assert self._results is not None
-        return [
-            result.metadata().merge_with_partial(partial_metadata)
-            for result, partial_metadata in zip(self._results, self.partial_metadatas)
-        ]
+        self.cache_metadata()
+        assert self._partition_metadatas is not None
+        return self._partition_metadatas
 
     def micropartition(self, index: int) -> MicroPartition:
         """Get the raw vPartition of the result."""
@@ -389,7 +426,7 @@ class WriteFile(SingleOutputInstruction):
         ]
 
     def _handle_file_write(self, input: MicroPartition) -> MicroPartition:
-        return table_io.write_tabular(
+        return recordbatch_io.write_tabular(
             input,
             path=self.root_dir,
             schema=self.schema,
@@ -400,6 +437,35 @@ class WriteFile(SingleOutputInstruction):
         )
 
 
+@dataclass(frozen=True)
+class OverwriteFiles(SingleOutputInstruction):
+    overwrite_partitions: bool
+    root_dir: str | pathlib.Path
+    io_config: IOConfig | None
+
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
+        files_to_overwrite = []
+        for input in inputs:
+            files_to_overwrite.extend(input.get_column_by_name("path").to_pylist())
+        overwrite_files(
+            files_to_overwrite,
+            self.root_dir,
+            self.io_config,
+            self.overwrite_partitions,
+        )
+        return [MicroPartition.concat(inputs)]
+
+    def run_partial_metadata(self, input_metadatas: list[PartialPartitionMetadata]) -> list[PartialPartitionMetadata]:
+        input_rows = [_.num_rows for _ in input_metadatas]
+        input_sizes = [_.size_bytes for _ in input_metadatas]
+        return [
+            PartialPartitionMetadata(
+                num_rows=(sum(input_rows) if all(_ is not None for _ in input_rows) else None),
+                size_bytes=(sum(input_sizes) if all(_ is not None for _ in input_sizes) else None),
+            )
+        ]
+
+
 @dataclass(frozen=True)
 class WriteIceberg(SingleOutputInstruction):
     base_path: str
@@ -429,7 +495,7 @@ class WriteIceberg(SingleOutputInstruction):
         ]
 
     def _handle_file_write(self, input: MicroPartition) -> MicroPartition:
-        return table_io.write_iceberg(
+        return recordbatch_io.write_iceberg(
             input,
             base_path=self.base_path,
             schema=self.iceberg_schema,
@@ -468,7 +534,7 @@ class WriteDeltaLake(SingleOutputInstruction):
         ]
 
     def _handle_file_write(self, input: MicroPartition) -> MicroPartition:
-        return table_io.write_deltalake(
+        return recordbatch_io.write_deltalake(
             input,
             large_dtypes=self.large_dtypes,
             base_path=self.base_path,
@@ -505,7 +571,7 @@ class WriteLance(SingleOutputInstruction):
         ]
 
     def _handle_file_write(self, input: MicroPartition) -> MicroPartition:
-        return table_io.write_lance(
+        return recordbatch_io.write_lance(
             input,
             base_path=self.base_path,
             mode=self.mode,
@@ -514,6 +580,30 @@ class WriteLance(SingleOutputInstruction):
         )
 
 
+@dataclass(frozen=True)
+class DataSinkWrite(SingleOutputInstruction):
+    sink: DataSink
+
+    def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
+        result_field_name = "write_results"
+        results = list(self.sink.write(iter(inputs)))
+        results_series = Series.from_pylist(results, result_field_name, pyobj="force")
+        series_dict = {result_field_name: results_series._series}
+        rb = RecordBatch._from_pyrecordbatch(PyRecordBatch.from_pylist_series(series_dict))
+        mp = MicroPartition._from_record_batches([rb])
+        return [mp]
+
+    def run_partial_metadata(self, input_metadatas: list[PartialPartitionMetadata]) -> list[PartialPartitionMetadata]:
+        # TODO(desmond): We can potentially do something more useful here. For now, copy the implementation for the other writers.
+        assert len(input_metadatas) == 1
+        return [
+            PartialPartitionMetadata(
+                num_rows=None,  # we can write more than 1 file per partition
+                size_bytes=None,
+            )
+        ]
+
+
 @dataclass(frozen=True)
 class Filter(SingleOutputInstruction):
     predicate: ExpressionsProjection
@@ -901,12 +991,15 @@ class ReduceMergeAndSort(ReduceInstruction):
     sort_by: ExpressionsProjection
     descending: list[bool]
     bounds: MicroPartition
+    nulls_first: list[bool] | None = None
 
     def run(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
         return self._reduce_merge_and_sort(inputs)
 
     def _reduce_merge_and_sort(self, inputs: list[MicroPartition]) -> list[MicroPartition]:
-        partition = MicroPartition.concat(inputs).sort(self.sort_by, descending=self.descending)
+        partition = MicroPartition.concat(inputs).sort(
+            self.sort_by, descending=self.descending, nulls_first=self.nulls_first
+        )
         return [partition]
 
     def run_partial_metadata(self, input_metadatas: list[PartialPartitionMetadata]) -> list[PartialPartitionMetadata]:
@@ -935,7 +1028,6 @@ class ReduceToQuantiles(ReduceInstruction):
         merged = MicroPartition.concat(inputs)
 
         nulls_first = self.nulls_first if self.nulls_first is not None else self.descending
-
         # Skip evaluation of expressions by converting to Column Expression, since evaluation was done in Sample
         merged_sorted = merged.sort(
             self.sort_by.to_column_expressions(), descending=self.descending, nulls_first=nulls_first
@@ -1054,7 +1146,7 @@ class FanoutRange(FanoutInstruction, Generic[PartitionT]):
         if self._num_outputs == 1:
             return [input]
 
-        table_boundaries = boundaries.to_table()
+        table_boundaries = boundaries.to_record_batch()
         partitioned_tables = input.partition_by_range(self.sort_by, table_boundaries, self.descending)
 
         # Pad the partitioned_tables with empty tables if fewer than self._num_outputs were returned
