@@ -2,13 +2,17 @@ from __future__ import annotations
 
 from typing import TYPE_CHECKING, Iterator
 
+from daft.daft import (
+    LocalPhysicalPlan,
+)
 from daft.daft import (
     NativeExecutor as _NativeExecutor,
 )
-from daft.daft import PyDaftExecutionConfig
-from daft.table import MicroPartition
+from daft.dataframe.display import MermaidOptions
+from daft.recordbatch import MicroPartition
 
 if TYPE_CHECKING:
+    from daft.daft import PyDaftExecutionConfig
     from daft.logical.builder import LogicalPlanBuilder
     from daft.runners.partitioning import (
         LocalMaterializedResult,
@@ -18,16 +22,12 @@ if TYPE_CHECKING:
 
 
 class NativeExecutor:
-    def __init__(self, executor: _NativeExecutor):
-        self._executor = executor
-
-    @classmethod
-    def from_logical_plan_builder(cls, builder: LogicalPlanBuilder) -> NativeExecutor:
-        executor = _NativeExecutor.from_logical_plan_builder(builder._builder)
-        return cls(executor)
+    def __init__(self):
+        self._executor = _NativeExecutor()
 
     def run(
         self,
+        local_physical_plan: LocalPhysicalPlan,
         psets: dict[str, list[MaterializedResult[PartitionT]]],
         daft_execution_config: PyDaftExecutionConfig,
         results_buffer_size: int | None,
@@ -39,5 +39,25 @@ class NativeExecutor:
         }
         return (
             LocalMaterializedResult(MicroPartition._from_pymicropartition(part))
-            for part in self._executor.run(psets_mp, daft_execution_config, results_buffer_size)
+            for part in self._executor.run(
+                local_physical_plan,
+                psets_mp,
+                daft_execution_config,
+                results_buffer_size,
+            )
         )
+
+    def pretty_print(
+        self,
+        builder: LogicalPlanBuilder,
+        daft_execution_config: PyDaftExecutionConfig,
+        simple: bool = False,
+        format: str = "ascii",
+    ) -> str:
+        """Pretty prints the current underlying logical plan."""
+        if format == "ascii":
+            return self._executor.repr_ascii(builder._builder, daft_execution_config, simple)
+        elif format == "mermaid":
+            return self._executor.repr_mermaid(builder._builder, daft_execution_config, MermaidOptions(simple))
+        else:
+            raise ValueError(f"Unknown format: {format}")
