@@ -31,7 +31,7 @@ from typing import (
 )
 
 from daft.context import get_context
-from daft.daft import JoinSide, ResourceRequest
+from daft.daft import JoinSide, ResourceRequest, WriteMode
 from daft.execution import execution_step
 from daft.execution.execution_step import (
     Instruction,
@@ -43,11 +43,11 @@ from daft.execution.execution_step import (
     calculate_cross_join_stats,
 )
 from daft.expressions import ExpressionsProjection
+from daft.recordbatch.micropartition import MicroPartition
 from daft.runners.partitioning import (
     MaterializedResult,
     PartitionT,
 )
-from daft.table.micropartition import MicroPartition
 
 logger = logging.getLogger(__name__)
 
@@ -60,6 +60,7 @@ if TYPE_CHECKING:
     from pyiceberg.table import TableProperties as IcebergTableProperties
 
     from daft.daft import FileFormat, IOConfig, JoinType
+    from daft.io import DataSink
     from daft.logical.schema import Schema
 
 
@@ -92,6 +93,7 @@ def partition_read(
 
 def file_write(
     child_plan: InProgressPhysicalPlan[PartitionT],
+    write_mode: WriteMode,
     file_format: FileFormat,
     schema: Schema,
     root_dir: str | pathlib.Path,
@@ -100,21 +102,52 @@ def file_write(
     io_config: IOConfig | None,
 ) -> InProgressPhysicalPlan[PartitionT]:
     """Write the results of `child_plan` into files described by `write_info`."""
-    yield from (
-        step.add_instruction(
-            execution_step.WriteFile(
-                file_format=file_format,
-                schema=schema,
+    if write_mode == WriteMode.Overwrite or write_mode == WriteMode.OverwritePartitions:
+        stage_id = next(stage_id_counter)
+        write_tasks: list[SingleOutputPartitionTask[PartitionT]] = []
+        for step in child_plan:
+            if isinstance(step, PartitionTaskBuilder):
+                step = step.add_instruction(
+                    execution_step.WriteFile(
+                        file_format=file_format,
+                        schema=schema,
+                        root_dir=root_dir,
+                        compression=compression,
+                        partition_cols=partition_cols,
+                        io_config=io_config,
+                    ),
+                ).finalize_partition_task_single_output(stage_id=stage_id)
+                write_tasks.append(step)
+            yield step
+        while any(not _.done() for _ in write_tasks):
+            yield None
+        partition_metadatas = [task.partition_metadata() for task in write_tasks]
+        inputs = [task.partition() for task in write_tasks]
+        yield PartitionTaskBuilder[PartitionT](
+            inputs=inputs, partial_metadatas=list(partition_metadatas)
+        ).add_instruction(
+            execution_step.OverwriteFiles(
+                overwrite_partitions=write_mode == WriteMode.OverwritePartitions,
                 root_dir=root_dir,
-                compression=compression,
-                partition_cols=partition_cols,
                 io_config=io_config,
-            ),
+            )
+        )
+    else:
+        yield from (
+            step.add_instruction(
+                execution_step.WriteFile(
+                    file_format=file_format,
+                    schema=schema,
+                    root_dir=root_dir,
+                    compression=compression,
+                    partition_cols=partition_cols,
+                    io_config=io_config,
+                ),
+            )
+            if isinstance(step, PartitionTaskBuilder)
+            else step
+            for step in child_plan
         )
-        if isinstance(step, PartitionTaskBuilder)
-        else step
-        for step in child_plan
-    )
 
 
 def iceberg_write(
@@ -192,6 +225,17 @@ def lance_write(
     )
 
 
+def data_sink_write(
+    child_plan: InProgressPhysicalPlan[PartitionT],
+    sink: DataSink,
+) -> InProgressPhysicalPlan[PartitionT]:
+    """Write the results of `child_plan` into a custom write sink described by `sink`."""
+    yield from (
+        step.add_instruction(execution_step.DataSinkWrite(sink)) if isinstance(step, PartitionTaskBuilder) else step
+        for step in child_plan
+    )
+
+
 def pipeline_instruction(
     child_plan: InProgressPhysicalPlan[PartitionT],
     pipeable_instruction: Instruction,
@@ -344,40 +388,44 @@ def hash_join(
     how: JoinType,
 ) -> InProgressPhysicalPlan[PartitionT]:
     """Hash-based pairwise join the partitions from `left_child_plan` and `right_child_plan` together."""
-    # Materialize the steps from the left and right sources to get partitions.
-    # As the materializations complete, emit new steps to join each left and right partition.
-    left_requests: deque[SingleOutputPartitionTask[PartitionT]] = deque()
-    right_requests: deque[SingleOutputPartitionTask[PartitionT]] = deque()
-    stage_id = next(stage_id_counter)
-    yield_left = True
+    left_tasks: dict[int, SingleOutputPartitionTask[PartitionT]] = {}
+    right_tasks: dict[int, SingleOutputPartitionTask[PartitionT]] = {}
 
-    while True:
-        # Emit new join steps if we have left and right partitions ready.
-        while (
-            len(left_requests) > 0 and len(right_requests) > 0 and left_requests[0].done() and right_requests[0].done()
-        ):
-            next_left = left_requests.popleft()
-            next_right = right_requests.popleft()
+    left_stage_id = next(stage_id_counter)
+    right_stage_id = next(stage_id_counter)
+    hash_join_stage_id = next(stage_id_counter)
 
-            # Calculate memory request for task.
-            left_size_bytes = next_left.partition_metadata().size_bytes
-            right_size_bytes = next_right.partition_metadata().size_bytes
-            if left_size_bytes is None and right_size_bytes is None:
-                size_bytes = None
-            elif left_size_bytes is None and right_size_bytes is not None:
-                # Use 2x the right side as the memory request, assuming that left and right side are ~ the same size.
-                size_bytes = 2 * right_size_bytes
-            elif right_size_bytes is None and left_size_bytes is not None:
-                # Use 2x the left side as the memory request, assuming that left and right side are ~ the same size.
-                size_bytes = 2 * left_size_bytes
-            elif left_size_bytes is not None and right_size_bytes is not None:
-                size_bytes = left_size_bytes + right_size_bytes
+    # First, fully materialize the left side of the join
+    for step in left_plan:
+        if isinstance(step, PartitionTaskBuilder):
+            step = step.finalize_partition_task_single_output(stage_id=left_stage_id)
+            left_tasks[len(left_tasks)] = step
+        yield step
 
-            join_step = PartitionTaskBuilder[PartitionT](
-                inputs=[next_left.partition(), next_right.partition()],
-                partial_metadatas=[next_left.partition_metadata(), next_right.partition_metadata()],
+    def create_join_step(
+        left_task: SingleOutputPartitionTask[PartitionT], right_task: SingleOutputPartitionTask[PartitionT]
+    ) -> SingleOutputPartitionTask[PartitionT]:
+        """Helper function to create a join step for a pair of tasks."""
+        left_size_bytes = left_task.partition_metadata().size_bytes
+        right_size_bytes = right_task.partition_metadata().size_bytes
+
+        # Calculate memory request for task
+        if left_size_bytes is None and right_size_bytes is None:
+            size_bytes = None
+        elif left_size_bytes is None and right_size_bytes is not None:
+            size_bytes = 2 * right_size_bytes  # Assume left ≈ right size
+        elif right_size_bytes is None and left_size_bytes is not None:
+            size_bytes = 2 * left_size_bytes  # Assume right ≈ left size
+        elif left_size_bytes is not None and right_size_bytes is not None:
+            size_bytes = left_size_bytes + right_size_bytes
+
+        join_step = (
+            PartitionTaskBuilder[PartitionT](
+                inputs=[left_task.partition(), right_task.partition()],
+                partial_metadatas=[left_task.partition_metadata(), right_task.partition_metadata()],
                 resource_request=ResourceRequest(memory_bytes=size_bytes),
-            ).add_instruction(
+            )
+            .add_instruction(
                 instruction=execution_step.HashJoin(
                     left_on=left_on,
                     right_on=right_on,
@@ -386,40 +434,73 @@ def hash_join(
                     is_swapped=False,
                 )
             )
-            yield join_step
+            .finalize_partition_task_single_output(stage_id=hash_join_stage_id)
+        )
+        return join_step
 
-        # Exhausted all ready inputs; execute a single child step to get more join inputs.
-        # Choose whether to execute from left child or right child (whichever one is more behind)
-        if len(left_requests) < len(right_requests):
-            next_plan, next_requests = left_plan, left_requests
-        elif len(left_requests) > len(right_requests):
-            next_plan, next_requests = right_plan, right_requests
-        elif len(left_requests) == len(right_requests):
-            # Both plans have progressed equally; alternate between the two plans to avoid starving either one
-            next_plan, next_requests = (left_plan, left_requests) if yield_left else (right_plan, right_requests)
-            yield_left = not yield_left
+    join_tasks: dict[int, SingleOutputPartitionTask[PartitionT]] = {}
+    right_partition_counter = 0
+    next_join_partition_to_emit = 0
+    while True:
+        # Check if we have any join tasks that are ready to be emitted
+        while next_join_partition_to_emit in join_tasks and join_tasks[next_join_partition_to_emit].done():
+            to_emit = join_tasks.pop(next_join_partition_to_emit)
+            size_bytes = to_emit.partition_metadata().size_bytes
+            yield PartitionTaskBuilder[PartitionT](
+                inputs=[to_emit.partition()],
+                partial_metadatas=[to_emit.partition_metadata()],
+                resource_request=ResourceRequest(memory_bytes=size_bytes),
+            )
+            next_join_partition_to_emit += 1
 
-        try:
-            step = next(next_plan)
-            if isinstance(step, PartitionTaskBuilder):
-                step = step.finalize_partition_task_single_output(stage_id=stage_id)
-                next_requests.append(step)
-            yield step
+        # Find all partitions that are ready to be joined
+        ready_partitions = [
+            partition_num
+            for partition_num in left_tasks.keys() & right_tasks.keys()  # Intersection of keys
+            if left_tasks[partition_num].done() and right_tasks[partition_num].done()
+        ]
 
-        except StopIteration:
-            # Left and right child plans have completed.
-            # Are we still waiting for materializations to complete? (We will emit more joins from them).
-            if len(left_requests) + len(right_requests) > 0:
-                logger.debug(
-                    "join blocked on completion of sources.\n Left sources: %s\nRight sources: %s",
-                    left_requests,
-                    right_requests,
-                )
-                yield None
+        if len(ready_partitions) > 0:
+            # Process all ready pairs
+            for partition in ready_partitions:
+                left_task = left_tasks.pop(partition)
+                right_task = right_tasks.pop(partition)
+                join_task = create_join_step(left_task, right_task)
+                join_tasks[partition] = join_task
+                yield join_task
+        else:
+            try:
+                # Process next right plan step
+                step = next(right_plan)
+                if isinstance(step, PartitionTaskBuilder):
+                    step = step.finalize_partition_task_single_output(stage_id=right_stage_id)
+                    right_tasks[right_partition_counter] = step
+                    right_partition_counter += 1
+                yield step
 
-            # Otherwise, we are entirely done.
-            else:
-                return
+            except StopIteration:
+                if left_tasks or right_tasks:
+                    logger.debug(
+                        "join blocked on completion of sources.\n Left sources: %s\nRight sources: %s",
+                        left_tasks,
+                        right_tasks,
+                    )
+                    yield None
+                else:
+                    break
+
+    # Emit the remaining join tasks in order of partition number
+    while len(join_tasks) > 0:
+        while not join_tasks[next_join_partition_to_emit].done():
+            yield None
+        to_emit = join_tasks.pop(next_join_partition_to_emit)
+        size_bytes = to_emit.partition_metadata().size_bytes
+        yield PartitionTaskBuilder[PartitionT](
+            inputs=[to_emit.partition()],
+            partial_metadatas=[to_emit.partition_metadata()],
+            resource_request=ResourceRequest(memory_bytes=size_bytes),
+        )
+        next_join_partition_to_emit += 1
 
 
 def _create_broadcast_join_step(
@@ -1646,6 +1727,7 @@ def sort(
             execution_step.ReduceMergeAndSort(
                 sort_by=sort_by,
                 descending=descending,
+                nulls_first=nulls_first,
                 bounds=per_part_boundaries,
             )
             for per_part_boundaries in per_partition_bounds
@@ -1653,6 +1735,30 @@ def sort(
     )
 
 
+def top_n(
+    child_plan: InProgressPhysicalPlan[PartitionT],
+    sort_by: ExpressionsProjection,
+    descending: list[bool],
+    nulls_first: list[bool],
+    limit: int,
+    num_partitions: int,
+) -> InProgressPhysicalPlan[PartitionT]:
+    """Take the top N values from the result of `child_plan` according to `sort_info` and `limit`."""
+    # TODO: The current distributed top_n implementation will perform a full sort
+    # followed by a limit. This is not optimal, but upcoming infrastructure changes
+    # to the distributed execution engine will make it easier to add a more efficient
+    # distributed top_n implementation.
+
+    child_plan = sort(
+        child_plan=child_plan,
+        sort_by=sort_by,
+        descending=descending,
+        nulls_first=nulls_first,
+        num_partitions=num_partitions,
+    )
+    yield from global_limit(child_plan=child_plan, limit_rows=limit, eager=False, num_partitions=num_partitions)
+
+
 def fanout_random(child_plan: InProgressPhysicalPlan[PartitionT], num_partitions: int):
     """Splits the results of `child_plan` randomly into a list of `node.num_partitions()` number of partitions."""
     seed = 0
@@ -1681,7 +1787,7 @@ def _best_effort_next_step(
         return (None, False)
     else:
         if isinstance(step, PartitionTaskBuilder):
-            step = step.finalize_partition_task_single_output(stage_id=stage_id)
+            step = step.finalize_partition_task_single_output(stage_id=stage_id, cache_metadata_on_done=False)
             return (step, True)
         elif isinstance(step, PartitionTask):
             return (step, False)
@@ -1771,7 +1877,7 @@ class Materialize:
             try:
                 step = next(self.child_plan)
                 if isinstance(step, PartitionTaskBuilder):
-                    step = step.finalize_partition_task_single_output(stage_id=stage_id)
+                    step = step.finalize_partition_task_single_output(stage_id=stage_id, cache_metadata_on_done=False)
                     self.materializations.append(step)
                     num_final_yielded += 1
                     logger.debug("[plan-%s] YIELDING final task (%s so far)", stage_id, num_final_yielded)
