@@ -11,6 +11,7 @@ from daft.daft import (
     PySchema,
     ResourceRequest,
     ScanTask,
+    WriteMode,
 )
 from daft.execution import execution_step, physical_plan
 from daft.expressions import Expression, ExpressionsProjection
@@ -22,7 +23,8 @@ if TYPE_CHECKING:
     from pyiceberg.schema import Schema as IcebergSchema
     from pyiceberg.table import TableProperties as IcebergTableProperties
 
-    from daft.table import MicroPartition
+    from daft.io import DataSink
+    from daft.recordbatch import MicroPartition
 
 
 def scan_with_tasks(
@@ -215,6 +217,25 @@ def sort(
     )
 
 
+def top_n(
+    input: physical_plan.InProgressPhysicalPlan[PartitionT],
+    sort_by: list[PyExpr],
+    descending: list[bool],
+    nulls_first: list[bool],
+    limit: int,
+    num_partitions: int,
+) -> physical_plan.InProgressPhysicalPlan[PartitionT]:
+    expr_projection = ExpressionsProjection([Expression._from_pyexpr(expr) for expr in sort_by])
+    return physical_plan.top_n(
+        child_plan=input,
+        sort_by=expr_projection,
+        descending=descending,
+        nulls_first=nulls_first,
+        limit=limit,
+        num_partitions=num_partitions,
+    )
+
+
 def fanout_by_hash(
     input: physical_plan.InProgressPhysicalPlan[PartitionT],
     num_partitions: int,
@@ -325,6 +346,7 @@ def broadcast_join(
 
 def write_file(
     input: physical_plan.InProgressPhysicalPlan[PartitionT],
+    write_mode: WriteMode,
     file_format: FileFormat,
     schema: PySchema,
     root_dir: str,
@@ -338,6 +360,7 @@ def write_file(
         expr_projection = None
     return physical_plan.file_write(
         input,
+        write_mode,
         file_format,
         Schema._from_pyschema(schema),
         root_dir,
@@ -393,3 +416,10 @@ def write_lance(
     kwargs: dict | None,
 ) -> physical_plan.InProgressPhysicalPlan[PartitionT]:
     return physical_plan.lance_write(input, path, mode, io_config, kwargs)
+
+
+def write_data_sink(
+    input: physical_plan.InProgressPhysicalPlan[PartitionT],
+    sink: DataSink,
+) -> physical_plan.InProgressPhysicalPlan[PartitionT]:
+    return physical_plan.data_sink_write(input, sink)
