@@ -5,10 +5,7 @@ import os
 from dataclasses import dataclass
 from enum import Enum
 
-import pyarrow as pa
-import pyarrow.fs as pafs
-import pyarrow.parquet as pq
-
+from daft.dependencies import pa, pafs, pq
 from daft.filesystem import join_path
 
 
