@@ -71,7 +71,7 @@ def _iceberg_partition_field_to_daft_partition_field(
         tfm = PartitionTransform.day()
         # pyiceberg uses date as the result type of a day transform, which is incorrect
         # so we cannot use transform.result_type() here
-        result_type = DataType.int32()
+        result_type = DataType.date()
     elif isinstance(transform, HourTransform):
         tfm = PartitionTransform.hour()
         result_type = DataType.int32()
@@ -124,10 +124,12 @@ class IcebergScanOperator(ScanOperator):
     def partitioning_keys(self) -> list[PartitionField]:
         return self._partition_keys
 
-    def _iceberg_record_to_partition_spec(self, spec: IcebergPartitionSpec, record: Record) -> daft.table.Table | None:
+    def _iceberg_record_to_partition_spec(
+        self, spec: IcebergPartitionSpec, record: Record
+    ) -> daft.recordbatch.RecordBatch | None:
         partition_fields = iceberg_partition_spec_to_fields(self._table.schema(), spec)
         arrays = dict()
-        assert len(record._position_to_field_name) == len(partition_fields)
+        assert len(record) == len(partition_fields)
         for idx, pfield in enumerate(partition_fields):
             field = Field._from_pyfield(pfield.field)
             field_name = field.name
@@ -137,7 +139,7 @@ class IcebergScanOperator(ScanOperator):
                 field_dtype
             )
         if len(arrays) > 0:
-            return daft.table.Table.from_pydict(arrays)
+            return daft.recordbatch.RecordBatch.from_pydict(arrays)
         else:
             return None
 
@@ -196,7 +198,7 @@ class IcebergScanOperator(ScanOperator):
                 size_bytes=file.file_size_in_bytes,
                 iceberg_delete_files=iceberg_delete_files,
                 pushdowns=pushdowns,
-                partition_values=pspec._table if pspec is not None else None,
+                partition_values=pspec._recordbatch if pspec is not None else None,
                 stats=None,
             )
             if st is None:
