@@ -16,18 +16,27 @@ from daft.io._json import read_json
 from daft.io._lance import read_lance
 from daft.io._parquet import read_parquet
 from daft.io._sql import read_sql
+from daft.io._warc import read_warc
+from daft.io._range import _range
 from daft.io.catalog import DataCatalogTable, DataCatalogType
 from daft.io.file_path import from_glob_path
+from daft.io.sink import DataSink, WriteOutput
+from daft.io.source import DataSource, DataSourceTask
 
 __all__ = [
     "AzureConfig",
     "DataCatalogTable",
     "DataCatalogType",
+    "DataSink",
+    "DataSource",
+    "DataSourceTask",
     "GCSConfig",
     "HTTPConfig",
     "IOConfig",
     "S3Config",
     "S3Credentials",
+    "WriteOutput",
+    "_range",
     "from_glob_path",
     "read_csv",
     "read_deltalake",
@@ -37,4 +46,5 @@ __all__ = [
     "read_lance",
     "read_parquet",
     "read_sql",
+    "read_warc",
 ]
