@@ -4,23 +4,26 @@ from typing import TYPE_CHECKING, Iterator, List, Optional
 
 from daft import context
 from daft.api_annotations import PublicAPI
-from daft.daft import IOConfig, Pushdowns, PyTable, ScanOperatorHandle, ScanTask
+from daft.daft import IOConfig, Pushdowns, PyRecordBatch, ScanOperatorHandle, ScanTask
 from daft.dataframe import DataFrame
 from daft.io.object_store_options import io_config_to_storage_options
 from daft.io.scan import PartitionField, ScanOperator
 from daft.logical.builder import LogicalPlanBuilder
 from daft.logical.schema import Schema
-from daft.table import Table
+from daft.recordbatch import RecordBatch
 
 if TYPE_CHECKING:
     import lance
 
 
 def _lancedb_table_factory_function(
-    fragment: "lance.LanceFragment", required_columns: Optional[List[str]]
-) -> Iterator["PyTable"]:
+    ds: "lance.LanceDataset", fragment_id: int, required_columns: Optional[List[str]]
+) -> Iterator["PyRecordBatch"]:
+    fragment = ds.get_fragment(fragment_id)
+    assert fragment is not None, RuntimeError(f"Unable to find lance fragment {fragment_id}")
     return (
-        Table.from_arrow_record_batches([rb], rb.schema)._table for rb in fragment.to_batches(columns=required_columns)
+        RecordBatch.from_arrow_record_batches([rb], rb.schema)._recordbatch
+        for rb in fragment.to_batches(columns=required_columns)
     )
 
 
@@ -28,28 +31,33 @@ def _lancedb_table_factory_function(
 def read_lance(url: str, io_config: Optional["IOConfig"] = None) -> DataFrame:
     """Create a DataFrame from a LanceDB table.
 
-    .. NOTE::
-        This function requires the use of `LanceDB <https://lancedb.github.io/lancedb/>`_, which is the Python
-        library for the LanceDB project.
-
-        To ensure that this is installed with Daft, you may install: ``pip install getdaft[lance]``
-
-    Example:
-    >>> df = daft.read_lance("s3://my-lancedb-bucket/data/")
-    >>> df.show()
-
     Args:
         url: URL to the LanceDB table (supports remote URLs to object stores such as `s3://` or `gs://`)
         io_config: A custom IOConfig to use when accessing LanceDB data. Defaults to None.
 
     Returns:
         DataFrame: a DataFrame with the schema converted from the specified LanceDB table
+
+    Note:
+        This function requires the use of [LanceDB](https://lancedb.github.io/lancedb/), which is the Python library for the LanceDB project.
+        To ensure that this is installed with Daft, you may install: `pip install daft[lance]`
+
+    Examples:
+        Read a local LanceDB table:
+        >>> df = daft.read_lance("s3://my-lancedb-bucket/data/")
+        >>> df.show()
+
+        Read a LanceDB table from a public S3 bucket:
+        >>> from daft.io import S3Config
+        >>> s3_config = S3Config(region="us-west-2", anonymous=True)
+        >>> df = daft.read_lance("s3://daft-public-data/lance/words-test-dataset", io_config=s3_config)
+        >>> df.show()
     """
     try:
         import lance
     except ImportError as e:
         raise ImportError(
-            "Unable to import the `lance` package, please ensure that Daft is installed with the lance extra dependency: `pip install getdaft[lance]`"
+            "Unable to import the `lance` package, please ensure that Daft is installed with the lance extra dependency: `pip install daft[lance]`"
         ) from e
 
     io_config = context.get_context().daft_planning_config.default_io_config if io_config is None else io_config
@@ -125,7 +133,7 @@ class LanceDBScanOperator(ScanOperator):
             yield ScanTask.python_factory_func_scan_task(
                 module=_lancedb_table_factory_function.__module__,
                 func_name=_lancedb_table_factory_function.__name__,
-                func_args=(fragment, required_columns),
+                func_args=(self._ds, fragment.fragment_id, required_columns),
                 schema=self.schema()._schema,
                 num_rows=num_rows,
                 size_bytes=size_bytes,
