@@ -2,6 +2,8 @@ from __future__ import annotations
 
 import logging
 
+from daft import refresh_logger
+
 
 def setup_debug_logger(
     exclude_prefix: list[str] = [],
@@ -9,6 +11,7 @@ def setup_debug_logger(
 ):
     logging.basicConfig(level="DEBUG")
     root_logger = logging.getLogger()
+    root_logger.setLevel(logging.DEBUG)
 
     if daft_only:
         for handler in root_logger.handlers:
@@ -18,3 +21,5 @@ def setup_debug_logger(
         for prefix in exclude_prefix:
             for handler in root_logger.handlers:
                 handler.addFilter(lambda record: not record.name.startswith(prefix))
+
+    refresh_logger()
