@@ -11,12 +11,13 @@ from dataclasses import dataclass
 from typing import TYPE_CHECKING, Callable, Iterator
 
 from daft.context import get_context
-from daft.daft import FileFormatConfig, FileInfos, IOConfig, ResourceRequest, SystemInfo
+from daft.daft import FileFormatConfig, FileInfos, IOConfig, LocalPhysicalPlan, ResourceRequest, SystemInfo
 from daft.execution.native_executor import NativeExecutor
 from daft.execution.physical_plan import ActorPoolManager
 from daft.expressions import ExpressionsProjection
 from daft.filesystem import glob_path_with_stats
 from daft.internal.gpu import cuda_visible_devices
+from daft.recordbatch import MicroPartition
 from daft.runners import runner_io
 from daft.runners.partitioning import (
     LocalMaterializedResult,
@@ -30,7 +31,7 @@ from daft.runners.partitioning import (
 from daft.runners.profiler import profiler
 from daft.runners.progress_bar import ProgressBar
 from daft.runners.runner import LOCAL_PARTITION_SET_CACHE, Runner
-from daft.table import MicroPartition
+from daft.scarf_telemetry import track_runner_on_scarf
 
 if TYPE_CHECKING:
     from daft.execution import physical_plan
@@ -290,11 +291,10 @@ class PyRunnerIO(runner_io.RunnerIO):
 class PyRunner(Runner[MicroPartition], ActorPoolManager):
     name = "py"
 
-    def __init__(self, use_thread_pool: bool | None) -> None:
+    def __init__(self, use_thread_pool: bool | None, num_threads: int | None) -> None:
         super().__init__()
 
         self._use_thread_pool: bool = use_thread_pool if use_thread_pool is not None else True
-        self._thread_pool = futures.ThreadPoolExecutor()
 
         # Registry of active ActorPools
         self._actor_pools: dict[str, PyActorPool] = {}
@@ -303,11 +303,12 @@ class PyRunner(Runner[MicroPartition], ActorPoolManager):
         self._inflight_futures: dict[tuple[ExecutionID, TaskID], futures.Future] = {}
 
         system_info = SystemInfo()
-        num_cpus = system_info.cpu_count()
+        num_cpus = num_threads if num_threads is not None and num_threads > 0 else system_info.cpu_count()
         if num_cpus is None:
             import multiprocessing
 
             num_cpus = multiprocessing.cpu_count()
+        self._thread_pool = futures.ThreadPoolExecutor(num_cpus)
 
         gpus = cuda_visible_devices()
         memory_bytes = system_info.total_memory()
@@ -344,6 +345,9 @@ class PyRunner(Runner[MicroPartition], ActorPoolManager):
             "Please switch to the NativeRunner now via `daft.context.set_runner_native()` or by setting the env variable `DAFT_RUNNER=native`. "
             "Please report any issues at github.com/Eventual-Inc/Daft/issues",
         )
+
+        track_runner_on_scarf(runner=self.name)
+
         # NOTE: Freeze and use this same execution config for the entire execution
         daft_execution_config = get_context().daft_execution_config
         execution_id = str(uuid.uuid4())
@@ -380,8 +384,10 @@ class PyRunner(Runner[MicroPartition], ActorPoolManager):
             if daft_execution_config.enable_native_executor:
                 logger.info("Using native executor")
 
-                executor = NativeExecutor.from_logical_plan_builder(builder)
+                executor = NativeExecutor()
+                plan = LocalPhysicalPlan.from_logical_plan_builder(builder._builder)
                 results_gen = executor.run(
+                    plan,
                     {k: v.values() for k, v in self._part_set_cache.get_all_partition_sets().items()},
                     daft_execution_config,
                     results_buffer_size,
