@@ -1,12 +1,17 @@
 from __future__ import annotations
 
-import pickle
-import random
-import statistics
-from typing import Any, Callable
+import asyncio
+from typing import TYPE_CHECKING, Any, AsyncIterator, Callable, Iterable, Union
 
 from daft.dependencies import pa
 
+if TYPE_CHECKING:
+    from daft.expressions import Expression
+
+# Column input type definitions
+ColumnInputType = Union["Expression", str]
+ManyColumnsInputType = Union[ColumnInputType, Iterable[ColumnInputType]]
+
 
 def get_arrow_version():
     return tuple(int(s) for s in pa.__version__.split(".") if s.isnumeric())
@@ -57,49 +62,13 @@ def freeze(input: dict | list | Any) -> frozenset | tuple | Any:
         return input
 
 
-def estimate_size_bytes_pylist(pylist: list) -> int:
-    """Estimate the size of this list by sampling and pickling its objects."""
-    if len(pylist) == 0:
-        return 0
-
-    # The pylist is non-empty.
-    # Sample up to 1MB or 10000 items to determine total size.
-    MAX_SAMPLE_QUANTITY = 10000
-    MAX_SAMPLE_SIZE = 1024 * 1024
-
-    sample_candidates = random.sample(pylist, min(len(pylist), MAX_SAMPLE_QUANTITY))
-
-    sampled_sizes = []
-    sample_size_allowed = MAX_SAMPLE_SIZE
-    for sample in sample_candidates:
-        size = len(pickle.dumps(sample))
-        sampled_sizes.append(size)
-        sample_size_allowed -= size
-        if sample_size_allowed <= 0:
-            break
-
-    # Sampling complete.
-    # If we ended up measuring the entire list, just return the exact value.
-    if len(sampled_sizes) == len(pylist):
-        return sum(sampled_sizes)
-
-    # Otherwise, reduce to a one-item estimate and extrapolate.
-    if len(sampled_sizes) == 1:
-        [one_item_size_estimate] = sampled_sizes
-    else:
-        mean, stdev = statistics.mean(sampled_sizes), statistics.stdev(sampled_sizes)
-        one_item_size_estimate = int(mean + stdev)
-
-    return one_item_size_estimate * len(pylist)
-
-
 def map_operator_arrow_semantics_bool(
     operator: Callable[[Any, Any], Any],
     left_pylist: list,
     right_pylist: list,
 ) -> list[bool | None]:
     return [
-        bool(operator(left, right)) if (left is not None and right is not None) else None
+        (bool(operator(left, right)) if (left is not None and right is not None) else None)
         for (left, right) in zip(left_pylist, right_pylist)
     ]
 
@@ -137,3 +106,59 @@ def pyarrow_supports_fixed_shape_tensor() -> bool:
     return hasattr(pa, "fixed_shape_tensor") and (
         (get_context().get_or_create_runner().name != "ray") or get_arrow_version() >= (13, 0, 0)
     )
+
+
+# Column utility functions
+def is_column_input(x: Any) -> bool:
+    from daft.expressions import Expression
+
+    return isinstance(x, str) or isinstance(x, Expression)
+
+
+def column_inputs_to_expressions(columns: ManyColumnsInputType) -> list[Expression]:
+    """Inputs to dataframe operations can be passed in as individual arguments or an iterable.
+
+    In addition, they may be strings or Expressions.
+    This method normalizes the inputs to a list of Expressions.
+    """
+    from daft.expressions import col
+
+    column_iter: Iterable[ColumnInputType] = [columns] if is_column_input(columns) else columns  # type: ignore
+    return [col(c) if isinstance(c, str) else c for c in column_iter]
+
+
+class SyncFromAsyncIterator:
+    """Convert an async iterator to a sync iterator.
+
+    Note that the async iterator is created lazily upon first iteration.
+    """
+
+    def __init__(self, async_iter_producer: Callable[[], AsyncIterator]):
+        self.async_iter_producer = async_iter_producer
+        self.async_iter = None
+        self.loop = asyncio.new_event_loop()
+        self.stopped = False
+
+    def __iter__(self):
+        if self.stopped:
+            raise StopIteration
+        return self
+
+    def __next__(self):
+        if self.stopped:
+            raise StopIteration
+
+        try:
+            return self.loop.run_until_complete(self._get_next())
+        except StopAsyncIteration:
+            self.stopped = True
+            self.loop.close()
+            raise StopIteration
+
+    async def _get_next(self):
+        if self.async_iter is None:
+            self.async_iter = self.async_iter_producer()
+        res = await self.async_iter.__anext__()
+        if res is None:
+            raise StopAsyncIteration
+        return res
