@@ -1,3 +1,5 @@
+use std::sync::Arc;
+
 use arrow2::types::months_days_ns;
 
 use super::as_arrow::AsArrow;
@@ -28,7 +30,7 @@ where
         let arrow_array = self.as_arrow();
         let is_valid = arrow_array
             .validity()
-            .map_or(true, |validity| validity.get_bit(idx));
+            .is_none_or(|validity| validity.get_bit(idx));
         if is_valid {
             Some(unsafe { arrow_array.value_unchecked(idx) })
         } else {
@@ -49,7 +51,7 @@ macro_rules! impl_array_arrow_get {
                 let arrow_array = self.as_arrow();
                 let is_valid = arrow_array
                     .validity()
-                    .map_or(true, |validity| validity.get_bit(idx));
+                    .is_none_or(|validity| validity.get_bit(idx));
                 if is_valid {
                     Some(unsafe { arrow_array.value_unchecked(idx) })
                 } else {
@@ -102,7 +104,7 @@ impl ExtensionArray {
         let is_valid = self
             .data
             .validity()
-            .map_or(true, |validity| validity.get_bit(idx));
+            .is_none_or(|validity| validity.get_bit(idx));
         if is_valid {
             Some(arrow2::scalar::new_scalar(self.data(), idx))
         } else {
@@ -114,7 +116,7 @@ impl ExtensionArray {
 #[cfg(feature = "python")]
 impl crate::datatypes::PythonArray {
     #[inline]
-    pub fn get(&self, idx: usize) -> pyo3::PyObject {
+    pub fn get(&self, idx: usize) -> Arc<pyo3::PyObject> {
         use arrow2::array::Array;
         use pyo3::prelude::*;
 
@@ -132,7 +134,7 @@ impl crate::datatypes::PythonArray {
         if valid {
             self.as_arrow().values().get(idx).unwrap().clone()
         } else {
-            Python::with_gil(|py| py.None())
+            Arc::new(Python::with_gil(|py| py.None()))
         }
     }
 }
