@@ -1,6 +1,6 @@
-use std::{iter::repeat, sync::Arc};
+use std::{iter::repeat_n, sync::Arc};
 
-use arrow2::offset::OffsetsBuffer;
+use arrow2::offset::{Offsets, OffsetsBuffer};
 use common_error::DaftResult;
 use indexmap::{
     map::{raw_entry_v1::RawEntryMut, RawEntryApiV1},
@@ -15,7 +15,10 @@ use crate::{
         FixedSizeListArray, ListArray, StructArray,
     },
     count_mode::CountMode,
-    datatypes::{BooleanArray, DataType, Field, Int64Array, UInt64Array, Utf8Array},
+    datatypes::{
+        try_mean_aggregation_supertype, BooleanArray, DataType, Field, Int64Array, UInt64Array,
+        Utf8Array,
+    },
     kernels::search_sorted::build_is_valid,
     prelude::MapArray,
     series::{IntoSeries, Series},
@@ -53,7 +56,7 @@ fn join_arrow_list_of_utf8s(
 // times, otherwise we simply take the original array.
 fn create_iter<'a>(arr: &'a Int64Array, len: usize) -> Box<dyn Iterator<Item = i64> + 'a> {
     match arr.len() {
-        1 => Box::new(repeat(arr.get(0).unwrap()).take(len)),
+        1 => Box::new(repeat_n(arr.get(0).unwrap(), len)),
         arr_len => {
             assert_eq!(arr_len, len);
             Box::new(arr.as_arrow().iter().map(|x| *x.unwrap()))
@@ -76,10 +79,7 @@ fn get_slices_helper(
     let mut starting_idx = parent_offsets.next().unwrap();
     for (i, ((start, end), ending_idx)) in start_iter.zip(end_iter).zip(parent_offsets).enumerate()
     {
-        let is_valid = match validity {
-            None => true,
-            Some(v) => v.get(i).unwrap(),
-        };
+        let is_valid = validity.is_none_or(|v| v.get(i).unwrap());
         let slice_start = if start >= 0 {
             starting_idx + start
         } else {
@@ -143,11 +143,11 @@ fn get_slices_helper(
 /// * `validity`    - The parent list's validity.
 /// * `size`        - The size for each chunk.
 /// * `total_elements_to_skip` - The number of elements in the Series that do not fit cleanly into
-///                              chunks. We take the fast path iff this value is 0.
+///   chunks. We take the fast path iff this value is 0.
 /// * `to_skip`     - An optional iterator of the number of elements to skip for each list. Elements
-///                   are skipped when they cannot fit into their parent list's chunks.
+///   are skipped when they cannot fit into their parent list's chunks.
 /// * `new_offsets` - The new offsets to use for the topmost list array, this is computed based on
-///                   the number of chunks extracted from each list.
+///   the number of chunks extracted from each list.
 fn get_chunks_helper(
     flat_child: &Series,
     field: Arc<Field>,
@@ -255,6 +255,31 @@ fn list_sort_helper_fixed_size(
         .collect()
 }
 
+fn general_list_fill_helper(element: &Series, num_array: &Int64Array) -> DaftResult<Vec<Series>> {
+    let num_iter = create_iter(num_array, element.len());
+    let mut result = Vec::with_capacity(element.len());
+    let element_data = element.as_physical()?;
+    for (row_index, num) in num_iter.enumerate() {
+        let list_arr = if element.is_valid(row_index) {
+            let mut list_growable = make_growable(
+                element.name(),
+                element.data_type(),
+                vec![&element_data],
+                false,
+                num as usize,
+            );
+            for _ in 0..num {
+                list_growable.extend(0, row_index, 1);
+            }
+            list_growable.build()?
+        } else {
+            Series::full_null(element.name(), element.data_type(), num as usize)
+        };
+        result.push(list_arr);
+    }
+    Ok(result)
+}
+
 impl ListArray {
     pub fn value_counts(&self) -> DaftResult<MapArray> {
         struct IndexRef {
@@ -398,7 +423,7 @@ impl ListArray {
                         .sum()
                 })
                 .collect(),
-            (CountMode::Null, None) => repeat(0).take(self.offsets().len() - 1).collect(),
+            (CountMode::Null, None) => repeat_n(0, self.offsets().len() - 1).collect(),
             (CountMode::Null, Some(validity)) => self
                 .offsets()
                 .windows(2)
@@ -456,7 +481,7 @@ impl ListArray {
         assert_eq!(self.child_data_type(), &DataType::Utf8,);
 
         let delimiter_iter: Box<dyn Iterator<Item = Option<&str>>> = if delimiter.len() == 1 {
-            Box::new(repeat(delimiter.get(0)).take(self.len()))
+            Box::new(repeat_n(delimiter.get(0), self.len()))
         } else {
             assert_eq!(delimiter.len(), self.len());
             Box::new(delimiter.as_arrow().iter())
@@ -575,8 +600,8 @@ impl ListArray {
     pub fn list_sort(&self, desc: &BooleanArray, nulls_first: &BooleanArray) -> DaftResult<Self> {
         let offsets = self.offsets();
         let child_series = if desc.len() == 1 {
-            let desc_iter = repeat(desc.get(0).unwrap()).take(self.len());
-            let nulls_first_iter = repeat(nulls_first.get(0).unwrap()).take(self.len());
+            let desc_iter = repeat_n(desc.get(0).unwrap(), self.len());
+            let nulls_first_iter = repeat_n(nulls_first.get(0).unwrap(), self.len());
             if let Some(validity) = self.validity() {
                 list_sort_helper(
                     &self.flat_child,
@@ -591,7 +616,7 @@ impl ListArray {
                     offsets,
                     desc_iter,
                     nulls_first_iter,
-                    repeat(true).take(self.len()),
+                    repeat_n(true, self.len()),
                 )?
             }
         } else {
@@ -611,13 +636,17 @@ impl ListArray {
                     offsets,
                     desc_iter,
                     nulls_first_iter,
-                    repeat(true).take(self.len()),
+                    repeat_n(true, self.len()),
                 )?
             }
         };
 
         let child_refs: Vec<&Series> = child_series.iter().collect();
-        let child = Series::concat(&child_refs)?;
+        let child = if child_refs.is_empty() {
+            Series::empty(self.name(), self.child_data_type())
+        } else {
+            Series::concat(&child_refs)?
+        };
         Ok(Self::new(
             self.field.clone(),
             child,
@@ -625,19 +654,157 @@ impl ListArray {
             self.validity().cloned(),
         ))
     }
+
+    pub fn list_fill(elem: &Series, num_array: &Int64Array) -> DaftResult<Self> {
+        let generated = general_list_fill_helper(elem, num_array)?;
+        let generated_refs: Vec<&Series> = generated.iter().collect();
+        let lengths = generated.iter().map(|arr| arr.len());
+        let offsets = Offsets::try_from_lengths(lengths)?;
+        let flat_child = if generated_refs.is_empty() {
+            // when there's no output, we should create an empty series
+            Series::empty(elem.name(), elem.data_type())
+        } else {
+            Series::concat(&generated_refs)?
+        };
+        Ok(Self::new(
+            elem.field().to_list_field()?,
+            flat_child,
+            offsets.into(),
+            None,
+        ))
+    }
+
+    pub fn list_bool_and(&self) -> DaftResult<BooleanArray> {
+        let child = &self.flat_child;
+        let offsets = self.offsets();
+        let validity = self.validity();
+
+        let mut result = Vec::with_capacity(self.len());
+        let mut result_validity = Vec::with_capacity(self.len());
+
+        for i in 0..self.len() {
+            let is_valid = validity.is_none_or(|v| v.get(i).unwrap());
+            if !is_valid {
+                result.push(false);
+                result_validity.push(false);
+                continue;
+            }
+
+            let start = *offsets.get(i).unwrap() as usize;
+            let end = *offsets.get(i + 1).unwrap() as usize;
+            let slice = child.slice(start, end)?;
+
+            // If slice is empty or all null, return null
+            if slice.is_empty()
+                || slice
+                    .validity()
+                    .is_some_and(|v| v.unset_bits() == slice.len())
+            {
+                result.push(false);
+                result_validity.push(false);
+                continue;
+            }
+
+            // Look for first non-null false value
+            let mut all_true = true;
+            let bool_slice = slice.bool()?;
+            let bool_validity = bool_slice.validity();
+            let bool_data = bool_slice.as_arrow().values();
+            for j in 0..bool_slice.len() {
+                if bool_validity.is_none_or(|v| v.get(j).unwrap()) && !bool_data.get_bit(j) {
+                    all_true = false;
+                    break;
+                }
+            }
+            result.push(all_true);
+            result_validity.push(true);
+        }
+
+        let validity_bitmap = arrow2::bitmap::Bitmap::from_iter(result_validity.iter().copied());
+        let values = arrow2::bitmap::Bitmap::from_iter(result.iter().copied());
+        let arrow_array = arrow2::array::BooleanArray::new(
+            arrow2::datatypes::DataType::Boolean,
+            values,
+            Some(validity_bitmap),
+        );
+        Ok(BooleanArray::from((self.name(), Box::new(arrow_array))))
+    }
+
+    pub fn list_bool_or(&self) -> DaftResult<BooleanArray> {
+        let child = &self.flat_child;
+        let offsets = self.offsets();
+        let validity = self.validity();
+
+        let mut result = Vec::with_capacity(self.len());
+        let mut result_validity = Vec::with_capacity(self.len());
+
+        for i in 0..self.len() {
+            let is_valid = validity.is_none_or(|v| v.get(i).unwrap());
+            if !is_valid {
+                result.push(false);
+                result_validity.push(false);
+                continue;
+            }
+
+            let start = *offsets.get(i).unwrap() as usize;
+            let end = *offsets.get(i + 1).unwrap() as usize;
+            let slice = child.slice(start, end)?;
+
+            // If slice is empty or all null, return null
+            if slice.is_empty()
+                || slice
+                    .validity()
+                    .is_some_and(|v| v.unset_bits() == slice.len())
+            {
+                result.push(false);
+                result_validity.push(false);
+                continue;
+            }
+
+            // Look for first non-null true value
+            let mut any_true = false;
+            let bool_slice = slice.bool()?;
+            let bool_validity = bool_slice.validity();
+            let bool_data = bool_slice.as_arrow().values();
+            for j in 0..bool_slice.len() {
+                if bool_validity.is_none_or(|v| v.get(j).unwrap()) && bool_data.get_bit(j) {
+                    any_true = true;
+                    break;
+                }
+            }
+            result.push(any_true);
+            result_validity.push(true);
+        }
+
+        let validity_bitmap = arrow2::bitmap::Bitmap::from_iter(result_validity.iter().copied());
+        let values = arrow2::bitmap::Bitmap::from_iter(result.iter().copied());
+        let arrow_array = arrow2::array::BooleanArray::new(
+            arrow2::datatypes::DataType::Boolean,
+            values,
+            Some(validity_bitmap),
+        );
+        Ok(BooleanArray::from((self.name(), Box::new(arrow_array))))
+    }
 }
 
 impl FixedSizeListArray {
+    pub fn list_bool_and(&self) -> DaftResult<BooleanArray> {
+        self.to_list().list_bool_and()
+    }
+
+    pub fn list_bool_or(&self) -> DaftResult<BooleanArray> {
+        self.to_list().list_bool_or()
+    }
+
     pub fn value_counts(&self) -> DaftResult<MapArray> {
-        let list = self.to_list();
-        list.value_counts()
+        self.to_list().value_counts()
     }
 
     pub fn count(&self, mode: CountMode) -> DaftResult<UInt64Array> {
         let size = self.fixed_element_len();
         let counts = match (mode, self.flat_child.validity()) {
             (CountMode::All, _) | (CountMode::Valid, None) => {
-                repeat(size as u64).take(self.len()).collect()
+                repeat_n(size as u64, self.len()).collect()
             }
             (CountMode::Valid, Some(validity)) => (0..self.len())
                 .map(|i| {
@@ -646,7 +813,7 @@ impl FixedSizeListArray {
                         .sum()
                 })
                 .collect(),
-            (CountMode::Null, None) => repeat(0).take(self.len()).collect(),
+            (CountMode::Null, None) => repeat_n(0, self.len()).collect(),
             (CountMode::Null, Some(validity)) => (0..self.len())
                 .map(|i| {
                     (0..size)
@@ -693,7 +860,7 @@ impl FixedSizeListArray {
         assert_eq!(self.child_data_type(), &DataType::Utf8,);
 
         let delimiter_iter: Box<dyn Iterator<Item = Option<&str>>> = if delimiter.len() == 1 {
-            Box::new(repeat(delimiter.get(0)).take(self.len()))
+            Box::new(repeat_n(delimiter.get(0), self.len()))
         } else {
             assert_eq!(delimiter.len(), self.len());
             Box::new(delimiter.as_arrow().iter())
@@ -766,7 +933,7 @@ impl FixedSizeListArray {
         let start_iter = create_iter(start, self.len());
         let end_iter = match end {
             Some(end) => create_iter(end, self.len()),
-            None => Box::new(repeat(list_size as i64).take(self.len())),
+            None => Box::new(repeat_n(list_size as i64, self.len())),
         };
         let new_field = Arc::new(self.field.to_exploded_field()?.to_list_field()?);
         get_slices_helper(
@@ -795,7 +962,7 @@ impl FixedSizeListArray {
         let to_skip = if total_elements_to_skip == 0 {
             None
         } else {
-            Some(std::iter::repeat(modulo).take(self.len()))
+            Some(std::iter::repeat_n(modulo, self.len()))
         };
         get_chunks_helper(
             &self.flat_child,
@@ -813,8 +980,8 @@ impl FixedSizeListArray {
         let fixed_size = self.fixed_element_len();
 
         let child_series = if desc.len() == 1 {
-            let desc_iter = repeat(desc.get(0).unwrap()).take(self.len());
-            let nulls_first_iter = repeat(nulls_first.get(0).unwrap()).take(self.len());
+            let desc_iter = repeat_n(desc.get(0).unwrap(), self.len());
+            let nulls_first_iter = repeat_n(nulls_first.get(0).unwrap(), self.len());
             if let Some(validity) = self.validity() {
                 list_sort_helper_fixed_size(
                     &self.flat_child,
@@ -829,7 +996,7 @@ impl FixedSizeListArray {
                     fixed_size,
                     desc_iter,
                     nulls_first_iter,
-                    repeat(true).take(self.len()),
+                    repeat_n(true, self.len()),
                 )?
             }
         } else {
@@ -849,13 +1016,17 @@ impl FixedSizeListArray {
                     fixed_size,
                     desc_iter,
                     nulls_first_iter,
-                    repeat(true).take(self.len()),
+                    repeat_n(true, self.len()),
                 )?
             }
         };
 
         let child_refs: Vec<&Series> = child_series.iter().collect();
-        let child = Series::concat(&child_refs)?;
+        let child = if child_refs.is_empty() {
+            Series::empty(self.name(), self.child_data_type())
+        } else {
+            Series::concat(&child_refs)?
+        };
         Ok(Self::new(
             self.field.clone(),
             child,
@@ -867,9 +1038,10 @@ impl FixedSizeListArray {
 macro_rules! impl_aggs_list_array {
     ($la:ident) => {
         impl $la {
-            fn agg_helper<T>(&self, op: T) -> DaftResult<Series>
+            fn agg_helper<T, F>(&self, op: T, target_type_getter: F) -> DaftResult<Series>
             where
                 T: Fn(&Series) -> DaftResult<Series>,
+                F: Fn(&DataType) -> DaftResult<DataType>,
             {
                 // TODO(Kevin): Currently this requires full materialization of one Series for every list. We could avoid this by implementing either sorted aggregation or an array builder
 
@@ -882,23 +1054,28 @@ macro_rules! impl_aggs_list_array {
 
                 let agg_refs: Vec<_> = aggs.iter().collect();
 
-                Series::concat(agg_refs.as_slice()).map(|s| s.rename(self.name()))
+                if agg_refs.is_empty() {
+                    let target_type = target_type_getter(self.child_data_type())?;
+                    Ok(Series::empty(self.name(), &target_type))
+                } else {
+                    Series::concat(agg_refs.as_slice()).map(|s| s.rename(self.name()))
+                }
             }
 
             pub fn sum(&self) -> DaftResult<Series> {
-                self.agg_helper(|s| s.sum(None))
+                self.agg_helper(|s| s.sum(None), |dtype| Ok(dtype.clone()))
             }
 
             pub fn mean(&self) -> DaftResult<Series> {
-                self.agg_helper(|s| s.mean(None))
+                self.agg_helper(|s| s.mean(None), try_mean_aggregation_supertype)
             }
 
             pub fn min(&self) -> DaftResult<Series> {
-                self.agg_helper(|s| s.min(None))
+                self.agg_helper(|s| s.min(None), |dtype| Ok(dtype.clone()))
             }
 
             pub fn max(&self) -> DaftResult<Series> {
-                self.agg_helper(|s| s.max(None))
+                self.agg_helper(|s| s.max(None), |dtype| Ok(dtype.clone()))
             }
         }
     };
