@@ -7,7 +7,9 @@ mod arithmetic;
 pub mod arrow2;
 pub mod as_arrow;
 mod between;
+mod binary;
 mod bitwise;
+mod bool_agg;
 pub(crate) mod broadcast;
 pub(crate) mod cast;
 mod cbrt;
@@ -47,6 +49,7 @@ mod search_sorted;
 mod shift;
 mod sign;
 mod sketch_percentile;
+mod skew;
 mod sort;
 pub(crate) mod sparse_tensor;
 mod sqrt;
@@ -75,6 +78,9 @@ pub trait DaftCompare<Rhs> {
     /// equality.
     fn equal(&self, rhs: Rhs) -> Self::Output;
 
+    /// null-safe equality.
+    fn eq_null_safe(&self, rhs: Rhs) -> Self::Output;
+
     /// inequality.
     fn not_equal(&self, rhs: Rhs) -> Self::Output;
 
@@ -236,3 +242,23 @@ pub trait DaftHllMergeAggable {
     fn hll_merge(&self) -> Self::Output;
     fn grouped_hll_merge(&self, groups: &GroupIndices) -> Self::Output;
 }
+
+pub trait DaftSetAggable {
+    type Output;
+    fn set(&self) -> Self::Output;
+    fn grouped_set(&self, groups: &GroupIndices) -> Self::Output;
+}
+
+pub trait DaftBoolAggable {
+    type Output;
+    fn bool_and(&self) -> Self::Output;
+    fn bool_or(&self) -> Self::Output;
+    fn grouped_bool_and(&self, groups: &GroupIndices) -> Self::Output;
+    fn grouped_bool_or(&self, groups: &GroupIndices) -> Self::Output;
+}
+
+pub trait DaftSkewAggable {
+    type Output;
+    fn skew(&self) -> Self::Output;
+    fn grouped_skew(&self, groups: &GroupIndices) -> Self::Output;
+}
