@@ -10,6 +10,18 @@ where
     pub fn sign(&self) -> DaftResult<Self> {
         self.apply(|v| v.signum())
     }
+
+    pub fn signum(&self) -> DaftResult<Self> {
+        self.apply(|v| v.signum())
+    }
+
+    pub fn negate(&self) -> DaftResult<Self> {
+        self.apply(|v| -v)
+    }
+
+    pub fn negative(&self) -> DaftResult<Self> {
+        self.apply(|v| -v)
+    }
 }
 
 impl<T: DaftNumericType> DataArray<T>
