@@ -1,11 +1,12 @@
 use std::{
     hash::BuildHasherDefault,
     ops::{Add, Div, Mul, Rem, Sub},
+    sync::Arc,
 };
 
 use common_arrow_ffi as ffi;
 use daft_hash::{HashFunctionKind, MurBuildHasher, Sha1Hasher};
-use daft_schema::python::PyDataType;
+use daft_schema::python::{PyDataType, PyTimeUnit};
 use pyo3::{
     exceptions::PyValueError,
     prelude::*,
@@ -15,10 +16,7 @@ use pyo3::{
 
 use crate::{
     array::{
-        ops::{
-            as_arrow::AsArrow, trigonometry::TrigonometricFunction, DaftLogical,
-            Utf8NormalizeOptions,
-        },
+        ops::{as_arrow::AsArrow, DaftLogical, Utf8NormalizeOptions},
         pseudo_arrow::PseudoArrowArray,
         DataArray,
     },
@@ -46,17 +44,22 @@ impl PySeries {
 
     // This ingests a Python list[object] directly into a Rust PythonArray.
     #[staticmethod]
-    pub fn from_pylist(name: &str, pylist: Bound<PyAny>, pyobj: &str) -> PyResult<Self> {
+    pub fn from_pylist(
+        py: Python<'_>,
+        name: &str,
+        pylist: Bound<PyAny>,
+        pyobj: &str,
+    ) -> PyResult<Self> {
         let vec_pyobj: Vec<PyObject> = pylist.extract()?;
-        let py = pylist.py();
         let dtype = match pyobj {
             "force" => DataType::Python,
             "allow" => infer_daft_dtype_for_sequence(&vec_pyobj, py, name)?.unwrap_or(DataType::Python),
             "disallow" => panic!("Cannot create a Series from a pylist and being strict about only using Arrow types by setting pyobj=disallow"),
             _ => panic!("Unsupported pyobj behavior when creating Series from pylist: {}", pyobj)
         };
+        let vec_pyobj_arced = vec_pyobj.into_iter().map(Arc::new).collect();
         let arrow_array: Box<dyn arrow2::array::Array> =
-            Box::new(PseudoArrowArray::<PyObject>::from_pyobj_vec(vec_pyobj));
+            Box::new(PseudoArrowArray::from_pyobj_vec(vec_pyobj_arced));
         let field = Field::new(name, DataType::Python);
 
         let data_array = DataArray::<PythonType>::new(field.into(), arrow_array)?;
@@ -66,17 +69,23 @@ impl PySeries {
 
     // This is for PythonArrays only,
     // to convert the Rust PythonArray to a Python list[object].
-    pub fn to_pylist(&self) -> PyResult<PyObject> {
+    pub fn to_pylist<'a>(&self, py: Python<'a>) -> PyResult<Bound<'a, PyList>> {
         let pseudo_arrow_array = self.series.python()?.as_arrow();
         let pyobj_vec = pseudo_arrow_array.to_pyobj_vec();
-        Python::with_gil(|py| Ok(PyList::new_bound(py, pyobj_vec).into()))
+
+        let pyobj_vec_cloned = pyobj_vec
+            .into_iter()
+            .map(|pyobj| pyobj.clone_ref(py))
+            .collect::<Vec<_>>();
+
+        PyList::new(py, pyobj_vec_cloned)
     }
 
     pub fn to_arrow(&self) -> PyResult<PyObject> {
         let arrow_array = self.series.to_arrow();
         let arrow_array = cast_array_from_daft_if_needed(arrow_array);
         Python::with_gil(|py| {
-            let pyarrow = py.import_bound(pyo3::intern!(py, "pyarrow"))?;
+            let pyarrow = py.import(pyo3::intern!(py, "pyarrow"))?;
             Ok(ffi::to_py_array(py, arrow_array, &pyarrow)?.unbind())
         })
     }
@@ -129,126 +138,6 @@ impl PySeries {
         Ok(self.series.floor_div(&other.series)?.into())
     }
 
-    pub fn ceil(&self) -> PyResult<Self> {
-        Ok(self.series.ceil()?.into())
-    }
-
-    pub fn floor(&self) -> PyResult<Self> {
-        Ok(self.series.floor()?.into())
-    }
-
-    pub fn sign(&self) -> PyResult<Self> {
-        Ok(self.series.sign()?.into())
-    }
-
-    pub fn round(&self, decimal: i32) -> PyResult<Self> {
-        if decimal < 0 {
-            return Err(PyValueError::new_err(format!(
-                "decimal value can not be negative: {decimal}"
-            )));
-        }
-        Ok(self.series.round(decimal)?.into())
-    }
-    pub fn clip(&self, min: &Self, max: &Self) -> PyResult<Self> {
-        Ok(self.series.clip(&min.series, &max.series)?.into())
-    }
-
-    pub fn sqrt(&self) -> PyResult<Self> {
-        Ok(self.series.sqrt()?.into())
-    }
-
-    pub fn cbrt(&self) -> PyResult<Self> {
-        Ok(self.series.cbrt()?.into())
-    }
-
-    pub fn sin(&self) -> PyResult<Self> {
-        Ok(self
-            .series
-            .trigonometry(&TrigonometricFunction::Sin)?
-            .into())
-    }
-
-    pub fn cos(&self) -> PyResult<Self> {
-        Ok(self
-            .series
-            .trigonometry(&TrigonometricFunction::Cos)?
-            .into())
-    }
-
-    pub fn tan(&self) -> PyResult<Self> {
-        Ok(self
-            .series
-            .trigonometry(&TrigonometricFunction::Tan)?
-            .into())
-    }
-
-    pub fn cot(&self) -> PyResult<Self> {
-        Ok(self
-            .series
-            .trigonometry(&TrigonometricFunction::Cot)?
-            .into())
-    }
-
-    pub fn arcsin(&self) -> PyResult<Self> {
-        Ok(self
-            .series
-            .trigonometry(&TrigonometricFunction::ArcSin)?
-            .into())
-    }
-
-    pub fn arccos(&self) -> PyResult<Self> {
-        Ok(self
-            .series
-            .trigonometry(&TrigonometricFunction::ArcCos)?
-            .into())
-    }
-
-    pub fn arctan(&self) -> PyResult<Self> {
-        Ok(self
-            .series
-            .trigonometry(&TrigonometricFunction::ArcTan)?
-            .into())
-    }
-
-    pub fn arctan2(&self, other: &Self) -> PyResult<Self> {
-        Ok(self.series.atan2(&other.series)?.into())
-    }
-
-    pub fn degrees(&self) -> PyResult<Self> {
-        Ok(self
-            .series
-            .trigonometry(&TrigonometricFunction::Degrees)?
-            .into())
-    }
-
-    pub fn radians(&self) -> PyResult<Self> {
-        Ok(self
-            .series
-            .trigonometry(&TrigonometricFunction::Radians)?
-            .into())
-    }
-
-    pub fn arctanh(&self) -> PyResult<Self> {
-        Ok(self
-            .series
-            .trigonometry(&TrigonometricFunction::ArcTanh)?
-            .into())
-    }
-
-    pub fn arccosh(&self) -> PyResult<Self> {
-        Ok(self
-            .series
-            .trigonometry(&TrigonometricFunction::ArcCosh)?
-            .into())
-    }
-
-    pub fn arcsinh(&self) -> PyResult<Self> {
-        Ok(self
-            .series
-            .trigonometry(&TrigonometricFunction::ArcSinh)?
-            .into())
-    }
-
     pub fn log2(&self) -> PyResult<Self> {
         Ok(self.series.log2()?.into())
     }
@@ -265,8 +154,8 @@ impl PySeries {
         Ok(self.series.ln()?.into())
     }
 
-    pub fn exp(&self) -> PyResult<Self> {
-        Ok(self.series.exp()?.into())
+    pub fn log1p(&self) -> PyResult<Self> {
+        Ok(self.series.log1p()?.into())
     }
 
     pub fn take(&self, idx: &Self) -> PyResult<Self> {
@@ -310,6 +199,7 @@ impl PySeries {
         Ok(self.series.argsort(descending, nulls_first)?.into())
     }
 
+    #[pyo3(signature = (seed=None))]
     pub fn hash(&self, seed: Option<Self>) -> PyResult<Self> {
         let seed_series;
         let mut seed_array = None;
@@ -417,6 +307,10 @@ impl PySeries {
         Ok((self.series).agg_list(None)?.into())
     }
 
+    pub fn agg_set(&self) -> PyResult<Self> {
+        Ok((self.series).agg_set(None)?.into())
+    }
+
     pub fn cast(&self, dtype: PyDataType) -> PyResult<Self> {
         Ok(self.series.cast(&dtype.into())?.into())
     }
@@ -567,6 +461,7 @@ impl PySeries {
         Ok(self.series.utf8_to_date(format)?.into())
     }
 
+    #[pyo3(signature = (format, timezone=None))]
     pub fn utf8_to_datetime(&self, format: &str, timezone: Option<&str>) -> PyResult<Self> {
         Ok(self.series.utf8_to_datetime(format, timezone)?.into())
     }
@@ -600,22 +495,6 @@ impl PySeries {
             .into())
     }
 
-    pub fn is_nan(&self) -> PyResult<Self> {
-        Ok(self.series.is_nan()?.into())
-    }
-
-    pub fn is_inf(&self) -> PyResult<Self> {
-        Ok(self.series.is_inf()?.into())
-    }
-
-    pub fn not_nan(&self) -> PyResult<Self> {
-        Ok(self.series.not_nan()?.into())
-    }
-
-    pub fn fill_nan(&self, fill_value: &Self) -> PyResult<Self> {
-        Ok(self.series.fill_nan(&fill_value.series)?.into())
-    }
-
     pub fn dt_date(&self) -> PyResult<Self> {
         Ok(self.series.dt_date()?.into())
     }
@@ -636,6 +515,22 @@ impl PySeries {
         Ok(self.series.dt_second()?.into())
     }
 
+    pub fn dt_millisecond(&self) -> PyResult<Self> {
+        Ok(self.series.dt_millisecond()?.into())
+    }
+
+    pub fn dt_microsecond(&self) -> PyResult<Self> {
+        Ok(self.series.dt_microsecond()?.into())
+    }
+
+    pub fn dt_nanosecond(&self) -> PyResult<Self> {
+        Ok(self.series.dt_nanosecond()?.into())
+    }
+
+    pub fn dt_unix_date(&self) -> PyResult<Self> {
+        Ok(self.series.dt_unix_date()?.into())
+    }
+
     pub fn dt_time(&self) -> PyResult<Self> {
         Ok(self.series.dt_time()?.into())
     }
@@ -644,6 +539,10 @@ impl PySeries {
         Ok(self.series.dt_month()?.into())
     }
 
+    pub fn dt_quarter(&self) -> PyResult<Self> {
+        Ok(self.series.dt_quarter()?.into())
+    }
+
     pub fn dt_year(&self) -> PyResult<Self> {
         Ok(self.series.dt_year()?.into())
     }
@@ -652,6 +551,18 @@ impl PySeries {
         Ok(self.series.dt_day_of_week()?.into())
     }
 
+    pub fn dt_day_of_month(&self) -> PyResult<Self> {
+        Ok(self.series.dt_day_of_month()?.into())
+    }
+
+    pub fn dt_day_of_year(&self) -> PyResult<Self> {
+        Ok(self.series.dt_day_of_year()?.into())
+    }
+
+    pub fn dt_week_of_year(&self) -> PyResult<Self> {
+        Ok(self.series.dt_week_of_year()?.into())
+    }
+
     pub fn dt_truncate(&self, interval: &str, relative_to: &Self) -> PyResult<Self> {
         Ok(self
             .series
@@ -659,6 +570,15 @@ impl PySeries {
             .into())
     }
 
+    pub fn dt_to_unix_epoch(&self, unit: PyTimeUnit) -> PyResult<Self> {
+        Ok(self.series.dt_to_unix_epoch(unit.timeunit)?.into())
+    }
+
+    #[pyo3(signature = (format=None))]
+    pub fn dt_strftime(&self, format: Option<&str>) -> PyResult<Self> {
+        Ok(self.series.dt_strftime(format)?.into())
+    }
+
     pub fn partitioning_days(&self) -> PyResult<Self> {
         Ok(self.series.partitioning_days()?.into())
     }
@@ -731,7 +651,7 @@ impl PySeries {
 
     pub fn _debug_bincode_serialize(&self, py: Python) -> PyResult<PyObject> {
         let values = bincode::serialize(&self.series).unwrap();
-        Ok(PyBytes::new_bound(py, &values).into())
+        Ok(PyBytes::new(py, &values).into())
     }
 
     #[staticmethod]
@@ -763,16 +683,16 @@ fn infer_daft_dtype_for_sequence(
     _name: &str,
 ) -> PyResult<Option<DataType>> {
     let py_pil_image_type = py
-        .import_bound(pyo3::intern!(py, "PIL.Image"))
+        .import(pyo3::intern!(py, "PIL.Image"))
         .and_then(|m| m.getattr(pyo3::intern!(py, "Image")));
     let np_ndarray_type = py
-        .import_bound(pyo3::intern!(py, "numpy"))
+        .import(pyo3::intern!(py, "numpy"))
         .and_then(|m| m.getattr(pyo3::intern!(py, "ndarray")));
     let np_generic_type = py
-        .import_bound(pyo3::intern!(py, "numpy"))
+        .import(pyo3::intern!(py, "numpy"))
         .and_then(|m| m.getattr(pyo3::intern!(py, "generic")));
     let from_numpy_dtype = {
-        py.import_bound(pyo3::intern!(py, "daft.datatype"))?
+        py.import(pyo3::intern!(py, "daft.datatype"))?
             .getattr(pyo3::intern!(py, "DataType"))?
             .getattr(pyo3::intern!(py, "from_numpy_dtype"))?
     };
