@@ -7,17 +7,13 @@ pub mod abs;
 pub mod agg;
 pub mod arithmetic;
 pub mod between;
+pub mod binary;
 pub mod broadcast;
 pub mod cast;
-pub mod cbrt;
-pub mod ceil;
-pub mod clip;
 pub mod comparison;
 pub mod concat;
 pub mod downcast;
-mod exp;
 pub mod filter;
-pub mod float;
 pub mod floor;
 pub mod groups;
 pub mod hash;
@@ -33,18 +29,15 @@ pub mod not;
 pub mod null;
 pub mod partitioning;
 pub mod repeat;
-pub mod round;
 pub mod search_sorted;
 pub mod shift;
-pub mod sign;
 pub mod sketch_percentile;
 pub mod sort;
-pub mod sqrt;
 pub mod struct_;
 pub mod take;
 pub mod time;
-mod trigonometry;
 pub mod utf8;
+pub mod zip;
 
 pub fn cast_series_to_supertype(series: &[&Series]) -> DaftResult<Vec<Series>> {
     let supertype = series
