@@ -15,9 +15,9 @@ use csv_async::AsyncReader;
 use daft_compression::CompressionCodec;
 use daft_core::{prelude::*, utils::arrow::cast_array_for_daft_if_needed};
 use daft_decoding::deserialize::deserialize_column;
-use daft_dsl::optimization::get_required_columns;
+use daft_dsl::{expr::bound_expr::BoundExpr, optimization::get_required_columns};
 use daft_io::{parse_url, GetResult, IOClient, IOStatsRef, SourceType};
-use daft_table::Table;
+use daft_recordbatch::RecordBatch;
 use futures::{stream::BoxStream, Stream, StreamExt, TryStreamExt};
 use rayon::{
     iter::{IndexedParallelIterator, IntoParallelIterator},
@@ -48,7 +48,7 @@ impl<S> ByteRecordChunkStream for S where
 use crate::local::{read_csv_local, stream_csv_local};
 
 type TableChunkResult =
-    super::Result<Context<JoinHandle<DaftResult<Table>>, super::JoinSnafu, super::Error>>;
+    super::Result<Context<JoinHandle<DaftResult<RecordBatch>>, super::JoinSnafu, super::Error>>;
 trait TableStream: Stream<Item = TableChunkResult> {}
 impl<S> TableStream for S where S: Stream<Item = TableChunkResult> {}
 
@@ -62,7 +62,7 @@ pub fn read_csv(
     io_stats: Option<IOStatsRef>,
     multithreaded_io: bool,
     max_chunks_in_flight: Option<usize>,
-) -> DaftResult<Table> {
+) -> DaftResult<RecordBatch> {
     let runtime_handle = get_io_runtime(multithreaded_io);
     runtime_handle.block_on_current_thread(async {
         read_csv_single_into_table(
@@ -89,7 +89,7 @@ pub fn read_csv_bulk(
     multithreaded_io: bool,
     max_chunks_in_flight: Option<usize>,
     num_parallel_tasks: usize,
-) -> DaftResult<Vec<Table>> {
+) -> DaftResult<Vec<RecordBatch>> {
     let runtime_handle = get_io_runtime(multithreaded_io);
     let tables = runtime_handle.block_on_current_thread(async move {
         // Launch a read task per URI, throttling the number of concurrent file reads to num_parallel tasks.
@@ -154,7 +154,7 @@ pub async fn stream_csv(
     io_client: Arc<IOClient>,
     io_stats: Option<IOStatsRef>,
     max_chunks_in_flight: Option<usize>,
-) -> DaftResult<BoxStream<'static, DaftResult<Table>>> {
+) -> DaftResult<BoxStream<'static, DaftResult<RecordBatch>>> {
     let uri = uri.as_str();
     let (source_type, _) = parse_url(uri)?;
     let is_compressed = CompressionCodec::from_uri(uri).is_some();
@@ -185,7 +185,7 @@ pub async fn stream_csv(
     }
 }
 
-pub fn tables_concat(mut tables: Vec<Table>) -> DaftResult<Table> {
+pub fn tables_concat(mut tables: Vec<RecordBatch>) -> DaftResult<RecordBatch> {
     if tables.is_empty() {
         return Err(DaftError::ValueError(
             "Need at least 1 Table to perform concat".to_string(),
@@ -209,17 +209,15 @@ pub fn tables_concat(mut tables: Vec<Table>) -> DaftResult<Table> {
     let new_series = (0..num_columns)
         .into_par_iter()
         .map(|i| {
-            let series_to_cat: Vec<&Series> = tables
-                .iter()
-                .map(|s| s.as_ref().get_column_by_index(i).unwrap())
-                .collect();
+            let series_to_cat: Vec<&Series> =
+                tables.iter().map(|s| s.as_ref().get_column(i)).collect();
             Series::concat(series_to_cat.as_slice())
         })
         .collect::<DaftResult<Vec<_>>>()?;
-    Table::new_with_size(
+    RecordBatch::new_with_size(
         first_table.schema.clone(),
         new_series,
-        tables.iter().map(daft_table::Table::len).sum(),
+        tables.iter().map(daft_recordbatch::RecordBatch::len).sum(),
     )
 }
 
@@ -232,7 +230,7 @@ async fn read_csv_single_into_table(
     io_client: Arc<IOClient>,
     io_stats: Option<IOStatsRef>,
     max_chunks_in_flight: Option<usize>,
-) -> DaftResult<Table> {
+) -> DaftResult<RecordBatch> {
     let (source_type, _) = parse_url(uri)?;
     let is_compressed = CompressionCodec::from_uri(uri).is_some();
     if matches!(source_type, SourceType::File) && !is_compressed {
@@ -314,13 +312,25 @@ async fn read_csv_single_into_table(
     };
 
     let schema: arrow2::datatypes::Schema = schema_fields.into();
-    let schema = Arc::new(Schema::try_from(&schema)?);
+    let schema = Arc::new(schema.into());
+
+    let predicate = predicate
+        .map(|expr| BoundExpr::try_new(expr, &schema))
+        .transpose()?;
+    let include_column_indices = include_columns
+        .map(|include_columns| {
+            include_columns
+                .iter()
+                .map(|name| schema.get_index(name))
+                .collect::<DaftResult<Vec<_>>>()
+        })
+        .transpose()?;
 
     let filtered_tables = tables.map_ok(move |table| {
         if let Some(predicate) = &predicate {
             let filtered = table?.filter(&[predicate.clone()])?;
-            if let Some(include_columns) = &include_columns {
-                filtered.get_columns(include_columns.as_slice())
+            if let Some(include_column_indices) = &include_column_indices {
+                Ok(filtered.get_columns(include_column_indices))
             } else {
                 Ok(filtered)
             }
@@ -350,7 +360,7 @@ async fn read_csv_single_into_table(
         .collect::<DaftResult<Vec<_>>>()?;
     // Handle empty table case.
     if collected_tables.is_empty() {
-        return Table::empty(Some(schema));
+        return RecordBatch::empty(Some(schema));
     }
 
     // // TODO(Clark): Don't concatenate all chunks from a file into a single table, since MicroPartition is natively chunked.
@@ -373,7 +383,7 @@ pub async fn stream_csv_single(
     io_client: Arc<IOClient>,
     io_stats: Option<IOStatsRef>,
     max_chunks_in_flight: Option<usize>,
-) -> DaftResult<impl Stream<Item = DaftResult<Table>> + Send> {
+) -> DaftResult<impl Stream<Item = DaftResult<RecordBatch>> + Send> {
     let predicate = convert_options
         .as_ref()
         .and_then(|opts| opts.predicate.clone());
@@ -428,9 +438,17 @@ pub async fn stream_csv_single(
     let filtered_tables = tables.map(move |table| {
         let table = table?;
         if let Some(predicate) = &predicate {
-            let filtered = table?.filter(&[predicate.clone()])?;
+            let table = table?;
+            let predicate = BoundExpr::try_new(predicate.clone(), &table.schema)?;
+
+            let filtered = table.filter(&[predicate])?;
             if let Some(include_columns) = &include_columns {
-                filtered.get_columns(include_columns.as_slice())
+                let include_column_indices = include_columns
+                    .iter()
+                    .map(|name| table.schema.get_index(name))
+                    .collect::<DaftResult<Vec<_>>>()?;
+
+                Ok(filtered.get_columns(&include_column_indices))
             } else {
                 Ok(filtered)
             }
@@ -633,12 +651,12 @@ fn parse_into_column_array_chunk_stream(
         .iter()
         .map(|i| fields.get(*i).unwrap().into())
         .collect::<Vec<daft_core::datatypes::Field>>();
-    let read_schema = Arc::new(daft_core::prelude::Schema::new(fields_subset)?);
+    let read_schema = Arc::new(daft_core::prelude::Schema::new(fields_subset));
     let read_daft_fields = Arc::new(
         read_schema
-            .fields
-            .values()
-            .map(|f| Arc::new(f.clone()))
+            .into_iter()
+            .cloned()
+            .map(Arc::new)
             .collect::<Vec<_>>(),
     );
 
@@ -667,7 +685,7 @@ fn parse_into_column_array_chunk_stream(
                         })
                         .collect::<DaftResult<Vec<Series>>>()?;
                     let num_rows = chunk.first().map_or(0, daft_core::series::Series::len);
-                    Ok(Table::new_unchecked(read_schema, chunk, num_rows))
+                    Ok(RecordBatch::new_unchecked(read_schema, chunk, num_rows))
                 })();
                 let _ = send.send(result);
             });
@@ -713,7 +731,7 @@ mod tests {
         utils::arrow::{cast_array_for_daft_if_needed, cast_array_from_daft_if_needed},
     };
     use daft_io::{IOClient, IOConfig};
-    use daft_table::Table;
+    use daft_recordbatch::RecordBatch;
     use rstest::rstest;
 
     use super::read_csv;
@@ -722,7 +740,7 @@ mod tests {
     #[allow(clippy::too_many_arguments)]
     fn check_equal_local_arrow2(
         path: &str,
-        out: &Table,
+        out: &RecordBatch,
         has_header: bool,
         delimiter: Option<char>,
         double_quote: bool,
@@ -774,7 +792,7 @@ mod tests {
         let schema = Schema::try_from(&schema).unwrap().to_arrow().unwrap();
         assert_eq!(out.schema.to_arrow().unwrap(), schema);
         let out_columns = (0..out.num_columns())
-            .map(|i| out.get_column_by_index(i).unwrap().to_arrow())
+            .map(|i| out.get_column(i).to_arrow())
             .collect::<Vec<_>>();
         assert_eq!(out_columns, columns);
     }
@@ -824,7 +842,7 @@ mod tests {
                 Field::new("petal.length", DataType::Float64),
                 Field::new("petal.width", DataType::Float64),
                 Field::new("variety", DataType::Utf8),
-            ])?
+            ])
             .into(),
         );
         if compression.is_none() {
@@ -886,7 +904,7 @@ mod tests {
                 Field::new("petal.length", DataType::Float64),
                 Field::new("petal.width", DataType::Float64),
                 Field::new("variety", DataType::Utf8),
-            ])?
+            ])
             .into(),
         );
         check_equal_local_arrow2(
@@ -937,7 +955,7 @@ mod tests {
                 Field::new("petal.length", DataType::Float64),
                 Field::new("petal.width", DataType::Float64),
                 Field::new("variety", DataType::Utf8),
-            ])?
+            ])
             .into(),
         );
         check_equal_local_arrow2(
@@ -988,7 +1006,7 @@ mod tests {
                 Field::new("petal.length", DataType::Float64),
                 Field::new("petal.width", DataType::Float64),
                 Field::new("variety", DataType::Utf8),
-            ])?
+            ])
             .into(),
         );
         check_equal_local_arrow2(
@@ -1038,7 +1056,7 @@ mod tests {
                 Field::new("petal.length", DataType::Float64),
                 Field::new("petal.width", DataType::Float64),
                 Field::new("variety", DataType::Utf8),
-            ])?
+            ])
             .into(),
         );
         check_equal_local_arrow2(
@@ -1086,7 +1104,7 @@ mod tests {
                 Field::new("petal.length", DataType::Float64),
                 Field::new("petal.width", DataType::Float64),
                 Field::new("variety", DataType::Utf8),
-            ])?
+            ])
             .into(),
         );
         check_equal_local_arrow2(
@@ -1134,7 +1152,7 @@ mod tests {
                 Field::new("petal.length", DataType::Float64),
                 Field::new("petal.width", DataType::Float64),
                 Field::new("variety", DataType::Utf8),
-            ])?
+            ])
             .into(),
         );
         check_equal_local_arrow2(
@@ -1181,7 +1199,7 @@ mod tests {
                 Field::new("petal.length", DataType::Float64),
                 Field::new("petal.width", DataType::Float64),
                 Field::new("variety", DataType::Utf8),
-            ])?
+            ])
             .into(),
         );
         check_equal_local_arrow2(
@@ -1229,7 +1247,7 @@ mod tests {
             Schema::new(vec![
                 Field::new("petal.length", DataType::Float64),
                 Field::new("petal.width", DataType::Float64),
-            ])?
+            ])
             .into(),
         );
         check_equal_local_arrow2(
@@ -1293,7 +1311,7 @@ mod tests {
             Schema::new(vec![
                 Field::new("petal.length", DataType::Float64),
                 Field::new("petal.width", DataType::Float64),
-            ])?
+            ])
             .into(),
         );
         check_equal_local_arrow2(
@@ -1341,7 +1359,7 @@ mod tests {
                 Field::new("petal.length", DataType::Float64),
                 Field::new("petal.width", DataType::Float64),
                 Field::new("variety", DataType::Utf8),
-            ])?
+            ])
             .into(),
         );
         check_equal_local_arrow2(
@@ -1389,7 +1407,7 @@ mod tests {
                 Field::new("petal.length", DataType::Float64),
                 Field::new("petal.width", DataType::Float64),
                 Field::new("variety", DataType::Utf8),
-            ])?
+            ])
             .into(),
         );
         check_equal_local_arrow2(
@@ -1437,7 +1455,7 @@ mod tests {
                 Field::new("petal.length", DataType::Float64),
                 Field::new("petal.width", DataType::Float64),
                 Field::new("variety", DataType::Utf8),
-            ])?
+            ])
             .into(),
         );
         check_equal_local_arrow2(
@@ -1476,7 +1494,7 @@ mod tests {
                 Field::new("petal.length", DataType::Float64),
                 Field::new("petal.width", DataType::Float64),
                 Field::new("variety", DataType::Utf8),
-            ])?
+            ])
             .into(),
         );
         check_equal_local_arrow2(
@@ -1519,10 +1537,10 @@ mod tests {
                 Field::new("petal.length", DataType::Null),
                 Field::new("petal.width", DataType::Float64),
                 Field::new("variety", DataType::Utf8),
-            ])?
+            ])
             .into(),
         );
-        let null_column = table.get_column("petal.length")?;
+        let null_column = table.get_column(2);
         assert_eq!(null_column.data_type(), &DataType::Null);
         assert_eq!(null_column.len(), 6);
         assert_eq!(
@@ -1553,7 +1571,7 @@ mod tests {
             Field::new("petal.length", DataType::Null),
             Field::new("petal.width", DataType::Float64),
             Field::new("variety", DataType::Utf8),
-        ])?;
+        ]);
 
         let table = read_csv(
             file.as_ref(),
@@ -1575,10 +1593,10 @@ mod tests {
                 Field::new("petal.length", DataType::Null),
                 Field::new("petal.width", DataType::Float64),
                 Field::new("variety", DataType::Utf8),
-            ])?
+            ])
             .into(),
         );
-        let null_column = table.get_column("petal.length")?;
+        let null_column = table.get_column(2);
         assert_eq!(null_column.data_type(), &DataType::Null);
         assert_eq!(null_column.len(), 6);
         assert_eq!(
@@ -1614,7 +1632,7 @@ mod tests {
                 Field::new("petal.length", DataType::Float64),
                 Field::new("petal.width", DataType::Float64),
                 Field::new("variety", DataType::Utf8),
-            ])?
+            ])
             .into(),
         );
         check_equal_local_arrow2(
@@ -1650,7 +1668,7 @@ mod tests {
             Field::new("petal.length", DataType::Boolean),
             Field::new("petal.width", DataType::Boolean),
             Field::new("variety", DataType::Int64),
-        ])?;
+        ]);
         let table = read_csv(
             file.as_ref(),
             Some(CsvConvertOptions::default().with_schema(Some(schema.into()))),
@@ -1665,7 +1683,7 @@ mod tests {
         assert_eq!(num_rows, 20);
         // Check that all columns are all null.
         for idx in 0..table.num_columns() {
-            let column = table.get_column_by_index(idx)?;
+            let column = table.get_column(idx);
             assert_eq!(column.to_arrow().null_count(), num_rows);
         }
 
@@ -1731,18 +1749,18 @@ mod tests {
                 Field::new("petal.length", DataType::Float64),
                 Field::new("petal.width", DataType::Float64),
                 Field::new("variety", DataType::Utf8),
-            ])?
+            ])
             .into(),
         );
 
         // First 4 cols should have no nulls
-        assert_eq!(table.get_column("sepal.length")?.to_arrow().null_count(), 0);
-        assert_eq!(table.get_column("sepal.width")?.to_arrow().null_count(), 0);
-        assert_eq!(table.get_column("petal.length")?.to_arrow().null_count(), 0);
-        assert_eq!(table.get_column("petal.width")?.to_arrow().null_count(), 0);
+        assert_eq!(table.get_column(0).to_arrow().null_count(), 0);
+        assert_eq!(table.get_column(1).to_arrow().null_count(), 0);
+        assert_eq!(table.get_column(2).to_arrow().null_count(), 0);
+        assert_eq!(table.get_column(3).to_arrow().null_count(), 0);
 
         // Last col should have 3 nulls because of the missing data
-        assert_eq!(table.get_column("variety")?.to_arrow().null_count(), 3);
+        assert_eq!(table.get_column(4).to_arrow().null_count(), 3);
 
         Ok(())
     }
@@ -1818,7 +1836,7 @@ mod tests {
                 Field::new("column_2", DataType::Float64),
                 Field::new("column_3", DataType::Float64),
                 Field::new("column_4", DataType::Float64),
-            ])?
+            ])
             .into(),
         );
 
@@ -1843,7 +1861,7 @@ mod tests {
             Field::new("petal.length", DataType::Float64),
             Field::new("petal.width", DataType::Float64),
             Field::new("variety", DataType::Utf8),
-        ])?;
+        ]);
 
         let table = read_csv(
             file.as_ref(),
@@ -1863,7 +1881,7 @@ mod tests {
         assert_eq!(table.len(), 3);
 
         assert_eq!(
-            table.get_column("variety")?.to_arrow(),
+            table.get_column(4).to_arrow(),
             Box::new(arrow2::array::Utf8Array::<i64>::from(vec![
                 None,
                 Some("Seratosa"),
@@ -1915,7 +1933,7 @@ mod tests {
             Schema::new(vec![
                 Field::new("a", DataType::Int64),
                 Field::new("b", DataType::Utf8)
-            ])?
+            ])
             .into(),
         );
 
@@ -1950,7 +1968,7 @@ mod tests {
             Schema::new(vec![
                 Field::new("a", DataType::Int64),
                 Field::new("b", DataType::Utf8)
-            ])?
+            ])
             .into(),
         );
 
@@ -1986,7 +2004,7 @@ mod tests {
         assert_eq!(table.len(), 100);
         assert_eq!(
             table.schema,
-            Schema::new(vec![Field::new("b", DataType::Utf8)])?.into(),
+            Schema::new(vec![Field::new("b", DataType::Utf8)]).into(),
         );
 
         Ok(())
@@ -2017,7 +2035,7 @@ mod tests {
             Schema::new(vec![
                 Field::new("a", DataType::Int64),
                 Field::new("b", DataType::Utf8)
-            ])?
+            ])
             .into(),
         );
 
@@ -2046,7 +2064,7 @@ mod tests {
         assert_eq!(table.len(), 100);
         assert_eq!(
             table.schema,
-            Schema::new(vec![Field::new("b", DataType::Utf8)])?.into(),
+            Schema::new(vec![Field::new("b", DataType::Utf8)]).into(),
         );
 
         Ok(())
