@@ -1,4 +1,5 @@
 pub mod agg;
+mod function_args;
 pub mod map;
 pub mod partitioning;
 pub mod python;
@@ -7,18 +8,21 @@ pub mod sketch;
 pub mod struct_;
 
 use std::{
+    collections::HashMap,
     fmt::{Display, Formatter, Result, Write},
     hash::Hash,
+    sync::{Arc, LazyLock, RwLock},
 };
 
 use common_error::DaftResult;
 use daft_core::prelude::*;
+pub use function_args::{FunctionArg, FunctionArgs};
 use python::PythonUDF;
 pub use scalar::*;
 use serde::{Deserialize, Serialize};
 
 use self::{map::MapExpr, partitioning::PartitioningExpr, sketch::SketchExpr, struct_::StructExpr};
-use crate::{Expr, ExprRef, Operator};
+use crate::ExprRef;
 
 #[derive(Debug, Clone, Serialize, Deserialize, PartialEq, Eq, Hash)]
 pub enum FunctionExpr {
@@ -106,39 +110,6 @@ pub fn function_display_without_formatter(
     Ok(f)
 }
 
-pub fn is_in_display_without_formatter(
-    expr: &ExprRef,
-    inputs: &[ExprRef],
-) -> std::result::Result<String, std::fmt::Error> {
-    let mut f = String::default();
-    write!(&mut f, "{expr} IN (")?;
-    for (i, input) in inputs.iter().enumerate() {
-        if i != 0 {
-            write!(&mut f, ", ")?;
-        }
-        write!(&mut f, "{input}")?;
-    }
-    write!(&mut f, ")")?;
-    Ok(f)
-}
-
-pub fn binary_op_display_without_formatter(
-    op: &Operator,
-    left: &ExprRef,
-    right: &ExprRef,
-) -> std::result::Result<String, std::fmt::Error> {
-    let mut f = String::default();
-    let write_out_expr = |f: &mut String, input: &Expr| match input {
-        Expr::Alias(e, _) => write!(f, "{e}"),
-        Expr::BinaryOp { .. } => write!(f, "[{input}]"),
-        _ => write!(f, "{input}"),
-    };
-    write_out_expr(&mut f, left)?;
-    write!(&mut f, " {op} ")?;
-    write_out_expr(&mut f, right)?;
-    Ok(f)
-}
-
 pub fn function_semantic_id(func: &FunctionExpr, inputs: &[ExprRef], schema: &Schema) -> FieldID {
     let inputs = inputs
         .iter()
@@ -148,3 +119,44 @@ pub fn function_semantic_id(func: &FunctionExpr, inputs: &[ExprRef], schema: &Sc
     // TODO: check for function idempotency here.
     FieldID::new(format!("Function_{func:?}({inputs})"))
 }
+
+#[derive(Default)]
+pub struct FunctionRegistry {
+    // Todo: Use the Bindings object instead, so we can get aliases and case handling.
+    map: HashMap<String, Arc<dyn ScalarUDF>>,
+}
+pub trait FunctionModule {
+    /// Register this module to the given [SQLFunctions] table.
+    fn register(_parent: &mut FunctionRegistry);
+}
+
+impl FunctionRegistry {
+    pub fn new() -> Self {
+        Self {
+            map: HashMap::new(),
+        }
+    }
+    pub fn register<Mod: FunctionModule>(&mut self) {
+        Mod::register(self);
+    }
+
+    pub fn add_fn<UDF: ScalarUDF + 'static>(&mut self, func: UDF) {
+        let func = Arc::new(func);
+        // todo: use bindings instead of hashmap so we don't need duplicate entries.
+        for alias in func.aliases() {
+            self.map.insert((*alias).to_string(), func.clone());
+        }
+        self.map.insert(func.name().to_string(), func);
+    }
+
+    pub fn get(&self, name: &str) -> Option<Arc<dyn ScalarUDF>> {
+        self.map.get(name).cloned()
+    }
+
+    pub fn entries(&self) -> impl Iterator<Item = (&String, &Arc<dyn ScalarUDF>)> {
+        self.map.iter()
+    }
+}
+
+pub static FUNCTION_REGISTRY: LazyLock<RwLock<FunctionRegistry>> =
+    LazyLock::new(|| RwLock::new(FunctionRegistry::new()));
