@@ -1,5 +1,7 @@
 #![allow(clippy::all, reason = "todo: remove; getting a rustc error")]
 
+use std::sync::Arc;
+
 use serde::{Deserialize, Serialize};
 
 /// A wrapper around PyObject that is safe to use even when the Python feature flag isn't turned on
@@ -12,9 +14,11 @@ pub struct RuntimePyObject {
 impl RuntimePyObject {
     #[cfg(feature = "test-utils")]
     pub fn new_testing_none() -> Self {
+        use std::sync::Arc;
+
         #[cfg(feature = "python")]
         {
-            let none_value = pyo3::Python::with_gil(|py| py.None());
+            let none_value = Arc::new(pyo3::Python::with_gil(|py| py.None()));
             Self {
                 obj: crate::pyobj_serde::PyObjectWrapper(none_value),
             }
@@ -26,14 +30,14 @@ impl RuntimePyObject {
     }
 
     #[cfg(feature = "python")]
-    pub fn new(value: pyo3::PyObject) -> Self {
+    pub fn new(value: Arc<pyo3::PyObject>) -> Self {
         Self {
             obj: crate::pyobj_serde::PyObjectWrapper(value),
         }
     }
 
     #[cfg(feature = "python")]
-    pub fn unwrap(self) -> pyo3::PyObject {
+    pub fn unwrap(self) -> Arc<pyo3::PyObject> {
         self.obj.0
     }
 }
@@ -49,6 +53,6 @@ impl AsRef<pyo3::PyObject> for RuntimePyObject {
 #[cfg(feature = "python")]
 impl From<pyo3::PyObject> for RuntimePyObject {
     fn from(value: pyo3::PyObject) -> Self {
-        Self::new(value)
+        Self::new(Arc::new(value))
     }
 }
