@@ -1,15 +1,19 @@
-#![feature(async_closure)]
+#![allow(
+    deprecated,
+    reason = "moving over all scalarUDFs to new pattern. Remove once completed!"
+)]
+pub mod binary;
 pub mod coalesce;
 pub mod count_matches;
 pub mod distance;
 pub mod float;
 pub mod hash;
-pub mod image;
 pub mod list;
 pub mod minhash;
 pub mod numeric;
 #[cfg(feature = "python")]
 pub mod python;
+pub mod sequence;
 pub mod temporal;
 pub mod to_struct;
 pub mod tokenize;
@@ -29,7 +33,7 @@ pub enum Error {
 
 impl From<Error> for std::io::Error {
     fn from(err: Error) -> Self {
-        Self::new(std::io::ErrorKind::Other, err)
+        Self::other(err)
     }
 }
 
@@ -38,3 +42,12 @@ impl From<Error> for DaftError {
         Self::External(err.into())
     }
 }
+
+/// TODO chore: cleanup function implementations using error macros
+#[macro_export]
+macro_rules! invalid_argument_err {
+    ($($arg:tt)*)  => {{
+        let msg = format!($($arg)*);
+        return Err(common_error::DaftError::TypeError(msg).into());
+    }};
+}
