@@ -14,8 +14,9 @@ pub struct Explode {}
 
 #[typetag::serde]
 impl ScalarUDF for Explode {
-    fn as_any(&self) -> &dyn std::any::Any {
-        self
+    fn evaluate(&self, inputs: daft_dsl::functions::FunctionArgs<Series>) -> DaftResult<Series> {
+        let inputs = inputs.into_inner();
+        self.evaluate_from_series(&inputs)
     }
 
     fn name(&self) -> &'static str {
@@ -35,7 +36,7 @@ impl ScalarUDF for Explode {
         }
     }
 
-    fn evaluate(&self, inputs: &[Series]) -> DaftResult<Series> {
+    fn evaluate_from_series(&self, inputs: &[Series]) -> DaftResult<Series> {
         match inputs {
             [input] => input.explode(),
             _ => Err(DaftError::ValueError(format!(
