@@ -1,27 +1,35 @@
+mod bool_and;
+mod bool_or;
 mod chunk;
 mod count;
+mod count_distinct;
+mod distinct;
 mod explode;
 mod get;
 mod join;
+mod list_fill;
 mod max;
 mod mean;
 mod min;
 mod slice;
 mod sort;
 mod sum;
-mod unique_count;
 mod value_counts;
 
+pub use bool_and::{list_bool_and as bool_and, ListBoolAnd};
+pub use bool_or::{list_bool_or as bool_or, ListBoolOr};
 pub use chunk::{list_chunk as chunk, ListChunk};
 pub use count::{list_count as count, ListCount};
+pub use count_distinct::{list_count_distinct as count_distinct, ListCountDistinct};
+pub use distinct::{list_distinct as distinct, ListDistinct};
 pub use explode::{explode, Explode};
 pub use get::{list_get as get, ListGet};
 pub use join::{list_join as join, ListJoin};
+pub use list_fill::list_fill;
 pub use max::{list_max as max, ListMax};
 pub use mean::{list_mean as mean, ListMean};
 pub use min::{list_min as min, ListMin};
 pub use slice::{list_slice as slice, ListSlice};
 pub use sort::{list_sort as sort, ListSort};
 pub use sum::{list_sum as sum, ListSum};
-pub use unique_count::{list_unique_count as unique_count, ListUniqueCount};
 pub use value_counts::list_value_counts as value_counts;
