@@ -1,4 +1,4 @@
-use common_error::DaftResult;
+use common_error::{ensure, DaftResult};
 use daft_core::{
     prelude::{Field, Schema},
     series::Series,
@@ -9,15 +9,18 @@ use daft_dsl::{
 };
 use serde::{Deserialize, Serialize};
 
-use super::{evaluate_single_numeric, to_field_single_numeric};
+use super::to_field_single_numeric;
 
 #[derive(Debug, Clone, Serialize, Deserialize, PartialEq, Eq, Hash)]
-pub struct Floor {}
+pub struct Floor;
 
 #[typetag::serde]
 impl ScalarUDF for Floor {
-    fn as_any(&self) -> &dyn std::any::Any {
-        self
+    fn evaluate(&self, inputs: daft_dsl::functions::FunctionArgs<Series>) -> DaftResult<Series> {
+        ensure!(inputs.len() == 1, "floor expects 1 argument");
+        let s = inputs.required((0, "input"))?;
+
+        s.floor()
     }
 
     fn name(&self) -> &'static str {
@@ -28,8 +31,8 @@ impl ScalarUDF for Floor {
         to_field_single_numeric(self, inputs, schema)
     }
 
-    fn evaluate(&self, inputs: &[Series]) -> DaftResult<Series> {
-        evaluate_single_numeric(inputs, Series::floor)
+    fn docstring(&self) -> &'static str {
+        "Rounds a number down to the nearest integer."
     }
 }
 
