@@ -11,30 +11,67 @@ use snafu::prelude::*;
 
 use crate::InvalidArgumentSnafu;
 
+/// Container for the keyword arguments of `url_download`
+/// ex:
+/// ```text
+/// url_decode(input)
+/// url_decode(input, max_connections=32)
+/// url_decode(input, on_error='raise')
+/// url_decode(input, on_error='null')
+/// url_decode(input, max_connections=32, on_error='raise')
+/// ```
 #[derive(Debug, Clone, Serialize, serde::Deserialize, PartialEq, Eq, Hash)]
-pub(super) struct DownloadFunction {
-    pub(super) max_connections: usize,
-    pub(super) raise_error_on_failure: bool,
-    pub(super) multi_thread: bool,
-    pub(super) config: Arc<IOConfig>,
+pub struct UrlDownloadArgs {
+    pub max_connections: usize,
+    pub raise_error_on_failure: bool,
+    pub multi_thread: bool,
+    pub io_config: Arc<IOConfig>,
+}
+
+impl UrlDownloadArgs {
+    pub fn new(
+        max_connections: usize,
+        raise_error_on_failure: bool,
+        multi_thread: bool,
+        io_config: Option<IOConfig>,
+    ) -> Self {
+        Self {
+            max_connections,
+            raise_error_on_failure,
+            multi_thread,
+            io_config: io_config.unwrap_or_default().into(),
+        }
+    }
+}
+
+impl Default for UrlDownloadArgs {
+    fn default() -> Self {
+        Self {
+            max_connections: 32,
+            raise_error_on_failure: true,
+            multi_thread: true,
+            io_config: IOConfig::default().into(),
+        }
+    }
 }
 
 #[typetag::serde]
-impl ScalarUDF for DownloadFunction {
-    fn as_any(&self) -> &dyn std::any::Any {
-        self
+impl ScalarUDF for UrlDownloadArgs {
+    fn evaluate(&self, inputs: daft_dsl::functions::FunctionArgs<Series>) -> DaftResult<Series> {
+        let inner = inputs.into_inner();
+        self.evaluate_from_series(&inner)
     }
 
     fn name(&self) -> &'static str {
         "download"
     }
 
-    fn evaluate(&self, inputs: &[Series]) -> DaftResult<Series> {
+    fn evaluate_from_series(&self, inputs: &[Series]) -> DaftResult<Series> {
         let Self {
             max_connections,
             raise_error_on_failure,
             multi_thread,
-            config,
+            io_config,
         } = self;
 
         match inputs {
@@ -47,7 +84,7 @@ impl ScalarUDF for DownloadFunction {
                         *max_connections,
                         *raise_error_on_failure,
                         *multi_thread,
-                        config.clone(),
+                        io_config.clone(),
                         Some(io_stats),
                     )?;
                     Ok(result.into_series())
