@@ -12,11 +12,13 @@ use google_cloud_storage::{
     },
 };
 use google_cloud_token::{TokenSource, TokenSourceProvider};
+use regex::Regex;
 use snafu::{IntoError, ResultExt, Snafu};
 use tokio::sync::Semaphore;
 
 use crate::{
     object_io::{FileMetadata, FileType, LSResult, ObjectSource},
+    retry::{ExponentialBackoff, RetryError},
     stats::IOStatsRef,
     stream_utils::io_stats_on_bytestream,
     FileFormat, GetResult,
@@ -37,11 +39,6 @@ enum Error {
     #[snafu(display("Unable to read data from {}: {}", path, source))]
     UnableToReadBytes { path: String, source: GError },
 
-    #[snafu(display("Unable to parse URL: \"{}\"", path))]
-    InvalidUrl {
-        path: String,
-        source: url::ParseError,
-    },
     #[snafu(display("Unable to load Credentials: {}", source))]
     UnableToLoadCredentials {
         source: google_cloud_storage::client::google_cloud_auth::error::Error,
@@ -62,8 +59,8 @@ enum Error {
 impl From<Error> for super::Error {
     fn from(error: Error) -> Self {
         use Error::{
-            InvalidUrl, NotAFile, NotFound, UnableToCreateClient, UnableToGrabSemaphore,
-            UnableToListObjects, UnableToLoadCredentials, UnableToOpenFile, UnableToReadBytes,
+            NotAFile, NotFound, UnableToCreateClient, UnableToGrabSemaphore, UnableToListObjects,
+            UnableToLoadCredentials, UnableToOpenFile, UnableToReadBytes,
         };
         match error {
             UnableToReadBytes { path, source }
@@ -128,7 +125,6 @@ impl From<Error> for super::Error {
                 path: path.into(),
                 source: error.into(),
             },
-            InvalidUrl { path, source } => Self::InvalidUrl { path, source },
             UnableToLoadCredentials { source } => Self::UnableToLoadCredentials {
                 store: super::SourceType::GCS,
                 source: source.into(),
@@ -154,17 +150,19 @@ struct GCSClientWrapper {
     connection_pool_sema: Arc<Semaphore>,
 }
 
-fn parse_uri(uri: &url::Url) -> super::Result<(&str, &str)> {
-    let bucket = match uri.host_str() {
-        Some(s) => Ok(s),
-        None => Err(Error::InvalidUrl {
-            path: uri.to_string(),
-            source: url::ParseError::EmptyHost,
-        }),
-    }?;
-    let key = uri.path();
-    let key = key.strip_prefix(GCS_DELIMITER).unwrap_or(key);
-    Ok((bucket, key))
+fn parse_raw_uri(uri: &str) -> super::Result<(&str, &str)> {
+    // We use regex here instead of the more robust url crate because we do not want to handle character escaping
+    // which is done by `google_cloud_storage::client::Client` already
+    let re = Regex::new(r"^gs://([^/]+)(?:/(.*))?$").unwrap();
+
+    if let Some(cap) = re.captures(uri) {
+        let bucket = cap.get(1).unwrap().as_str();
+        let key = cap.get(2).map_or("", |key| key.as_str());
+
+        Ok((bucket, key))
+    } else {
+        Err(Error::NotAFile { path: uri.into() }.into())
+    }
 }
 
 impl GCSClientWrapper {
@@ -174,8 +172,7 @@ impl GCSClientWrapper {
         range: Option<Range<usize>>,
         io_stats: Option<IOStatsRef>,
     ) -> super::Result<GetResult> {
-        let uri = url::Url::parse(uri).with_context(|_| InvalidUrlSnafu { path: uri })?;
-        let (bucket, key) = parse_uri(&uri)?;
+        let (bucket, key) = parse_raw_uri(uri)?;
         if key.is_empty() {
             return Err(Error::NotAFile { path: uri.into() }.into());
         }
@@ -226,8 +223,7 @@ impl GCSClientWrapper {
     }
 
     async fn get_size(&self, uri: &str, io_stats: Option<IOStatsRef>) -> super::Result<usize> {
-        let uri = url::Url::parse(uri).with_context(|_| InvalidUrlSnafu { path: uri })?;
-        let (bucket, key) = parse_uri(&uri)?;
+        let (bucket, key) = parse_raw_uri(uri)?;
         if key.is_empty() {
             return Err(Error::NotAFile { path: uri.into() }.into());
         }
@@ -257,7 +253,7 @@ impl GCSClientWrapper {
         Ok(response.size as usize)
     }
     #[allow(clippy::too_many_arguments)]
-    async fn _ls_impl(
+    async fn ls_impl(
         &self,
         client: &Client,
         bucket: &str,
@@ -315,8 +311,7 @@ impl GCSClientWrapper {
         page_size: Option<i32>,
         io_stats: Option<IOStatsRef>,
     ) -> super::Result<LSResult> {
-        let uri = url::Url::parse(path).with_context(|_| InvalidUrlSnafu { path })?;
-        let (bucket, key) = parse_uri(&uri)?;
+        let (bucket, key) = parse_raw_uri(path)?;
 
         let _permit = self
             .connection_pool_sema
@@ -334,7 +329,7 @@ impl GCSClientWrapper {
                 format!("{}{GCS_DELIMITER}", key.trim_end_matches(GCS_DELIMITER))
             };
             let forced_directory_ls_result = self
-                ._ls_impl(
+                .ls_impl(
                     client,
                     bucket,
                     forced_directory_key.as_str(),
@@ -349,7 +344,7 @@ impl GCSClientWrapper {
             // details as the one-and-only-one entry
             if forced_directory_ls_result.files.is_empty() {
                 let mut file_result = self
-                    ._ls_impl(
+                    .ls_impl(
                         client,
                         bucket,
                         key,
@@ -377,7 +372,7 @@ impl GCSClientWrapper {
                 Ok(forced_directory_ls_result)
             }
         } else {
-            self._ls_impl(
+            self.ls_impl(
                 client,
                 bucket,
                 key,
@@ -438,8 +433,28 @@ impl GCSSource {
                 ..Default::default()
             }
         } else {
-            let attempted = ClientConfig::default()
-                .with_auth()
+            let backoff = ExponentialBackoff::default();
+
+            let get_client_config = async || {
+                ClientConfig::default().with_auth().await.map_err(|e| {
+                    use google_cloud_storage::client::google_cloud_auth::error::Error::HttpError;
+
+                    // retry when we may receive an error from hitting the credentials server too much
+                    if let HttpError(reqwest_err) = &e
+                        && (reqwest_err.is_request()
+                            || reqwest_err.is_timeout()
+                            || matches!(reqwest_err.status().map(|s| s.as_u16()), Some(429) | Some(500..599)))
+                    {
+                        log::warn!("Encountered HTTP error while attempting to fetch Google Cloud Storage client: {reqwest_err}. Retrying...");
+                        RetryError::Transient(e)
+                    } else {
+                        RetryError::Permanent(e)
+                    }
+                })
+            };
+
+            let attempted = backoff
+                .retry(get_client_config)
                 .await
                 .context(UnableToLoadCredentialsSnafu {});
 
