@@ -1,35 +1,39 @@
-#![feature(async_closure)]
 #![feature(let_chains)]
 #![feature(io_error_more)]
 #![feature(if_let_guard)]
 mod azure_blob;
+mod counting_reader;
 mod google_cloud;
 mod http;
 mod huggingface;
 mod local;
 mod object_io;
 mod object_store_glob;
+mod retry;
 mod s3_like;
 mod stats;
 mod stream_utils;
 
+use std::sync::LazyLock;
+
 use azure_blob::AzureBlobSource;
 use common_file_formats::FileFormat;
+pub use counting_reader::CountingReader;
 use google_cloud::GCSSource;
 use huggingface::HFSource;
-use lazy_static::lazy_static;
 #[cfg(feature = "python")]
 pub mod python;
 
 use std::{borrow::Cow, collections::HashMap, hash::Hash, ops::Range, sync::Arc};
 
 use common_error::{DaftError, DaftResult};
-pub use common_io_config::{AzureConfig, IOConfig, S3Config};
+pub use common_io_config::{AzureConfig, GCSConfig, HTTPConfig, IOConfig, S3Config};
 use futures::stream::BoxStream;
 use object_io::StreamingRetryParams;
 pub use object_io::{FileMetadata, GetResult};
 #[cfg(feature = "python")]
 pub use python::register_modules;
+pub use s3_like::s3_config_from_env;
 use s3_like::S3LikeSource;
 use snafu::{prelude::*, Snafu};
 pub use stats::{IOStatsContext, IOStatsRef};
@@ -173,7 +177,7 @@ impl From<Error> for DaftError {
 
 impl From<Error> for std::io::Error {
     fn from(err: Error) -> Self {
-        Self::new(std::io::ErrorKind::Other, err)
+        Self::other(err)
     }
 }
 
@@ -317,9 +321,9 @@ impl IOClient {
                     Err(err)
                 } else {
                     log::warn!(
-                    "Error occurred during url_download at index: {index} {} (falling back to Null)",
-                    err
-                );
+                        "Error occurred during url_download at index: {index} {} (falling back to Null)",
+                        err
+                    );
                     Ok(None)
                 }
             }
@@ -332,6 +336,7 @@ impl IOClient {
         index: usize,
         dest: String,
         data: Option<bytes::Bytes>,
+        raise_error_on_failure: bool,
         io_stats: Option<IOStatsRef>,
     ) -> Result<Option<String>> {
         let value = if let Some(data) = data {
@@ -344,11 +349,15 @@ impl IOClient {
         match value {
             Some(Ok(())) => Ok(Some(dest)),
             Some(Err(err)) => {
-                log::warn!(
-                    "Error occurred during file upload at index: {index} {} (falling back to Null)",
-                    err
-                );
-                Err(err)
+                if raise_error_on_failure {
+                    Err(err)
+                } else {
+                    log::warn!(
+                        "Error occurred during file upload at index: {index} {} (falling back to Null)",
+                        err
+                    );
+                    Ok(None)
+                }
             }
             None => Ok(None),
         }
@@ -422,10 +431,8 @@ pub fn parse_url(input: &str) -> Result<(SourceType, Cow<'_, str>)> {
 }
 type CacheKey = (bool, Arc<IOConfig>);
 
-lazy_static! {
-    static ref CLIENT_CACHE: std::sync::RwLock<HashMap<CacheKey, Arc<IOClient>>> =
-        std::sync::RwLock::new(HashMap::new());
-}
+static CLIENT_CACHE: LazyLock<std::sync::RwLock<HashMap<CacheKey, Arc<IOClient>>>> =
+    LazyLock::new(|| std::sync::RwLock::new(HashMap::new()));
 
 pub fn get_io_client(multi_thread: bool, config: Arc<IOConfig>) -> DaftResult<Arc<IOClient>> {
     let read_handle = CLIENT_CACHE.read().unwrap();
