@@ -1,15 +1,16 @@
 use std::sync::Arc;
 
-use common_runtime::RuntimeRef;
 use daft_dsl::ExprRef;
 use daft_functions::list::explode;
 use daft_micropartition::MicroPartition;
-use tracing::instrument;
+use itertools::Itertools;
+use tracing::{instrument, Span};
 
 use super::intermediate_op::{
     IntermediateOpExecuteResult, IntermediateOpState, IntermediateOperator,
     IntermediateOperatorResult,
 };
+use crate::ExecutionTaskSpawner;
 
 pub struct ExplodeOperator {
     to_explode: Arc<Vec<ExprRef>>,
@@ -29,21 +30,31 @@ impl IntermediateOperator for ExplodeOperator {
         &self,
         input: Arc<MicroPartition>,
         state: Box<dyn IntermediateOpState>,
-        runtime: &RuntimeRef,
+        task_spawner: &ExecutionTaskSpawner,
     ) -> IntermediateOpExecuteResult {
         let to_explode = self.to_explode.clone();
-        runtime
-            .spawn(async move {
-                let out = input.explode(&to_explode)?;
-                Ok((
-                    state,
-                    IntermediateOperatorResult::NeedMoreInput(Some(Arc::new(out))),
-                ))
-            })
+        task_spawner
+            .spawn(
+                async move {
+                    let out = input.explode(&to_explode)?;
+                    Ok((
+                        state,
+                        IntermediateOperatorResult::NeedMoreInput(Some(Arc::new(out))),
+                    ))
+                },
+                Span::current(),
+            )
             .into()
     }
 
+    fn multiline_display(&self) -> Vec<String> {
+        vec![format!(
+            "Explode: {}",
+            self.to_explode.iter().map(|e| e.to_string()).join(", ")
+        )]
+    }
+
     fn name(&self) -> &'static str {
-        "ExplodeOperator"
+        "Explode"
     }
 }
