@@ -2,7 +2,8 @@ use std::sync::Arc;
 
 use common_display::tree::TreeDisplay;
 use common_error::DaftResult;
-use common_runtime::{get_compute_runtime, RuntimeRef};
+use common_runtime::{get_compute_pool_num_threads, get_compute_runtime};
+use daft_logical_plan::stats::StatsState;
 use daft_micropartition::MicroPartition;
 use snafu::ResultExt;
 use tracing::{info_span, instrument};
@@ -15,8 +16,9 @@ use crate::{
     dispatcher::{DispatchSpawner, RoundRobinDispatcher, UnorderedDispatcher},
     pipeline::PipelineNode,
     progress_bar::ProgressBarColor,
+    resource_manager::MemoryManager,
     runtime_stats::{CountingReceiver, CountingSender, RuntimeStatsContext},
-    ExecutionRuntimeContext, OperatorOutput, PipelineExecutionSnafu, NUM_CPUS,
+    ExecutionRuntimeContext, ExecutionTaskSpawner, OperatorOutput, PipelineExecutionSnafu,
 };
 
 pub(crate) trait IntermediateOpState: Send + Sync {
@@ -42,9 +44,10 @@ pub trait IntermediateOperator: Send + Sync {
         &self,
         input: Arc<MicroPartition>,
         state: Box<dyn IntermediateOpState>,
-        runtime: &RuntimeRef,
+        task_spawner: &ExecutionTaskSpawner,
     ) -> IntermediateOpExecuteResult;
     fn name(&self) -> &'static str;
+    fn multiline_display(&self) -> Vec<String>;
     fn make_state(&self) -> DaftResult<Box<dyn IntermediateOpState>> {
         Ok(Box::new(DefaultIntermediateOperatorState {}))
     }
@@ -52,7 +55,11 @@ pub trait IntermediateOperator: Send + Sync {
     /// Each worker will has its own IntermediateOperatorState.
     /// This method should be overridden if the operator needs to limit the number of concurrent workers, i.e. UDFs with resource requests.
     fn max_concurrency(&self) -> DaftResult<usize> {
-        Ok(*NUM_CPUS)
+        Ok(get_compute_pool_num_threads())
+    }
+
+    fn morsel_size(&self, runtime_handle: &ExecutionRuntimeContext) -> Option<usize> {
+        Some(runtime_handle.default_morsel_size())
     }
 
     fn dispatch_spawner(
@@ -61,13 +68,9 @@ pub trait IntermediateOperator: Send + Sync {
         maintain_order: bool,
     ) -> Arc<dyn DispatchSpawner> {
         if maintain_order {
-            Arc::new(RoundRobinDispatcher::new(Some(
-                runtime_handle.default_morsel_size(),
-            )))
+            Arc::new(RoundRobinDispatcher::new(self.morsel_size(runtime_handle)))
         } else {
-            Arc::new(UnorderedDispatcher::new(Some(
-                runtime_handle.default_morsel_size(),
-            )))
+            Arc::new(UnorderedDispatcher::new(self.morsel_size(runtime_handle)))
         }
     }
 }
@@ -76,26 +79,30 @@ pub struct IntermediateNode {
     intermediate_op: Arc<dyn IntermediateOperator>,
     children: Vec<Box<dyn PipelineNode>>,
     runtime_stats: Arc<RuntimeStatsContext>,
+    plan_stats: StatsState,
 }
 
 impl IntermediateNode {
     pub(crate) fn new(
         intermediate_op: Arc<dyn IntermediateOperator>,
         children: Vec<Box<dyn PipelineNode>>,
+        plan_stats: StatsState,
     ) -> Self {
-        let rts = RuntimeStatsContext::new();
-        Self::new_with_runtime_stats(intermediate_op, children, rts)
+        let rts = RuntimeStatsContext::new(intermediate_op.name());
+        Self::new_with_runtime_stats(intermediate_op, children, rts, plan_stats)
     }
 
     pub(crate) fn new_with_runtime_stats(
         intermediate_op: Arc<dyn IntermediateOperator>,
         children: Vec<Box<dyn PipelineNode>>,
         runtime_stats: Arc<RuntimeStatsContext>,
+        plan_stats: StatsState,
     ) -> Self {
         Self {
             intermediate_op,
             children,
             runtime_stats,
+            plan_stats,
         }
     }
 
@@ -109,17 +116,16 @@ impl IntermediateNode {
         receiver: Receiver<Arc<MicroPartition>>,
         sender: Sender<Arc<MicroPartition>>,
         rt_context: Arc<RuntimeStatsContext>,
+        memory_manager: Arc<MemoryManager>,
     ) -> DaftResult<()> {
         let span = info_span!("IntermediateOp::execute");
         let compute_runtime = get_compute_runtime();
+        let task_spawner =
+            ExecutionTaskSpawner::new(compute_runtime, memory_manager, rt_context, span);
         let mut state = op.make_state()?;
         while let Some(morsel) = receiver.recv().await {
             loop {
-                let result = rt_context
-                    .in_span(&span, || {
-                        op.execute(morsel.clone(), state, &compute_runtime)
-                    })
-                    .await??;
+                let result = op.execute(morsel.clone(), state, &task_spawner).await??;
                 state = result.0;
                 match result.1 {
                     IntermediateOperatorResult::NeedMoreInput(Some(mp)) => {
@@ -147,6 +153,7 @@ impl IntermediateNode {
         input_receivers: Vec<Receiver<Arc<MicroPartition>>>,
         runtime_handle: &mut ExecutionRuntimeContext,
         maintain_order: bool,
+        memory_manager: Arc<MemoryManager>,
     ) -> OrderingAwareReceiver<Arc<MicroPartition>> {
         let (output_sender, output_receiver) =
             create_ordering_aware_receiver_channel(maintain_order, input_receivers.len());
@@ -157,6 +164,7 @@ impl IntermediateNode {
                     input_receiver,
                     output_sender,
                     self.runtime_stats.clone(),
+                    memory_manager.clone(),
                 ),
                 self.intermediate_op.name(),
             );
@@ -169,12 +177,24 @@ impl TreeDisplay for IntermediateNode {
     fn display_as(&self, level: common_display::DisplayLevel) -> String {
         use std::fmt::Write;
         let mut display = String::new();
-        writeln!(display, "{}", self.intermediate_op.name()).unwrap();
-        use common_display::DisplayLevel::Compact;
-        if matches!(level, Compact) {
-        } else {
-            let rt_result = self.runtime_stats.result();
-            rt_result.display(&mut display, true, true, true).unwrap();
+
+        use common_display::DisplayLevel;
+        match level {
+            DisplayLevel::Compact => {
+                writeln!(display, "{}", self.intermediate_op.name()).unwrap();
+            }
+            level => {
+                let multiline_display = self.intermediate_op.multiline_display().join("\n");
+                writeln!(display, "{}", multiline_display).unwrap();
+                if let StatsState::Materialized(stats) = &self.plan_stats {
+                    writeln!(display, "Stats = {}", stats).unwrap();
+                }
+                if matches!(level, DisplayLevel::Verbose) {
+                    writeln!(display).unwrap();
+                    let rt_result = self.runtime_stats.result();
+                    rt_result.display(&mut display, true, true, true).unwrap();
+                }
+            }
         }
         display
     }
@@ -220,7 +240,7 @@ impl PipelineNode for IntermediateNode {
         let num_workers = op.max_concurrency().context(PipelineExecutionSnafu {
             node_name: self.name(),
         })?;
-        let (destination_sender, destination_receiver) = create_channel(1);
+        let (destination_sender, destination_receiver) = create_channel(0);
         let counting_sender =
             CountingSender::new(destination_sender, self.runtime_stats.clone(), progress_bar);
 
@@ -241,6 +261,7 @@ impl PipelineNode for IntermediateNode {
             spawned_dispatch_result.worker_receivers,
             runtime_handle,
             maintain_order,
+            runtime_handle.memory_manager(),
         );
         runtime_handle.spawn(
             async move {
