@@ -1,20 +1,21 @@
 use std::sync::Arc;
 
 use common_daft_config::DaftExecutionConfig;
-use common_display::{mermaid::MermaidDisplayVisitor, tree::TreeDisplay};
-use common_error::DaftResult;
-use common_file_formats::FileFormat;
-use daft_core::{
-    datatypes::Field,
-    join::JoinSide,
-    prelude::{Schema, SchemaRef},
-    utils::supertype,
+use common_display::{
+    ascii::fmt_tree_gitstyle,
+    mermaid::{MermaidDisplayVisitor, SubgraphOptions},
+    tree::TreeDisplay,
+    DisplayLevel,
 };
-use daft_dsl::{col, join::get_common_join_keys};
+use common_error::{DaftError, DaftResult};
+use common_file_formats::FileFormat;
+use daft_core::{join::JoinSide, prelude::Schema};
+use daft_dsl::{join::get_common_join_cols, resolved_col};
 use daft_local_plan::{
     ActorPoolProject, Concat, CrossJoin, EmptyScan, Explode, Filter, HashAggregate, HashJoin,
     InMemoryScan, Limit, LocalPhysicalPlan, MonotonicallyIncreasingId, PhysicalWrite, Pivot,
-    Project, Sample, Sort, UnGroupedAggregate, Unpivot,
+    Project, Sample, Sort, TopN, UnGroupedAggregate, Unpivot, WindowOrderByOnly,
+    WindowPartitionAndDynamicFrame, WindowPartitionAndOrderBy, WindowPartitionOnly,
 };
 use daft_logical_plan::{stats::StatsState, JoinType};
 use daft_micropartition::{
@@ -29,14 +30,14 @@ use snafu::ResultExt;
 use crate::{
     channel::Receiver,
     intermediate_ops::{
-        actor_pool_project::ActorPoolProjectOperator,
-        anti_semi_hash_join_probe::AntiSemiProbeOperator, cross_join::CrossJoinOperator,
+        actor_pool_project::ActorPoolProjectOperator, cross_join::CrossJoinOperator,
         explode::ExplodeOperator, filter::FilterOperator,
         inner_hash_join_probe::InnerHashJoinProbeOperator, intermediate_op::IntermediateNode,
         project::ProjectOperator, sample::SampleOperator, unpivot::UnpivotOperator,
     },
     sinks::{
         aggregate::AggregateSink,
+        anti_semi_hash_join_probe::AntiSemiProbeSink,
         blocking_sink::BlockingSinkNode,
         concat::ConcatSink,
         cross_join_collect::CrossJoinCollectSink,
@@ -48,9 +49,14 @@ use crate::{
         pivot::PivotSink,
         sort::SortSink,
         streaming_sink::StreamingSinkNode,
+        top_n::TopNSink,
+        window_order_by_only::WindowOrderByOnlySink,
+        window_partition_and_dynamic_frame::WindowPartitionAndDynamicFrameSink,
+        window_partition_and_order_by::WindowPartitionAndOrderBySink,
+        window_partition_only::WindowPartitionOnlySink,
         write::{WriteFormat, WriteSink},
     },
-    sources::{empty_scan::EmptyScanSource, in_memory::InMemorySource},
+    sources::{empty_scan::EmptyScanSource, in_memory::InMemorySource, source::SourceNode},
     state_bridge::BroadcastStateBridge,
     ExecutionRuntimeContext, PipelineCreationSnafu,
 };
@@ -67,18 +73,30 @@ pub(crate) trait PipelineNode: Sync + Send + TreeDisplay {
     fn as_tree_display(&self) -> &dyn TreeDisplay;
 }
 
-pub fn viz_pipeline(root: &dyn PipelineNode) -> String {
+pub fn viz_pipeline_mermaid(
+    root: &dyn PipelineNode,
+    display_type: DisplayLevel,
+    bottom_up: bool,
+    subgraph_options: Option<SubgraphOptions>,
+) -> String {
     let mut output = String::new();
-    let mut visitor = MermaidDisplayVisitor::new(
-        &mut output,
-        common_display::DisplayLevel::Default,
-        true,
-        Default::default(),
-    );
+    let mut visitor =
+        MermaidDisplayVisitor::new(&mut output, display_type, bottom_up, subgraph_options);
     visitor.fmt(root.as_tree_display()).unwrap();
     output
 }
 
+pub fn viz_pipeline_ascii(root: &dyn PipelineNode, simple: bool) -> String {
+    let mut s = String::new();
+    let level = if simple {
+        DisplayLevel::Compact
+    } else {
+        DisplayLevel::Default
+    };
+    fmt_tree_gitstyle(root.as_tree_display(), 0, &mut s, level).unwrap();
+    s
+}
+
 pub fn physical_plan_to_pipeline(
     physical_plan: &LocalPhysicalPlan,
     psets: &(impl PartitionSetCache<MicroPartitionRef, Arc<MicroPartitionSet>> + ?Sized),
@@ -88,15 +106,18 @@ pub fn physical_plan_to_pipeline(
 
     use crate::sources::scan_task::ScanTaskSource;
     let out: Box<dyn PipelineNode> = match physical_plan {
-        LocalPhysicalPlan::EmptyScan(EmptyScan { schema, .. }) => {
+        LocalPhysicalPlan::EmptyScan(EmptyScan {
+            schema,
+            stats_state,
+        }) => {
             let source = EmptyScanSource::new(schema.clone());
-            source.arced().into()
+            SourceNode::new(source.arced(), stats_state.clone()).boxed()
         }
         LocalPhysicalPlan::PhysicalScan(PhysicalScan {
             scan_tasks,
             pushdowns,
             schema,
-            ..
+            stats_state,
         }) => {
             let scan_tasks = scan_tasks
                 .iter()
@@ -105,75 +126,220 @@ pub fn physical_plan_to_pipeline(
 
             let scan_task_source =
                 ScanTaskSource::new(scan_tasks, pushdowns.clone(), schema.clone(), cfg);
-            scan_task_source.arced().into()
+            SourceNode::new(scan_task_source.arced(), stats_state.clone()).boxed()
         }
-        LocalPhysicalPlan::InMemoryScan(InMemoryScan { info, .. }) => {
+        LocalPhysicalPlan::WindowPartitionOnly(WindowPartitionOnly {
+            input,
+            partition_by,
+            schema,
+            stats_state,
+            aggregations,
+            aliases,
+        }) => {
+            let input_node = physical_plan_to_pipeline(input, psets, cfg)?;
+            let window_partition_only_sink =
+                WindowPartitionOnlySink::new(aggregations, aliases, partition_by, schema)
+                    .with_context(|_| PipelineCreationSnafu {
+                        plan_name: physical_plan.name(),
+                    })?;
+            BlockingSinkNode::new(
+                Arc::new(window_partition_only_sink),
+                input_node,
+                stats_state.clone(),
+            )
+            .boxed()
+        }
+        LocalPhysicalPlan::WindowPartitionAndOrderBy(WindowPartitionAndOrderBy {
+            input,
+            partition_by,
+            order_by,
+            descending,
+            schema,
+            stats_state,
+            functions,
+            aliases,
+        }) => {
+            let input_node = physical_plan_to_pipeline(input, psets, cfg)?;
+            let window_partition_and_order_by_sink = WindowPartitionAndOrderBySink::new(
+                functions,
+                aliases,
+                partition_by,
+                order_by,
+                descending,
+                schema,
+            )
+            .with_context(|_| PipelineCreationSnafu {
+                plan_name: physical_plan.name(),
+            })?;
+            BlockingSinkNode::new(
+                Arc::new(window_partition_and_order_by_sink),
+                input_node,
+                stats_state.clone(),
+            )
+            .boxed()
+        }
+        LocalPhysicalPlan::WindowPartitionAndDynamicFrame(WindowPartitionAndDynamicFrame {
+            input,
+            partition_by,
+            order_by,
+            descending,
+            frame,
+            min_periods,
+            schema,
+            stats_state,
+            aggregations,
+            aliases,
+        }) => {
+            let input_node = physical_plan_to_pipeline(input, psets, cfg)?;
+            let window_partition_and_dynamic_frame_sink = WindowPartitionAndDynamicFrameSink::new(
+                aggregations,
+                *min_periods,
+                aliases,
+                partition_by,
+                order_by,
+                descending,
+                frame,
+                schema,
+            )
+            .with_context(|_| PipelineCreationSnafu {
+                plan_name: physical_plan.name(),
+            })?;
+            BlockingSinkNode::new(
+                Arc::new(window_partition_and_dynamic_frame_sink),
+                input_node,
+                stats_state.clone(),
+            )
+            .boxed()
+        }
+        LocalPhysicalPlan::WindowOrderByOnly(WindowOrderByOnly {
+            input,
+            order_by,
+            descending,
+            schema,
+            stats_state,
+            functions,
+            aliases,
+        }) => {
+            let input_node = physical_plan_to_pipeline(input, psets, cfg)?;
+            let window_order_by_only_op =
+                WindowOrderByOnlySink::new(functions, aliases, order_by, descending, schema)
+                    .with_context(|_| PipelineCreationSnafu {
+                        plan_name: physical_plan.name(),
+                    })?;
+            BlockingSinkNode::new(
+                Arc::new(window_order_by_only_op),
+                input_node,
+                stats_state.clone(),
+            )
+            .boxed()
+        }
+        LocalPhysicalPlan::InMemoryScan(InMemoryScan { info, stats_state }) => {
             let cache_key: Arc<str> = info.cache_key.clone().into();
 
-            let materialized_pset = psets
-                .get_partition_set(&cache_key)
-                .unwrap_or_else(|| panic!("Cache key not found: {:?}", info.cache_key));
-
-            InMemorySource::new(materialized_pset, info.source_schema.clone())
-                .arced()
-                .into()
+            let materialized_pset = psets.get_partition_set(&cache_key);
+            let in_memory_source = InMemorySource::new(
+                materialized_pset,
+                info.source_schema.clone(),
+                info.size_bytes,
+            )
+            .arced();
+            SourceNode::new(in_memory_source, stats_state.clone()).boxed()
         }
         LocalPhysicalPlan::Project(Project {
-            input, projection, ..
+            input,
+            projection,
+            stats_state,
+            ..
         }) => {
-            let proj_op = ProjectOperator::new(projection.clone());
+            let proj_op = ProjectOperator::new(projection.clone()).with_context(|_| {
+                PipelineCreationSnafu {
+                    plan_name: physical_plan.name(),
+                }
+            })?;
             let child_node = physical_plan_to_pipeline(input, psets, cfg)?;
-            IntermediateNode::new(Arc::new(proj_op), vec![child_node]).boxed()
+            IntermediateNode::new(Arc::new(proj_op), vec![child_node], stats_state.clone()).boxed()
         }
         LocalPhysicalPlan::ActorPoolProject(ActorPoolProject {
-            input, projection, ..
+            input,
+            projection,
+            stats_state,
+            ..
         }) => {
-            let proj_op = ActorPoolProjectOperator::new(projection.clone());
+            let proj_op =
+                ActorPoolProjectOperator::try_new(projection.clone()).with_context(|_| {
+                    PipelineCreationSnafu {
+                        plan_name: physical_plan.name(),
+                    }
+                })?;
             let child_node = physical_plan_to_pipeline(input, psets, cfg)?;
-            IntermediateNode::new(Arc::new(proj_op), vec![child_node]).boxed()
+            IntermediateNode::new(Arc::new(proj_op), vec![child_node], stats_state.clone()).boxed()
         }
         LocalPhysicalPlan::Sample(Sample {
             input,
             fraction,
             with_replacement,
             seed,
+            stats_state,
             ..
         }) => {
             let sample_op = SampleOperator::new(*fraction, *with_replacement, *seed);
             let child_node = physical_plan_to_pipeline(input, psets, cfg)?;
-            IntermediateNode::new(Arc::new(sample_op), vec![child_node]).boxed()
+            IntermediateNode::new(Arc::new(sample_op), vec![child_node], stats_state.clone())
+                .boxed()
         }
         LocalPhysicalPlan::Filter(Filter {
-            input, predicate, ..
+            input,
+            predicate,
+            stats_state,
+            ..
         }) => {
             let filter_op = FilterOperator::new(predicate.clone());
             let child_node = physical_plan_to_pipeline(input, psets, cfg)?;
-            IntermediateNode::new(Arc::new(filter_op), vec![child_node]).boxed()
+            IntermediateNode::new(Arc::new(filter_op), vec![child_node], stats_state.clone())
+                .boxed()
         }
         LocalPhysicalPlan::Explode(Explode {
-            input, to_explode, ..
+            input,
+            to_explode,
+            stats_state,
+            ..
         }) => {
             let explode_op = ExplodeOperator::new(to_explode.clone());
             let child_node = physical_plan_to_pipeline(input, psets, cfg)?;
-            IntermediateNode::new(Arc::new(explode_op), vec![child_node]).boxed()
+            IntermediateNode::new(Arc::new(explode_op), vec![child_node], stats_state.clone())
+                .boxed()
         }
         LocalPhysicalPlan::Limit(Limit {
-            input, num_rows, ..
+            input,
+            num_rows,
+            stats_state,
+            ..
         }) => {
             let sink = LimitSink::new(*num_rows as usize);
             let child_node = physical_plan_to_pipeline(input, psets, cfg)?;
-            StreamingSinkNode::new(Arc::new(sink), vec![child_node]).boxed()
+            StreamingSinkNode::new(Arc::new(sink), vec![child_node], stats_state.clone()).boxed()
         }
-        LocalPhysicalPlan::Concat(Concat { input, other, .. }) => {
+        LocalPhysicalPlan::Concat(Concat {
+            input,
+            other,
+            stats_state,
+            ..
+        }) => {
             let left_child = physical_plan_to_pipeline(input, psets, cfg)?;
             let right_child = physical_plan_to_pipeline(other, psets, cfg)?;
             let sink = ConcatSink {};
-            StreamingSinkNode::new(Arc::new(sink), vec![left_child, right_child]).boxed()
+            StreamingSinkNode::new(
+                Arc::new(sink),
+                vec![left_child, right_child],
+                stats_state.clone(),
+            )
+            .boxed()
         }
         LocalPhysicalPlan::UnGroupedAggregate(UnGroupedAggregate {
             input,
             aggregations,
             schema,
+            stats_state,
             ..
         }) => {
             let child_node = physical_plan_to_pipeline(input, psets, cfg)?;
@@ -182,13 +348,14 @@ pub fn physical_plan_to_pipeline(
                     plan_name: physical_plan.name(),
                 }
             })?;
-            BlockingSinkNode::new(Arc::new(agg_sink), child_node).boxed()
+            BlockingSinkNode::new(Arc::new(agg_sink), child_node, stats_state.clone()).boxed()
         }
         LocalPhysicalPlan::HashAggregate(HashAggregate {
             input,
             aggregations,
             group_by,
             schema,
+            stats_state,
             ..
         }) => {
             let child_node = physical_plan_to_pipeline(input, psets, cfg)?;
@@ -196,7 +363,7 @@ pub fn physical_plan_to_pipeline(
                 .with_context(|_| PipelineCreationSnafu {
                     plan_name: physical_plan.name(),
                 })?;
-            BlockingSinkNode::new(Arc::new(agg_sink), child_node).boxed()
+            BlockingSinkNode::new(Arc::new(agg_sink), child_node, stats_state.clone()).boxed()
         }
         LocalPhysicalPlan::Unpivot(Unpivot {
             input,
@@ -204,6 +371,7 @@ pub fn physical_plan_to_pipeline(
             values,
             variable_name,
             value_name,
+            stats_state,
             ..
         }) => {
             let child_node = physical_plan_to_pipeline(input, psets, cfg)?;
@@ -213,7 +381,8 @@ pub fn physical_plan_to_pipeline(
                 variable_name.clone(),
                 value_name.clone(),
             );
-            IntermediateNode::new(Arc::new(unpivot_op), vec![child_node]).boxed()
+            IntermediateNode::new(Arc::new(unpivot_op), vec![child_node], stats_state.clone())
+                .boxed()
         }
         LocalPhysicalPlan::Pivot(Pivot {
             input,
@@ -222,6 +391,7 @@ pub fn physical_plan_to_pipeline(
             value_column,
             aggregation,
             names,
+            stats_state,
             ..
         }) => {
             let child_node = physical_plan_to_pipeline(input, psets, cfg)?;
@@ -232,30 +402,54 @@ pub fn physical_plan_to_pipeline(
                 aggregation.clone(),
                 names.clone(),
             );
-            BlockingSinkNode::new(Arc::new(pivot_sink), child_node).boxed()
+            BlockingSinkNode::new(Arc::new(pivot_sink), child_node, stats_state.clone()).boxed()
         }
         LocalPhysicalPlan::Sort(Sort {
             input,
             sort_by,
             descending,
             nulls_first,
+            stats_state,
             ..
         }) => {
             let sort_sink = SortSink::new(sort_by.clone(), descending.clone(), nulls_first.clone());
             let child_node = physical_plan_to_pipeline(input, psets, cfg)?;
-            BlockingSinkNode::new(Arc::new(sort_sink), child_node).boxed()
+            BlockingSinkNode::new(Arc::new(sort_sink), child_node, stats_state.clone()).boxed()
+        }
+        LocalPhysicalPlan::TopN(TopN {
+            input,
+            sort_by,
+            descending,
+            nulls_first,
+            limit,
+            stats_state,
+            ..
+        }) => {
+            let sink = TopNSink::new(
+                sort_by.clone(),
+                descending.clone(),
+                nulls_first.clone(),
+                *limit as usize,
+            );
+            let child_node = physical_plan_to_pipeline(input, psets, cfg)?;
+            BlockingSinkNode::new(Arc::new(sink), child_node, stats_state.clone()).boxed()
         }
         LocalPhysicalPlan::MonotonicallyIncreasingId(MonotonicallyIncreasingId {
             input,
             column_name,
             schema,
+            stats_state,
             ..
         }) => {
             let child_node = physical_plan_to_pipeline(input, psets, cfg)?;
             let monotonically_increasing_id_sink =
                 MonotonicallyIncreasingIdSink::new(column_name.clone(), schema.clone());
-            StreamingSinkNode::new(Arc::new(monotonically_increasing_id_sink), vec![child_node])
-                .boxed()
+            StreamingSinkNode::new(
+                Arc::new(monotonically_increasing_id_sink),
+                vec![child_node],
+                stats_state.clone(),
+            )
+            .boxed()
         }
         LocalPhysicalPlan::HashJoin(HashJoin {
             left,
@@ -265,6 +459,7 @@ pub fn physical_plan_to_pipeline(
             null_equals_null,
             join_type,
             schema,
+            stats_state,
             ..
         }) => {
             let left_schema = left.schema();
@@ -283,17 +478,15 @@ pub fn physical_plan_to_pipeline(
                         StatsState::Materialized(left_stats),
                         StatsState::Materialized(right_stats),
                     ) => {
-                        let left_size = left_stats.approx_stats.upper_bound_bytes;
-                        let right_size = right_stats.approx_stats.upper_bound_bytes;
-                        left_size.zip(right_size).map_or(true, |(l, r)| l <= r)
+                        let left_size = left_stats.approx_stats.num_rows;
+                        let right_size = right_stats.approx_stats.num_rows;
+                        left_size <= right_size
                     }
                     // If stats are only available on the right side of the join, and the upper bound bytes on the
                     // right are under the broadcast join size threshold, we build on the right instead of the left.
                     (StatsState::NotMaterialized, StatsState::Materialized(right_stats)) => {
-                        right_stats
-                            .approx_stats
-                            .upper_bound_bytes
-                            .map_or(true, |size| size > cfg.broadcast_join_size_bytes_threshold)
+                        right_stats.approx_stats.size_bytes
+                            > cfg.broadcast_join_size_bytes_threshold
                     }
                     _ => true,
                 },
@@ -304,21 +497,15 @@ pub fn physical_plan_to_pipeline(
                         StatsState::Materialized(left_stats),
                         StatsState::Materialized(right_stats),
                     ) => {
-                        let left_size = left_stats.approx_stats.upper_bound_bytes;
-                        let right_size = right_stats.approx_stats.upper_bound_bytes;
-                        left_size
-                            .zip(right_size)
-                            .map_or(false, |(l, r)| (r as f64) >= ((l as f64) * 1.5))
+                        let left_size = left_stats.approx_stats.num_rows;
+                        let right_size = right_stats.approx_stats.num_rows;
+                        right_size as f64 >= left_size as f64 * 1.5
                     }
                     // If stats are only available on the left side of the join, and the upper bound bytes on the left
                     // are under the broadcast join size threshold, we build on the left instead of the right.
                     (StatsState::Materialized(left_stats), StatsState::NotMaterialized) => {
-                        left_stats
-                            .approx_stats
-                            .upper_bound_bytes
-                            .map_or(false, |size| {
-                                size <= cfg.broadcast_join_size_bytes_threshold
-                            })
+                        left_stats.approx_stats.size_bytes
+                            <= cfg.broadcast_join_size_bytes_threshold
                     }
                     _ => false,
                 },
@@ -329,27 +516,38 @@ pub fn physical_plan_to_pipeline(
                         StatsState::Materialized(left_stats),
                         StatsState::Materialized(right_stats),
                     ) => {
-                        let left_size = left_stats.approx_stats.upper_bound_bytes;
-                        let right_size = right_stats.approx_stats.upper_bound_bytes;
-                        left_size
-                            .zip(right_size)
-                            .map_or(true, |(l, r)| (r as f64) < ((l as f64) * 1.5))
+                        let left_size = left_stats.approx_stats.num_rows;
+                        let right_size = right_stats.approx_stats.num_rows;
+                        (right_size as f64 * 1.5) >= left_size as f64
                     }
                     // If stats are only available on the right side of the join, and the upper bound bytes on the
                     // right are under the broadcast join size threshold, we build on the right instead of the left.
                     (StatsState::NotMaterialized, StatsState::Materialized(right_stats)) => {
-                        right_stats
-                            .approx_stats
-                            .upper_bound_bytes
-                            .map_or(false, |size| {
-                                size <= cfg.broadcast_join_size_bytes_threshold
-                            })
+                        right_stats.approx_stats.size_bytes
+                            > cfg.broadcast_join_size_bytes_threshold
                     }
+                    _ => true,
+                },
+                // Anti/semi joins can build on the left side, but prefer building on the right because building on left requires keeping track
+                // of used indices in a bitmap. If stats are available, only select the left side if its smaller than the right side by a factor of 1.5.
+                JoinType::Anti | JoinType::Semi => match (left_stats_state, right_stats_state) {
+                    (
+                        StatsState::Materialized(left_stats),
+                        StatsState::Materialized(right_stats),
+                    ) => {
+                        let left_size = left_stats.approx_stats.num_rows;
+                        let right_size = right_stats.approx_stats.num_rows;
+                        right_size as f64 > left_size as f64 * 1.5
+                    }
+                    // If stats are only available on the left side of the join, and the upper bound bytes on the left
+                    // are under the broadcast join size threshold, we build on the left instead of the right.
+                    (StatsState::Materialized(left_stats), StatsState::NotMaterialized) => {
+                        left_stats.approx_stats.size_bytes
+                            <= cfg.broadcast_join_size_bytes_threshold
+                    }
+                    // Else, default to building on the right
                     _ => false,
                 },
-
-                // Anti and semi joins always build on the right
-                JoinType::Anti | JoinType::Semi => false,
             };
             let (build_on, probe_on, build_child, probe_child) = match build_on_left {
                 true => (left_on, right_on, left, right),
@@ -359,7 +557,7 @@ pub fn physical_plan_to_pipeline(
             let build_schema = build_child.schema();
             let probe_schema = probe_child.schema();
             || -> DaftResult<_> {
-                let common_join_keys: IndexSet<_> = get_common_join_keys(left_on, right_on)
+                let common_join_cols: IndexSet<_> = get_common_join_cols(left_schema, right_schema)
                     .map(std::string::ToString::to_string)
                     .collect();
                 let build_key_fields = build_on
@@ -370,81 +568,81 @@ pub fn physical_plan_to_pipeline(
                     .iter()
                     .map(|e| e.to_field(probe_schema))
                     .collect::<DaftResult<Vec<_>>>()?;
-                let key_schema: SchemaRef = Schema::new(
-                    build_key_fields
-                        .into_iter()
-                        .zip(probe_key_fields.into_iter())
-                        .map(|(l, r)| {
-                            // TODO we should be using the comparison_op function here instead but i'm just using existing behavior for now
-                            let dtype = supertype::try_get_supertype(&l.dtype, &r.dtype)?;
-                            Ok(Field::new(l.name, dtype))
-                        })
-                        .collect::<DaftResult<Vec<_>>>()?,
-                )?
-                .into();
 
-                let casted_build_on = build_on
-                    .iter()
-                    .zip(key_schema.fields.values())
-                    .map(|(e, f)| e.clone().cast(&f.dtype))
-                    .collect::<Vec<_>>();
-                let casted_probe_on = probe_on
-                    .iter()
-                    .zip(key_schema.fields.values())
-                    .map(|(e, f)| e.clone().cast(&f.dtype))
-                    .collect::<Vec<_>>();
+                for (build_field, probe_field) in build_key_fields.iter().zip(probe_key_fields.iter()) {
+                    if build_field.dtype != probe_field.dtype {
+                        return Err(DaftError::SchemaMismatch(
+                            format!("Expected build and probe key field datatypes to match, found: {} vs {}", build_field.dtype, probe_field.dtype)
+                        ));
+                    }
+                }
+                let key_schema = Arc::new(Schema::new(build_key_fields));
+
                 // we should move to a builder pattern
                 let probe_state_bridge = BroadcastStateBridge::new();
+                let track_indices = if matches!(join_type, JoinType::Anti | JoinType::Semi) {
+                    build_on_left
+                } else {
+                    true
+                };
                 let build_sink = HashJoinBuildSink::new(
                     key_schema,
-                    casted_build_on,
+                    build_on.clone(),
                     null_equals_null.clone(),
-                    join_type,
+                    track_indices,
                     probe_state_bridge.clone(),
                 )?;
                 let build_child_node = physical_plan_to_pipeline(build_child, psets, cfg)?;
-                let build_node =
-                    BlockingSinkNode::new(Arc::new(build_sink), build_child_node).boxed();
+                let build_node = BlockingSinkNode::new(
+                    Arc::new(build_sink),
+                    build_child_node,
+                    build_child.get_stats_state().clone(),
+                )
+                .boxed();
 
                 let probe_child_node = physical_plan_to_pipeline(probe_child, psets, cfg)?;
 
                 match join_type {
-                    JoinType::Anti | JoinType::Semi => Ok(IntermediateNode::new(
-                        Arc::new(AntiSemiProbeOperator::new(
-                            casted_probe_on,
+                    JoinType::Anti | JoinType::Semi => Ok(StreamingSinkNode::new(
+                        Arc::new(AntiSemiProbeSink::new(
+                            probe_on.clone(),
                             join_type,
                             schema,
                             probe_state_bridge,
+                            build_on_left,
                         )),
                         vec![build_node, probe_child_node],
+                        stats_state.clone(),
                     )
                     .boxed()),
                     JoinType::Inner => Ok(IntermediateNode::new(
                         Arc::new(InnerHashJoinProbeOperator::new(
-                            casted_probe_on,
+                            probe_on.clone(),
                             left_schema,
                             right_schema,
                             build_on_left,
-                            common_join_keys,
+                            common_join_cols,
                             schema,
                             probe_state_bridge,
                         )),
                         vec![build_node, probe_child_node],
+                        stats_state.clone(),
                     )
                     .boxed()),
                     JoinType::Left | JoinType::Right | JoinType::Outer => {
                         Ok(StreamingSinkNode::new(
                             Arc::new(OuterHashJoinProbeSink::new(
-                                casted_probe_on,
+                                probe_on.clone(),
                                 left_schema,
                                 right_schema,
                                 *join_type,
                                 build_on_left,
-                                common_join_keys,
+                                common_join_cols,
                                 schema,
                                 probe_state_bridge,
-                            )),
+                            )?),
                             vec![build_node, probe_child_node],
+                            stats_state.clone(),
                         )
                         .boxed())
                     }
@@ -458,6 +656,7 @@ pub fn physical_plan_to_pipeline(
             left,
             right,
             schema,
+            stats_state,
             ..
         }) => {
             let left_stats_state = left.get_stats_state();
@@ -467,15 +666,13 @@ pub fn physical_plan_to_pipeline(
             // the larger side to stream so that it can be parallelized via an intermediate op. Default to left side.
             let stream_on_left = match (left_stats_state, right_stats_state) {
                 (StatsState::Materialized(left_stats), StatsState::Materialized(right_stats)) => {
-                    left_stats.approx_stats.upper_bound_bytes
-                        > right_stats.approx_stats.upper_bound_bytes
+                    left_stats.approx_stats.num_rows > right_stats.approx_stats.num_rows
                 }
                 // If stats are only available on the left side of the join, and the upper bound bytes on the
                 // left are under the broadcast join size threshold, we stream on the right.
-                (StatsState::Materialized(left_stats), StatsState::NotMaterialized) => left_stats
-                    .approx_stats
-                    .upper_bound_bytes
-                    .map_or(true, |size| size > cfg.broadcast_join_size_bytes_threshold),
+                (StatsState::Materialized(left_stats), StatsState::NotMaterialized) => {
+                    left_stats.approx_stats.size_bytes > cfg.broadcast_join_size_bytes_threshold
+                }
                 // If stats are not available, we fall back and stream on the left by default.
                 _ => true,
             };
@@ -498,6 +695,7 @@ pub fn physical_plan_to_pipeline(
             let collect_node = BlockingSinkNode::new(
                 Arc::new(CrossJoinCollectSink::new(state_bridge.clone())),
                 collect_child_node,
+                collect_child.get_stats_state().clone(),
             )
             .boxed();
 
@@ -508,6 +706,7 @@ pub fn physical_plan_to_pipeline(
                     state_bridge,
                 )),
                 vec![collect_node, stream_child_node],
+                stats_state.clone(),
             )
             .boxed()
         }
@@ -515,6 +714,7 @@ pub fn physical_plan_to_pipeline(
             input,
             file_info,
             file_schema,
+            stats_state,
             ..
         }) => {
             let child_node = physical_plan_to_pipeline(input, psets, cfg)?;
@@ -531,14 +731,16 @@ pub fn physical_plan_to_pipeline(
                 writer_factory,
                 file_info.partition_cols.clone(),
                 file_schema.clone(),
+                Some(file_info.clone()),
             );
-            BlockingSinkNode::new(Arc::new(write_sink), child_node).boxed()
+            BlockingSinkNode::new(Arc::new(write_sink), child_node, stats_state.clone()).boxed()
         }
         #[cfg(feature = "python")]
         LocalPhysicalPlan::CatalogWrite(daft_local_plan::CatalogWrite {
             input,
             catalog_type,
             file_schema,
+            stats_state,
             ..
         }) => {
             use daft_logical_plan::CatalogType;
@@ -561,7 +763,7 @@ pub fn physical_plan_to_pipeline(
                     {
                         let partition_col_exprs = partition_cols
                             .iter()
-                            .map(|name| col(name.as_str()))
+                            .map(|name| resolved_col(name.as_str()))
                             .collect::<Vec<_>>();
                         (Some(partition_col_exprs), WriteFormat::PartitionedDeltalake)
                     } else {
@@ -577,14 +779,16 @@ pub fn physical_plan_to_pipeline(
                 writer_factory,
                 partition_by,
                 file_schema.clone(),
+                None,
             );
-            BlockingSinkNode::new(Arc::new(write_sink), child_node).boxed()
+            BlockingSinkNode::new(Arc::new(write_sink), child_node, stats_state.clone()).boxed()
         }
         #[cfg(feature = "python")]
         LocalPhysicalPlan::LanceWrite(daft_local_plan::LanceWrite {
             input,
             lance_info,
             file_schema,
+            stats_state,
             ..
         }) => {
             let child_node = physical_plan_to_pipeline(input, psets, cfg)?;
@@ -594,8 +798,28 @@ pub fn physical_plan_to_pipeline(
                 writer_factory,
                 None,
                 file_schema.clone(),
+                None,
             );
-            BlockingSinkNode::new(Arc::new(write_sink), child_node).boxed()
+            BlockingSinkNode::new(Arc::new(write_sink), child_node, stats_state.clone()).boxed()
+        }
+        #[cfg(feature = "python")]
+        LocalPhysicalPlan::DataSink(daft_local_plan::DataSink {
+            input,
+            data_sink_info,
+            file_schema,
+            stats_state,
+        }) => {
+            let child_node = physical_plan_to_pipeline(input, psets, cfg)?;
+            let writer_factory =
+                daft_writers::make_data_sink_writer_factory(data_sink_info.clone());
+            let write_sink = WriteSink::new(
+                WriteFormat::DataSink,
+                writer_factory,
+                None,
+                file_schema.clone(),
+                None,
+            );
+            BlockingSinkNode::new(Arc::new(write_sink), child_node, stats_state.clone()).boxed()
         }
     };
 
