@@ -1,21 +1,20 @@
 use std::sync::Arc;
 
+use bitmap::{and, Bitmap, MutableBitmap};
 use common_error::DaftResult;
-use common_runtime::RuntimeRef;
 use daft_core::{
-    prelude::{
-        bitmap::{and, Bitmap, MutableBitmap},
-        BooleanArray, Schema, SchemaRef,
-    },
+    prelude::*,
     series::{IntoSeries, Series},
+    utils::supertype::try_get_supertype,
 };
-use daft_dsl::ExprRef;
+use daft_dsl::{expr::bound_expr::BoundExpr, ExprRef};
 use daft_logical_plan::JoinType;
 use daft_micropartition::MicroPartition;
-use daft_table::{GrowableTable, ProbeState, Table};
+use daft_recordbatch::{get_columns_by_name, GrowableRecordBatch, ProbeState, RecordBatch};
 use futures::{stream, StreamExt};
 use indexmap::IndexSet;
-use tracing::{info_span, instrument};
+use itertools::Itertools;
+use tracing::{info_span, instrument, Span};
 
 use super::streaming_sink::{
     StreamingSink, StreamingSinkExecuteResult, StreamingSinkFinalizeResult, StreamingSinkOutput,
@@ -24,15 +23,15 @@ use super::streaming_sink::{
 use crate::{
     dispatcher::{DispatchSpawner, RoundRobinDispatcher, UnorderedDispatcher},
     state_bridge::BroadcastStateBridgeRef,
-    ExecutionRuntimeContext,
+    ExecutionRuntimeContext, ExecutionTaskSpawner,
 };
 
-struct IndexBitmapBuilder {
+pub(crate) struct IndexBitmapBuilder {
     mutable_bitmaps: Vec<MutableBitmap>,
 }
 
 impl IndexBitmapBuilder {
-    fn new(tables: &[Table]) -> Self {
+    pub fn new(tables: &[RecordBatch]) -> Self {
         Self {
             mutable_bitmaps: tables
                 .iter()
@@ -42,23 +41,23 @@ impl IndexBitmapBuilder {
     }
 
     #[inline]
-    fn mark_used(&mut self, table_idx: usize, row_idx: usize) {
+    pub fn mark_used(&mut self, table_idx: usize, row_idx: usize) {
         self.mutable_bitmaps[table_idx].set(row_idx, false);
     }
 
-    fn build(self) -> IndexBitmap {
+    pub fn build(self) -> IndexBitmap {
         IndexBitmap {
             bitmaps: self.mutable_bitmaps.into_iter().map(|b| b.into()).collect(),
         }
     }
 }
 
-struct IndexBitmap {
+pub(crate) struct IndexBitmap {
     bitmaps: Vec<Bitmap>,
 }
 
 impl IndexBitmap {
-    fn merge(&self, other: &Self) -> Self {
+    pub fn merge(&self, other: &Self) -> Self {
         Self {
             bitmaps: self
                 .bitmaps
@@ -69,7 +68,13 @@ impl IndexBitmap {
         }
     }
 
-    fn convert_to_boolean_arrays(self) -> impl Iterator<Item = BooleanArray> {
+    pub fn negate(&self) -> Self {
+        Self {
+            bitmaps: self.bitmaps.iter().map(|b| !b).collect(),
+        }
+    }
+
+    pub fn convert_to_boolean_arrays(self) -> impl Iterator<Item = BooleanArray> {
         self.bitmaps
             .into_iter()
             .map(|b| BooleanArray::from(("bitmap", b)))
@@ -119,9 +124,10 @@ impl StreamingSinkState for OuterHashJoinState {
 
 struct OuterHashJoinParams {
     probe_on: Vec<ExprRef>,
-    common_join_keys: Vec<String>,
+    common_join_cols: Vec<String>,
     left_non_join_columns: Vec<String>,
     right_non_join_columns: Vec<String>,
+    outer_common_col_schema: SchemaRef,
     left_non_join_schema: SchemaRef,
     right_non_join_schema: SchemaRef,
     join_type: JoinType,
@@ -143,10 +149,10 @@ impl OuterHashJoinProbeSink {
         right_schema: &SchemaRef,
         join_type: JoinType,
         build_on_left: bool,
-        common_join_keys: IndexSet<String>,
+        common_join_cols: IndexSet<String>,
         output_schema: &SchemaRef,
         probe_state_bridge: BroadcastStateBridgeRef<ProbeState>,
-    ) -> Self {
+    ) -> DaftResult<Self> {
         let needs_bitmap = join_type == JoinType::Outer
             || join_type == JoinType::Right && !build_on_left
             || join_type == JoinType::Left && build_on_left;
@@ -155,31 +161,45 @@ impl OuterHashJoinProbeSink {
             (JoinType::Outer, false) => (right_schema, left_schema),
             _ => (left_schema, right_schema),
         };
+        let outer_common_col_fields = common_join_cols
+            .iter()
+            .map(|name| {
+                let supertype = try_get_supertype(
+                    &left_schema.get_field(name)?.dtype,
+                    &right_schema.get_field(name)?.dtype,
+                )?;
+
+                Ok(Field::new(name.clone(), supertype))
+            })
+            .collect::<DaftResult<Vec<_>>>()?;
+        let outer_common_col_schema = Arc::new(Schema::new(outer_common_col_fields));
         let left_non_join_fields = left_schema
-            .fields
-            .values()
-            .filter(|f| !common_join_keys.contains(&f.name))
-            .cloned()
+            .into_iter()
+            .filter(|f| !common_join_cols.contains(&f.name))
+            .cloned();
+        let left_non_join_schema = Arc::new(Schema::new(left_non_join_fields));
+        let left_non_join_columns = left_non_join_schema
+            .field_names()
+            .map(ToString::to_string)
             .collect();
-        let left_non_join_schema =
-            Arc::new(Schema::new(left_non_join_fields).expect("left schema should be valid"));
-        let left_non_join_columns = left_non_join_schema.fields.keys().cloned().collect();
         let right_non_join_fields = right_schema
-            .fields
-            .values()
-            .filter(|f| !common_join_keys.contains(&f.name))
-            .cloned()
+            .fields()
+            .iter()
+            .filter(|f| !common_join_cols.contains(&f.name))
+            .cloned();
+        let right_non_join_schema = Arc::new(Schema::new(right_non_join_fields));
+        let right_non_join_columns = right_non_join_schema
+            .field_names()
+            .map(ToString::to_string)
             .collect();
-        let right_non_join_schema =
-            Arc::new(Schema::new(right_non_join_fields).expect("right schema should be valid"));
-        let right_non_join_columns = right_non_join_schema.fields.keys().cloned().collect();
-        let common_join_keys = common_join_keys.into_iter().collect();
-        Self {
+        let common_join_cols = common_join_cols.into_iter().collect();
+        Ok(Self {
             params: Arc::new(OuterHashJoinParams {
                 probe_on,
-                common_join_keys,
+                common_join_cols,
                 left_non_join_columns,
                 right_non_join_columns,
+                outer_common_col_schema,
                 left_non_join_schema,
                 right_non_join_schema,
                 join_type,
@@ -188,7 +208,7 @@ impl OuterHashJoinProbeSink {
             needs_bitmap,
             output_schema: output_schema.clone(),
             probe_state_bridge,
-        }
+        })
     }
 
     fn probe_left_right_with_bitmap(
@@ -196,8 +216,8 @@ impl OuterHashJoinProbeSink {
         bitmap_builder: &mut IndexBitmapBuilder,
         probe_state: &ProbeState,
         join_type: JoinType,
-        probe_on: &[ExprRef],
-        common_join_keys: &[String],
+        probe_on: &[BoundExpr],
+        common_join_cols: &[String],
         left_non_join_columns: &[String],
         right_non_join_columns: &[String],
     ) -> DaftResult<Arc<MicroPartition>> {
@@ -205,7 +225,7 @@ impl OuterHashJoinProbeSink {
         let tables = probe_state.get_tables();
 
         let _growables = info_span!("OuterHashJoinProbeSink::build_growables").entered();
-        let mut build_side_growable = GrowableTable::new(
+        let mut build_side_growable = GrowableRecordBatch::new(
             &tables.iter().collect::<Vec<_>>(),
             false,
             tables.iter().map(|t| t.len()).sum(),
@@ -213,7 +233,7 @@ impl OuterHashJoinProbeSink {
 
         let input_tables = input.get_tables()?;
         let mut probe_side_growable =
-            GrowableTable::new(&input_tables.iter().collect::<Vec<_>>(), false, input.len())?;
+            GrowableRecordBatch::new(&input_tables.iter().collect::<Vec<_>>(), false, input.len())?;
 
         drop(_growables);
         {
@@ -242,14 +262,14 @@ impl OuterHashJoinProbeSink {
         let probe_side_table = probe_side_growable.build()?;
 
         let final_table = if join_type == JoinType::Left {
-            let join_table = build_side_table.get_columns(common_join_keys)?;
-            let left = build_side_table.get_columns(left_non_join_columns)?;
-            let right = probe_side_table.get_columns(right_non_join_columns)?;
+            let join_table = get_columns_by_name(&build_side_table, common_join_cols)?;
+            let left = get_columns_by_name(&build_side_table, left_non_join_columns)?;
+            let right = get_columns_by_name(&probe_side_table, right_non_join_columns)?;
             join_table.union(&left)?.union(&right)?
         } else {
-            let join_table = build_side_table.get_columns(common_join_keys)?;
-            let left = probe_side_table.get_columns(left_non_join_columns)?;
-            let right = build_side_table.get_columns(right_non_join_columns)?;
+            let join_table = get_columns_by_name(&build_side_table, common_join_cols)?;
+            let left = get_columns_by_name(&probe_side_table, left_non_join_columns)?;
+            let right = get_columns_by_name(&build_side_table, right_non_join_columns)?;
             join_table.union(&left)?.union(&right)?
         };
         Ok(Arc::new(MicroPartition::new_loaded(
@@ -263,8 +283,8 @@ impl OuterHashJoinProbeSink {
         input: &Arc<MicroPartition>,
         probe_state: &ProbeState,
         join_type: JoinType,
-        probe_on: &[ExprRef],
-        common_join_keys: &[String],
+        probe_on: &[BoundExpr],
+        common_join_cols: &[String],
         left_non_join_columns: &[String],
         right_non_join_columns: &[String],
     ) -> DaftResult<Arc<MicroPartition>> {
@@ -272,7 +292,7 @@ impl OuterHashJoinProbeSink {
         let tables = probe_state.get_tables();
 
         let _growables = info_span!("OuterHashJoinProbeSink::build_growables").entered();
-        let mut build_side_growable = GrowableTable::new(
+        let mut build_side_growable = GrowableRecordBatch::new(
             &tables.iter().collect::<Vec<_>>(),
             true,
             tables.iter().map(|t| t.len()).sum(),
@@ -280,7 +300,7 @@ impl OuterHashJoinProbeSink {
 
         let input_tables = input.get_tables()?;
         let mut probe_side_growable =
-            GrowableTable::new(&input_tables.iter().collect::<Vec<_>>(), false, input.len())?;
+            GrowableRecordBatch::new(&input_tables.iter().collect::<Vec<_>>(), false, input.len())?;
 
         drop(_growables);
         {
@@ -311,14 +331,14 @@ impl OuterHashJoinProbeSink {
         let probe_side_table = probe_side_growable.build()?;
 
         let final_table = if join_type == JoinType::Left {
-            let join_table = probe_side_table.get_columns(common_join_keys)?;
-            let left = probe_side_table.get_columns(left_non_join_columns)?;
-            let right = build_side_table.get_columns(right_non_join_columns)?;
+            let join_table = get_columns_by_name(&probe_side_table, common_join_cols)?;
+            let left = get_columns_by_name(&probe_side_table, left_non_join_columns)?;
+            let right = get_columns_by_name(&build_side_table, right_non_join_columns)?;
             join_table.union(&left)?.union(&right)?
         } else {
-            let join_table = probe_side_table.get_columns(common_join_keys)?;
-            let left = build_side_table.get_columns(left_non_join_columns)?;
-            let right = probe_side_table.get_columns(right_non_join_columns)?;
+            let join_table = get_columns_by_name(&probe_side_table, common_join_cols)?;
+            let left = get_columns_by_name(&build_side_table, left_non_join_columns)?;
+            let right = get_columns_by_name(&probe_side_table, right_non_join_columns)?;
             join_table.union(&left)?.union(&right)?
         };
         Ok(Arc::new(MicroPartition::new_loaded(
@@ -333,8 +353,9 @@ impl OuterHashJoinProbeSink {
         input: &Arc<MicroPartition>,
         probe_state: &ProbeState,
         bitmap_builder: &mut IndexBitmapBuilder,
-        probe_on: &[ExprRef],
-        common_join_keys: &[String],
+        probe_on: &[BoundExpr],
+        common_join_cols: &[String],
+        outer_common_col_schema: &SchemaRef,
         left_non_join_columns: &[String],
         right_non_join_columns: &[String],
         build_on_left: bool,
@@ -344,7 +365,7 @@ impl OuterHashJoinProbeSink {
 
         let _growables = info_span!("OuterHashJoinProbeSink::build_growables").entered();
         // Need to set use_validity to true here because we add nulls to the build side
-        let mut build_side_growable = GrowableTable::new(
+        let mut build_side_growable = GrowableRecordBatch::new(
             &tables.iter().collect::<Vec<_>>(),
             true,
             tables.iter().map(|t| t.len()).sum(),
@@ -352,7 +373,7 @@ impl OuterHashJoinProbeSink {
 
         let input_tables = input.get_tables()?;
         let mut probe_side_growable =
-            GrowableTable::new(&input_tables.iter().collect::<Vec<_>>(), false, input.len())?;
+            GrowableRecordBatch::new(&input_tables.iter().collect::<Vec<_>>(), false, input.len())?;
 
         drop(_growables);
         {
@@ -381,9 +402,10 @@ impl OuterHashJoinProbeSink {
         let build_side_table = build_side_growable.build()?;
         let probe_side_table = probe_side_growable.build()?;
 
-        let join_table = probe_side_table.get_columns(common_join_keys)?;
-        let left = build_side_table.get_columns(left_non_join_columns)?;
-        let right = probe_side_table.get_columns(right_non_join_columns)?;
+        let join_table = get_columns_by_name(&probe_side_table, common_join_cols)?
+            .cast_to_schema(outer_common_col_schema)?;
+        let left = get_columns_by_name(&build_side_table, left_non_join_columns)?;
+        let right = get_columns_by_name(&probe_side_table, right_non_join_columns)?;
         // If we built the probe table on the right, flip the order of union.
         let (left, right) = if build_on_left {
             (left, right)
@@ -400,7 +422,7 @@ impl OuterHashJoinProbeSink {
 
     async fn merge_bitmaps_and_construct_null_table(
         mut states: Vec<Box<dyn StreamingSinkState>>,
-    ) -> DaftResult<Table> {
+    ) -> DaftResult<RecordBatch> {
         let mut states_iter = states.iter_mut();
         let first_state = states_iter
             .next()
@@ -450,26 +472,28 @@ impl OuterHashJoinProbeSink {
             .map(|(bitmap, table)| table.mask_filter(&bitmap.into_series()))
             .collect::<DaftResult<Vec<_>>>()?;
 
-        Table::concat(&leftovers)
+        RecordBatch::concat(&leftovers)
     }
 
     async fn finalize_outer(
         states: Vec<Box<dyn StreamingSinkState>>,
-        common_join_keys: &[String],
+        common_join_cols: &[String],
+        outer_common_col_schema: &SchemaRef,
         left_non_join_columns: &[String],
         right_non_join_schema: &SchemaRef,
         build_on_left: bool,
     ) -> DaftResult<Option<Arc<MicroPartition>>> {
         let build_side_table = Self::merge_bitmaps_and_construct_null_table(states).await?;
-        let join_table = build_side_table.get_columns(common_join_keys)?;
-        let left = build_side_table.get_columns(left_non_join_columns)?;
+        let join_table = get_columns_by_name(&build_side_table, common_join_cols)?
+            .cast_to_schema(outer_common_col_schema)?;
+        let left = get_columns_by_name(&build_side_table, left_non_join_columns)?;
         let right = {
             let columns = right_non_join_schema
-                .fields
-                .values()
+                .fields()
+                .iter()
                 .map(|field| Series::full_null(&field.name, &field.dtype, left.len()))
                 .collect::<Vec<_>>();
-            Table::new_unchecked(right_non_join_schema.clone(), columns, left.len())
+            RecordBatch::new_unchecked(right_non_join_schema.clone(), columns, left.len())
         };
         // If we built the probe table on the right, flip the order of union.
         let (left, right) = if build_on_left {
@@ -487,20 +511,20 @@ impl OuterHashJoinProbeSink {
 
     async fn finalize_left(
         states: Vec<Box<dyn StreamingSinkState>>,
-        common_join_keys: &[String],
+        common_join_cols: &[String],
         left_non_join_columns: &[String],
         right_non_join_schema: &SchemaRef,
     ) -> DaftResult<Option<Arc<MicroPartition>>> {
         let build_side_table = Self::merge_bitmaps_and_construct_null_table(states).await?;
-        let join_table = build_side_table.get_columns(common_join_keys)?;
-        let left = build_side_table.get_columns(left_non_join_columns)?;
+        let join_table = get_columns_by_name(&build_side_table, common_join_cols)?;
+        let left = get_columns_by_name(&build_side_table, left_non_join_columns)?;
         let right = {
             let columns = right_non_join_schema
-                .fields
-                .values()
+                .fields()
+                .iter()
                 .map(|field| Series::full_null(&field.name, &field.dtype, left.len()))
                 .collect::<Vec<_>>();
-            Table::new_unchecked(right_non_join_schema.clone(), columns, left.len())
+            RecordBatch::new_unchecked(right_non_join_schema.clone(), columns, left.len())
         };
         let final_table = join_table.union(&left)?.union(&right)?;
         Ok(Some(Arc::new(MicroPartition::new_loaded(
@@ -512,25 +536,25 @@ impl OuterHashJoinProbeSink {
 
     async fn finalize_right(
         states: Vec<Box<dyn StreamingSinkState>>,
-        common_join_keys: &[String],
+        common_join_cols: &[String],
         right_non_join_columns: &[String],
         left_non_join_schema: &SchemaRef,
     ) -> DaftResult<Option<Arc<MicroPartition>>> {
         let build_side_table = Self::merge_bitmaps_and_construct_null_table(states).await?;
-        let join_table = build_side_table.get_columns(common_join_keys)?;
+        let join_table = get_columns_by_name(&build_side_table, common_join_cols)?;
         let left = {
             let columns = left_non_join_schema
-                .fields
-                .values()
+                .fields()
+                .iter()
                 .map(|field| Series::full_null(&field.name, &field.dtype, build_side_table.len()))
                 .collect::<Vec<_>>();
-            Table::new_unchecked(
+            RecordBatch::new_unchecked(
                 left_non_join_schema.clone(),
                 columns,
                 build_side_table.len(),
             )
         };
-        let right = build_side_table.get_columns(right_non_join_columns)?;
+        let right = get_columns_by_name(&build_side_table, right_non_join_columns)?;
         let final_table = join_table.union(&left)?.union(&right)?;
         Ok(Some(Arc::new(MicroPartition::new_loaded(
             final_table.schema.clone(),
@@ -546,7 +570,7 @@ impl StreamingSink for OuterHashJoinProbeSink {
         &self,
         input: Arc<MicroPartition>,
         mut state: Box<dyn StreamingSinkState>,
-        runtime_ref: &RuntimeRef,
+        spawner: &ExecutionTaskSpawner,
     ) -> StreamingSinkExecuteResult {
         if input.is_empty() {
             let empty = Arc::new(MicroPartition::empty(Some(self.output_schema.clone())));
@@ -555,67 +579,100 @@ impl StreamingSink for OuterHashJoinProbeSink {
 
         let needs_bitmap = self.needs_bitmap;
         let params = self.params.clone();
-        runtime_ref
-            .spawn(async move {
-                let outer_join_state = state
-                    .as_any_mut()
-                    .downcast_mut::<OuterHashJoinState>()
-                    .expect("OuterHashJoinProbeSink should have OuterHashJoinProbeState");
-                let probe_state = outer_join_state.get_or_build_probe_state().await;
-                let out = match params.join_type {
-                    JoinType::Left | JoinType::Right if needs_bitmap => {
-                        Self::probe_left_right_with_bitmap(
+        spawner
+            .spawn(
+                async move {
+                    let outer_join_state = state
+                        .as_any_mut()
+                        .downcast_mut::<OuterHashJoinState>()
+                        .expect("OuterHashJoinProbeSink should have OuterHashJoinProbeState");
+                    let probe_state = outer_join_state.get_or_build_probe_state().await;
+
+                    let probe_on = params
+                        .probe_on
+                        .iter()
+                        .map(|expr| BoundExpr::try_new(expr.clone(), &input.schema()))
+                        .collect::<DaftResult<Vec<_>>>()?;
+
+                    let out = match params.join_type {
+                        JoinType::Left | JoinType::Right if needs_bitmap => {
+                            Self::probe_left_right_with_bitmap(
+                                &input,
+                                outer_join_state
+                                    .get_or_build_bitmap()
+                                    .await
+                                    .as_mut()
+                                    .expect("bitmap should be set"),
+                                &probe_state,
+                                params.join_type,
+                                &probe_on,
+                                &params.common_join_cols,
+                                &params.left_non_join_columns,
+                                &params.right_non_join_columns,
+                            )
+                        }
+                        JoinType::Left | JoinType::Right => Self::probe_left_right(
                             &input,
-                            outer_join_state
+                            &probe_state,
+                            params.join_type,
+                            &probe_on,
+                            &params.common_join_cols,
+                            &params.left_non_join_columns,
+                            &params.right_non_join_columns,
+                        ),
+                        JoinType::Outer => {
+                            let bitmap_builder = outer_join_state
                                 .get_or_build_bitmap()
                                 .await
                                 .as_mut()
-                                .expect("bitmap should be set"),
-                            &probe_state,
-                            params.join_type,
-                            &params.probe_on,
-                            &params.common_join_keys,
-                            &params.left_non_join_columns,
-                            &params.right_non_join_columns,
-                        )
-                    }
-                    JoinType::Left | JoinType::Right => Self::probe_left_right(
-                        &input,
-                        &probe_state,
-                        params.join_type,
-                        &params.probe_on,
-                        &params.common_join_keys,
-                        &params.left_non_join_columns,
-                        &params.right_non_join_columns,
-                    ),
-                    JoinType::Outer => {
-                        let bitmap_builder = outer_join_state
-                            .get_or_build_bitmap()
-                            .await
-                            .as_mut()
-                            .expect("bitmap should be set");
-                        Self::probe_outer(
-                            &input,
-                            &probe_state,
-                            bitmap_builder,
-                            &params.probe_on,
-                            &params.common_join_keys,
-                            &params.left_non_join_columns,
-                            &params.right_non_join_columns,
-                            params.build_on_left,
-                        )
-                    }
-                    _ => unreachable!(
+                                .expect("bitmap should be set");
+                            Self::probe_outer(
+                                &input,
+                                &probe_state,
+                                bitmap_builder,
+                                &probe_on,
+                                &params.common_join_cols,
+                                &params.outer_common_col_schema,
+                                &params.left_non_join_columns,
+                                &params.right_non_join_columns,
+                                params.build_on_left,
+                            )
+                        }
+                        _ => unreachable!(
                         "Only Left, Right, and Outer joins are supported in OuterHashJoinProbeSink"
                     ),
-                }?;
-                Ok((state, StreamingSinkOutput::NeedMoreInput(Some(out))))
-            })
+                    }?;
+                    Ok((state, StreamingSinkOutput::NeedMoreInput(Some(out))))
+                },
+                Span::current(),
+            )
             .into()
     }
 
     fn name(&self) -> &'static str {
-        "OuterHashJoinProbeSink"
+        "OuterHashJoinProbe"
+    }
+
+    fn multiline_display(&self) -> Vec<String> {
+        let mut res = vec![];
+        match self.params.join_type {
+            JoinType::Left => res.push("LeftHashJoinProbe:".to_string()),
+            JoinType::Right => res.push("RightHashJoinProbe:".to_string()),
+            JoinType::Outer => res.push("OuterHashJoinProbe:".to_string()),
+            _ => unreachable!(
+                "Only Left, Right, and Outer joins are supported in OuterHashJoinProbeSink"
+            ),
+        }
+        res.push(format!(
+            "Probe on: [{}]",
+            self.params
+                .probe_on
+                .iter()
+                .map(|e| e.to_string())
+                .join(", ")
+        ));
+        res.push(format!("Build on left: {}", self.params.build_on_left));
+        res
     }
 
     fn make_state(&self) -> Box<dyn StreamingSinkState> {
@@ -625,43 +682,48 @@ impl StreamingSink for OuterHashJoinProbeSink {
         ))
     }
 
+    #[instrument(skip_all, name = "OuterHashJoinProbeSink::finalize")]
     fn finalize(
         &self,
         states: Vec<Box<dyn StreamingSinkState>>,
-        runtime_ref: &RuntimeRef,
+        spawner: &ExecutionTaskSpawner,
     ) -> StreamingSinkFinalizeResult {
         if self.needs_bitmap {
             let params = self.params.clone();
-            runtime_ref
-                .spawn(async move {
-                    match params.join_type {
+            spawner
+                .spawn(
+                    async move {
+                        match params.join_type {
                         JoinType::Left => Self::finalize_left(
                             states,
-                            &params.common_join_keys,
+                            &params.common_join_cols,
                             &params.left_non_join_columns,
                             &params.right_non_join_schema,
                         )
                         .await,
                         JoinType::Right => Self::finalize_right(
                             states,
-                            &params.common_join_keys,
+                            &params.common_join_cols,
                             &params.right_non_join_columns,
                             &params.left_non_join_schema,
                         )
                         .await,
                         JoinType::Outer => Self::finalize_outer(
-                            states,
-                            &params.common_join_keys,
-                            &params.left_non_join_columns,
-                            &params.right_non_join_schema,
-                            params.build_on_left,
-                        )
-                        .await,
+                                states,
+                                &params.common_join_cols,
+                                &params.outer_common_col_schema,
+                                &params.left_non_join_columns,
+                                &params.right_non_join_schema,
+                                params.build_on_left,
+                            )
+                            .await,
                         _ => unreachable!(
                             "Only Left, Right, and Outer joins are supported in OuterHashJoinProbeSink"
                         ),
                     }
-                })
+                    },
+                    Span::current(),
+                )
                 .into()
         } else {
             Ok(None).into()
