@@ -1,20 +1,24 @@
 use std::sync::Arc;
 
-use common_treenode::Transformed;
+use common_error::DaftResult;
+use common_treenode::{Transformed, TreeNode, TreeNodeRecursion};
 use daft_core::prelude::*;
-use daft_dsl::{optimization, AggExpr, ApproxPercentileParams, Expr, ExprRef, ExprResolver};
+use daft_dsl::{
+    functions::FunctionArgs, optimization, resolved_col, AggExpr, ApproxPercentileParams, Column,
+    Expr, ExprRef,
+};
 use indexmap::{IndexMap, IndexSet};
 use itertools::Itertools;
-use snafu::ResultExt;
 
 use crate::{
-    logical_plan::{CreationSnafu, Result},
+    logical_plan::{self},
     stats::StatsState,
     LogicalPlan,
 };
 
 #[derive(Clone, Debug, PartialEq, Eq, Hash)]
 pub struct Project {
+    pub plan_id: Option<usize>,
     // Upstream node.
     pub input: Arc<LogicalPlan>,
     pub projection: Vec<ExprRef>,
@@ -23,20 +27,23 @@ pub struct Project {
 }
 
 impl Project {
-    pub(crate) fn try_new(input: Arc<LogicalPlan>, projection: Vec<ExprRef>) -> Result<Self> {
-        let expr_resolver = ExprResolver::builder().allow_actor_pool_udf(true).build();
-
-        let (projection, fields) = expr_resolver
-            .resolve(projection, &input.schema())
-            .context(CreationSnafu)?;
-
+    pub(crate) fn try_new(
+        input: Arc<LogicalPlan>,
+        projection: Vec<ExprRef>,
+    ) -> logical_plan::Result<Self> {
         // Factor the projection and see if there are any substitutions to factor out.
         let (factored_input, factored_projection) =
             Self::try_factor_subexpressions(input, projection)?;
 
-        let projected_schema = Schema::new(fields).context(CreationSnafu)?.into();
+        let fields = factored_projection
+            .iter()
+            .map(|expr| expr.to_field(&factored_input.schema()))
+            .collect::<DaftResult<Vec<_>>>()?;
+
+        let projected_schema = Schema::new(fields).into();
 
         Ok(Self {
+            plan_id: None,
             input: factored_input,
             projection: factored_projection,
             projected_schema,
@@ -44,13 +51,17 @@ impl Project {
         })
     }
 
+    pub fn with_plan_id(mut self, plan_id: usize) -> Self {
+        self.plan_id = Some(plan_id);
+        self
+    }
+
     /// Create a new Projection using the specified output schema
-    pub(crate) fn new_from_schema(input: Arc<LogicalPlan>, schema: SchemaRef) -> Result<Self> {
-        let expr: Vec<ExprRef> = schema
-            .names()
-            .into_iter()
-            .map(|n| Arc::new(Expr::Column(Arc::from(n))))
-            .collect();
+    pub(crate) fn new_from_schema(
+        input: Arc<LogicalPlan>,
+        schema: SchemaRef,
+    ) -> logical_plan::Result<Self> {
+        let expr: Vec<ExprRef> = schema.names().into_iter().map(resolved_col).collect();
         Self::try_new(input, expr)
     }
 
@@ -75,7 +86,28 @@ impl Project {
     fn try_factor_subexpressions(
         input: Arc<LogicalPlan>,
         projection: Vec<ExprRef>,
-    ) -> Result<(Arc<LogicalPlan>, Vec<ExprRef>)> {
+    ) -> logical_plan::Result<(Arc<LogicalPlan>, Vec<ExprRef>)> {
+        // Check if projection contains any window functions, if so, return without factoring.
+        // This is because window functions may implicitly reference columns via the window spec.
+        let has_window = projection.iter().any(|expr| {
+            let mut has_window = false;
+            expr.apply(|e| {
+                if matches!(e.as_ref(), Expr::Over(..)) {
+                    has_window = true;
+                    Ok(TreeNodeRecursion::Stop)
+                } else {
+                    Ok(TreeNodeRecursion::Continue)
+                }
+            })
+            .unwrap();
+
+            has_window
+        });
+
+        if has_window {
+            return Ok((input, projection));
+        }
+
         // Given construction parameters for a projection,
         // see if we can factor out common subexpressions.
         // Returns a new set of projection parameters
@@ -137,7 +169,7 @@ impl Project {
                         expr.children()
                     } else {
                         let expr_id = expr.semantic_id(schema);
-                        if let Expr::Column(..) = expr.as_ref() {
+                        if let Expr::Column(Column::Resolved(..)) = expr.as_ref() {
                             column_name_substitutions.insert(expr_id.clone(), expr.clone());
                         }
                         // Mark expr as seen
@@ -197,31 +229,28 @@ impl Project {
     }
 }
 
+/// Constructs a new copy of this expression
+/// with all occurrences of subexprs_to_replace replaced with a column selection.
+/// e.g. e := (a+b)+c, subexprs := {FieldID("(a + b)")}
+///  -> Col("(a + b)") + c
 fn replace_column_with_semantic_id(
     e: ExprRef,
     subexprs_to_replace: &IndexSet<FieldID>,
     schema: &Schema,
 ) -> Transformed<ExprRef> {
-    // Constructs a new copy of this expression
-    // with all occurrences of subexprs_to_replace replaced with a column selection.
-    // e.g. e := (a+b)+c, subexprs := {FieldID("(a + b)")}
-    //  -> Col("(a + b)") + c
-
     let sem_id = e.semantic_id(schema);
     if subexprs_to_replace.contains(&sem_id) {
-        let new_expr = Expr::Column(sem_id.id);
+        let new_expr = resolved_col(sem_id.id);
         let new_expr = match e.as_ref() {
-            Expr::Alias(_, name) => Expr::Alias(new_expr.into(), name.clone()),
+            Expr::Alias(_, name) => Expr::Alias(new_expr, name.clone()).into(),
             _ => new_expr,
         };
-        Transformed::yes(new_expr.into())
+        Transformed::yes(new_expr)
     } else {
         match e.as_ref() {
-            Expr::Column(_)
-            | Expr::Literal(_)
-            | Expr::Subquery(_)
-            | Expr::Exists(_)
-            | Expr::OuterReferenceColumn { .. } => Transformed::no(e),
+            Expr::Column(_) | Expr::Literal(_) | Expr::Subquery(_) | Expr::Exists(_) => {
+                Transformed::no(e)
+            }
             Expr::Agg(agg_expr) => replace_column_with_semantic_id_aggexpr(
                 agg_expr.clone(),
                 subexprs_to_replace,
@@ -231,6 +260,18 @@ fn replace_column_with_semantic_id(
                 |transformed_child| Expr::Agg(transformed_child).into(),
                 |_| e,
             ),
+            Expr::Over(inner_expr, window_spec) => {
+                let expr_ref: ExprRef = ExprRef::from(inner_expr);
+
+                replace_column_with_semantic_id(expr_ref, subexprs_to_replace, schema).map_yes_no(
+                    |transformed_child| {
+                        Expr::Over(transformed_child.try_into().unwrap(), window_spec.clone())
+                            .into()
+                    },
+                    |_| e.clone(),
+                )
+            }
+            Expr::WindowFunction(_) => Transformed::no(e),
             Expr::Alias(child, name) => {
                 replace_column_with_semantic_id(child.clone(), subexprs_to_replace, schema)
                     .map_yes_no(
@@ -238,6 +279,7 @@ fn replace_column_with_semantic_id(
                         |_| e.clone(),
                     )
             }
+
             Expr::Cast(child, datatype) => {
                 replace_column_with_semantic_id(child.clone(), subexprs_to_replace, schema)
                     .map_yes_no(
@@ -302,6 +344,22 @@ fn replace_column_with_semantic_id(
                     )
                 }
             }
+            Expr::List(items) => {
+                let mut transformed = false;
+                let mut new_items = Vec::<ExprRef>::new();
+                for item in items {
+                    let new_item =
+                        replace_column_with_semantic_id(item.clone(), subexprs_to_replace, schema);
+                    if new_item.transformed {
+                        new_items.push(new_item.data.clone());
+                        transformed = true;
+                    }
+                }
+                if transformed {
+                    return Transformed::yes(Expr::List(new_items).into());
+                }
+                Transformed::no(e)
+            }
             Expr::Between(child, lower, upper) => {
                 let child =
                     replace_column_with_semantic_id(child.clone(), subexprs_to_replace, schema);
@@ -382,13 +440,19 @@ fn replace_column_with_semantic_id(
                     .inputs
                     .iter()
                     .map(|e| {
-                        replace_column_with_semantic_id(e.clone(), subexprs_to_replace, schema)
+                        e.map(|e| {
+                            replace_column_with_semantic_id(e.clone(), subexprs_to_replace, schema)
+                        })
                     })
                     .collect::<Vec<_>>();
-                if transforms.iter().all(|e| !e.transformed) {
+                if transforms.iter().all(|e| !e.inner().transformed) {
                     Transformed::no(e)
                 } else {
-                    func.inputs = transforms.iter().map(|t| t.data.clone()).collect();
+                    let inputs = transforms
+                        .iter()
+                        .map(|t| t.map(|t| t.data.clone()))
+                        .collect::<Vec<_>>();
+                    func.inputs = FunctionArgs::new_unchecked(inputs);
                     Transformed::yes(Expr::ScalarFunction(func).into())
                 }
             }
@@ -477,6 +541,14 @@ fn replace_column_with_semantic_id_aggexpr(
             replace_column_with_semantic_id(child.clone(), subexprs_to_replace, schema)
                 .map_yes_no(AggExpr::Max, |_| e)
         }
+        AggExpr::BoolAnd(ref child) => {
+            replace_column_with_semantic_id(child.clone(), subexprs_to_replace, schema)
+                .map_yes_no(AggExpr::BoolAnd, |_| e)
+        }
+        AggExpr::BoolOr(ref child) => {
+            replace_column_with_semantic_id(child.clone(), subexprs_to_replace, schema)
+                .map_yes_no(AggExpr::BoolOr, |_| e)
+        }
         AggExpr::AnyValue(ref child, ignore_nulls) => {
             replace_column_with_semantic_id(child.clone(), subexprs_to_replace, schema).map_yes_no(
                 |transformed_child| AggExpr::AnyValue(transformed_child, ignore_nulls),
@@ -487,10 +559,18 @@ fn replace_column_with_semantic_id_aggexpr(
             replace_column_with_semantic_id(child.clone(), subexprs_to_replace, schema)
                 .map_yes_no(AggExpr::List, |_| e)
         }
+        AggExpr::Set(ref child) => {
+            replace_column_with_semantic_id(child.clone(), subexprs_to_replace, schema)
+                .map_yes_no(AggExpr::Set, |_| e)
+        }
         AggExpr::Concat(ref child) => {
             replace_column_with_semantic_id(child.clone(), subexprs_to_replace, schema)
                 .map_yes_no(AggExpr::Concat, |_| e)
         }
+        AggExpr::Skew(ref child) => {
+            replace_column_with_semantic_id(child.clone(), subexprs_to_replace, schema)
+                .map_yes_no(AggExpr::Skew, |_| e)
+        }
         AggExpr::MapGroups { func, inputs } => {
             let transforms = inputs
                 .iter()
@@ -512,7 +592,7 @@ fn replace_column_with_semantic_id_aggexpr(
 mod tests {
     use common_error::DaftResult;
     use daft_core::prelude::*;
-    use daft_dsl::{binary_op, col, lit, Operator};
+    use daft_dsl::{binary_op, lit, resolved_col, Operator};
 
     use crate::{
         ops::Project,
@@ -535,7 +615,7 @@ mod tests {
             Field::new("b", DataType::Int64),
         ]))
         .build();
-        let a2 = binary_op(Operator::Plus, col("a"), col("a"));
+        let a2 = binary_op(Operator::Plus, resolved_col("a"), resolved_col("a"));
         let a2_colname = a2.semantic_id(&source.schema()).id;
 
         let a4 = binary_op(Operator::Plus, a2.clone(), a2.clone());
@@ -545,12 +625,12 @@ mod tests {
         let expressions = vec![a8.alias("x")];
         let result_projection = Project::try_new(source, expressions)?;
 
-        let a4_col = col(a4_colname.clone());
+        let a4_col = resolved_col(a4_colname.clone());
         let expected_result_projection =
             vec![binary_op(Operator::Plus, a4_col.clone(), a4_col).alias("x")];
         assert_eq!(result_projection.projection, expected_result_projection);
 
-        let a2_col = col(a2_colname.clone());
+        let a2_col = resolved_col(a2_colname.clone());
         let expected_subprojection =
             vec![binary_op(Operator::Plus, a2_col.clone(), a2_col).alias(a4_colname)];
         let LogicalPlan::Project(subprojection) = result_projection.input.as_ref() else {
@@ -581,23 +661,23 @@ mod tests {
             Field::new("b", DataType::Int64),
         ]))
         .build();
-        let a2 = binary_op(Operator::Plus, col("a"), col("a"));
+        let a2 = binary_op(Operator::Plus, resolved_col("a"), resolved_col("a"));
         let a2_colname = a2.semantic_id(&source.schema()).id;
 
         let expressions = vec![
             a2.alias("x"),
-            binary_op(Operator::Plus, a2.clone(), col("a")).alias("y"),
+            binary_op(Operator::Plus, a2.clone(), resolved_col("a")).alias("y"),
         ];
         let result_projection = Project::try_new(source, expressions)?;
 
-        let a2_col = col(a2_colname.clone());
+        let a2_col = resolved_col(a2_colname.clone());
         let expected_result_projection = vec![
             a2_col.alias("x"),
-            binary_op(Operator::Plus, a2_col, col("a")).alias("y"),
+            binary_op(Operator::Plus, a2_col, resolved_col("a")).alias("y"),
         ];
         assert_eq!(result_projection.projection, expected_result_projection);
 
-        let expected_subprojection = vec![a2.alias(a2_colname), col("a").alias("a")];
+        let expected_subprojection = vec![a2.alias(a2_colname), resolved_col("a").alias("a")];
         let LogicalPlan::Project(subprojection) = result_projection.input.as_ref() else {
             panic!()
         };
@@ -622,8 +702,8 @@ mod tests {
         let expressions = vec![
             lit(3).alias("x"),
             lit(3).alias("y"),
-            col("a").alias("w"),
-            col("a").alias("z"),
+            resolved_col("a").alias("w"),
+            resolved_col("a").alias("z"),
         ];
         let result_projection = Project::try_new(source, expressions.clone())?;
 
