@@ -6,9 +6,11 @@ use common_treenode::Transformed;
 use super::{
     logical_plan_tracker::LogicalPlanTracker,
     rules::{
-        DropRepartition, EliminateCrossJoin, EnrichWithStats, FilterNullJoinKey,
-        LiftProjectFromAgg, MaterializeScans, OptimizerRule, PushDownFilter, PushDownLimit,
-        PushDownProjection, SimplifyExpressionsRule, SplitActorPoolProjects,
+        DetectMonotonicId, DropRepartition, EliminateCrossJoin, EliminateSubqueryAliasRule,
+        EnrichWithStats, ExtractWindowFunction, FilterNullJoinKey, LiftProjectFromAgg,
+        MaterializeScans, OptimizerRule, PushDownAntiSemiJoin, PushDownFilter,
+        PushDownJoinPredicate, PushDownLimit, PushDownProjection, ReorderJoins,
+        SimplifyExpressionsRule, SimplifyNullFilteredJoin, SplitActorPoolProjects,
         UnnestPredicateSubquery, UnnestScalarSubquery,
     },
 };
@@ -32,7 +34,7 @@ impl OptimizerConfig {
 impl Default for OptimizerConfig {
     fn default() -> Self {
         // Default to a max of 5 optimizer passes for a given batch.
-        Self::new(5)
+        Self::new(20)
     }
 }
 
@@ -80,6 +82,153 @@ pub enum RuleExecutionStrategy {
     FixedPoint(Option<usize>),
 }
 
+pub struct OptimizerBuilder {
+    // Batches of rules for the optimizer to apply.
+    rule_batches: Vec<RuleBatch>,
+    // Config for optimizer.
+    config: OptimizerConfig,
+}
+
+impl Default for OptimizerBuilder {
+    fn default() -> Self {
+        Self {
+            rule_batches: vec![
+                // --- Rewrite rules ---
+                RuleBatch::new(
+                    vec![
+                        Box::new(LiftProjectFromAgg::new()),
+                        Box::new(UnnestScalarSubquery::new()),
+                        Box::new(UnnestPredicateSubquery::new()),
+                        Box::new(EliminateSubqueryAliasRule::new()),
+                        Box::new(SplitActorPoolProjects::new()),
+                        Box::new(DetectMonotonicId::new()),
+                        Box::new(ExtractWindowFunction::new()),
+                    ],
+                    RuleExecutionStrategy::FixedPoint(None),
+                ),
+                // we want to simplify expressions first to make the rest of the rules easier
+                RuleBatch::new(
+                    vec![Box::new(SimplifyExpressionsRule::new())],
+                    RuleExecutionStrategy::FixedPoint(None),
+                ),
+                // --- Filter out null join keys ---
+                // This rule should be run once, before any filter pushdown rules.
+                RuleBatch::new(
+                    vec![Box::new(FilterNullJoinKey::new())],
+                    RuleExecutionStrategy::Once,
+                ),
+                // --- Anti/semi join pushdowns ---
+                // This needs to be separate from PushDownProjection and PushDownFilter
+                // because otherwise they will just keep swapping places.
+                // We ultimately do want filters to go before joins, so we run this rule before PushDownFilter.
+                RuleBatch::new(
+                    vec![Box::new(PushDownAntiSemiJoin::new())],
+                    RuleExecutionStrategy::FixedPoint(None),
+                ),
+                // --- Bulk of our rules ---
+                RuleBatch::new(
+                    vec![
+                        Box::new(DropRepartition::new()),
+                        Box::new(PushDownFilter::new()),
+                        Box::new(PushDownProjection::new()),
+                        Box::new(EliminateCrossJoin::new()),
+                        Box::new(SimplifyNullFilteredJoin::new()),
+                        Box::new(PushDownJoinPredicate::new()),
+                    ],
+                    // Use a fixed-point policy for the pushdown rules: PushDownProjection can produce a Filter node
+                    // at the current node, which would require another batch application in order to have a chance to push
+                    // that Filter node through upstream nodes.
+                    // TODO(Clark): Refine this fixed-point policy.
+                    RuleExecutionStrategy::FixedPoint(None),
+                ),
+                // --- Limit pushdowns ---
+                // This needs to be separate from PushDownProjection because otherwise the limit and
+                // projection just keep swapping places, preventing optimization
+                // (see https://github.com/Eventual-Inc/Daft/issues/2616)
+                RuleBatch::new(
+                    vec![Box::new(PushDownLimit::new())],
+                    RuleExecutionStrategy::FixedPoint(Some(3)),
+                ),
+                // --- Simplify expressions before scans are materialized ---
+                RuleBatch::new(
+                    vec![Box::new(SimplifyExpressionsRule::new())],
+                    RuleExecutionStrategy::FixedPoint(None),
+                ),
+                // --- Materialize scan nodes ---
+                RuleBatch::new(
+                    vec![Box::new(MaterializeScans::new())],
+                    RuleExecutionStrategy::Once,
+                ),
+                // --- Enrich logical plan with stats ---
+                RuleBatch::new(
+                    vec![Box::new(EnrichWithStats::new())],
+                    RuleExecutionStrategy::Once,
+                ),
+            ],
+            config: Default::default(),
+        }
+    }
+}
+
+impl OptimizerBuilder {
+    pub fn new() -> Self {
+        Self {
+            rule_batches: vec![],
+            config: Default::default(),
+        }
+    }
+
+    pub fn reorder_joins(mut self) -> Self {
+        self.rule_batches.push(RuleBatch::new(
+            vec![
+                Box::new(ReorderJoins::new()),
+                Box::new(PushDownFilter::new()),
+                Box::new(PushDownProjection::new()),
+                Box::new(EnrichWithStats::new()),
+            ],
+            RuleExecutionStrategy::Once,
+        ));
+        self
+    }
+
+    pub fn enrich_with_stats(mut self) -> Self {
+        self.rule_batches.push(RuleBatch::new(
+            vec![Box::new(EnrichWithStats::new())],
+            RuleExecutionStrategy::Once,
+        ));
+        self
+    }
+
+    pub fn simplify_expressions(mut self) -> Self {
+        // Try to simplify expressions again as other rules could introduce new exprs.
+        self.rule_batches.push(RuleBatch::new(
+            vec![Box::new(SimplifyExpressionsRule::new())],
+            RuleExecutionStrategy::FixedPoint(None),
+        ));
+        self
+    }
+
+    pub fn with_optimizer_config(mut self, config: OptimizerConfig) -> Self {
+        self.config = config;
+        self
+    }
+
+    pub fn build(self) -> Optimizer {
+        Optimizer {
+            rule_batches: self.rule_batches,
+            config: self.config,
+        }
+    }
+
+    pub fn when(self, condition: bool, f: impl FnOnce(Self) -> Self) -> Self {
+        if condition {
+            f(self)
+        } else {
+            self
+        }
+    }
+}
+
 /// Logical rule-based optimizer.
 pub struct Optimizer {
     // Batches of rules for the optimizer to apply.
@@ -89,66 +238,6 @@ pub struct Optimizer {
 }
 
 impl Optimizer {
-    pub fn new(config: OptimizerConfig) -> Self {
-        let rule_batches = vec![
-            // --- Rewrite rules ---
-            RuleBatch::new(
-                vec![
-                    Box::new(LiftProjectFromAgg::new()),
-                    Box::new(UnnestScalarSubquery::new()),
-                    Box::new(UnnestPredicateSubquery::new()),
-                    Box::new(SplitActorPoolProjects::new()),
-                ],
-                RuleExecutionStrategy::FixedPoint(None),
-            ),
-            // we want to simplify expressions first to make the rest of the rules easier
-            RuleBatch::new(
-                vec![Box::new(SimplifyExpressionsRule::new())],
-                RuleExecutionStrategy::FixedPoint(Some(3)),
-            ),
-            // --- Bulk of our rules ---
-            RuleBatch::new(
-                vec![
-                    Box::new(DropRepartition::new()),
-                    Box::new(FilterNullJoinKey::new()),
-                    Box::new(PushDownFilter::new()),
-                    Box::new(PushDownProjection::new()),
-                    Box::new(EliminateCrossJoin::new()),
-                ],
-                // Use a fixed-point policy for the pushdown rules: PushDownProjection can produce a Filter node
-                // at the current node, which would require another batch application in order to have a chance to push
-                // that Filter node through upstream nodes.
-                // TODO(Clark): Refine this fixed-point policy.
-                RuleExecutionStrategy::FixedPoint(Some(3)),
-            ),
-            // --- Limit pushdowns ---
-            // This needs to be separate from PushDownProjection because otherwise the limit and
-            // projection just keep swapping places, preventing optimization
-            // (see https://github.com/Eventual-Inc/Daft/issues/2616)
-            RuleBatch::new(
-                vec![Box::new(PushDownLimit::new())],
-                RuleExecutionStrategy::FixedPoint(Some(3)),
-            ),
-            // --- Materialize scan nodes ---
-            RuleBatch::new(
-                vec![Box::new(MaterializeScans::new())],
-                RuleExecutionStrategy::Once,
-            ),
-            // --- Enrich logical plan with stats ---
-            RuleBatch::new(
-                vec![Box::new(EnrichWithStats::new())],
-                RuleExecutionStrategy::Once,
-            ),
-            // try to simplify expressions again as other rules could introduce new exprs
-            RuleBatch::new(
-                vec![Box::new(SimplifyExpressionsRule::new())],
-                RuleExecutionStrategy::FixedPoint(Some(3)),
-            ),
-        ];
-
-        Self::with_rule_batches(rule_batches, config)
-    }
-
     pub fn with_rule_batches(rule_batches: Vec<RuleBatch>, config: OptimizerConfig) -> Self {
         Self {
             rule_batches,
@@ -247,7 +336,7 @@ mod tests {
     use common_error::DaftResult;
     use common_treenode::{Transformed, TreeNode};
     use daft_core::prelude::*;
-    use daft_dsl::{col, lit};
+    use daft_dsl::{lit, unresolved_col};
 
     use super::{Optimizer, OptimizerConfig, RuleBatch, RuleExecutionStrategy};
     use crate::{
@@ -316,9 +405,9 @@ mod tests {
             OptimizerConfig::new(20),
         );
         let proj_exprs = vec![
-            col("a").add(lit(1)),
-            col("a").add(lit(2)).alias("b"),
-            col("a").add(lit(3)).alias("c"),
+            unresolved_col("a").add(lit(1)),
+            unresolved_col("a").add(lit(2)).alias("b"),
+            unresolved_col("a").add(lit(3)).alias("c"),
         ];
         let plan = dummy_scan_node(dummy_scan_operator(vec![Field::new("a", DataType::Int64)]))
             .select(proj_exprs)?
@@ -351,9 +440,9 @@ mod tests {
             OptimizerConfig::new(20),
         );
         let proj_exprs = vec![
-            col("a").add(lit(1)),
-            col("a").add(lit(2)).alias("b"),
-            col("a").add(lit(3)).alias("c"),
+            unresolved_col("a").add(lit(1)),
+            unresolved_col("a").add(lit(2)).alias("b"),
+            unresolved_col("a").add(lit(3)).alias("c"),
         ];
         let plan = dummy_scan_node(dummy_scan_operator(vec![Field::new("a", DataType::Int64)]))
             .select(proj_exprs)?
@@ -402,11 +491,11 @@ mod tests {
             OptimizerConfig::new(20),
         );
         let proj_exprs = vec![
-            col("a").add(lit(1)),
-            col("a").add(lit(2)).alias("b"),
-            col("a").add(lit(3)).alias("c"),
+            unresolved_col("a").add(lit(1)),
+            unresolved_col("a").add(lit(2)).alias("b"),
+            unresolved_col("a").add(lit(3)).alias("c"),
         ];
-        let filter_predicate = col("a").lt(lit(2));
+        let filter_predicate = unresolved_col("a").lt(lit(2));
         let scan_op = dummy_scan_operator(vec![Field::new("a", DataType::Int64)]);
         let plan = dummy_scan_node(scan_op.clone())
             .select(proj_exprs.clone())?
