@@ -1,28 +1,41 @@
+mod detect_monotonic_id;
 mod drop_repartition;
 mod eliminate_cross_join;
+mod eliminate_subquery_alias;
 mod enrich_with_stats;
+mod extract_window_function;
 mod filter_null_join_key;
 mod lift_project_from_agg;
 mod materialize_scans;
+mod push_down_anti_semi_join;
 mod push_down_filter;
+mod push_down_join_predicate;
 mod push_down_limit;
 mod push_down_projection;
 mod reorder_joins;
 mod rule;
 mod simplify_expressions;
+mod simplify_null_filtered_join;
 mod split_actor_pool_projects;
 mod unnest_subquery;
 
+pub use detect_monotonic_id::DetectMonotonicId;
 pub use drop_repartition::DropRepartition;
 pub use eliminate_cross_join::EliminateCrossJoin;
+pub use eliminate_subquery_alias::EliminateSubqueryAliasRule;
 pub use enrich_with_stats::EnrichWithStats;
+pub use extract_window_function::ExtractWindowFunction;
 pub use filter_null_join_key::FilterNullJoinKey;
 pub use lift_project_from_agg::LiftProjectFromAgg;
 pub use materialize_scans::MaterializeScans;
+pub use push_down_anti_semi_join::PushDownAntiSemiJoin;
 pub use push_down_filter::PushDownFilter;
+pub use push_down_join_predicate::PushDownJoinPredicate;
 pub use push_down_limit::PushDownLimit;
 pub use push_down_projection::PushDownProjection;
+pub use reorder_joins::ReorderJoins;
 pub use rule::OptimizerRule;
 pub use simplify_expressions::SimplifyExpressionsRule;
+pub use simplify_null_filtered_join::SimplifyNullFilteredJoin;
 pub use split_actor_pool_projects::SplitActorPoolProjects;
 pub use unnest_subquery::{UnnestPredicateSubquery, UnnestScalarSubquery};
