@@ -1,11 +1,14 @@
 use std::{
     collections::{BTreeMap, HashMap, HashSet},
     fmt::Display,
+    io::Cursor,
+    pin::Pin,
     sync::{Arc, Mutex},
+    task::{Context, Poll},
 };
 
 use arrow2::io::parquet::read::schema::infer_schema_with_options;
-use common_error::DaftResult;
+use common_error::{DaftError, DaftResult};
 #[cfg(feature = "python")]
 use common_file_formats::DatabaseSourceConfig;
 use common_file_formats::{FileFormatConfig, ParquetSourceConfig};
@@ -19,9 +22,11 @@ use daft_json::{JsonConvertOptions, JsonParseOptions, JsonReadOptions};
 use daft_parquet::read::{
     read_parquet_bulk, read_parquet_metadata_bulk, ParquetSchemaInferenceOptions,
 };
+use daft_recordbatch::RecordBatch;
 use daft_scan::{storage_config::StorageConfig, ChunkSpec, DataSource, ScanTask};
 use daft_stats::{PartitionSpec, TableMetadata, TableStatistics};
-use daft_table::Table;
+use daft_warc::WarcConvertOptions;
+use futures::{Future, Stream};
 use parquet2::metadata::FileMetaData;
 use snafu::ResultExt;
 
@@ -30,7 +35,7 @@ use crate::{DaftCSVSnafu, DaftCoreComputeSnafu};
 #[derive(Debug)]
 pub enum TableState {
     Unloaded(Arc<ScanTask>),
-    Loaded(Arc<Vec<Table>>),
+    Loaded(Arc<Vec<RecordBatch>>),
 }
 
 impl Display for TableState {
@@ -84,7 +89,7 @@ pub struct MicroPartition {
     pub(crate) statistics: Option<TableStatistics>,
 }
 
-/// Helper to run all the IO and compute required to materialize a [`ScanTask`] into a `Vec<Table>`
+/// Helper to run all the IO and compute required to materialize a [`ScanTask`] into a `Vec<RecordBatch>`
 ///
 /// All [`Table`] objects returned will have the same [`Schema`] as [`ScanTask::materialized_schema`].
 ///
@@ -95,14 +100,14 @@ pub struct MicroPartition {
 fn materialize_scan_task(
     scan_task: Arc<ScanTask>,
     io_stats: Option<IOStatsRef>,
-) -> crate::Result<(Vec<Table>, SchemaRef)> {
+) -> crate::Result<(Vec<RecordBatch>, SchemaRef)> {
     let pushdown_columns = scan_task.pushdowns.columns.as_ref().map(|v| {
         v.iter()
             .map(std::string::String::as_str)
             .collect::<Vec<&str>>()
     });
     let file_column_names =
-        _get_file_column_names(pushdown_columns.as_deref(), scan_task.partition_spec());
+        get_file_column_names(pushdown_columns.as_deref(), scan_task.partition_spec());
 
     let urls = scan_task
         .sources
@@ -148,7 +153,7 @@ fn materialize_scan_task(
                 .into_iter()
                 .collect::<Vec<_>>();
 
-            let delete_map = _read_delete_files(
+            let delete_map = read_delete_files(
                 iceberg_delete_files.as_slice(),
                 urls.as_slice(),
                 io_client.clone(),
@@ -191,13 +196,7 @@ fn materialize_scan_task(
         FileFormatConfig::Csv(cfg) => {
             let schema_of_file = scan_task.schema.clone();
             let col_names = if !cfg.has_headers {
-                Some(
-                    schema_of_file
-                        .fields
-                        .values()
-                        .map(|f| f.name.as_str())
-                        .collect::<Vec<_>>(),
-                )
+                Some(schema_of_file.field_names().collect::<Vec<_>>())
             } else {
                 None
             };
@@ -266,6 +265,32 @@ fn materialize_scan_task(
             )
             .context(DaftCoreComputeSnafu)?
         }
+
+        // ****************
+        // Native Warc Reads
+        // ****************
+        FileFormatConfig::Warc(_) => {
+            let schema_of_file = scan_task.schema.clone();
+            let convert_options = WarcConvertOptions {
+                limit: scan_task.pushdowns.limit,
+                include_columns: file_column_names
+                    .as_ref()
+                    .map(|cols| cols.iter().map(|col| (*col).to_string()).collect()),
+                schema: schema_of_file,
+                predicate: scan_task.pushdowns.filters.clone(),
+            };
+            let uris = urls.collect::<Vec<_>>();
+            daft_warc::read_warc_bulk(
+                uris.as_slice(),
+                convert_options,
+                io_client,
+                io_stats,
+                scan_task.storage_config.multithreaded_io,
+                None,
+                8,
+            )
+            .context(DaftCoreComputeSnafu)?
+        }
         #[cfg(feature = "python")]
         FileFormatConfig::Database(DatabaseSourceConfig { sql, conn }) => {
             let predicate = scan_task
@@ -341,22 +366,22 @@ impl MicroPartition {
         }
     }
 
-    /// Create a new "loaded" MicroPartition using the materialized tables
+    /// Create a new "loaded" MicroPartition using the materialized record batches.
     ///
     /// Schema invariants:
-    /// 1. `schema` must match each Table's schema exactly
+    /// 1. `schema` must match each record batch's schema exactly
     /// 2. If `statistics` is provided, each Loaded column statistic must be castable to the corresponding column in the MicroPartition's schema
     #[must_use]
     pub fn new_loaded(
         schema: SchemaRef,
-        tables: Arc<Vec<Table>>,
+        record_batches: Arc<Vec<RecordBatch>>,
         statistics: Option<TableStatistics>,
     ) -> Self {
         // Check and validate invariants with asserts
-        for table in tables.iter() {
+        for batch in record_batches.iter() {
             assert!(
-                table.schema == schema,
-                "Loaded MicroPartition's tables' schema must match its own schema exactly"
+                batch.schema == schema,
+                "Loaded MicroPartition's batch schema must match its own schema exactly"
             );
         }
 
@@ -365,18 +390,31 @@ impl MicroPartition {
                 .cast_to_schema(schema.clone())
                 .expect("Statistics cannot be casted to schema")
         });
-        let tables_len_sum = tables.iter().map(daft_table::Table::len).sum();
+
+        // micropartition length is the length of all batches combined
+        let length = record_batches
+            .iter()
+            .map(daft_recordbatch::RecordBatch::len)
+            .sum();
 
         Self {
             schema,
-            state: Mutex::new(TableState::Loaded(tables)),
-            metadata: TableMetadata {
-                length: tables_len_sum,
-            },
+            state: Mutex::new(TableState::Loaded(record_batches)),
+            metadata: TableMetadata { length },
             statistics,
         }
     }
 
+    pub fn from_arrow<S: Into<SchemaRef>>(
+        schema: S,
+        arrays: Vec<Box<dyn arrow2::array::Array>>,
+    ) -> DaftResult<Self> {
+        let schema = schema.into();
+        let batch = RecordBatch::from_arrow(schema.clone(), arrays)?;
+        let batches = Arc::new(vec![batch]);
+        Ok(Self::new_loaded(schema, batches, None))
+    }
+
     pub fn from_scan_task(scan_task: Arc<ScanTask>, io_stats: IOStatsRef) -> crate::Result<Self> {
         let schema = scan_task.materialized_schema();
         match (
@@ -505,7 +543,7 @@ impl MicroPartition {
         let size_bytes = if let TableState::Loaded(tables) = &*guard {
             let total_size: usize = tables
                 .iter()
-                .map(daft_table::Table::size_bytes)
+                .map(daft_recordbatch::RecordBatch::size_bytes)
                 .collect::<DaftResult<Vec<_>>>()?
                 .iter()
                 .sum();
@@ -529,7 +567,10 @@ impl MicroPartition {
     ///
     /// "Reading if necessary" means I/O operations only occur for unloaded data,
     /// optimizing performance by avoiding redundant reads.
-    pub(crate) fn tables_or_read(&self, io_stats: IOStatsRef) -> crate::Result<Arc<Vec<Table>>> {
+    pub(crate) fn tables_or_read(
+        &self,
+        io_stats: IOStatsRef,
+    ) -> crate::Result<Arc<Vec<RecordBatch>>> {
         let mut guard = self.state.lock().unwrap();
         match &*guard {
             TableState::Unloaded(scan_task) => {
@@ -545,12 +586,12 @@ impl MicroPartition {
         }
     }
 
-    pub fn get_tables(&self) -> crate::Result<Arc<Vec<Table>>> {
+    pub fn get_tables(&self) -> crate::Result<Arc<Vec<RecordBatch>>> {
         let tables = self.tables_or_read(IOStatsContext::new("get tables"))?;
         Ok(tables)
     }
 
-    pub fn concat_or_get(&self, io_stats: IOStatsRef) -> crate::Result<Arc<Vec<Table>>> {
+    pub fn concat_or_get(&self, io_stats: IOStatsRef) -> crate::Result<Arc<Vec<RecordBatch>>> {
         let tables = self.tables_or_read(io_stats)?;
         if tables.len() <= 1 {
             return Ok(tables);
@@ -559,10 +600,10 @@ impl MicroPartition {
         let mut guard = self.state.lock().unwrap();
 
         if tables.len() > 1 {
-            let new_table = Table::concat(tables.iter().collect::<Vec<_>>().as_slice())
+            let new_table = RecordBatch::concat(tables.iter().collect::<Vec<_>>().as_slice())
                 .context(DaftCoreComputeSnafu)?;
             *guard = TableState::Loaded(Arc::new(vec![new_table]));
-        };
+        }
         if let TableState::Loaded(tables) = &*guard {
             assert_eq!(tables.len(), 1);
             Ok(tables.clone())
@@ -589,15 +630,10 @@ impl MicroPartition {
             })
             .collect::<DaftResult<Vec<_>>>()?;
 
-        let mut schema_with_id_index_map = self.schema.fields.clone();
-        schema_with_id_index_map.shift_insert(
-            0,
-            column_name.to_string(),
-            Field::new(column_name, DataType::UInt64),
-        );
-        let schema_with_id = Schema {
-            fields: schema_with_id_index_map,
-        };
+        let schema_with_id_index_map = std::iter::once(Field::new(column_name, DataType::UInt64))
+            .chain(self.schema.into_iter().cloned());
+
+        let schema_with_id = Schema::new(schema_with_id_index_map);
 
         Ok(Self::new_loaded(
             Arc::new(schema_with_id),
@@ -605,6 +641,45 @@ impl MicroPartition {
             self.statistics.clone(),
         ))
     }
+
+    pub fn write_to_ipc_stream(&self) -> DaftResult<Vec<u8>> {
+        let buffer = Vec::with_capacity(self.size_bytes()?.unwrap_or(0));
+        let schema = self.schema.to_arrow()?;
+        let options = arrow2::io::ipc::write::WriteOptions { compression: None };
+        let mut writer = arrow2::io::ipc::write::StreamWriter::new(buffer, options);
+        writer.start(&schema, None)?;
+        let tables = self.tables_or_read(IOStatsContext::new("write to stream"))?;
+        for table in tables.iter() {
+            let chunk = table.to_chunk();
+            writer.write(&chunk, None)?;
+        }
+        writer.finish()?;
+        let mut finished_buffer = writer.into_inner();
+        finished_buffer.shrink_to_fit();
+        Ok(finished_buffer)
+    }
+
+    pub fn read_from_ipc_stream(buffer: &[u8]) -> DaftResult<Self> {
+        let mut cursor = Cursor::new(buffer);
+        let stream_metadata = arrow2::io::ipc::read::read_stream_metadata(&mut cursor).unwrap();
+        let schema = Arc::new(Schema::from(stream_metadata.schema.clone()));
+        let reader = arrow2::io::ipc::read::StreamReader::new(cursor, stream_metadata, None);
+        let tables = reader
+            .into_iter()
+            .map(|state| {
+                let state = state?;
+                let arrow_chunk = match state {
+                    arrow2::io::ipc::read::StreamState::Some(chunk) => chunk,
+                    _ => panic!("State should not be waiting when reading from IPC buffer"),
+                };
+                let record_batch =
+                    RecordBatch::from_arrow(schema.clone(), arrow_chunk.into_arrays())?;
+                Ok(record_batch)
+            })
+            .collect::<DaftResult<Vec<_>>>()?;
+
+        Ok(Self::new_loaded(schema.into(), Arc::new(tables), None))
+    }
 }
 
 fn prune_fields_from_schema(
@@ -612,11 +687,7 @@ fn prune_fields_from_schema(
     columns: Option<&[&str]>,
 ) -> DaftResult<Arc<Schema>> {
     if let Some(columns) = columns {
-        let avail_names = schema
-            .fields
-            .keys()
-            .map(std::string::String::as_str)
-            .collect::<HashSet<_>>();
+        let avail_names = schema.field_names().collect::<HashSet<_>>();
         let mut names_to_keep = HashSet::new();
         for col_name in columns {
             if avail_names.contains(col_name) {
@@ -630,13 +701,10 @@ fn prune_fields_from_schema(
             }
         }
         let filtered_columns = schema
-            .as_ref()
-            .fields
-            .values()
+            .into_iter()
             .filter(|field| names_to_keep.contains(field.name.as_str()))
-            .cloned()
-            .collect::<Vec<_>>();
-        Ok(Arc::new(Schema::new(filtered_columns)?))
+            .cloned();
+        Ok(Arc::new(Schema::new(filtered_columns)))
     } else {
         Ok(schema)
     }
@@ -692,7 +760,7 @@ pub fn read_csv_into_micropartition(
             let unioned_schema = tables
                 .iter()
                 .map(|tbl| tbl.schema.clone())
-                .reduce(|s1, s2| Arc::new(s1.non_distinct_union(s2.as_ref())))
+                .try_reduce(|s1, s2| s1.non_distinct_union(s2.as_ref()).map(Arc::new))?
                 .unwrap();
             let tables = tables
                 .into_iter()
@@ -741,7 +809,7 @@ pub fn read_json_into_micropartition(
             let unioned_schema = tables
                 .iter()
                 .map(|tbl| tbl.schema.clone())
-                .reduce(|s1, s2| Arc::new(s1.non_distinct_union(s2.as_ref())))
+                .try_reduce(|s1, s2| s1.non_distinct_union(s2.as_ref()).map(Arc::new))?
                 .unwrap();
             let tables = tables
                 .into_iter()
@@ -758,7 +826,42 @@ pub fn read_json_into_micropartition(
     }
 }
 
-fn _get_file_column_names<'a>(
+pub fn read_warc_into_micropartition(
+    uris: &[&str],
+    schema: SchemaRef,
+    io_config: Arc<IOConfig>,
+    multithreaded_io: bool,
+    io_stats: Option<IOStatsRef>,
+) -> DaftResult<MicroPartition> {
+    let io_client = daft_io::get_io_client(multithreaded_io, io_config)?;
+    let convert_options = WarcConvertOptions {
+        limit: None,
+        include_columns: None,
+        schema: schema.clone(),
+        predicate: None,
+    };
+
+    match uris {
+        [] => Ok(MicroPartition::empty(None)),
+        uris => {
+            // Perform a bulk read of URIs, materializing a table per URI.
+            let tables = daft_warc::read_warc_bulk(
+                uris,
+                convert_options,
+                io_client,
+                io_stats,
+                multithreaded_io,
+                None,
+                8,
+            )
+            .context(DaftCoreComputeSnafu)?;
+
+            // Construct MicroPartition from tables and unioned schema
+            Ok(MicroPartition::new_loaded(schema, Arc::new(tables), None))
+        }
+    }
+}
+fn get_file_column_names<'a>(
     columns: Option<&'a [&'a str]>,
     partition_spec: Option<&PartitionSpec>,
 ) -> Option<Vec<&'a str>> {
@@ -783,7 +886,7 @@ fn _get_file_column_names<'a>(
     }
 }
 
-fn _read_delete_files(
+fn read_delete_files(
     delete_files: &[&str],
     uris: &[&str],
     io_client: Arc<IOClient>,
@@ -820,8 +923,20 @@ fn _read_delete_files(
     for table in &tables {
         // values in the file_path column are guaranteed by the iceberg spec to match the full URI of the corresponding data file
         // https://iceberg.apache.org/spec/#position-delete-files
-        let file_paths = table.get_column("file_path")?.downcast::<Utf8Array>()?;
-        let positions = table.get_column("pos")?.downcast::<Int64Array>()?;
+
+        let get_column_by_name = |name| {
+            if let [(idx, _)] = table.schema.get_fields_with_name(name)[..] {
+                Ok(table.get_column(idx))
+            } else {
+                Err(DaftError::SchemaMismatch(format!(
+                    "Iceberg delete files must have columns \"file_path\" and \"pos\", found: {}",
+                    table.schema
+                )))
+            }
+        };
+
+        let file_paths = get_column_by_name("file_path")?.downcast::<Utf8Array>()?;
+        let positions = get_column_by_name("pos")?.downcast::<Int64Array>()?;
 
         for i in 0..table.len() {
             let file = file_paths.get(i);
@@ -840,7 +955,7 @@ fn _read_delete_files(
 }
 
 #[allow(clippy::too_many_arguments)]
-fn _read_parquet_into_loaded_micropartition<T: AsRef<str>>(
+fn read_parquet_into_loaded_micropartition<T: AsRef<str>>(
     io_client: Arc<IOClient>,
     multithreaded_io: bool,
     uris: &[&str],
@@ -860,7 +975,7 @@ fn _read_parquet_into_loaded_micropartition<T: AsRef<str>>(
 ) -> DaftResult<MicroPartition> {
     let delete_map = iceberg_delete_files
         .map(|files| {
-            _read_delete_files(
+            read_delete_files(
                 files.as_slice(),
                 uris,
                 io_client.clone(),
@@ -878,7 +993,7 @@ fn _read_parquet_into_loaded_micropartition<T: AsRef<str>>(
             .collect::<Vec<&str>>()
     });
 
-    let file_column_names = _get_file_column_names(columns.as_deref(), partition_spec);
+    let file_column_names = get_file_column_names(columns.as_deref(), partition_spec);
     let all_tables = read_parquet_bulk(
         uris,
         file_column_names.as_deref(),
@@ -904,7 +1019,7 @@ fn _read_parquet_into_loaded_micropartition<T: AsRef<str>>(
         let unioned_schema = all_tables
             .iter()
             .map(|t| t.schema.clone())
-            .reduce(|l, r| l.non_distinct_union(&r).into());
+            .try_reduce(|l, r| l.non_distinct_union(&r).map(Arc::new))?;
         unioned_schema.expect("we need at least 1 schema")
     };
 
@@ -961,10 +1076,10 @@ pub fn read_parquet_into_micropartition<T: AsRef<str>>(
     // on the MicroPartition (e.g. its length). Hence we need to perform an eager read.
     if iceberg_delete_files
         .as_ref()
-        .map_or(false, |files| !files.is_empty())
+        .is_some_and(|files| !files.is_empty())
         || predicate.is_some()
     {
-        return _read_parquet_into_loaded_micropartition(
+        return read_parquet_into_loaded_micropartition(
             io_client,
             multithreaded_io,
             uris,
@@ -993,7 +1108,7 @@ pub fn read_parquet_into_micropartition<T: AsRef<str>>(
             .iter()
             .map(|m| {
                 let schema = infer_schema_with_options(m, Some((*schema_infer_options).into()))?;
-                let daft_schema = daft_core::prelude::Schema::try_from(&schema)?;
+                let daft_schema = schema.into();
                 DaftResult::Ok(Arc::new(daft_schema))
             })
             .collect::<DaftResult<Vec<_>>>()?;
@@ -1017,7 +1132,7 @@ pub fn read_parquet_into_micropartition<T: AsRef<str>>(
             .iter()
             .map(|m| {
                 let schema = infer_schema_with_options(m, Some((*schema_infer_options).into()))?;
-                let daft_schema = daft_core::prelude::Schema::try_from(&schema)?;
+                let daft_schema = schema.into();
                 DaftResult::Ok(Arc::new(daft_schema))
             })
             .collect::<DaftResult<Vec<_>>>()?;
@@ -1053,7 +1168,7 @@ pub fn read_parquet_into_micropartition<T: AsRef<str>>(
         } else {
             let unioned_schema = schemas
                 .into_iter()
-                .reduce(|l, r| Arc::new(l.non_distinct_union(&r)));
+                .try_reduce(|l, r| l.non_distinct_union(&r).map(Arc::new))?;
             unioned_schema.expect("we need at least 1 schema")
         };
 
@@ -1091,7 +1206,7 @@ pub fn read_parquet_into_micropartition<T: AsRef<str>>(
                 .zip(
                     row_groups
                         .clone()
-                        .unwrap_or_else(|| std::iter::repeat(None).take(uris.len()).collect()),
+                        .unwrap_or_else(|| std::iter::repeat_n(None, uris.len()).collect()),
                 )
                 .map(|((url, metadata), rgs)| DataSource::File {
                     path: url,
@@ -1139,7 +1254,7 @@ pub fn read_parquet_into_micropartition<T: AsRef<str>>(
         ))
     } else {
         // If no TableStatistics are available, we perform an eager read
-        _read_parquet_into_loaded_micropartition(
+        read_parquet_into_loaded_micropartition(
             io_client,
             multithreaded_io,
             uris,
@@ -1171,12 +1286,12 @@ impl Display for MicroPartition {
                 writeln!(f, "{}\n{}", self.schema, guard)?;
             }
             TableState::Loaded(tables) => {
-                if tables.len() == 0 {
+                if tables.is_empty() {
                     writeln!(f, "{}", self.schema)?;
                 }
                 writeln!(f, "{guard}")?;
             }
-        };
+        }
 
         match &self.statistics {
             Some(t) => writeln!(f, "Statistics\n{t}")?,
@@ -1187,5 +1302,112 @@ impl Display for MicroPartition {
     }
 }
 
+struct MicroPartitionStreamAdapter {
+    state: TableState,
+    current: usize,
+    pending_task: Option<tokio::task::JoinHandle<DaftResult<Vec<RecordBatch>>>>,
+}
+
+impl Stream for MicroPartitionStreamAdapter {
+    type Item = DaftResult<RecordBatch>;
+
+    fn poll_next(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Option<Self::Item>> {
+        let this = self.get_mut();
+
+        if let Some(handle) = &mut this.pending_task {
+            match Pin::new(handle).poll(cx) {
+                Poll::Ready(Ok(Ok(tables))) => {
+                    let tables = Arc::new(tables);
+                    this.state = TableState::Loaded(tables.clone());
+                    this.current = 0;
+                    this.pending_task = None;
+                    return Poll::Ready(tables.first().cloned().map(Ok));
+                }
+                Poll::Ready(Ok(Err(e))) => return Poll::Ready(Some(Err(e))),
+                Poll::Ready(Err(e)) => {
+                    return Poll::Ready(Some(Err(DaftError::InternalError(e.to_string()))))
+                }
+                Poll::Pending => return Poll::Pending,
+            }
+        }
+
+        match &this.state {
+            // if the state is unloaded, we spawn a task to load the tables
+            // and set the state to loaded
+            TableState::Unloaded(scan_task) => {
+                let scan_task = scan_task.clone();
+                let handle = tokio::spawn(async move {
+                    materialize_scan_task(scan_task, None)
+                        .map(|(tables, _)| tables)
+                        .map_err(DaftError::from)
+                });
+                this.pending_task = Some(handle);
+                cx.waker().wake_by_ref();
+                Poll::Pending
+            }
+            TableState::Loaded(tables) => {
+                let current = this.current;
+                if current < tables.len() {
+                    this.current = current + 1;
+                    Poll::Ready(tables.get(current).cloned().map(Ok))
+                } else {
+                    Poll::Ready(None)
+                }
+            }
+        }
+    }
+}
+impl MicroPartition {
+    pub fn into_stream(self: Arc<Self>) -> DaftResult<impl Stream<Item = DaftResult<RecordBatch>>> {
+        let state = match &*self.state.lock().unwrap() {
+            TableState::Unloaded(scan_task) => TableState::Unloaded(scan_task.clone()),
+            TableState::Loaded(tables) => TableState::Loaded(tables.clone()),
+        };
+
+        Ok(MicroPartitionStreamAdapter {
+            state,
+            current: 0,
+            pending_task: None,
+        })
+    }
+}
+
 #[cfg(test)]
-mod test {}
+mod tests {
+    use std::sync::Arc;
+
+    use common_error::DaftResult;
+    use daft_core::{
+        datatypes::{DataType, Field, Int32Array},
+        prelude::Schema,
+        series::IntoSeries,
+    };
+    use daft_recordbatch::RecordBatch;
+    use futures::StreamExt;
+
+    use crate::MicroPartition;
+
+    #[tokio::test]
+    async fn test_mp_stream() -> DaftResult<()> {
+        let columns = vec![Int32Array::from_values("a", vec![1].into_iter()).into_series()];
+        let columns2 = vec![Int32Array::from_values("a", vec![2].into_iter()).into_series()];
+        let schema = Schema::new(vec![Field::new("a", DataType::Int32)]);
+
+        let table1 = RecordBatch::from_nonempty_columns(columns)?;
+        let table2 = RecordBatch::from_nonempty_columns(columns2)?;
+
+        let mp = MicroPartition::new_loaded(
+            Arc::new(schema),
+            Arc::new(vec![table1.clone(), table2.clone()]),
+            None,
+        );
+        let mp = Arc::new(mp);
+
+        let mut stream = mp.into_stream()?;
+        let tbl = stream.next().await.unwrap().unwrap();
+        assert_eq!(tbl, table1);
+        let tbl = stream.next().await.unwrap().unwrap();
+        assert_eq!(tbl, table2);
+        Ok(())
+    }
+}
