@@ -1,33 +1,34 @@
 use std::sync::{Arc, Mutex};
 
-use common_error::DaftResult;
+use common_partitioning::{Partition, PartitionId, PartitionSet};
 use daft_core::{
     join::JoinSide,
     prelude::*,
     python::{PySchema, PySeries, PyTimeUnit},
 };
 use daft_csv::{CsvConvertOptions, CsvParseOptions, CsvReadOptions};
-use daft_dsl::python::PyExpr;
+use daft_dsl::{expr::bound_expr::BoundExpr, python::PyExpr};
 use daft_io::{python::IOConfig, IOStatsContext};
 use daft_json::{JsonConvertOptions, JsonParseOptions, JsonReadOptions};
 use daft_parquet::read::ParquetSchemaInferenceOptions;
+use daft_recordbatch::{python::PyRecordBatch, RecordBatch};
 use daft_scan::{
     python::pylib::PyScanTask, storage_config::StorageConfig, DataSource, ScanTask, ScanTaskRef,
 };
 use daft_stats::{TableMetadata, TableStatistics};
-use daft_table::{python::PyTable, Table};
 use pyo3::{exceptions::PyValueError, prelude::*, types::PyBytes, PyTypeInfo};
 use snafu::ResultExt;
 
 use crate::{
     micropartition::{MicroPartition, TableState},
+    partitioning::MicroPartitionSet,
     DaftCoreComputeSnafu, PyIOSnafu,
 };
 
 #[pyclass(module = "daft.daft", frozen)]
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 pub struct PyMicroPartition {
-    inner: Arc<MicroPartition>,
+    pub inner: Arc<MicroPartition>,
 }
 
 #[pymethods]
@@ -42,21 +43,71 @@ impl PyMicroPartition {
         Ok(self.inner.column_names())
     }
 
-    pub fn get_column(&self, name: &str, py: Python) -> PyResult<PySeries> {
+    #[deprecated(since = "TBD", note = "name-referenced columns")]
+    pub fn get_column_by_name(&self, name: &str, py: Python) -> PyResult<PySeries> {
+        let index = self.inner.schema().get_index(name)?;
+
         let tables = py.allow_threads(|| {
-            let io_stats = IOStatsContext::new(format!("PyMicroPartition::get_column: {name}"));
+            let io_stats =
+                IOStatsContext::new(format!("PyMicroPartition::get_column_by_name: {name}"));
             self.inner.concat_or_get(io_stats)
         })?;
         let columns = tables
             .iter()
-            .map(|t| t.get_column(name))
-            .collect::<DaftResult<Vec<_>>>()?;
+            .map(|t| t.get_column(index))
+            .collect::<Vec<_>>();
         match columns.as_slice() {
             [] => Ok(Series::empty(name, &self.inner.schema.get_field(name)?.dtype).into()),
             columns => Ok(Series::concat(columns)?.into()),
         }
     }
 
+    pub fn get_column(&self, idx: usize, py: Python) -> PyResult<PySeries> {
+        let tables = py.allow_threads(|| {
+            let io_stats = IOStatsContext::new(format!("PyMicroPartition::get_column: {idx}"));
+            self.inner.concat_or_get(io_stats)
+        })?;
+
+        if tables.is_empty() {
+            let field = &self.inner.schema()[idx];
+            Ok(Series::empty(&field.name, &field.dtype).into())
+        } else {
+            let columns = tables.iter().map(|t| t.get_column(idx)).collect::<Vec<_>>();
+
+            Ok(Series::concat(&columns)?.into())
+        }
+    }
+
+    pub fn columns(&self, py: Python) -> PyResult<Vec<PySeries>> {
+        let tables = py.allow_threads(|| {
+            let io_stats = IOStatsContext::new("PyMicroPartition::columns");
+            self.inner.concat_or_get(io_stats)
+        })?;
+
+        (0..self.inner.schema().len())
+            .map(|idx| {
+                if tables.is_empty() {
+                    let field = &self.inner.schema()[idx];
+                    Ok(Series::empty(&field.name, &field.dtype).into())
+                } else {
+                    let columns = tables.iter().map(|t| t.get_column(idx)).collect::<Vec<_>>();
+
+                    Ok(Series::concat(&columns)?.into())
+                }
+            })
+            .collect()
+    }
+
+    pub fn get_record_batches(&self, py: Python) -> PyResult<Vec<PyRecordBatch>> {
+        let record_batches = py.allow_threads(|| self.inner.get_tables())?;
+        Ok(record_batches
+            .iter()
+            .map(|rb| PyRecordBatch {
+                record_batch: rb.clone(),
+            })
+            .collect())
+    }
+
     pub fn size_bytes(&self) -> PyResult<Option<usize>> {
         Ok(self.inner.size_bytes()?)
     }
@@ -88,14 +139,19 @@ impl PyMicroPartition {
     }
 
     #[staticmethod]
-    pub fn from_tables(tables: Vec<PyTable>) -> PyResult<Self> {
-        match &tables[..] {
+    pub fn from_record_batches(record_batches: Vec<PyRecordBatch>) -> PyResult<Self> {
+        match &record_batches[..] {
             [] => Ok(MicroPartition::empty(None).into()),
             [first, ..] => {
-                let tables = Arc::new(tables.iter().map(|t| t.table.clone()).collect::<Vec<_>>());
+                let record_batches = Arc::new(
+                    record_batches
+                        .iter()
+                        .map(|t| t.record_batch.clone())
+                        .collect::<Vec<_>>(),
+                );
                 Ok(MicroPartition::new_loaded(
-                    first.table.schema.clone(),
-                    tables,
+                    first.record_batch.schema.clone(),
+                    record_batches,
                     // Don't compute statistics if data is already materialized
                     None,
                 )
@@ -105,6 +161,7 @@ impl PyMicroPartition {
     }
 
     #[staticmethod]
+    #[pyo3(signature = (schema=None))]
     pub fn empty(schema: Option<PySchema>) -> PyResult<Self> {
         Ok(MicroPartition::empty(match schema {
             Some(s) => Some(s.schema),
@@ -122,22 +179,24 @@ impl PyMicroPartition {
         // TODO: Cleanup and refactor code for sharing with Table
         let tables = record_batches
             .into_iter()
-            .map(|rb| daft_table::ffi::record_batches_to_table(py, &[rb], schema.schema.clone()))
+            .map(|rb| {
+                daft_recordbatch::ffi::record_batch_from_arrow(py, &[rb], schema.schema.clone())
+            })
             .collect::<PyResult<Vec<_>>>()?;
 
         Ok(MicroPartition::new_loaded(schema.schema.clone(), Arc::new(tables), None).into())
     }
 
     // Export Methods
-    pub fn to_table(&self, py: Python) -> PyResult<PyTable> {
+    pub fn to_record_batch(&self, py: Python) -> PyResult<PyRecordBatch> {
         let concatted = py.allow_threads(|| {
-            let io_stats = IOStatsContext::new("PyMicroPartition::to_table");
+            let io_stats = IOStatsContext::new("PyMicroPartition::to_record_batch");
             self.inner.concat_or_get(io_stats)
         })?;
         match &concatted.as_ref()[..] {
-            [] => PyTable::empty(Some(self.schema()?)),
-            [table] => Ok(PyTable {
-                table: table.clone(),
+            [] => PyRecordBatch::empty(Some(self.schema()?)),
+            [batch] => Ok(PyRecordBatch {
+                record_batch: batch.clone(),
             }),
             [..] => unreachable!("concat_or_get should return one or none"),
         }
@@ -264,6 +323,13 @@ impl PyMicroPartition {
         })
     }
 
+    #[pyo3(signature = (
+        right,
+        left_on,
+        right_on,
+        how,
+        null_equals_nulls=None
+    ))]
     pub fn hash_join(
         &self,
         py: Python,
@@ -368,6 +434,7 @@ impl PyMicroPartition {
         })
     }
 
+    #[pyo3(signature = (fraction, with_replacement, seed=None))]
     pub fn sample_by_fraction(
         &self,
         py: Python,
@@ -393,6 +460,7 @@ impl PyMicroPartition {
         })
     }
 
+    #[pyo3(signature = (size, with_replacement, seed=None))]
     pub fn sample_by_size(
         &self,
         py: Python,
@@ -478,7 +546,7 @@ impl PyMicroPartition {
         &self,
         py: Python,
         partition_keys: Vec<PyExpr>,
-        boundaries: &PyTable,
+        boundaries: &PyRecordBatch,
         descending: Vec<bool>,
     ) -> PyResult<Vec<Self>> {
         let exprs: Vec<daft_dsl::ExprRef> = partition_keys
@@ -488,7 +556,11 @@ impl PyMicroPartition {
         py.allow_threads(|| {
             Ok(self
                 .inner
-                .partition_by_range(exprs.as_slice(), &boundaries.table, descending.as_slice())?
+                .partition_by_range(
+                    exprs.as_slice(),
+                    &boundaries.record_batch,
+                    descending.as_slice(),
+                )?
                 .into_iter()
                 .map(std::convert::Into::into)
                 .collect::<Vec<Self>>())
@@ -530,6 +602,13 @@ impl PyMicroPartition {
     }
 
     #[staticmethod]
+    #[pyo3(signature = (
+        uri,
+        schema,
+        storage_config,
+        include_columns=None,
+        num_rows=None
+    ))]
     pub fn read_json(
         py: Python,
         uri: &str,
@@ -555,6 +634,14 @@ impl PyMicroPartition {
     }
 
     #[staticmethod]
+    #[pyo3(signature = (
+        uri,
+        convert_options=None,
+        parse_options=None,
+        read_options=None,
+        io_config=None,
+        multithreaded_io=None
+    ))]
     pub fn read_json_native(
         py: Python,
         uri: &str,
@@ -582,6 +669,14 @@ impl PyMicroPartition {
     }
 
     #[staticmethod]
+    #[pyo3(signature = (
+        uri,
+        convert_options=None,
+        parse_options=None,
+        read_options=None,
+        io_config=None,
+        multithreaded_io=None
+    ))]
     pub fn read_csv(
         py: Python,
         uri: &str,
@@ -609,6 +704,17 @@ impl PyMicroPartition {
 
     #[allow(clippy::too_many_arguments)]
     #[staticmethod]
+    #[pyo3(signature = (
+        uri,
+        columns=None,
+        start_offset=None,
+        num_rows=None,
+        row_groups=None,
+        predicate=None,
+        io_config=None,
+        multithreaded_io=None,
+        coerce_int96_timestamp_unit=None
+    ))]
     pub fn read_parquet(
         py: Python,
         uri: &str,
@@ -655,6 +761,19 @@ impl PyMicroPartition {
 
     #[allow(clippy::too_many_arguments)]
     #[staticmethod]
+    #[pyo3(signature = (
+        uris,
+        columns=None,
+        start_offset=None,
+        num_rows=None,
+        row_groups=None,
+        predicate=None,
+        io_config=None,
+        num_parallel_tasks=None,
+        multithreaded_io=None,
+        coerce_int96_timestamp_unit=None,
+        chunk_size=None
+    ))]
     pub fn read_parquet_bulk(
         py: Python,
         uris: Vec<String>,
@@ -701,6 +820,48 @@ impl PyMicroPartition {
         Ok(mp.into())
     }
 
+    #[staticmethod]
+    #[pyo3(signature = (
+        uri,
+        io_config=None,
+        multithreaded_io=None
+    ))]
+    pub fn read_warc(
+        py: Python,
+        uri: &str,
+        io_config: Option<IOConfig>,
+        multithreaded_io: Option<bool>,
+    ) -> PyResult<Self> {
+        let schema = Arc::new(Schema::new(vec![
+            Field::new("WARC-Record-ID", daft_core::prelude::DataType::Utf8),
+            Field::new("WARC-Type", daft_core::prelude::DataType::Utf8),
+            Field::new(
+                "WARC-Date",
+                daft_core::prelude::DataType::Timestamp(
+                    TimeUnit::Nanoseconds,
+                    Some("Etc/UTC".to_string()),
+                ),
+            ),
+            Field::new("Content-Length", daft_core::prelude::DataType::Int64),
+            Field::new(
+                "WARC-Identified-Payload-Type",
+                daft_core::prelude::DataType::Utf8,
+            ),
+            Field::new("warc_content", daft_core::prelude::DataType::Binary),
+            Field::new("warc_headers", daft_core::prelude::DataType::Utf8),
+        ]));
+        let mp = py.allow_threads(|| {
+            crate::micropartition::read_warc_into_micropartition(
+                &[uri],
+                schema.into(),
+                io_config.unwrap_or_default().config.into(),
+                multithreaded_io.unwrap_or(true),
+                None,
+            )
+        })?;
+        Ok(mp.into())
+    }
+
     #[staticmethod]
     pub fn _from_unloaded_table_state(
         schema_bytes: &[u8],
@@ -737,9 +898,9 @@ impl PyMicroPartition {
         let tables = table_objs
             .into_iter()
             .map(|p| {
-                Ok(p.getattr(py, pyo3::intern!(py, "_table"))?
-                    .extract::<PyTable>(py)?
-                    .table)
+                Ok(p.getattr(py, pyo3::intern!(py, "_recordbatch"))?
+                    .extract::<PyRecordBatch>(py)?
+                    .record_batch)
             })
             .collect::<PyResult<Vec<_>>>()?;
 
@@ -753,34 +914,37 @@ impl PyMicroPartition {
     }
 
     pub fn __reduce__(&self, py: Python) -> PyResult<(PyObject, PyObject)> {
-        let schema_bytes = PyBytes::new_bound(py, &bincode::serialize(&self.inner.schema).unwrap());
+        let schema_bytes = PyBytes::new(py, &bincode::serialize(&self.inner.schema).unwrap());
 
         let py_metadata_bytes =
-            PyBytes::new_bound(py, &bincode::serialize(&self.inner.metadata).unwrap());
-        let py_stats_bytes =
-            PyBytes::new_bound(py, &bincode::serialize(&self.inner.statistics).unwrap());
+            PyBytes::new(py, &bincode::serialize(&self.inner.metadata).unwrap());
+        let py_stats_bytes = PyBytes::new(py, &bincode::serialize(&self.inner.statistics).unwrap());
 
         let guard = self.inner.state.lock().unwrap();
         if let TableState::Loaded(tables) = &*guard {
             let _from_pytable = py
-                .import_bound(pyo3::intern!(py, "daft.table"))?
-                .getattr(pyo3::intern!(py, "Table"))?
-                .getattr(pyo3::intern!(py, "_from_pytable"))?;
+                .import(pyo3::intern!(py, "daft.recordbatch"))?
+                .getattr(pyo3::intern!(py, "RecordBatch"))?
+                .getattr(pyo3::intern!(py, "_from_pyrecordbatch"))?;
 
-            let pytables = tables.iter().map(|t| PyTable { table: t.clone() });
+            let pytables = tables.iter().map(|t| PyRecordBatch {
+                record_batch: t.clone(),
+            });
             let pyobjs = pytables
                 .map(|pt| _from_pytable.call1((pt,)))
                 .collect::<PyResult<Vec<_>>>()?;
             Ok((
-                Self::type_object_bound(py)
+                Self::type_object(py)
                     .getattr(pyo3::intern!(py, "_from_loaded_table_state"))?
                     .into(),
-                (schema_bytes, pyobjs, py_metadata_bytes, py_stats_bytes).to_object(py),
+                (schema_bytes, pyobjs, py_metadata_bytes, py_stats_bytes)
+                    .into_pyobject(py)?
+                    .into(),
             ))
         } else if let TableState::Unloaded(params) = &*guard {
-            let py_params_bytes = PyBytes::new_bound(py, &bincode::serialize(params).unwrap());
+            let py_params_bytes = PyBytes::new(py, &bincode::serialize(params).unwrap());
             Ok((
-                Self::type_object_bound(py)
+                Self::type_object(py)
                     .getattr(pyo3::intern!(py, "_from_unloaded_table_state"))?
                     .into(),
                 (
@@ -789,12 +953,26 @@ impl PyMicroPartition {
                     py_metadata_bytes,
                     py_stats_bytes,
                 )
-                    .to_object(py),
+                    .into_pyobject(py)?
+                    .into(),
             ))
         } else {
             unreachable!()
         }
     }
+
+    pub fn write_to_ipc_stream<'a>(&'a self, py: Python<'a>) -> PyResult<Bound<'a, PyBytes>> {
+        let buffer = py.allow_threads(|| self.inner.write_to_ipc_stream())?;
+        let bytes = PyBytes::new(py, &buffer);
+        Ok(bytes)
+    }
+
+    #[staticmethod]
+    pub fn read_from_ipc_stream(bytes: Bound<'_, PyBytes>, py: Python) -> PyResult<Self> {
+        let buffer = bytes.as_bytes();
+        let mp = py.allow_threads(|| MicroPartition::read_from_ipc_stream(buffer))?;
+        Ok(mp.into())
+    }
 }
 
 pub fn read_json_into_py_table(
@@ -804,22 +982,22 @@ pub fn read_json_into_py_table(
     storage_config: StorageConfig,
     include_columns: Option<Vec<String>>,
     num_rows: Option<usize>,
-) -> PyResult<PyTable> {
+) -> PyResult<PyRecordBatch> {
     let read_options = py
-        .import_bound(pyo3::intern!(py, "daft.runners.partitioning"))?
+        .import(pyo3::intern!(py, "daft.runners.partitioning"))?
         .getattr(pyo3::intern!(py, "TableReadOptions"))?
         .call1((num_rows, include_columns))?;
     let py_schema = py
-        .import_bound(pyo3::intern!(py, "daft.logical.schema"))?
+        .import(pyo3::intern!(py, "daft.logical.schema"))?
         .getattr(pyo3::intern!(py, "Schema"))?
         .getattr(pyo3::intern!(py, "_from_pyschema"))?
         .call1((schema,))?;
-    py.import_bound(pyo3::intern!(py, "daft.table.table_io"))?
+    py.import(pyo3::intern!(py, "daft.recordbatch.recordbatch_io"))?
         .getattr(pyo3::intern!(py, "read_json"))?
         .call1((uri, py_schema, storage_config, read_options))?
-        .getattr(pyo3::intern!(py, "to_table"))?
+        .getattr(pyo3::intern!(py, "to_record_batch"))?
         .call0()?
-        .getattr(pyo3::intern!(py, "_table"))?
+        .getattr(pyo3::intern!(py, "_recordbatch"))?
         .extract()
 }
 
@@ -834,27 +1012,27 @@ pub fn read_csv_into_py_table(
     storage_config: StorageConfig,
     include_columns: Option<Vec<String>>,
     num_rows: Option<usize>,
-) -> PyResult<PyTable> {
+) -> PyResult<PyRecordBatch> {
     let py_schema = py
-        .import_bound(pyo3::intern!(py, "daft.logical.schema"))?
+        .import(pyo3::intern!(py, "daft.logical.schema"))?
         .getattr(pyo3::intern!(py, "Schema"))?
         .getattr(pyo3::intern!(py, "_from_pyschema"))?
         .call1((schema,))?;
     let read_options = py
-        .import_bound(pyo3::intern!(py, "daft.runners.partitioning"))?
+        .import(pyo3::intern!(py, "daft.runners.partitioning"))?
         .getattr(pyo3::intern!(py, "TableReadOptions"))?
         .call1((num_rows, include_columns))?;
     let header_idx = if has_header { Some(0) } else { None };
     let parse_options = py
-        .import_bound(pyo3::intern!(py, "daft.runners.partitioning"))?
+        .import(pyo3::intern!(py, "daft.runners.partitioning"))?
         .getattr(pyo3::intern!(py, "TableParseCSVOptions"))?
         .call1((delimiter, header_idx, double_quote))?;
-    py.import_bound(pyo3::intern!(py, "daft.table.table_io"))?
+    py.import(pyo3::intern!(py, "daft.recordbatch.recordbatch_io"))?
         .getattr(pyo3::intern!(py, "read_csv"))?
         .call1((uri, py_schema, storage_config, parse_options, read_options))?
-        .getattr(pyo3::intern!(py, "to_table"))?
+        .getattr(pyo3::intern!(py, "to_record_batch"))?
         .call0()?
-        .getattr(pyo3::intern!(py, "_table"))?
+        .getattr(pyo3::intern!(py, "_recordbatch"))?
         .extract()
 }
 
@@ -866,31 +1044,31 @@ pub fn read_parquet_into_py_table(
     storage_config: StorageConfig,
     include_columns: Option<Vec<String>>,
     num_rows: Option<usize>,
-) -> PyResult<PyTable> {
+) -> PyResult<PyRecordBatch> {
     let py_schema = py
-        .import_bound(pyo3::intern!(py, "daft.logical.schema"))?
+        .import(pyo3::intern!(py, "daft.logical.schema"))?
         .getattr(pyo3::intern!(py, "Schema"))?
         .getattr(pyo3::intern!(py, "_from_pyschema"))?
         .call1((schema,))?;
     let read_options = py
-        .import_bound(pyo3::intern!(py, "daft.runners.partitioning"))?
+        .import(pyo3::intern!(py, "daft.runners.partitioning"))?
         .getattr(pyo3::intern!(py, "TableReadOptions"))?
         .call1((num_rows, include_columns))?;
     let py_coerce_int96_timestamp_unit = py
-        .import_bound(pyo3::intern!(py, "daft.datatype"))?
+        .import(pyo3::intern!(py, "daft.datatype"))?
         .getattr(pyo3::intern!(py, "TimeUnit"))?
         .getattr(pyo3::intern!(py, "_from_pytimeunit"))?
         .call1((coerce_int96_timestamp_unit,))?;
     let parse_options = py
-        .import_bound(pyo3::intern!(py, "daft.runners.partitioning"))?
+        .import(pyo3::intern!(py, "daft.runners.partitioning"))?
         .getattr(pyo3::intern!(py, "TableParseParquetOptions"))?
         .call1((py_coerce_int96_timestamp_unit,))?;
-    py.import_bound(pyo3::intern!(py, "daft.table.table_io"))?
+    py.import(pyo3::intern!(py, "daft.recordbatch.recordbatch_io"))?
         .getattr(pyo3::intern!(py, "read_parquet"))?
         .call1((uri, py_schema, storage_config, read_options, parse_options))?
-        .getattr(pyo3::intern!(py, "to_table"))?
+        .getattr(pyo3::intern!(py, "to_record_batch"))?
         .call0()?
-        .getattr(pyo3::intern!(py, "_table"))?
+        .getattr(pyo3::intern!(py, "_recordbatch"))?
         .extract()
 }
 
@@ -902,15 +1080,15 @@ pub fn read_sql_into_py_table(
     schema: PySchema,
     include_columns: Option<Vec<String>>,
     num_rows: Option<usize>,
-) -> PyResult<PyTable> {
+) -> PyResult<PyRecordBatch> {
     let py_schema = py
-        .import_bound(pyo3::intern!(py, "daft.logical.schema"))?
+        .import(pyo3::intern!(py, "daft.logical.schema"))?
         .getattr(pyo3::intern!(py, "Schema"))?
         .getattr(pyo3::intern!(py, "_from_pyschema"))?
         .call1((schema,))?;
     let py_predicate = match predicate {
         Some(p) => Some(
-            py.import_bound(pyo3::intern!(py, "daft.expressions.expressions"))?
+            py.import(pyo3::intern!(py, "daft.expressions.expressions"))?
                 .getattr(pyo3::intern!(py, "Expression"))?
                 .getattr(pyo3::intern!(py, "_from_pyexpr"))?
                 .call1((p,))?,
@@ -918,21 +1096,21 @@ pub fn read_sql_into_py_table(
         None => None,
     };
     let read_options = py
-        .import_bound(pyo3::intern!(py, "daft.runners.partitioning"))?
+        .import(pyo3::intern!(py, "daft.runners.partitioning"))?
         .getattr(pyo3::intern!(py, "TableReadOptions"))?
         .call1((num_rows, include_columns))?;
-    py.import_bound(pyo3::intern!(py, "daft.table.table_io"))?
+    py.import(pyo3::intern!(py, "daft.recordbatch.recordbatch_io"))?
         .getattr(pyo3::intern!(py, "read_sql"))?
         .call1((sql, conn, py_schema, read_options, py_predicate))?
-        .getattr(pyo3::intern!(py, "to_table"))?
+        .getattr(pyo3::intern!(py, "to_record_batch"))?
         .call0()?
-        .getattr(pyo3::intern!(py, "_table"))?
+        .getattr(pyo3::intern!(py, "_recordbatch"))?
         .extract()
 }
 
 pub fn read_pyfunc_into_table_iter(
     scan_task: &ScanTaskRef,
-) -> crate::Result<impl Iterator<Item = crate::Result<Table>>> {
+) -> crate::Result<impl Iterator<Item = crate::Result<RecordBatch>>> {
     let table_iterators = scan_task.sources.iter().map(|source| {
         // Call Python function to create an Iterator (Grabs the GIL and then releases it)
         match source {
@@ -943,11 +1121,11 @@ pub fn read_pyfunc_into_table_iter(
                 ..
             } => {
                 Python::with_gil(|py| {
-                    let func = py.import_bound(module.as_str())
+                    let func = py.import(module.as_str())
                         .unwrap_or_else(|_| panic!("Cannot import factory function from module {module}"))
                         .getattr(func_name.as_str())
                         .unwrap_or_else(|_| panic!("Cannot find function {func_name} in module {module}"));
-                    func.call(func_args.to_pytuple(py), None)
+                    func.call(func_args.to_pytuple(py).with_context(|_| PyIOSnafu)?, None)
                         .with_context(|_| PyIOSnafu)
                         .map(Into::<PyObject>::into)
                 })
@@ -970,9 +1148,9 @@ pub fn read_pyfunc_into_table_iter(
                         .map(|result| {
                             result
                                 .map(|tbl| {
-                                    tbl.extract::<daft_table::python::PyTable>()
-                                        .expect("Must be a PyTable")
-                                        .table
+                                    tbl.extract::<daft_recordbatch::python::PyRecordBatch>()
+                                        .expect("Must be a PyRecordBatch")
+                                        .record_batch
                                 })
                                 .with_context(|_| PyIOSnafu)
                         })
@@ -990,10 +1168,13 @@ pub fn read_pyfunc_into_table_iter(
                 Err(e) => Some(Err(e)),
                 Ok(table) => {
                     // Apply filters
-                    let post_pushdown_table = || -> crate::Result<Table> {
+                    let post_pushdown_table = || -> crate::Result<RecordBatch> {
                         let table = if let Some(filters) = scan_task_filters.as_ref() {
+                            let filters = BoundExpr::try_new(filters.clone(), &table.schema)
+                                .with_context(|_| DaftCoreComputeSnafu)?;
+
                             table
-                                .filter(&[filters.clone()])
+                                .filter(&[filters])
                                 .with_context(|_| DaftCoreComputeSnafu)?
                         } else {
                             table
@@ -1044,7 +1225,66 @@ impl From<PyMicroPartition> for Arc<MicroPartition> {
     }
 }
 
+/// TODO chore: cutover LocalPartitionSet to use this pyclass.
+#[pyclass(module = "daft.daft")]
+#[derive(Clone, Debug)]
+pub struct PyMicroPartitionSet(Arc<MicroPartitionSet>);
+
+#[pymethods]
+impl PyMicroPartitionSet {
+    fn get_partition(&self, idx: PartitionId) -> PyResult<PyMicroPartition> {
+        Ok(self.0.get_partition(&idx)?.into())
+    }
+
+    fn set_partition(&mut self, idx: PartitionId, part: PyMicroPartition) -> PyResult<()> {
+        Ok(self.0.set_partition(idx, &part.inner)?)
+    }
+
+    fn delete_partition(&mut self, idx: PartitionId) -> PyResult<()> {
+        Ok(self.0.delete_partition(&idx)?)
+    }
+
+    fn has_partition(&self, idx: PartitionId) -> PyResult<bool> {
+        Ok(self.0.has_partition(&idx))
+    }
+
+    fn __len__(&self) -> PyResult<usize> {
+        Ok(self.0.len())
+    }
+
+    fn size_bytes(&self) -> PyResult<usize> {
+        Ok(self.0.size_bytes()?)
+    }
+
+    fn num_partitions(&self) -> PyResult<usize> {
+        Ok(self.0.num_partitions())
+    }
+
+    fn wait(&self) -> PyResult<()> {
+        Ok(())
+    }
+}
+
+impl From<MicroPartitionSet> for PyMicroPartitionSet {
+    fn from(value: MicroPartitionSet) -> Self {
+        Arc::new(value).into()
+    }
+}
+
+impl From<Arc<MicroPartitionSet>> for PyMicroPartitionSet {
+    fn from(value: Arc<MicroPartitionSet>) -> Self {
+        Self(value)
+    }
+}
+
+impl From<PyMicroPartitionSet> for Arc<MicroPartitionSet> {
+    fn from(value: PyMicroPartitionSet) -> Self {
+        value.0
+    }
+}
+
 pub fn register_modules(parent: &Bound<PyModule>) -> PyResult<()> {
     parent.add_class::<PyMicroPartition>()?;
+    parent.add_class::<PyMicroPartitionSet>()?;
     Ok(())
 }
