@@ -6,12 +6,12 @@ use std::{
 
 use arrow2::io::parquet::read::{column_iter_to_arrays, schema::infer_schema_with_options};
 use common_error::DaftResult;
-use common_runtime::get_compute_runtime;
+use common_runtime::get_io_runtime;
 use daft_core::{prelude::*, utils::arrow::cast_array_for_daft_if_needed};
 use daft_dsl::ExprRef;
 use daft_io::{IOClient, IOStatsRef};
+use daft_recordbatch::RecordBatch;
 use daft_stats::TruthValue;
-use daft_table::Table;
 use futures::{future::try_join_all, stream::BoxStream, StreamExt};
 use parquet2::{
     page::{CompressedPage, Page},
@@ -30,8 +30,8 @@ use crate::{
     stream_reader::spawn_column_iters_to_table_task,
     utils::combine_stream,
     JoinSnafu, OneShotRecvSnafu, UnableToConvertRowGroupMetadataToStatsSnafu,
-    UnableToConvertSchemaToDaftSnafu, UnableToCreateParquetPageStreamSnafu,
-    UnableToParseSchemaFromMetadataSnafu, UnableToRunExpressionOnStatsSnafu, PARQUET_MORSEL_SIZE,
+    UnableToCreateParquetPageStreamSnafu, UnableToParseSchemaFromMetadataSnafu,
+    UnableToRunExpressionOnStatsSnafu, PARQUET_MORSEL_SIZE,
 };
 
 pub struct ParquetReaderBuilder {
@@ -279,10 +279,7 @@ impl ParquetReaderBuilder {
                 .retain(|f| names_to_keep.contains(f.name.as_str()));
         }
 
-        let daft_schema =
-            Schema::try_from(&arrow_schema).with_context(|_| UnableToConvertSchemaToDaftSnafu {
-                path: self.uri.to_string(),
-            })?;
+        let daft_schema = Schema::from(&arrow_schema);
         let row_ranges = build_row_ranges(
             self.limit,
             self.row_start_offset,
@@ -405,10 +402,8 @@ impl ParquetFileReader {
         original_columns: Option<Vec<String>>,
         original_num_rows: Option<usize>,
         delete_rows: Option<Vec<i64>>,
-    ) -> DaftResult<BoxStream<'static, DaftResult<Table>>> {
-        let daft_schema = Arc::new(daft_core::prelude::Schema::try_from(
-            self.arrow_schema.as_ref(),
-        )?);
+    ) -> DaftResult<BoxStream<'static, DaftResult<RecordBatch>>> {
+        let daft_schema = Arc::new(self.arrow_schema.as_ref().into());
 
         let num_parallel_tasks = determine_parquet_parallelism(&daft_schema);
         let semaphore = Arc::new(tokio::sync::Semaphore::new(num_parallel_tasks));
@@ -503,9 +498,9 @@ impl ParquetFileReader {
                 (row_range, chunk_iter_task)
             });
 
-        let compute_runtime = get_compute_runtime();
+        let io_runtime = get_io_runtime(true);
         let uri = self.uri.clone();
-        let parquet_task = compute_runtime.spawn(async move {
+        let parquet_task = io_runtime.spawn(async move {
             let mut table_tasks = Vec::with_capacity(chunk_iter_handles.len());
             for ((row_range, chunk_iter_handle), output_sender) in chunk_iter_handles.zip(senders) {
                 // We want to ensure that the channel capacity can hold one morsel worth of data for better deserialization performance.
@@ -556,7 +551,7 @@ impl ParquetFileReader {
     pub async fn read_from_ranges_into_table(
         self,
         ranges: Arc<RangesContainer>,
-    ) -> DaftResult<Table> {
+    ) -> DaftResult<RecordBatch> {
         let metadata = self.metadata;
         let all_handles = self
             .arrow_schema
@@ -665,7 +660,6 @@ impl ParquetFileReader {
                                         if (curr_index + arr.len()) < row_range.start {
                                             // throw arrays less than what we need
                                             curr_index += arr.len();
-                                            continue;
                                         } else if curr_index < row_range.start {
                                             let offset = row_range.start.saturating_sub(curr_index);
                                             all_arrays.push(arr.sliced(offset, arr.len() - offset));
@@ -731,8 +725,8 @@ impl ParquetFileReader {
             .into_iter()
             .collect::<DaftResult<Vec<_>>>()?;
 
-        Table::new_with_size(
-            Schema::new(all_series.iter().map(|s| s.field().clone()).collect())?,
+        RecordBatch::new_with_size(
+            Schema::new(all_series.iter().map(|s| s.field().clone())),
             all_series,
             self.row_ranges.as_ref().iter().map(|rr| rr.num_rows).sum(),
         )
@@ -848,7 +842,6 @@ impl ParquetFileReader {
                                         if (curr_index + arr.len()) < row_range.start {
                                             // throw arrays less than what we need
                                             curr_index += arr.len();
-                                            continue;
                                         } else if curr_index < row_range.start {
                                             let offset = row_range.start.saturating_sub(curr_index);
                                             all_arrays.push(arr.sliced(offset, arr.len() - offset));
