@@ -7,12 +7,14 @@ use daft_logical_plan::LogicalPlan;
 mod planner;
 
 use planner::PhysicalPlanTranslator;
-pub use planner::{AdaptivePlanner, MaterializedResults, QueryStageOutput};
+pub use planner::{AdaptivePlanner, MaterializedResults, QueryStageOutput, StageStats};
 
 use crate::{optimization::optimizer::PhysicalOptimizer, PhysicalPlanRef};
 mod translate;
 pub use translate::{extract_agg_expr, populate_aggregation_stages};
 
+mod display;
+
 /// Translate a logical plan to a physical plan.
 pub fn logical_to_physical(
     logical_plan: Arc<LogicalPlan>,
