@@ -19,6 +19,7 @@ pub enum PhysicalPlan {
     InMemoryScan(InMemoryScan),
     TabularScan(TabularScan),
     EmptyScan(EmptyScan),
+    PreviousStageScan(PreviousStageScan),
     Project(Project),
     ActorPoolProject(ActorPoolProject),
     Filter(Filter),
@@ -26,6 +27,7 @@ pub enum PhysicalPlan {
     Explode(Explode),
     Unpivot(Unpivot),
     Sort(Sort),
+    TopN(TopN),
     Sample(Sample),
     MonotonicallyIncreasingId(MonotonicallyIncreasingId),
     Aggregate(Aggregate),
@@ -45,6 +47,8 @@ pub enum PhysicalPlan {
     DeltaLakeWrite(DeltaLakeWrite),
     #[cfg(feature = "python")]
     LanceWrite(LanceWrite),
+    #[cfg(feature = "python")]
+    DataSink(DataSink),
 }
 
 impl PhysicalPlan {
@@ -64,6 +68,9 @@ impl PhysicalPlan {
             Self::EmptyScan(EmptyScan {
                 clustering_spec, ..
             }) => clustering_spec.clone(),
+            Self::PreviousStageScan(PreviousStageScan {
+                clustering_spec, ..
+            }) => clustering_spec.clone(),
             Self::Project(Project {
                 clustering_spec, ..
             }) => clustering_spec.clone(),
@@ -94,6 +101,17 @@ impl PhysicalPlan {
                 descending.clone(),
             ))
             .into(),
+            Self::TopN(TopN {
+                input,
+                sort_by,
+                descending,
+                ..
+            }) => ClusteringSpec::Range(RangeClusteringConfig::new(
+                input.clustering_spec().num_partitions(),
+                sort_by.clone(),
+                descending.clone(),
+            ))
+            .into(),
             Self::ShuffleExchange(shuffle_exchange) => shuffle_exchange.clustering_spec(),
             Self::Aggregate(Aggregate {
                 input,
@@ -168,7 +186,7 @@ impl PhysicalPlan {
                 ),
                 left_on.clone(),
                 // TODO(Clark): Propagate descending vec once sort-merge join supports descending sort orders.
-                std::iter::repeat(false).take(left_on.len()).collect(),
+                std::iter::repeat_n(false, left_on.len()).collect(),
             ))
             .into(),
             Self::CrossJoin(CrossJoin {
@@ -178,78 +196,85 @@ impl PhysicalPlan {
             Self::TabularWriteCsv(TabularWriteCsv { input, .. }) => input.clustering_spec(),
             Self::TabularWriteJson(TabularWriteJson { input, .. }) => input.clustering_spec(),
             #[cfg(feature = "python")]
-            Self::IcebergWrite(_) | Self::DeltaLakeWrite(_) | Self::LanceWrite(_) => {
-                ClusteringSpec::Unknown(UnknownClusteringConfig::new(1)).into()
-            }
+            Self::IcebergWrite(_)
+            | Self::DeltaLakeWrite(_)
+            | Self::LanceWrite(_)
+            | Self::DataSink(_) => ClusteringSpec::Unknown(UnknownClusteringConfig::new(1)).into(),
         }
     }
 
     pub fn approximate_stats(&self) -> ApproxStats {
         match self {
             Self::InMemoryScan(InMemoryScan { in_memory_info, .. }) => ApproxStats {
-                lower_bound_rows: in_memory_info.num_rows,
-                upper_bound_rows: Some(in_memory_info.num_rows),
-                lower_bound_bytes: in_memory_info.size_bytes,
-                upper_bound_bytes: Some(in_memory_info.size_bytes),
+                num_rows: in_memory_info.num_rows,
+                size_bytes: in_memory_info.size_bytes,
+                acc_selectivity: 1.0,
             },
             Self::TabularScan(TabularScan { scan_tasks, .. }) => {
-                let mut stats = ApproxStats::empty();
+                let mut approx_stats = ApproxStats::empty();
                 for st in scan_tasks.iter() {
-                    stats.lower_bound_rows += st.num_rows().unwrap_or(0);
-                    let in_memory_size = st.estimate_in_memory_size_bytes(None);
-                    stats.lower_bound_bytes += in_memory_size.unwrap_or(0);
-                    if let Some(st_ub) = st.upper_bound_rows() {
-                        if let Some(ub) = stats.upper_bound_rows {
-                            stats.upper_bound_rows = Some(ub + st_ub);
-                        } else {
-                            stats.upper_bound_rows = st.upper_bound_rows();
-                        }
-                    }
-                    if let Some(st_ub) = in_memory_size {
-                        if let Some(ub) = stats.upper_bound_bytes {
-                            stats.upper_bound_bytes = Some(ub + st_ub);
-                        } else {
-                            stats.upper_bound_bytes = in_memory_size;
-                        }
+                    if let Some(num_rows) = st.num_rows() {
+                        approx_stats.num_rows += num_rows;
+                    } else if let Some(approx_num_rows) = st.approx_num_rows(None) {
+                        approx_stats.num_rows += approx_num_rows as usize;
                     }
+                    approx_stats.size_bytes += st.estimate_in_memory_size_bytes(None).unwrap_or(0);
                 }
-                stats
+                approx_stats.acc_selectivity = if scan_tasks.is_empty() {
+                    0.0
+                } else {
+                    let st = scan_tasks.first().unwrap();
+                    st.pushdowns().estimated_selectivity(st.schema().as_ref())
+                };
+                approx_stats
             }
             Self::EmptyScan(..) => ApproxStats {
-                lower_bound_rows: 0,
-                upper_bound_rows: Some(0),
-                lower_bound_bytes: 0,
-                upper_bound_bytes: Some(0),
+                num_rows: 0,
+                size_bytes: 0,
+                acc_selectivity: 0.0,
+            },
+            Self::PreviousStageScan(..) => ApproxStats {
+                num_rows: 0,
+                size_bytes: 0,
+                acc_selectivity: 0.0,
             },
             // Assume no row/column pruning in cardinality-affecting operations.
             // TODO(Clark): Estimate row/column pruning to get a better size approximation.
-            Self::Filter(Filter { input, .. }) => {
+            Self::Filter(Filter {
+                input,
+                estimated_selectivity,
+                ..
+            }) => {
                 let input_stats = input.approximate_stats();
                 ApproxStats {
-                    lower_bound_rows: 0,
-                    upper_bound_rows: input_stats.upper_bound_rows,
-                    lower_bound_bytes: 0,
-                    upper_bound_bytes: input_stats.upper_bound_bytes,
+                    num_rows: (input_stats.num_rows as f64 * estimated_selectivity).ceil() as usize,
+                    size_bytes: (input_stats.size_bytes as f64 * estimated_selectivity).ceil()
+                        as usize,
+                    acc_selectivity: input_stats.acc_selectivity * estimated_selectivity,
                 }
             }
-            Self::Limit(Limit { input, limit, .. }) => {
-                let limit = *limit as usize;
+            Self::Limit(Limit { input, limit, .. }) | Self::TopN(TopN { input, limit, .. }) => {
                 let input_stats = input.approximate_stats();
-                let est_bytes_per_row_lower =
-                    input_stats.lower_bound_bytes / (input_stats.lower_bound_rows.max(1));
-                let est_bytes_per_row_upper = input_stats
-                    .upper_bound_bytes
-                    .and_then(|bytes| input_stats.upper_bound_rows.map(|rows| bytes / rows.max(1)));
-                let new_lower_rows = input_stats.lower_bound_rows.min(limit);
-                let new_upper_rows = input_stats
-                    .upper_bound_rows
-                    .map(|ub| ub.min(limit))
-                    .unwrap_or(limit);
+                let limit = *limit as usize;
+                let limit_selectivity = if input_stats.num_rows > limit {
+                    if input_stats.num_rows == 0 {
+                        0.0
+                    } else {
+                        limit as f64 / input_stats.num_rows as f64
+                    }
+                } else {
+                    1.0
+                };
                 ApproxStats {
-                    lower_bound_rows: new_lower_rows,
-                    upper_bound_rows: Some(new_upper_rows),
-                    lower_bound_bytes: new_lower_rows * est_bytes_per_row_lower,
-                    upper_bound_bytes: est_bytes_per_row_upper.map(|x| x * new_upper_rows),
+                    num_rows: limit.min(input_stats.num_rows),
+                    size_bytes: if input_stats.num_rows > limit {
+                        let est_bytes_per_row =
+                            input_stats.size_bytes / input_stats.num_rows.max(1);
+                        limit * est_bytes_per_row
+                    } else {
+                        input_stats.size_bytes
+                    },
+                    acc_selectivity: input_stats.acc_selectivity * limit_selectivity,
                 }
             }
             Self::Project(Project { input, .. })
@@ -265,11 +290,17 @@ impl PhysicalPlan {
                 .apply(|v| ((v as f64) * fraction) as usize),
             Self::Explode(Explode { input, .. }) => {
                 let input_stats = input.approximate_stats();
+                const EXPLODE_FACTOR: usize = 4;
+                let est_num_exploded_rows = input_stats.num_rows * EXPLODE_FACTOR;
+                let acc_selectivity = if input_stats.num_rows == 0 {
+                    0.0
+                } else {
+                    input_stats.acc_selectivity * EXPLODE_FACTOR as f64
+                };
                 ApproxStats {
-                    lower_bound_rows: input_stats.lower_bound_rows,
-                    upper_bound_rows: None,
-                    lower_bound_bytes: input_stats.lower_bound_bytes,
-                    upper_bound_bytes: None,
+                    num_rows: est_num_exploded_rows,
+                    size_bytes: input_stats.size_bytes,
+                    acc_selectivity,
                 }
             }
             // Propagate child approximation for operations that don't affect cardinality.
@@ -293,54 +324,54 @@ impl PhysicalPlan {
                 let left_stats = left.approximate_stats();
                 let right_stats = right.approximate_stats();
 
+                // We assume that if one side of a join had its cardinality reduced by some operations
+                // (e.g. filters, limits, aggregations), then assuming a pk-fk join, the total number of
+                // rows output from the join will be reduced proportionally. Hence, apply the right side's
+                // selectivity to the number of rows/size in bytes on the left and vice versa.
+                let left_num_rows = left_stats.num_rows as f64 * right_stats.acc_selectivity;
+                let right_num_rows = right_stats.num_rows as f64 * left_stats.acc_selectivity;
+                let left_size = left_stats.size_bytes as f64 * right_stats.acc_selectivity;
+                let right_size = right_stats.size_bytes as f64 * left_stats.acc_selectivity;
                 ApproxStats {
-                    lower_bound_rows: 0,
-                    upper_bound_rows: left_stats
-                        .upper_bound_rows
-                        .and_then(|l| right_stats.upper_bound_rows.map(|r| l.max(r))),
-                    lower_bound_bytes: 0,
-                    upper_bound_bytes: left_stats
-                        .upper_bound_bytes
-                        .and_then(|l| right_stats.upper_bound_bytes.map(|r| l.max(r))),
+                    num_rows: left_num_rows.max(right_num_rows).ceil() as usize,
+                    size_bytes: left_size.max(right_size).ceil() as usize,
+                    acc_selectivity: left_stats.acc_selectivity * right_stats.acc_selectivity,
                 }
             }
             // TODO(Clark): Approximate post-aggregation sizes via grouping estimates + aggregation type.
             Self::Aggregate(Aggregate { input, groupby, .. }) => {
                 let input_stats = input.approximate_stats();
                 // TODO we should use schema inference here
-                let est_bytes_per_row_lower =
-                    input_stats.lower_bound_bytes / (input_stats.lower_bound_rows.max(1));
-                let est_bytes_per_row_upper = input_stats
-                    .upper_bound_bytes
-                    .and_then(|bytes| input_stats.upper_bound_rows.map(|rows| bytes / rows.max(1)));
+                let est_bytes_per_row = input_stats.size_bytes / (input_stats.num_rows.max(1));
+                let acc_selectivity = if input_stats.num_rows == 0 {
+                    0.0
+                } else {
+                    input_stats.acc_selectivity / input_stats.num_rows as f64
+                };
                 if groupby.is_empty() {
                     ApproxStats {
-                        lower_bound_rows: input_stats.lower_bound_rows.min(1),
-                        upper_bound_rows: Some(1),
-                        lower_bound_bytes: input_stats.lower_bound_bytes.min(1)
-                            * est_bytes_per_row_lower,
-                        upper_bound_bytes: est_bytes_per_row_upper,
+                        num_rows: 1,
+                        size_bytes: est_bytes_per_row,
+                        acc_selectivity,
                     }
                 } else {
-                    // we should use the new schema here
+                    // Assume high cardinality for group by columns, and 80% of rows are unique.
+                    let est_num_groups = input_stats.num_rows * 4 / 5;
                     ApproxStats {
-                        lower_bound_rows: input_stats.lower_bound_rows.min(1),
-                        upper_bound_rows: input_stats.upper_bound_rows,
-                        lower_bound_bytes: input_stats.lower_bound_bytes.min(1)
-                            * est_bytes_per_row_lower,
-                        upper_bound_bytes: input_stats.upper_bound_bytes,
+                        num_rows: est_num_groups,
+                        size_bytes: est_bytes_per_row * est_num_groups,
+                        acc_selectivity: input_stats.acc_selectivity * est_num_groups as f64
+                            / input_stats.num_rows as f64,
                     }
                 }
             }
             Self::Unpivot(Unpivot { input, values, .. }) => {
                 let input_stats = input.approximate_stats();
                 let num_values = values.len();
-                // the number of bytes should be the name but nows should be multiplied by num_values
                 ApproxStats {
-                    lower_bound_rows: input_stats.lower_bound_rows * num_values,
-                    upper_bound_rows: input_stats.upper_bound_rows.map(|v| v * num_values),
-                    lower_bound_bytes: input_stats.lower_bound_bytes,
-                    upper_bound_bytes: input_stats.upper_bound_bytes,
+                    num_rows: input_stats.num_rows * num_values,
+                    size_bytes: input_stats.size_bytes,
+                    acc_selectivity: input_stats.acc_selectivity * num_values as f64,
                 }
             }
             // Post-write DataFrame will contain paths to files that were written.
@@ -349,16 +380,17 @@ impl PhysicalPlan {
                 ApproxStats::empty()
             }
             #[cfg(feature = "python")]
-            Self::IcebergWrite(_) | Self::DeltaLakeWrite(_) | Self::LanceWrite(_) => {
-                ApproxStats::empty()
-            }
+            Self::IcebergWrite(_)
+            | Self::DeltaLakeWrite(_)
+            | Self::LanceWrite(_)
+            | Self::DataSink(_) => ApproxStats::empty(),
         }
     }
 
     pub fn children(&self) -> Vec<&Self> {
         match self {
             Self::InMemoryScan(..) => vec![],
-            Self::TabularScan(..) | Self::EmptyScan(..) => vec![],
+            Self::TabularScan(..) | Self::EmptyScan(..) | Self::PreviousStageScan(..) => vec![],
             Self::Project(Project { input, .. }) => vec![input],
             Self::ActorPoolProject(ActorPoolProject { input, .. }) => vec![input],
             Self::Filter(Filter { input, .. }) => vec![input],
@@ -367,6 +399,7 @@ impl PhysicalPlan {
             Self::Unpivot(Unpivot { input, .. }) => vec![input],
             Self::Sample(Sample { input, .. }) => vec![input],
             Self::Sort(Sort { input, .. }) => vec![input],
+            Self::TopN(TopN { input, .. }) => vec![input],
             Self::Aggregate(Aggregate { input, .. }) => vec![input],
             Self::Pivot(Pivot { input, .. }) => vec![input],
             Self::TabularWriteParquet(TabularWriteParquet { input, .. }) => vec![input],
@@ -379,6 +412,8 @@ impl PhysicalPlan {
             Self::DeltaLakeWrite(DeltaLakeWrite { input, .. }) => vec![input],
             #[cfg(feature = "python")]
             Self::LanceWrite(LanceWrite { input, .. }) => vec![input],
+            #[cfg(feature = "python")]
+            Self::DataSink(DataSink { input, .. }) => vec![input],
             Self::HashJoin(HashJoin { left, right, .. }) => vec![left, right],
             Self::BroadcastJoin(BroadcastJoin {
                 broadcaster,
@@ -401,15 +436,17 @@ impl PhysicalPlan {
             [input] => match self {
                 Self::InMemoryScan(..) => panic!("Source nodes don't have children, with_new_children() should never be called for source ops"),
                 Self::TabularScan(..)
-                | Self::EmptyScan(..) => panic!("Source nodes don't have children, with_new_children() should never be called for source ops"),
+                | Self::EmptyScan(..)
+                | Self::PreviousStageScan(..) => panic!("Source nodes don't have children, with_new_children() should never be called for source ops"),
                 Self::Project(Project { projection, clustering_spec, .. }) =>
                     Self::Project(Project::new_with_clustering_spec(
                     input.clone(), projection.clone(), clustering_spec.clone(),
                 ).unwrap()),
 
                 Self::ActorPoolProject(ActorPoolProject {projection, ..}) => Self::ActorPoolProject(ActorPoolProject::try_new(input.clone(), projection.clone()).unwrap()),
-                Self::Filter(Filter { predicate, .. }) => Self::Filter(Filter::new(input.clone(), predicate.clone())),
+                Self::Filter(Filter { predicate, estimated_selectivity,.. }) => Self::Filter(Filter::new(input.clone(), predicate.clone(), *estimated_selectivity)),
                 Self::Limit(Limit { limit, eager, num_partitions, .. }) => Self::Limit(Limit::new(input.clone(), *limit, *eager, *num_partitions)),
+                Self::TopN(TopN { sort_by, descending, nulls_first, limit, num_partitions, .. }) => Self::TopN(TopN::new(input.clone(), sort_by.clone(), descending.clone(), nulls_first.clone(), *limit, *num_partitions)),
                 Self::Explode(Explode { to_explode, .. }) => Self::Explode(Explode::try_new(input.clone(), to_explode.clone()).unwrap()),
                 Self::Unpivot(Unpivot { ids, values, variable_name, value_name, .. }) => Self::Unpivot(Unpivot::new(input.clone(), ids.clone(), values.clone(), variable_name, value_name)),
                 Self::Pivot(Pivot { group_by, pivot_column, value_column, names, .. }) => Self::Pivot(Pivot::new(input.clone(), group_by.clone(), pivot_column.clone(), value_column.clone(), names.clone())),
@@ -427,6 +464,8 @@ impl PhysicalPlan {
                 Self::DeltaLakeWrite(DeltaLakeWrite {schema, delta_lake_info, .. }) => Self::DeltaLakeWrite(DeltaLakeWrite::new(schema.clone(), delta_lake_info.clone(), input.clone())),
                 #[cfg(feature = "python")]
                 Self::LanceWrite(LanceWrite { schema, lance_info, .. }) => Self::LanceWrite(LanceWrite::new(schema.clone(), lance_info.clone(), input.clone())),
+                #[cfg(feature = "python")]
+                Self::DataSink(DataSink { schema, data_sink_info, .. }) => Self::DataSink(DataSink::new(schema.clone(), data_sink_info.clone(), input.clone())),
                 Self::Concat(_) | Self::HashJoin(_) | Self::SortMergeJoin(_) | Self::BroadcastJoin(_) | Self::CrossJoin(_) => panic!("{} requires more than 1 input, but received: {}", self, children.len()),
             },
             [input1, input2] => match self {
@@ -457,10 +496,12 @@ impl PhysicalPlan {
             Self::InMemoryScan(..) => "InMemoryScan",
             Self::TabularScan(..) => "TabularScan",
             Self::EmptyScan(..) => "EmptyScan",
+            Self::PreviousStageScan(..) => "PreviousStageScan",
             Self::Project(..) => "Project",
             Self::ActorPoolProject(..) => "ActorPoolProject",
             Self::Filter(..) => "Filter",
             Self::Limit(..) => "Limit",
+            Self::TopN(..) => "TopN",
             Self::Explode(..) => "Explode",
             Self::Unpivot(..) => "Unpivot",
             Self::Sample(..) => "Sample",
@@ -483,6 +524,8 @@ impl PhysicalPlan {
             Self::DeltaLakeWrite(..) => "DeltaLakeWrite",
             #[cfg(feature = "python")]
             Self::LanceWrite(..) => "LanceWrite",
+            #[cfg(feature = "python")]
+            Self::DataSink(..) => "DataSinkWrite",
         };
         name.to_string()
     }
@@ -492,10 +535,12 @@ impl PhysicalPlan {
             Self::InMemoryScan(in_memory_scan) => in_memory_scan.multiline_display(),
             Self::TabularScan(tabular_scan) => tabular_scan.multiline_display(),
             Self::EmptyScan(empty_scan) => empty_scan.multiline_display(),
+            Self::PreviousStageScan(previous_stage_scan) => previous_stage_scan.multiline_display(),
             Self::Project(project) => project.multiline_display(),
             Self::ActorPoolProject(ap_project) => ap_project.multiline_display(),
             Self::Filter(filter) => filter.multiline_display(),
             Self::Limit(limit) => limit.multiline_display(),
+            Self::TopN(top_n) => top_n.multiline_display(),
             Self::Explode(explode) => explode.multiline_display(),
             Self::Unpivot(unpivot) => unpivot.multiline_display(),
             Self::Sample(sample) => sample.multiline_display(),
@@ -522,6 +567,8 @@ impl PhysicalPlan {
             Self::DeltaLakeWrite(delta_lake_info) => delta_lake_info.multiline_display(),
             #[cfg(feature = "python")]
             Self::LanceWrite(lance_info) => lance_info.multiline_display(),
+            #[cfg(feature = "python")]
+            Self::DataSink(data_sink_info) => data_sink_info.multiline_display(),
         }
     }
 
