@@ -6,15 +6,16 @@ use common_runtime::RuntimeRef;
 use common_scan_info::{PartitionField, Pushdowns, ScanOperator, ScanTaskLike, ScanTaskLikeRef};
 use daft_core::{prelude::Utf8Array, series::IntoSeries};
 use daft_csv::CsvParseOptions;
+use daft_dsl::expr::bound_expr::BoundExpr;
 use daft_io::{parse_url, FileMetadata, IOClient, IOStatsContext, IOStatsRef};
 use daft_parquet::read::ParquetSchemaInferenceOptions;
+use daft_recordbatch::RecordBatch;
 use daft_schema::{
     dtype::DataType,
     field::Field,
     schema::{Schema, SchemaRef},
 };
-use daft_stats::PartitionSpec;
-use daft_table::Table;
+use daft_stats::{PartitionSpec, TableMetadata};
 use futures::{stream::BoxStream, Stream, StreamExt, TryStreamExt};
 use snafu::Snafu;
 
@@ -33,6 +34,9 @@ pub struct GlobScanOperator {
     hive_partitioning: bool,
     partitioning_keys: Vec<PartitionField>,
     generated_fields: SchemaRef,
+    // When creating the glob scan operator, we might collect file metadata for the first file during schema inference.
+    // Cache this metadata (along with the first filepath) so we can use it to populate the stats for the first scan task.
+    first_metadata: Option<(String, TableMetadata)>,
 }
 
 /// Wrapper struct that implements a sync Iterator for a BoxStream
@@ -41,7 +45,7 @@ struct BoxStreamIterator<'a, T> {
     runtime_handle: RuntimeRef,
 }
 
-impl<'a, T> Iterator for BoxStreamIterator<'a, T> {
+impl<T> Iterator for BoxStreamIterator<'_, T> {
     type Item = T;
 
     fn next(&mut self) -> Option<Self::Item> {
@@ -184,22 +188,22 @@ impl GlobScanOperator {
         let (partitioning_keys, generated_fields) = if partition_fields.is_empty() {
             (vec![], Schema::empty())
         } else {
-            let generated_fields = Schema::new(partition_fields)?;
+            let generated_fields = Schema::new(partition_fields);
             let generated_fields = match user_provided_schema.clone() {
                 Some(hint) => generated_fields.apply_hints(&hint)?,
                 None => generated_fields,
             };
             // Extract partitioning keys only after the user's schema hints have been applied.
-            let partitioning_keys = (&generated_fields.fields)
+            let partitioning_keys = generated_fields
                 .into_iter()
-                .map(|(_, field)| PartitionField::new(field.clone(), None, None))
+                .map(|field| PartitionField::new(field.clone(), None, None))
                 .collect::<Result<Vec<_>, _>>()?;
             (partitioning_keys, generated_fields)
         };
 
-        let schema = match infer_schema {
+        let (schema, first_metadata) = match infer_schema {
             true => {
-                let inferred_schema = match file_format_config.as_ref() {
+                let (inferred_schema, first_metadata) = match file_format_config.as_ref() {
                     &FileFormatConfig::Parquet(ParquetSourceConfig {
                         coerce_int96_timestamp_unit,
                         ref field_id_mapping,
@@ -209,19 +213,25 @@ impl GlobScanOperator {
                             "GlobScanOperator constructor read_parquet_schema: for uri {first_filepath}"
                         ));
 
-                        let (schema, _metadata) = daft_parquet::read::read_parquet_schema(
-                            first_filepath.as_str(),
-                            io_client,
-                            Some(io_stats),
-                            ParquetSchemaInferenceOptions {
-                                coerce_int96_timestamp_unit,
-                                ..Default::default()
+                        let (schema, metadata) =
+                            daft_parquet::read::read_parquet_schema_and_metadata(
+                                first_filepath.as_str(),
+                                io_client,
+                                Some(io_stats),
+                                ParquetSchemaInferenceOptions {
+                                    coerce_int96_timestamp_unit,
+                                    ..Default::default()
+                                },
+                                field_id_mapping.clone(),
+                            )
+                            .await?;
+                        let metadata = Some((
+                            first_filepath,
+                            TableMetadata {
+                                length: metadata.num_rows,
                             },
-                            field_id_mapping.clone(),
-                        )
-                        .await?;
-
-                        schema
+                        ));
+                        (schema, metadata)
                     }
                     FileFormatConfig::Csv(CsvSourceConfig {
                         delimiter,
@@ -249,17 +259,23 @@ impl GlobScanOperator {
                             Some(io_stats),
                         )
                         .await?;
-                        schema
+                        (schema, None)
                     }
                     FileFormatConfig::Json(_) => {
-                        daft_json::schema::read_json_schema(
+                        let schema = daft_json::schema::read_json_schema(
                             first_filepath.as_str(),
                             None,
                             None,
                             io_client,
                             Some(io_stats),
                         )
-                        .await?
+                        .await?;
+                        (schema, None)
+                    }
+                    FileFormatConfig::Warc(_) => {
+                        return Err(DaftError::ValueError(
+                            "Warc schemas do not need to be inferred".to_string(),
+                        ))
                     }
                     #[cfg(feature = "python")]
                     FileFormatConfig::Database(_) => {
@@ -275,13 +291,17 @@ impl GlobScanOperator {
                     }
                 };
                 match user_provided_schema {
-                    Some(hint) => Arc::new(inferred_schema.apply_hints(&hint)?),
-                    None => Arc::new(inferred_schema),
+                    Some(hint) => (
+                        Arc::new(inferred_schema.apply_hints(&hint)?),
+                        first_metadata,
+                    ),
+                    None => (Arc::new(inferred_schema), first_metadata),
                 }
             }
-            false => {
-                user_provided_schema.expect("Schema must be provided if infer_schema is false")
-            }
+            false => (
+                user_provided_schema.expect("Schema must be provided if infer_schema is false"),
+                None,
+            ),
         };
         Ok(Self {
             glob_paths,
@@ -292,6 +312,7 @@ impl GlobScanOperator {
             hive_partitioning,
             partitioning_keys,
             generated_fields: Arc::new(generated_fields),
+            first_metadata,
         })
     }
 }
@@ -391,9 +412,14 @@ impl ScanOperator for GlobScanOperator {
         let partition_fields = self
             .partitioning_keys
             .iter()
-            .map(|partition_spec| partition_spec.clone_field())
-            .collect();
-        let partition_schema = Schema::new(partition_fields)?;
+            .map(|partition_spec| partition_spec.clone_field());
+        let partition_schema = Schema::new(partition_fields);
+        let (first_filepath, first_metadata) =
+            if let Some((first_filepath, first_metadata)) = &self.first_metadata {
+                (Some(first_filepath), Some(first_metadata))
+            } else {
+                (None, None)
+            };
         // Create one ScanTask per file.
         files
             .enumerate()
@@ -421,11 +447,15 @@ impl ScanOperator for GlobScanOperator {
                     }
                     let (partition_spec, generated_fields) = if !partition_values.is_empty() {
                         let partition_values_table =
-                            Table::from_nonempty_columns(partition_values)?;
+                            RecordBatch::from_nonempty_columns(partition_values)?;
                         // If there are partition values, evaluate them against partition filters, if any.
                         if let Some(partition_filters) = &pushdowns.partition_filters {
+                            let partition_filters = BoundExpr::try_new(
+                                partition_filters.clone(),
+                                &partition_values_table.schema,
+                            )?;
                             let filter_result =
-                                partition_values_table.filter(&[partition_filters.clone()])?;
+                                partition_values_table.filter(&[partition_filters])?;
                             if filter_result.is_empty() {
                                 // Skip the current file since it does not satisfy the partition filters.
                                 return Ok(None);
@@ -446,11 +476,17 @@ impl ScanOperator for GlobScanOperator {
                     let chunk_spec = row_group.map(ChunkSpec::Parquet);
                     Ok(Some(ScanTask::new(
                         vec![DataSource::File {
+                            metadata: if let Some(first_filepath) = first_filepath
+                                && path == *first_filepath
+                            {
+                                first_metadata.cloned()
+                            } else {
+                                None
+                            },
                             path,
                             chunk_spec,
                             size_bytes,
                             iceberg_delete_files: None,
-                            metadata: None,
                             partition_spec,
                             statistics: None,
                             parquet_metadata: None,
