@@ -3,9 +3,10 @@
 use std::{
     any::Any,
     borrow::Cow,
+    collections::HashMap,
     fmt::Debug,
     hash::{Hash, Hasher},
-    sync::Arc,
+    sync::{Arc, OnceLock},
 };
 
 use common_display::DisplayAs;
@@ -448,7 +449,7 @@ impl ScanTaskLike for ScanTask {
         other
             .as_any()
             .downcast_ref::<Self>()
-            .map_or(false, |a| a == self)
+            .is_some_and(|a| a == self)
     }
 
     fn dyn_hash(&self, mut state: &mut dyn Hasher) {
@@ -500,6 +501,23 @@ impl From<ScanTask> for ScanTaskLikeRef {
 
 pub type ScanTaskRef = Arc<ScanTask>;
 
+static WARC_COLUMN_SIZES: OnceLock<HashMap<&'static str, usize>> = OnceLock::new();
+
+fn warc_column_sizes() -> &'static HashMap<&'static str, usize> {
+    WARC_COLUMN_SIZES.get_or_init(|| {
+        let mut m = HashMap::new();
+        // Average sizes based on analysis of Common Crawl WARC files.
+        m.insert("WARC-Record-ID", 36); // UUID-style identifiers.
+        m.insert("WARC-Type", 8); // e.g. "response".
+        m.insert("WARC-Date", 8); // Timestamp stored as i64 nanoseconds.
+        m.insert("Content-Length", 8); // i64.
+        m.insert("WARC-Identified-Payload-Type", 5); // e.g. "text/html". Typically null.
+        m.insert("warc_content", 27282); // Average content size.
+        m.insert("warc_headers", 350); // Average headers size.
+        m
+    })
+}
+
 impl ScanTask {
     #[must_use]
     pub fn new(
@@ -609,24 +627,22 @@ impl ScanTask {
         match (&self.generated_fields, &self.pushdowns.columns) {
             (None, None) => self.schema.clone(),
             _ => {
-                let mut fields = self.schema.fields.clone();
-                // Extend the schema with generated fields.
-                if let Some(generated_fields) = &self.generated_fields {
-                    fields.extend(
-                        generated_fields
-                            .fields
-                            .iter()
-                            .map(|(name, field)| (name.clone(), field.clone())),
-                    );
-                }
+                let schema_with_generated_fields =
+                    if let Some(generated_fields) = &self.generated_fields {
+                        // Extend the schema with generated fields.
+                        Arc::new(self.schema.non_distinct_union(generated_fields).unwrap())
+                    } else {
+                        self.schema.clone()
+                    };
+
+                let mut fields = schema_with_generated_fields.fields().to_vec();
+
                 // Filter the schema based on the pushdown column filters.
                 if let Some(columns) = &self.pushdowns.columns {
-                    fields = fields
-                        .into_iter()
-                        .filter(|(name, _)| columns.contains(name))
-                        .collect();
+                    fields.retain(|field| columns.contains(&field.name));
                 }
-                Arc::new(Schema { fields })
+
+                Arc::new(Schema::new(fields))
             }
         }
     }
@@ -671,13 +687,26 @@ impl ScanTask {
                         FileFormatConfig::Csv(_) | FileFormatConfig::Json(_) => {
                             config.csv_inflation_factor
                         }
+                        FileFormatConfig::Warc(_) => {
+                            if self.is_gzipped() {
+                                5.0
+                            } else {
+                                1.0
+                            }
+                        }
                         #[cfg(feature = "python")]
                         FileFormatConfig::Database(_) => 1.0,
                         #[cfg(feature = "python")]
                         FileFormatConfig::PythonFunction => 1.0,
                     };
                     let in_mem_size: f64 = (file_size as f64) * inflation_factor;
-                    let read_row_size = self.schema.estimate_row_size_bytes();
+                    let read_row_size = if self.is_warc() {
+                        // Across 100 Common Crawl WARC files, the average record size is 470 (metadata) + 27282 (content) bytes.
+                        // This is 27752 bytes per record.
+                        27752.0
+                    } else {
+                        self.schema.estimate_row_size_bytes()
+                    };
                     in_mem_size / read_row_size
                 })
             });
@@ -686,7 +715,10 @@ impl ScanTask {
             if self.pushdowns.filters.is_some() {
                 // HACK: This might not be a good idea? We could also just return None here
                 // Assume that filters filter out about 80% of the data
-                approx_total_num_rows_before_pushdowns / 5.
+                let estimated_selectivity =
+                    self.pushdowns.estimated_selectivity(self.schema.as_ref());
+                // Set the lower bound approximated number of rows to 1 to avoid underestimation.
+                (approx_total_num_rows_before_pushdowns * estimated_selectivity).max(1.0)
             } else if let Some(limit) = self.pushdowns.limit {
                 (limit as f64).min(approx_total_num_rows_before_pushdowns)
             } else {
@@ -712,30 +744,67 @@ impl ScanTask {
         self.size_bytes_on_disk.map(|s| s as usize)
     }
 
+    fn is_warc(&self) -> bool {
+        matches!(self.file_format_config.as_ref(), FileFormatConfig::Warc(_))
+    }
+
+    fn is_gzipped(&self) -> bool {
+        self.sources
+            .first()
+            .and_then(|s| match s {
+                DataSource::File { path, .. } => {
+                    let filename = std::path::Path::new(path);
+                    Some(
+                        filename
+                            .extension()
+                            .is_some_and(|ext| ext.eq_ignore_ascii_case("gz")),
+                    )
+                }
+                _ => None,
+            })
+            .unwrap_or(false)
+    }
+
     #[must_use]
     pub fn estimate_in_memory_size_bytes(
         &self,
         config: Option<&DaftExecutionConfig>,
     ) -> Option<usize> {
-        let mat_schema = self.materialized_schema();
-        self.statistics
-            .as_ref()
-            .and_then(|s| {
-                // Derive in-memory size estimate from table stats.
-                self.num_rows().and_then(|num_rows| {
-                    let row_size = s.estimate_row_size(Some(mat_schema.as_ref())).ok()?;
-                    let estimate = (num_rows as f64) * row_size;
-                    Some(estimate as usize)
+        // WARC files that are gzipped are often 5x smaller than the uncompressed size.
+        // For example, see this blog post by Common Crawl: https://commoncrawl.org/blog/february-2025-crawl-archive-now-available
+        if self.is_warc() {
+            let approx_num_rows = self.approx_num_rows(config)?;
+            let mat_schema = self.materialized_schema();
+
+            // Calculate size based on materialized schema and WARC column sizes
+            let row_size: usize = mat_schema
+                .field_names()
+                .map(|name| warc_column_sizes().get(name).copied().unwrap_or(8))
+                .sum();
+
+            let estimate = (approx_num_rows * row_size as f64) as usize;
+            Some(estimate)
+        } else {
+            let mat_schema = self.materialized_schema();
+            self.statistics
+                .as_ref()
+                .and_then(|s| {
+                    // Derive in-memory size estimate from table stats.
+                    self.num_rows().and_then(|num_rows| {
+                        let row_size = s.estimate_row_size(Some(mat_schema.as_ref())).ok()?;
+                        let estimate = (num_rows as f64) * row_size;
+                        Some(estimate as usize)
+                    })
                 })
-            })
-            .or_else(|| {
-                // use approximate number of rows multiplied by an approximate bytes-per-row
-                self.approx_num_rows(config).map(|approx_num_rows| {
-                    let row_size = mat_schema.estimate_row_size_bytes();
-                    let estimate = approx_num_rows * row_size;
-                    estimate as usize
+                .or_else(|| {
+                    // use approximate number of rows multiplied by an approximate bytes-per-row
+                    self.approx_num_rows(config).map(|approx_num_rows| {
+                        let row_size = mat_schema.estimate_row_size_bytes();
+                        let estimate = approx_num_rows * row_size;
+                        estimate as usize
+                    })
                 })
-            })
+        }
     }
 
     #[must_use]
@@ -873,7 +942,7 @@ mod test {
         )
     }
 
-    async fn make_glob_scan_operator(num_sources: usize) -> GlobScanOperator {
+    async fn make_glob_scan_operator(num_sources: usize, infer_schema: bool) -> GlobScanOperator {
         let file_format_config: FileFormatConfig = FileFormatConfig::Parquet(ParquetSourceConfig {
             coerce_int96_timestamp_unit: TimeUnit::Seconds,
             field_id_mapping: None,
@@ -891,7 +960,7 @@ mod test {
             sources,
             Arc::new(file_format_config),
             Arc::new(StorageConfig::new_internal(false, None)),
-            false,
+            infer_schema,
             Some(Arc::new(Schema::empty())),
             None,
             false,
@@ -904,12 +973,43 @@ mod test {
 
     #[tokio::test]
     async fn test_glob_display_condenses() -> DaftResult<()> {
-        let glob_scan_operator: GlobScanOperator = make_glob_scan_operator(8).await;
+        let glob_scan_operator: GlobScanOperator = make_glob_scan_operator(8, false).await;
         let condensed_glob_paths: Vec<String> = glob_scan_operator.multiline_display();
         assert_eq!(condensed_glob_paths[1], "Glob paths = [../../tests/assets/parquet-data/mvp.parquet, ../../tests/assets/parquet-data/mvp.parquet, ../../tests/assets/parquet-data/mvp.parquet, ..., ../../tests/assets/parquet-data/mvp.parquet, ../../tests/assets/parquet-data/mvp.parquet, ../../tests/assets/parquet-data/mvp.parquet]");
         Ok(())
     }
 
+    #[tokio::test]
+    async fn test_glob_single_file_stats() -> DaftResult<()> {
+        let glob_scan_operator = make_glob_scan_operator(1, true).await;
+        let scan_tasks = tokio::task::spawn_blocking(move || {
+            glob_scan_operator.to_scan_tasks(Pushdowns::default())
+        })
+        .await
+        .unwrap()?;
+        assert_eq!(scan_tasks.len(), 1, "Expected 1 scan task");
+        assert_eq!(
+            scan_tasks[0].num_rows(),
+            Some(100),
+            "Expected 100 rows in the first scan task"
+        );
+        Ok(())
+    }
+
+    #[tokio::test]
+    async fn test_glob_multiple_files_stats() -> DaftResult<()> {
+        let glob_scan_operator = make_glob_scan_operator(8, true).await;
+        let scan_tasks = tokio::task::spawn_blocking(move || {
+            glob_scan_operator.to_scan_tasks(Pushdowns::default())
+        })
+        .await
+        .unwrap()?;
+        assert!(scan_tasks.len() > 1, "Expected more than 1 scan tasks");
+        assert_eq!(scan_tasks[0].num_rows(), Some(100), "We should be able to populate stats from inference when the scan task's file matches the file used during inference");
+        assert_eq!(scan_tasks[1].num_rows(), Some(100), "We should be able to populate stats from inference when the scan task's file matches the file used during inference");
+        Ok(())
+    }
+
     #[test]
     fn test_display_condenses() -> DaftResult<()> {
         let scan_task = make_scan_task(7);
