@@ -1,4 +1,7 @@
-use std::hash::{Hash, Hasher};
+use std::{
+    hash::{Hash, Hasher},
+    sync::Arc,
+};
 
 use common_py_serde::{deserialize_py_object, serialize_py_object};
 use pyo3::{prelude::*, types::PyTuple};
@@ -12,7 +15,7 @@ struct PyObjectSerializableWrapper(
         serialize_with = "serialize_py_object",
         deserialize_with = "deserialize_py_object"
     )]
-    pub PyObject,
+    pub Arc<PyObject>,
 );
 
 /// Python arguments to a Python function that produces Tables
@@ -29,7 +32,7 @@ impl Hash for PythonTablesFactoryArgs {
 }
 
 impl PythonTablesFactoryArgs {
-    pub fn new(args: Vec<PyObject>) -> Self {
+    pub fn new(args: Vec<Arc<PyObject>>) -> Self {
         let mut hasher = std::collections::hash_map::DefaultHasher::new();
         Python::with_gil(|py| {
             for obj in &args {
@@ -45,9 +48,8 @@ impl PythonTablesFactoryArgs {
         }
     }
 
-    #[must_use]
-    pub fn to_pytuple<'a>(&self, py: Python<'a>) -> Bound<'a, PyTuple> {
-        pyo3::types::PyTuple::new_bound(py, self.args.iter().map(|x| x.0.bind(py)))
+    pub fn to_pytuple<'a>(&self, py: Python<'a>) -> PyResult<Bound<'a, PyTuple>> {
+        pyo3::types::PyTuple::new(py, self.args.iter().map(|x| x.0.bind(py)))
     }
 }
 
@@ -74,15 +76,12 @@ pub mod pylib {
         python::pylib::{PyPartitionField, PyPushdowns},
         PartitionField, Pushdowns, ScanOperator, ScanOperatorRef, ScanTaskLike, ScanTaskLikeRef,
     };
+    use daft_dsl::expr::bound_expr::BoundExpr;
     use daft_logical_plan::{LogicalPlanBuilder, PyLogicalPlanBuilder};
+    use daft_recordbatch::{python::PyRecordBatch, RecordBatch};
     use daft_schema::{python::schema::PySchema, schema::SchemaRef};
     use daft_stats::{PartitionSpec, TableMetadata, TableStatistics};
-    use daft_table::{python::PyTable, Table};
-    use pyo3::{
-        prelude::*,
-        pyclass,
-        types::{PyIterator, PyList},
-    };
+    use pyo3::{prelude::*, pyclass, types::PyIterator};
     use serde::{Deserialize, Serialize};
 
     use super::PythonTablesFactoryArgs;
@@ -126,6 +125,15 @@ pub mod pylib {
 
         #[staticmethod]
         #[allow(clippy::too_many_arguments)]
+        #[pyo3(signature = (
+            glob_path,
+            file_format_config,
+            storage_config,
+            hive_partitioning,
+            infer_schema,
+            schema=None,
+            file_path_column=None
+        ))]
         pub fn glob_scan(
             py: Python,
             glob_path: Vec<String>,
@@ -186,10 +194,10 @@ pub mod pylib {
         }
         fn _partitioning_keys(abc: &PyObject, py: Python) -> PyResult<Vec<PartitionField>> {
             let result = abc.call_method0(py, pyo3::intern!(py, "partitioning_keys"))?;
-            let result = result.extract::<&PyList>(py)?;
             result
-                .into_iter()
-                .map(|p| Ok(p.extract::<PyPartitionField>()?.0.as_ref().clone()))
+                .bind(py)
+                .try_iter()?
+                .map(|p| Ok(p?.extract::<PyPartitionField>()?.0.as_ref().clone()))
                 .collect()
         }
 
@@ -280,11 +288,11 @@ pub mod pylib {
 
         fn to_scan_tasks(&self, pushdowns: Pushdowns) -> DaftResult<Vec<ScanTaskLikeRef>> {
             let scan_tasks = Python::with_gil(|py| {
-                let pypd = PyPushdowns(pushdowns.clone().into()).into_py(py);
+                let pypd = PyPushdowns(pushdowns.clone().into()).into_pyobject(py)?;
                 let pyiter =
                     self.operator
                         .call_method1(py, pyo3::intern!(py, "to_scan_tasks"), (pypd,))?;
-                let pyiter = PyIterator::from_bound_object(pyiter.bind(py))?;
+                let pyiter = PyIterator::from_object(pyiter.bind(py))?;
                 DaftResult::Ok(
                     pyiter
                         .map(|v| {
@@ -340,6 +348,18 @@ pub mod pylib {
     impl PyScanTask {
         #[allow(clippy::too_many_arguments)]
         #[staticmethod]
+        #[pyo3(signature = (
+            file,
+            file_format,
+            schema,
+            storage_config,
+            num_rows=None,
+            size_bytes=None,
+            iceberg_delete_files=None,
+            pushdowns=None,
+            partition_values=None,
+            stats=None
+        ))]
         pub fn catalog_scan_task(
             file: String,
             file_format: PyFileFormatConfig,
@@ -349,17 +369,19 @@ pub mod pylib {
             size_bytes: Option<u64>,
             iceberg_delete_files: Option<Vec<String>>,
             pushdowns: Option<PyPushdowns>,
-            partition_values: Option<PyTable>,
-            stats: Option<PyTable>,
+            partition_values: Option<PyRecordBatch>,
+            stats: Option<PyRecordBatch>,
         ) -> PyResult<Option<Self>> {
             if let Some(ref pvalues) = partition_values
                 && let Some(Some(ref partition_filters)) =
                     pushdowns.as_ref().map(|p| &p.0.partition_filters)
             {
-                let table = &pvalues.table;
-                let eval_pred = table.eval_expression_list(&[partition_filters.clone()])?;
+                let table = &pvalues.record_batch;
+                let partition_filters =
+                    BoundExpr::try_new(partition_filters.clone(), &table.schema)?;
+                let eval_pred = table.eval_expression_list(&[partition_filters])?;
                 assert_eq!(eval_pred.num_columns(), 1);
-                let series = eval_pred.get_column_by_index(0)?;
+                let series = eval_pred.get_column(0);
                 assert_eq!(series.data_type(), &daft_core::datatypes::DataType::Boolean);
                 let boolean = series.bool()?;
                 assert_eq!(boolean.len(), 1);
@@ -372,10 +394,11 @@ pub mod pylib {
             // TODO(Clark): Filter out scan tasks with pushed down filters + table stats?
 
             let pspec = PartitionSpec {
-                keys: partition_values.map_or_else(|| Table::empty(None).unwrap(), |p| p.table),
+                keys: partition_values
+                    .map_or_else(|| RecordBatch::empty(None).unwrap(), |p| p.record_batch),
             };
             let statistics = stats
-                .map(|s| TableStatistics::from_stats_table(&s.table))
+                .map(|s| TableStatistics::from_stats_table(&s.record_batch))
                 .transpose()?;
 
             let metadata = num_rows.map(|n| TableMetadata { length: n as usize });
@@ -404,6 +427,16 @@ pub mod pylib {
 
         #[allow(clippy::too_many_arguments)]
         #[staticmethod]
+        #[pyo3(signature = (
+            url,
+            file_format,
+            schema,
+            storage_config,
+            num_rows=None,
+            size_bytes=None,
+            pushdowns=None,
+            stats=None
+        ))]
         pub fn sql_scan_task(
             url: String,
             file_format: PyFileFormatConfig,
@@ -412,10 +445,10 @@ pub mod pylib {
             num_rows: Option<i64>,
             size_bytes: Option<u64>,
             pushdowns: Option<PyPushdowns>,
-            stats: Option<PyTable>,
+            stats: Option<PyRecordBatch>,
         ) -> PyResult<Self> {
             let statistics = stats
-                .map(|s| TableStatistics::from_stats_table(&s.table))
+                .map(|s| TableStatistics::from_stats_table(&s.record_batch))
                 .transpose()?;
             let data_source = DataSource::Database {
                 path: url,
@@ -437,25 +470,34 @@ pub mod pylib {
 
         #[allow(clippy::too_many_arguments)]
         #[staticmethod]
+        #[pyo3(signature = (
+            module,
+            func_name,
+            func_args,
+            schema,
+            num_rows=None,
+            size_bytes=None,
+            pushdowns=None,
+            stats=None
+        ))]
         pub fn python_factory_func_scan_task(
-            py: Python,
             module: String,
             func_name: String,
-            func_args: Vec<Bound<PyAny>>,
+            func_args: Vec<PyObject>,
             schema: PySchema,
             num_rows: Option<i64>,
             size_bytes: Option<u64>,
             pushdowns: Option<PyPushdowns>,
-            stats: Option<PyTable>,
+            stats: Option<PyRecordBatch>,
         ) -> PyResult<Self> {
             let statistics = stats
-                .map(|s| TableStatistics::from_stats_table(&s.table))
+                .map(|s| TableStatistics::from_stats_table(&s.record_batch))
                 .transpose()?;
             let data_source = DataSource::PythonFactoryFunction {
                 module,
                 func_name,
                 func_args: PythonTablesFactoryArgs::new(
-                    func_args.iter().map(|pyany| pyany.into_py(py)).collect(),
+                    func_args.into_iter().map(Arc::new).collect(),
                 ),
                 size_bytes,
                 metadata: num_rows.map(|num_rows| TableMetadata {
@@ -523,7 +565,7 @@ pub mod pylib {
     ///
     /// Returns an `i64` representing the estimated size in bytes.
     ///
-    #[pyfunction]
+    #[pyfunction(signature = (uri, file_size, columns=None, has_metadata=None))]
     pub fn estimate_in_memory_size_bytes(
         uri: &str,
         file_size: u64,
@@ -531,14 +573,15 @@ pub mod pylib {
         has_metadata: Option<bool>,
     ) -> PyResult<usize> {
         let io_runtime = common_runtime::get_io_runtime(true);
-        let (schema, metadata) =
-            io_runtime.block_on_current_thread(daft_parquet::read::read_parquet_schema(
+        let (schema, metadata) = io_runtime.block_on_current_thread(
+            daft_parquet::read::read_parquet_schema_and_metadata(
                 uri,
                 default::Default::default(),
                 None,
                 default::Default::default(),
                 None,
-            ))?;
+            ),
+        )?;
         let data_source = DataSource::File {
             path: uri.to_string(),
             chunk_spec: None,
@@ -572,16 +615,13 @@ pub fn register_modules(parent: &Bound<PyModule>) -> PyResult<()> {
 
     parent.add_class::<pylib::ScanOperatorHandle>()?;
     parent.add_class::<pylib::PyScanTask>()?;
-    parent.add_function(wrap_pyfunction_bound!(
-        pylib::logical_plan_table_scan,
-        parent
-    )?)?;
+    parent.add_function(wrap_pyfunction!(pylib::logical_plan_table_scan, parent)?)?;
 
     Ok(())
 }
 
 pub fn register_testing_modules(parent: &Bound<PyModule>) -> PyResult<()> {
-    parent.add_function(wrap_pyfunction_bound!(
+    parent.add_function(wrap_pyfunction!(
         pylib::estimate_in_memory_size_bytes,
         parent
     )?)?;
