@@ -3,11 +3,13 @@ use std::sync::Arc;
 use common_display::mermaid::MermaidDisplayOptions;
 use common_error::DaftResult;
 use common_file_formats::FileFormat;
+#[cfg(feature = "python")]
+use common_file_formats::WriteMode;
 use common_py_serde::impl_bincode_py_state_serialization;
 use daft_dsl::ExprRef;
 use daft_logical_plan::InMemoryInfo;
 #[cfg(feature = "python")]
-use daft_logical_plan::{DeltaLakeCatalogInfo, IcebergCatalogInfo, LanceCatalogInfo};
+use daft_logical_plan::{DataSinkInfo, DeltaLakeCatalogInfo, IcebergCatalogInfo, LanceCatalogInfo};
 #[cfg(feature = "python")]
 use daft_physical_plan::ops::{DeltaLakeWrite, IcebergWrite, LanceWrite};
 use daft_physical_plan::{
@@ -15,7 +17,7 @@ use daft_physical_plan::{
     ops::{
         ActorPoolProject, Aggregate, BroadcastJoin, Concat, EmptyScan, Explode, Filter, HashJoin,
         InMemoryScan, Limit, MonotonicallyIncreasingId, Pivot, Project, Sample, Sort,
-        SortMergeJoin, TabularScan, TabularWriteCsv, TabularWriteJson, TabularWriteParquet,
+        SortMergeJoin, TabularScan, TabularWriteCsv, TabularWriteJson, TabularWriteParquet, TopN,
         Unpivot,
     },
     PhysicalPlan, PhysicalPlanRef, QueryStageOutput,
@@ -30,8 +32,11 @@ use {
     daft_dsl::python::PyExpr,
     daft_logical_plan::{OutputFileInfo, PyLogicalPlanBuilder},
     daft_scan::python::pylib::PyScanTask,
-    pyo3::{pyclass, pymethods, types::PyAnyMethods, PyObject, PyRef, PyRefMut, PyResult, Python},
-    std::collections::HashMap,
+    pyo3::{
+        pyclass, pymethods,
+        types::{PyAnyMethods, PyDict, PyList},
+        Bound, Py, PyAny, PyObject, PyRef, PyRefMut, PyResult, Python,
+    },
 };
 
 /// A work scheduler for physical plans.
@@ -88,7 +93,7 @@ impl PhysicalPlanScheduler {
     pub fn to_partition_tasks(
         &self,
         py: Python,
-        psets: HashMap<String, Vec<PyObject>>,
+        psets: Bound<PyDict>,
         actor_pool_manager: PyObject,
     ) -> PyResult<PyObject> {
         physical_plan_to_partition_tasks(self.plan().as_ref(), py, &psets, &actor_pool_manager)
@@ -98,7 +103,7 @@ impl PhysicalPlanScheduler {
 #[cfg(feature = "python")]
 #[pyclass]
 struct StreamingPartitionIterator {
-    iter: Box<dyn Iterator<Item = DaftResult<PyObject>> + Send>,
+    iter: Box<dyn Iterator<Item = DaftResult<PyObject>> + Send + Sync>,
 }
 
 #[cfg(feature = "python")]
@@ -126,7 +131,7 @@ impl From<QueryStageOutput> for PhysicalPlanScheduler {
 #[cfg(feature = "python")]
 #[pyclass]
 struct PartitionIterator {
-    parts: Vec<PyObject>,
+    parts: Py<PyList>,
     index: usize,
 }
 
@@ -136,10 +141,10 @@ impl PartitionIterator {
     fn __iter__(slf: PyRef<'_, Self>) -> PyRef<'_, Self> {
         slf
     }
-    fn __next__(mut slf: PyRefMut<'_, Self>) -> Option<PyObject> {
+    fn __next__(mut slf: PyRefMut<'_, Self>) -> Option<Bound<PyAny>> {
         let index = slf.index;
         slf.index += 1;
-        slf.parts.get(index).map(|part| part.clone_ref(slf.py()))
+        slf.parts.bind(slf.py()).get_item(index).ok()
     }
 }
 
@@ -153,28 +158,28 @@ fn exprs_to_pyexprs(exprs: &[ExprRef]) -> Vec<PyExpr> {
 fn tabular_write(
     py: Python,
     upstream_iter: PyObject,
+    write_mode: &WriteMode,
     file_format: &FileFormat,
     schema: &SchemaRef,
     root_dir: &String,
-    compression: &Option<String>,
-    partition_cols: &Option<Vec<ExprRef>>,
-    io_config: &Option<IOConfig>,
+    compression: Option<&String>,
+    partition_cols: Option<&Vec<ExprRef>>,
+    io_config: Option<&IOConfig>,
 ) -> PyResult<PyObject> {
     let py_iter = py
-        .import_bound(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+        .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
         .getattr(pyo3::intern!(py, "write_file"))?
         .call1((
             upstream_iter,
+            *write_mode,
             *file_format,
             PySchema::from(schema.clone()),
             root_dir,
-            compression.clone(),
-            partition_cols.as_ref().map(|cols| exprs_to_pyexprs(cols)),
-            io_config
-                .as_ref()
-                .map(|cfg| common_io_config::python::IOConfig {
-                    config: cfg.clone(),
-                }),
+            compression,
+            partition_cols.map(|cols| exprs_to_pyexprs(cols)),
+            io_config.map(|cfg| common_io_config::python::IOConfig {
+                config: cfg.clone(),
+            }),
         ))?;
     Ok(py_iter.into())
 }
@@ -187,13 +192,13 @@ fn iceberg_write(
     iceberg_info: &IcebergCatalogInfo,
 ) -> PyResult<PyObject> {
     let py_iter = py
-        .import_bound(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+        .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
         .getattr(pyo3::intern!(py, "write_iceberg"))?
         .call1((
             upstream_iter,
             &iceberg_info.table_location,
-            &iceberg_info.iceberg_schema,
-            &iceberg_info.iceberg_properties,
+            &iceberg_info.iceberg_schema.clone_ref(py),
+            &iceberg_info.iceberg_properties.clone_ref(py),
             iceberg_info.partition_spec_id,
             exprs_to_pyexprs(&iceberg_info.partition_cols),
             iceberg_info
@@ -214,7 +219,7 @@ fn deltalake_write(
     delta_lake_info: &DeltaLakeCatalogInfo,
 ) -> PyResult<PyObject> {
     let py_iter = py
-        .import_bound(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+        .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
         .getattr(pyo3::intern!(py, "write_deltalake"))?
         .call1((
             upstream_iter,
@@ -239,7 +244,7 @@ fn lance_write(
     lance_info: &LanceCatalogInfo,
 ) -> PyResult<PyObject> {
     let py_iter = py
-        .import_bound(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+        .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
         .getattr(pyo3::intern!(py, "write_lance"))?
         .call1((
             upstream_iter,
@@ -251,39 +256,54 @@ fn lance_write(
                 .map(|cfg| common_io_config::python::IOConfig {
                     config: cfg.clone(),
                 }),
-            lance_info.kwargs.clone(),
+            lance_info.kwargs.clone_ref(py),
         ))?;
     Ok(py_iter.into())
 }
 
+#[cfg(feature = "python")]
+fn data_sink_write(
+    py: Python,
+    upstream_iter: PyObject,
+    data_sink_info: &DataSinkInfo,
+) -> PyResult<PyObject> {
+    let py_iter = py
+        .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+        .getattr(pyo3::intern!(py, "write_data_sink"))?
+        .call1((upstream_iter, &data_sink_info.sink.clone_ref(py)))?;
+    Ok(py_iter.into())
+}
+
 #[cfg(feature = "python")]
 fn physical_plan_to_partition_tasks(
     physical_plan: &PhysicalPlan,
     py: Python,
-    psets: &HashMap<String, Vec<PyObject>>,
+    psets: &Bound<PyDict>,
     actor_pool_manager: &PyObject,
 ) -> PyResult<PyObject> {
     use daft_dsl::Expr;
-    use daft_physical_plan::ops::{CrossJoin, ShuffleExchange, ShuffleExchangeStrategy};
-
+    use daft_physical_plan::ops::{CrossJoin, DataSink, ShuffleExchange, ShuffleExchangeStrategy};
     match physical_plan {
+        PhysicalPlan::PreviousStageScan(..) => {
+            panic!("PreviousStageScan should be optimized away before reaching the scheduler")
+        }
         PhysicalPlan::InMemoryScan(InMemoryScan {
             in_memory_info: InMemoryInfo { cache_key, .. },
             ..
         }) => {
             let partition_iter = PartitionIterator {
-                parts: psets[cache_key].clone(),
+                parts: psets.get_item(cache_key)?.extract()?,
                 index: 0usize,
             };
             let py_iter = py
-                .import_bound(pyo3::intern!(py, "daft.execution.physical_plan"))?
+                .import(pyo3::intern!(py, "daft.execution.physical_plan"))?
                 .getattr(pyo3::intern!(py, "partition_read"))?
                 .call1((partition_iter,))?;
             Ok(py_iter.into())
         }
         PhysicalPlan::TabularScan(TabularScan { scan_tasks, .. }) => {
             let py_iter = py
-                .import_bound(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+                .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
                 .getattr(pyo3::intern!(py, "scan_with_tasks"))?
                 .call1((scan_tasks
                     .iter()
@@ -292,7 +312,7 @@ fn physical_plan_to_partition_tasks(
             Ok(py_iter.into())
         }
         PhysicalPlan::EmptyScan(EmptyScan { schema, .. }) => {
-            let schema_mod = py.import_bound(pyo3::intern!(py, "daft.logical.schema"))?;
+            let schema_mod = py.import(pyo3::intern!(py, "daft.logical.schema"))?;
             let python_schema = schema_mod
                 .getattr(pyo3::intern!(py, "Schema"))?
                 .getattr(pyo3::intern!(py, "_from_pyschema"))?
@@ -301,7 +321,7 @@ fn physical_plan_to_partition_tasks(
                 },))?;
 
             let py_iter = py
-                .import_bound(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+                .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
                 .getattr(pyo3::intern!(py, "empty_scan"))?
                 .call1((python_schema,))?;
             Ok(py_iter.into())
@@ -319,7 +339,7 @@ fn physical_plan_to_partition_tasks(
                 .map(|expr| PyExpr::from(expr.clone()))
                 .collect();
             let py_iter = py
-                .import_bound(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+                .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
                 .getattr(pyo3::intern!(py, "project"))?
                 .call1((
                     upstream_iter,
@@ -337,7 +357,7 @@ fn physical_plan_to_partition_tasks(
             let upstream_iter =
                 physical_plan_to_partition_tasks(input, py, psets, actor_pool_manager)?;
             let py_iter = py
-                .import_bound(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+                .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
                 .getattr(pyo3::intern!(py, "actor_pool_project"))?
                 .call1((
                     upstream_iter,
@@ -352,11 +372,12 @@ fn physical_plan_to_partition_tasks(
             Ok(py_iter.into())
         }
 
-        PhysicalPlan::Filter(Filter { input, predicate }) => {
+        PhysicalPlan::Filter(Filter {
+            input, predicate, ..
+        }) => {
             let upstream_iter =
                 physical_plan_to_partition_tasks(input, py, psets, actor_pool_manager)?;
-            let expressions_mod =
-                py.import_bound(pyo3::intern!(py, "daft.expressions.expressions"))?;
+            let expressions_mod = py.import(pyo3::intern!(py, "daft.expressions.expressions"))?;
             let py_predicate = expressions_mod
                 .getattr(pyo3::intern!(py, "Expression"))?
                 .getattr(pyo3::intern!(py, "_from_pyexpr"))?
@@ -365,7 +386,7 @@ fn physical_plan_to_partition_tasks(
                 .getattr(pyo3::intern!(py, "ExpressionsProjection"))?
                 .call1((vec![py_predicate],))?;
             let execution_step_mod =
-                py.import_bound(pyo3::intern!(py, "daft.execution.execution_step"))?;
+                py.import(pyo3::intern!(py, "daft.execution.execution_step"))?;
             let filter_step = execution_step_mod
                 .getattr(pyo3::intern!(py, "Filter"))?
                 .call1((expressions_projection,))?;
@@ -373,7 +394,7 @@ fn physical_plan_to_partition_tasks(
                 .getattr(pyo3::intern!(py, "ResourceRequest"))?
                 .call0()?;
             let py_iter = py
-                .import_bound(pyo3::intern!(py, "daft.execution.physical_plan"))?
+                .import(pyo3::intern!(py, "daft.execution.physical_plan"))?
                 .getattr(pyo3::intern!(py, "pipeline_instruction"))?
                 .call1((upstream_iter, filter_step, resource_request))?;
             Ok(py_iter.into())
@@ -386,8 +407,7 @@ fn physical_plan_to_partition_tasks(
         }) => {
             let upstream_iter =
                 physical_plan_to_partition_tasks(input, py, psets, actor_pool_manager)?;
-            let py_physical_plan =
-                py.import_bound(pyo3::intern!(py, "daft.execution.physical_plan"))?;
+            let py_physical_plan = py.import(pyo3::intern!(py, "daft.execution.physical_plan"))?;
             let global_limit_iter = py_physical_plan
                 .getattr(pyo3::intern!(py, "global_limit"))?
                 .call1((upstream_iter, *limit, *eager, *num_partitions))?;
@@ -403,7 +423,7 @@ fn physical_plan_to_partition_tasks(
                 .map(|expr| PyExpr::from(expr.clone()))
                 .collect();
             let py_iter = py
-                .import_bound(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+                .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
                 .getattr(pyo3::intern!(py, "explode"))?
                 .call1((upstream_iter, explode_pyexprs))?;
             Ok(py_iter.into())
@@ -425,7 +445,7 @@ fn physical_plan_to_partition_tasks(
                 .map(|expr| PyExpr::from(expr.clone()))
                 .collect();
             let py_iter = py
-                .import_bound(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+                .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
                 .getattr(pyo3::intern!(py, "unpivot"))?
                 .call1((
                     upstream_iter,
@@ -445,7 +465,7 @@ fn physical_plan_to_partition_tasks(
             let upstream_iter =
                 physical_plan_to_partition_tasks(input, py, psets, actor_pool_manager)?;
             let py_iter = py
-                .import_bound(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+                .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
                 .getattr(pyo3::intern!(py, "sample"))?
                 .call1((upstream_iter, *fraction, *with_replacement, *seed))?;
             Ok(py_iter.into())
@@ -457,7 +477,7 @@ fn physical_plan_to_partition_tasks(
             let upstream_iter =
                 physical_plan_to_partition_tasks(input, py, psets, actor_pool_manager)?;
             let py_iter = py
-                .import_bound(pyo3::intern!(py, "daft.execution.physical_plan"))?
+                .import(pyo3::intern!(py, "daft.execution.physical_plan"))?
                 .getattr(pyo3::intern!(py, "monotonically_increasing_id"))?
                 .call1((upstream_iter, column_name))?;
             Ok(py_iter.into())
@@ -476,7 +496,7 @@ fn physical_plan_to_partition_tasks(
                 .map(|expr| PyExpr::from(expr.clone()))
                 .collect();
             let py_iter = py
-                .import_bound(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+                .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
                 .getattr(pyo3::intern!(py, "sort"))?
                 .call1((
                     upstream_iter,
@@ -487,6 +507,35 @@ fn physical_plan_to_partition_tasks(
                 ))?;
             Ok(py_iter.into())
         }
+        PhysicalPlan::TopN(TopN {
+            input,
+            sort_by,
+            descending,
+            nulls_first,
+            limit,
+            num_partitions,
+        }) => {
+            let upstream_iter =
+                physical_plan_to_partition_tasks(input, py, psets, actor_pool_manager)?;
+            let py_physical_plan =
+                py.import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?;
+
+            let sort_by_pyexprs: Vec<PyExpr> = sort_by
+                .iter()
+                .map(|expr| PyExpr::from(expr.clone()))
+                .collect();
+            let global_limit_iter = py_physical_plan
+                .getattr(pyo3::intern!(py, "top_n"))?
+                .call1((
+                    upstream_iter,
+                    sort_by_pyexprs,
+                    descending.clone(),
+                    nulls_first.clone(),
+                    *limit,
+                    *num_partitions,
+                ))?;
+            Ok(global_limit_iter.into())
+        }
         PhysicalPlan::ShuffleExchange(ShuffleExchange { input, strategy }) => {
             let upstream_iter =
                 physical_plan_to_partition_tasks(input, py, psets, actor_pool_manager)?;
@@ -500,19 +549,16 @@ fn physical_plan_to_partition_tasks(
                                 .iter()
                                 .map(|expr| PyExpr::from(expr.clone()))
                                 .collect();
-                            py.import_bound(pyo3::intern!(
-                                py,
-                                "daft.execution.rust_physical_plan_shim"
-                            ))?
-                            .getattr(pyo3::intern!(py, "fanout_by_hash"))?
-                            .call1((
-                                upstream_iter,
-                                hash_clustering_config.num_partitions,
-                                partition_by_pyexprs,
-                            ))?
+                            py.import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+                                .getattr(pyo3::intern!(py, "fanout_by_hash"))?
+                                .call1((
+                                    upstream_iter,
+                                    hash_clustering_config.num_partitions,
+                                    partition_by_pyexprs,
+                                ))?
                         }
                         daft_logical_plan::ClusteringSpec::Random(random_clustering_config) => py
-                            .import_bound(pyo3::intern!(py, "daft.execution.physical_plan"))?
+                            .import(pyo3::intern!(py, "daft.execution.physical_plan"))?
                             .getattr(pyo3::intern!(py, "fanout_random"))?
                             .call1((upstream_iter, random_clustering_config.num_partitions()))?,
                         daft_logical_plan::ClusteringSpec::Range(_) => {
@@ -523,7 +569,7 @@ fn physical_plan_to_partition_tasks(
                         }
                     };
                     let reduced = py
-                        .import_bound(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+                        .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
                         .getattr(pyo3::intern!(py, "reduce_merge"))?
                         .call1((mapped,))?;
                     Ok(reduced.into())
@@ -533,7 +579,7 @@ fn physical_plan_to_partition_tasks(
                     pre_shuffle_merge_threshold,
                 } => {
                     let merged = py
-                        .import_bound(pyo3::intern!(
+                        .import(pyo3::intern!(
                             py,
                             "daft.execution.shuffles.pre_shuffle_merge"
                         ))?
@@ -546,19 +592,16 @@ fn physical_plan_to_partition_tasks(
                                 .iter()
                                 .map(|expr| PyExpr::from(expr.clone()))
                                 .collect();
-                            py.import_bound(pyo3::intern!(
-                                py,
-                                "daft.execution.rust_physical_plan_shim"
-                            ))?
-                            .getattr(pyo3::intern!(py, "fanout_by_hash"))?
-                            .call1((
-                                merged,
-                                hash_clustering_config.num_partitions,
-                                partition_by_pyexprs,
-                            ))?
+                            py.import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+                                .getattr(pyo3::intern!(py, "fanout_by_hash"))?
+                                .call1((
+                                    merged,
+                                    hash_clustering_config.num_partitions,
+                                    partition_by_pyexprs,
+                                ))?
                         }
                         daft_logical_plan::ClusteringSpec::Random(random_clustering_config) => py
-                            .import_bound(pyo3::intern!(py, "daft.execution.physical_plan"))?
+                            .import(pyo3::intern!(py, "daft.execution.physical_plan"))?
                             .getattr(pyo3::intern!(py, "fanout_random"))?
                             .call1((merged, random_clustering_config.num_partitions()))?,
                         daft_logical_plan::ClusteringSpec::Range(_) => {
@@ -569,11 +612,54 @@ fn physical_plan_to_partition_tasks(
                         }
                     };
                     let reduced = py
-                        .import_bound(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+                        .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
                         .getattr(pyo3::intern!(py, "reduce_merge"))?
                         .call1((mapped,))?;
                     Ok(reduced.into())
                 }
+                ShuffleExchangeStrategy::FlightShuffle {
+                    target_spec,
+                    shuffle_dirs,
+                } => {
+                    let shuffled = match target_spec.as_ref() {
+                        daft_logical_plan::ClusteringSpec::Hash(hash_clustering_config) => {
+                            let partition_by_pyexprs: Vec<PyExpr> = hash_clustering_config
+                                .by
+                                .iter()
+                                .map(|expr| PyExpr::from(expr.clone()))
+                                .collect();
+                            py.import(pyo3::intern!(
+                                py,
+                                "daft.execution.shuffles.flight_shuffle.flight_shuffle"
+                            ))?
+                            .getattr(pyo3::intern!(py, "flight_shuffle"))?
+                            .call1((
+                                upstream_iter,
+                                hash_clustering_config.num_partitions,
+                                shuffle_dirs,
+                                Some(partition_by_pyexprs),
+                            ))?
+                        }
+                        daft_logical_plan::ClusteringSpec::Random(random_clustering_config) => py
+                            .import(pyo3::intern!(
+                                py,
+                                "daft.execution.shuffles.flight_shuffle.flight_shuffle"
+                            ))?
+                            .getattr(pyo3::intern!(py, "flight_shuffle"))?
+                            .call1((
+                                upstream_iter,
+                                random_clustering_config.num_partitions(),
+                                shuffle_dirs,
+                            ))?,
+                        daft_logical_plan::ClusteringSpec::Range(_) => {
+                            unimplemented!("FanoutByRange not implemented, since only use case (sorting) doesn't need it yet.");
+                        }
+                        daft_logical_plan::ClusteringSpec::Unknown(_) => {
+                            unreachable!("Cannot use NaiveFullyMaterializingMapReduce ShuffleExchange to map to an Unknown ClusteringSpec");
+                        }
+                    };
+                    Ok(shuffled.into())
+                }
                 ShuffleExchangeStrategy::SplitOrCoalesceToTargetNum {
                     target_num_partitions,
                 } => {
@@ -582,7 +668,7 @@ fn physical_plan_to_partition_tasks(
                         std::cmp::Ordering::Greater => {
                             // Split if more outputs than inputs
                             let split = py
-                                .import_bound(pyo3::intern!(py, "daft.execution.physical_plan"))?
+                                .import(pyo3::intern!(py, "daft.execution.physical_plan"))?
                                 .getattr(pyo3::intern!(py, "split"))?
                                 .call1((
                                     upstream_iter,
@@ -590,7 +676,7 @@ fn physical_plan_to_partition_tasks(
                                     *target_num_partitions,
                                 ))?;
                             let flattened = py
-                                .import_bound(pyo3::intern!(py, "daft.execution.physical_plan"))?
+                                .import(pyo3::intern!(py, "daft.execution.physical_plan"))?
                                 .getattr(pyo3::intern!(py, "flatten_plan"))?
                                 .call1((split,))?;
                             Ok(flattened.into())
@@ -598,7 +684,7 @@ fn physical_plan_to_partition_tasks(
                         std::cmp::Ordering::Less => {
                             // Coalesce if fewer outputs than inputs
                             let coalesced = py
-                                .import_bound(pyo3::intern!(py, "daft.execution.physical_plan"))?
+                                .import(pyo3::intern!(py, "daft.execution.physical_plan"))?
                                 .getattr(pyo3::intern!(py, "coalesce"))?
                                 .call1((
                                     upstream_iter,
@@ -628,7 +714,7 @@ fn physical_plan_to_partition_tasks(
                 .map(|expr| PyExpr::from(expr.clone()))
                 .collect();
             let py_iter = py
-                .import_bound(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+                .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
                 .getattr(pyo3::intern!(py, "local_aggregate"))?
                 .call1((upstream_iter, aggs_as_pyexprs, groupbys_as_pyexprs))?;
             Ok(py_iter.into())
@@ -649,7 +735,7 @@ fn physical_plan_to_partition_tasks(
             let pivot_column_pyexpr = PyExpr::from(pivot_column.clone());
             let value_column_pyexpr = PyExpr::from(value_column.clone());
             let py_iter = py
-                .import_bound(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+                .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
                 .getattr(pyo3::intern!(py, "pivot"))?
                 .call1((
                     upstream_iter,
@@ -666,7 +752,7 @@ fn physical_plan_to_partition_tasks(
             let upstream_other_iter =
                 physical_plan_to_partition_tasks(other, py, psets, actor_pool_manager)?;
             let py_iter = py
-                .import_bound(pyo3::intern!(py, "daft.execution.physical_plan"))?
+                .import(pyo3::intern!(py, "daft.execution.physical_plan"))?
                 .getattr(pyo3::intern!(py, "concat"))?
                 .call1((upstream_input_iter, upstream_other_iter))?;
             Ok(py_iter.into())
@@ -693,7 +779,7 @@ fn physical_plan_to_partition_tasks(
                 .map(|expr| PyExpr::from(expr.clone()))
                 .collect();
             let py_iter = py
-                .import_bound(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+                .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
                 .getattr(pyo3::intern!(py, "hash_join"))?
                 .call1((
                     upstream_left_iter,
@@ -728,7 +814,7 @@ fn physical_plan_to_partition_tasks(
                 .collect();
             // TODO(Clark): Elide sorting one side of the join if already range-partitioned, where we'd use that side's boundaries to sort the other side.
             let py_iter = if *needs_presort {
-                py.import_bound(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+                py.import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
                     .getattr(pyo3::intern!(py, "sort_merge_join_aligned_boundaries"))?
                     .call1((
                         left_iter,
@@ -740,7 +826,7 @@ fn physical_plan_to_partition_tasks(
                         *left_is_larger,
                     ))?
             } else {
-                py.import_bound(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+                py.import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
                     .getattr(pyo3::intern!(py, "merge_join_sorted"))?
                     .call1((
                         left_iter,
@@ -775,7 +861,7 @@ fn physical_plan_to_partition_tasks(
                 .map(|expr| PyExpr::from(expr.clone()))
                 .collect();
             let py_iter = py
-                .import_bound(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
+                .import(pyo3::intern!(py, "daft.execution.rust_physical_plan_shim"))?
                 .getattr(pyo3::intern!(py, "broadcast_join"))?
                 .call1((
                     upstream_left_iter,
@@ -799,7 +885,7 @@ fn physical_plan_to_partition_tasks(
             let upstream_right_iter =
                 physical_plan_to_partition_tasks(right, py, psets, actor_pool_manager)?;
             let py_iter = py
-                .import_bound(pyo3::intern!(py, "daft.execution.physical_plan"))?
+                .import(pyo3::intern!(py, "daft.execution.physical_plan"))?
                 .getattr(pyo3::intern!(py, "cross_join"))?
                 .call1((upstream_left_iter, upstream_right_iter, *outer_loop_side))?;
             Ok(py_iter.into())
@@ -809,6 +895,7 @@ fn physical_plan_to_partition_tasks(
             file_info:
                 OutputFileInfo {
                     root_dir,
+                    write_mode,
                     file_format,
                     partition_cols,
                     compression,
@@ -818,18 +905,20 @@ fn physical_plan_to_partition_tasks(
         }) => tabular_write(
             py,
             physical_plan_to_partition_tasks(input, py, psets, actor_pool_manager)?,
+            write_mode,
             file_format,
             schema,
             root_dir,
-            compression,
-            partition_cols,
-            io_config,
+            compression.as_ref(),
+            partition_cols.as_ref(),
+            io_config.as_ref(),
         ),
         PhysicalPlan::TabularWriteCsv(TabularWriteCsv {
             schema,
             file_info:
                 OutputFileInfo {
                     root_dir,
+                    write_mode,
                     file_format,
                     partition_cols,
                     compression,
@@ -839,18 +928,20 @@ fn physical_plan_to_partition_tasks(
         }) => tabular_write(
             py,
             physical_plan_to_partition_tasks(input, py, psets, actor_pool_manager)?,
+            write_mode,
             file_format,
             schema,
             root_dir,
-            compression,
-            partition_cols,
-            io_config,
+            compression.as_ref(),
+            partition_cols.as_ref(),
+            io_config.as_ref(),
         ),
         PhysicalPlan::TabularWriteJson(TabularWriteJson {
             schema,
             file_info:
                 OutputFileInfo {
                     root_dir,
+                    write_mode,
                     file_format,
                     partition_cols,
                     compression,
@@ -860,12 +951,13 @@ fn physical_plan_to_partition_tasks(
         }) => tabular_write(
             py,
             physical_plan_to_partition_tasks(input, py, psets, actor_pool_manager)?,
+            write_mode,
             file_format,
             schema,
             root_dir,
-            compression,
-            partition_cols,
-            io_config,
+            compression.as_ref(),
+            partition_cols.as_ref(),
+            io_config.as_ref(),
         ),
         #[cfg(feature = "python")]
         PhysicalPlan::IcebergWrite(IcebergWrite {
@@ -897,5 +989,15 @@ fn physical_plan_to_partition_tasks(
             physical_plan_to_partition_tasks(input, py, psets, actor_pool_manager)?,
             lance_info,
         ),
+        #[cfg(feature = "python")]
+        PhysicalPlan::DataSink(DataSink {
+            schema: _,
+            data_sink_info,
+            input,
+        }) => data_sink_write(
+            py,
+            physical_plan_to_partition_tasks(input, py, psets, actor_pool_manager)?,
+            data_sink_info,
+        ),
     }
 }
