@@ -1,16 +1,18 @@
 #![feature(let_chains)]
 
-pub mod catalog;
 pub mod error;
 pub mod functions;
+
+mod exec;
 mod modules;
-
 mod planner;
-pub use planner::*;
+mod schema;
+mod statement;
+mod table_provider;
 
+pub use planner::*;
 #[cfg(feature = "python")]
 pub mod python;
-mod table_provider;
 
 #[cfg(feature = "python")]
 use pyo3::prelude::*;
@@ -18,52 +20,50 @@ use pyo3::prelude::*;
 #[cfg(feature = "python")]
 pub fn register_modules(parent: &Bound<PyModule>) -> PyResult<()> {
     parent.add_class::<python::PyCatalog>()?;
-    parent.add_function(wrap_pyfunction_bound!(python::sql, parent)?)?;
-    parent.add_function(wrap_pyfunction_bound!(python::sql_expr, parent)?)?;
-    parent.add_function(wrap_pyfunction_bound!(python::list_sql_functions, parent)?)?;
+    parent.add_function(wrap_pyfunction!(python::sql_exec, parent)?)?;
+    parent.add_function(wrap_pyfunction!(python::sql, parent)?)?;
+    parent.add_function(wrap_pyfunction!(python::sql_expr, parent)?)?;
+    parent.add_function(wrap_pyfunction!(python::sql_datatype, parent)?)?;
+    parent.add_function(wrap_pyfunction!(python::list_sql_functions, parent)?)?;
     Ok(())
 }
 
 #[cfg(test)]
 mod tests {
-    use std::sync::Arc;
+    use std::sync::{Arc, LazyLock};
 
-    use catalog::SQLCatalog;
     use daft_core::prelude::*;
-    use daft_dsl::{col, lit, Expr, OuterReferenceColumn, Subquery};
+    use daft_dsl::{lit, unresolved_col, Expr, ExprRef, PlanRef, Subquery, UnresolvedColumn};
     use daft_logical_plan::{
-        logical_plan::Source, source_info::PlaceHolderInfo, ClusteringSpec, LogicalPlan,
-        LogicalPlanBuilder, LogicalPlanRef, SourceInfo,
+        logical_plan::Source, source_info::PlaceHolderInfo, ClusteringSpec, JoinOptions,
+        LogicalPlan, LogicalPlanBuilder, LogicalPlanRef, SourceInfo,
     };
+    use daft_session::Session;
     use error::SQLPlannerResult;
     use rstest::{fixture, rstest};
 
     use super::*;
-    use crate::planner::SQLPlanner;
+    use crate::{error::PlannerError, planner::SQLPlanner};
 
     #[fixture]
     fn tbl_1() -> LogicalPlanRef {
-        let schema = Arc::new(
-            Schema::new(vec![
-                Field::new("test", DataType::Utf8),
-                Field::new("utf8", DataType::Utf8),
-                Field::new("i32", DataType::Int32),
-                Field::new("i64", DataType::Int64),
-                Field::new("f32", DataType::Float32),
-                Field::new("f64", DataType::Float64),
-                Field::new("bool", DataType::Boolean),
-                Field::new("date", DataType::Date),
-                Field::new("time", DataType::Time(TimeUnit::Microseconds)),
-                Field::new("list_utf8", DataType::new_list(DataType::Utf8)),
-            ])
-            .unwrap(),
-        );
+        let schema = Arc::new(Schema::new(vec![
+            Field::new("test", DataType::Utf8),
+            Field::new("utf8", DataType::Utf8),
+            Field::new("i32", DataType::Int32),
+            Field::new("i64", DataType::Int64),
+            Field::new("f32", DataType::Float32),
+            Field::new("f64", DataType::Float64),
+            Field::new("bool", DataType::Boolean),
+            Field::new("date", DataType::Date),
+            Field::new("time", DataType::Time(TimeUnit::Microseconds)),
+            Field::new("list_utf8", DataType::new_list(DataType::Utf8)),
+        ]));
         LogicalPlan::Source(Source::new(
             schema.clone(),
             Arc::new(SourceInfo::PlaceHolder(PlaceHolderInfo {
                 source_schema: schema,
                 clustering_spec: Arc::new(ClusteringSpec::unknown()),
-                source_id: 0,
             })),
         ))
         .arced()
@@ -71,20 +71,16 @@ mod tests {
 
     #[fixture]
     fn tbl_2() -> LogicalPlanRef {
-        let schema = Arc::new(
-            Schema::new(vec![
-                Field::new("text", DataType::Utf8),
-                Field::new("id", DataType::Int32),
-                Field::new("val", DataType::Int32),
-            ])
-            .unwrap(),
-        );
+        let schema = Arc::new(Schema::new(vec![
+            Field::new("text", DataType::Utf8),
+            Field::new("id", DataType::Int32),
+            Field::new("val", DataType::Int32),
+        ]));
         LogicalPlan::Source(Source::new(
             schema.clone(),
             Arc::new(SourceInfo::PlaceHolder(PlaceHolderInfo {
                 source_schema: schema,
                 clustering_spec: Arc::new(ClusteringSpec::unknown()),
-                source_id: 0,
             })),
         ))
         .arced()
@@ -92,20 +88,16 @@ mod tests {
 
     #[fixture]
     fn tbl_3() -> LogicalPlanRef {
-        let schema = Arc::new(
-            Schema::new(vec![
-                Field::new("first_name", DataType::Utf8),
-                Field::new("last_name", DataType::Utf8),
-                Field::new("id", DataType::Int32),
-            ])
-            .unwrap(),
-        );
+        let schema = Arc::new(Schema::new(vec![
+            Field::new("first_name", DataType::Utf8),
+            Field::new("last_name", DataType::Utf8),
+            Field::new("id", DataType::Int32),
+        ]));
         LogicalPlan::Source(Source::new(
             schema.clone(),
             Arc::new(SourceInfo::PlaceHolder(PlaceHolderInfo {
                 source_schema: schema,
                 clustering_spec: Arc::new(ClusteringSpec::unknown()),
-                source_id: 0,
             })),
         ))
         .arced()
@@ -113,13 +105,23 @@ mod tests {
 
     #[fixture]
     fn planner() -> SQLPlanner<'static> {
-        let mut catalog = SQLCatalog::new();
+        static SESSION: LazyLock<Session> = LazyLock::new(|| {
+            let session = Session::default();
+            // construct views from the tables and attach to the session
+            session
+                .create_temp_table("tbl1", &tbl_1().into(), false)
+                .unwrap();
+            session
+                .create_temp_table("tbl2", &tbl_2().into(), false)
+                .unwrap();
+            session
+                .create_temp_table("tbl3", &tbl_3().into(), false)
+                .unwrap();
 
-        catalog.register_table("tbl1", tbl_1());
-        catalog.register_table("tbl2", tbl_2());
-        catalog.register_table("tbl3", tbl_3());
+            session
+        });
 
-        SQLPlanner::new(catalog)
+        SQLPlanner::new(&SESSION)
     }
 
     #[rstest]
@@ -145,34 +147,15 @@ mod tests {
     #[case::join_with_filter("select * from tbl2 join tbl3 on tbl2.id = tbl3.id and tbl2.val > 0")]
     #[case::from("select tbl2.text from tbl2")]
     #[case::using("select tbl2.text from tbl2 join tbl3 using (id)")]
-    #[case(
-        r"
-    select
-        abs(i32) as abs,
-        ceil(i32) as ceil,
-        floor(i32) as floor,
-        sign(i32) as sign
-    from tbl1"
-    )]
-    #[case("select round(i32, 1) from tbl1")]
-    #[case::clip_int_to_float("select clip(i32, 1.5, 2.5) from tbl1")]
-    #[case::clip_float_to_int("select clip(f32, 1, 2) from tbl1")]
-    #[case::clip_null_lower("select clip(i32, NULL, 2) from tbl1")]
-    #[case::clip_null_upper("select clip(i32, 1, NULL) from tbl1")]
-    #[case::clip_float_null_lower("select clip(f32, NULL, 2.5) from tbl1")]
-    #[case::clip_float_null_upper("select clip(f32, 1.5, NULL) from tbl1")]
-    #[case::clip_float_lower_int_upper("select clip(i32, 1.5, 2) from tbl1")]
-    #[case::clip_int_lower_float_upper("select clip(i32, 1, 2.5) from tbl1")]
-    #[case::clip_float_lower_int_upper_float_col("select clip(f32, 1.5, 2) from tbl1")]
-    #[case::clip_int_lower_float_upper_float_col("select clip(f32, 1, 2.5) from tbl1")]
-    #[case::groupby("select max(i32) from tbl1 group by utf8")]
     #[case::orderby("select * from tbl1 order by i32")]
     #[case::orderby("select * from tbl1 order by i32 desc")]
     #[case::orderby("select * from tbl1 order by i32 asc")]
     #[case::orderby_multi("select * from tbl1 order by i32 desc, f32 asc")]
     #[case::whenthen("select case when i32 = 1 then 'a' else 'b' end from tbl1")]
-    #[case::globalagg("select max(i32) from tbl1")]
     #[case::cte("with cte as (select * from tbl1) select * from cte")]
+    #[case::double_alias("select * from tbl1 as tbl2, tbl2 as tbl1")]
+    #[case::double_alias_qualified("select tbl1.val from tbl1 as tbl2, tbl2 as tbl1")]
+    #[case::interval_arithmetic("select interval '1 day' * 3 from tbl1")]
     fn test_compiles(mut planner: SQLPlanner, #[case] query: &str) -> SQLPlannerResult<()> {
         let plan = planner.plan_sql(query);
         assert!(&plan.is_ok(), "query: {query}\nerror: {plan:?}");
@@ -205,7 +188,8 @@ mod tests {
         let plan = planner.plan_sql(sql).unwrap();
 
         let expected = LogicalPlanBuilder::from(tbl_1)
-            .select(vec![col("test").alias("a")])
+            .alias("tbl1")
+            .select(vec![unresolved_col("test").alias("a")])
             .unwrap()
             .build();
         assert_eq!(plan, expected);
@@ -217,8 +201,9 @@ mod tests {
         let plan = planner.plan_sql(sql)?;
 
         let expected = LogicalPlanBuilder::from(tbl_1)
-            .filter(col("test").eq(lit("a")))?
-            .select(vec![col("test").alias("a")])?
+            .alias("tbl1")
+            .filter(unresolved_col("test").eq(lit("a")))?
+            .select(vec![unresolved_col("test").alias("a")])?
             .build();
 
         assert_eq!(plan, expected);
@@ -230,7 +215,8 @@ mod tests {
         let plan = planner.plan_sql(sql)?;
 
         let expected = LogicalPlanBuilder::from(tbl_1)
-            .select(vec![col("test").alias("a")])?
+            .alias("tbl1")
+            .select(vec![unresolved_col("test").alias("a")])?
             .limit(10, true)?
             .build();
 
@@ -244,46 +230,15 @@ mod tests {
         let plan = planner.plan_sql(sql)?;
 
         let expected = LogicalPlanBuilder::from(tbl_1)
-            .select(vec![col("utf8")])?
-            .sort(vec![col("utf8")], vec![true], vec![true])?
+            .alias("tbl1")
+            .sort(vec![unresolved_col("utf8")], vec![true], vec![true])?
+            .select(vec![unresolved_col("utf8")])?
             .build();
 
         assert_eq!(plan, expected);
         Ok(())
     }
 
-    #[rstest]
-    fn test_cast(mut planner: SQLPlanner, tbl_1: LogicalPlanRef) -> SQLPlannerResult<()> {
-        let builder = LogicalPlanBuilder::from(tbl_1);
-        let cases = vec![
-            (
-                "select bool::text from tbl1",
-                vec![col("bool").cast(&DataType::Utf8)],
-            ),
-            (
-                "select utf8::bytes from tbl1",
-                vec![col("utf8").cast(&DataType::Binary)],
-            ),
-            (
-                r#"select CAST("bool" as text) from tbl1"#,
-                vec![col("bool").cast(&DataType::Utf8)],
-            ),
-        ];
-        for (sql, expected) in cases {
-            let actual = planner.plan_sql(sql)?;
-            let expected = builder.clone().select(expected)?.build();
-            assert_eq!(
-                actual,
-                expected,
-                "query: {}\n expected:{}",
-                sql,
-                expected.repr_ascii(false)
-            );
-        }
-
-        Ok(())
-    }
-
     #[rstest(
         null_equals_null => [false, true]
     )]
@@ -298,19 +253,37 @@ mod tests {
             if null_equals_null { "<=>" } else { "=" }
         );
         let plan = planner.plan_sql(&sql)?;
+
+        let left_on: ExprRef = UnresolvedColumn {
+            name: "id".into(),
+            plan_ref: PlanRef::Alias("tbl2".into()),
+            plan_schema: None,
+        }
+        .into();
+        let right_on: ExprRef = UnresolvedColumn {
+            name: "id".into(),
+            plan_ref: PlanRef::Alias("tbl3".into()),
+            plan_schema: None,
+        }
+        .into();
+
+        let on = if null_equals_null {
+            left_on.eq_null_safe(right_on)
+        } else {
+            left_on.eq(right_on)
+        };
+
         let expected = LogicalPlanBuilder::from(tbl_2)
-            .join_with_null_safe_equal(
-                tbl_3,
-                vec![col("id")],
-                vec![col("id")],
-                Some(vec![null_equals_null]),
+            .alias("tbl2")
+            .join(
+                LogicalPlanBuilder::from(tbl_3).alias("tbl3"),
+                on.into(),
+                vec![],
                 JoinType::Inner,
                 None,
-                None,
-                Some("tbl3."),
-                true,
+                JoinOptions::default().prefix("tbl3."),
             )?
-            .select(vec![col("*")])?
+            .select(vec![unresolved_col("*")])?
             .build();
         assert_eq!(plan, expected);
         Ok(())
@@ -323,93 +296,52 @@ mod tests {
         tbl_3: LogicalPlanRef,
     ) -> SQLPlannerResult<()> {
         let sql = "select * from tbl2 join tbl3 on tbl2.id = tbl3.id and tbl2.val > 0";
-        let plan = planner.plan_sql(&sql)?;
+        let plan = planner.plan_sql(sql)?;
+
+        let tbl2_id: ExprRef = UnresolvedColumn {
+            name: "id".into(),
+            plan_ref: PlanRef::Alias("tbl2".into()),
+            plan_schema: None,
+        }
+        .into();
+        let tbl3_id: ExprRef = UnresolvedColumn {
+            name: "id".into(),
+            plan_ref: PlanRef::Alias("tbl3".into()),
+            plan_schema: None,
+        }
+        .into();
+        let tbl2_val: ExprRef = UnresolvedColumn {
+            name: "val".into(),
+            plan_ref: PlanRef::Alias("tbl2".into()),
+            plan_schema: None,
+        }
+        .into();
 
         let expected = LogicalPlanBuilder::from(tbl_2)
-            .filter(col("val").gt(lit(0 as i64)))?
-            .join_with_null_safe_equal(
-                tbl_3,
-                vec![col("id")],
-                vec![col("id")],
-                Some(vec![false]),
+            .alias("tbl2")
+            .join(
+                LogicalPlanBuilder::from(tbl_3).alias("tbl3"),
+                (tbl2_id.eq(tbl3_id)).and(tbl2_val.gt(lit(0 as i64))).into(),
+                vec![],
                 JoinType::Inner,
                 None,
-                None,
-                Some("tbl3."),
-                true,
+                JoinOptions::default().prefix("tbl3."),
             )?
-            .select(vec![col("*")])?
+            .select(vec![unresolved_col("*")])?
             .build();
         assert_eq!(plan, expected);
         Ok(())
     }
 
-    #[rstest]
-    #[case::abs("select abs(i32) as abs from tbl1")]
-    #[case::ceil("select ceil(i32) as ceil from tbl1")]
-    #[case::floor("select floor(i32) as floor from tbl1")]
-    #[case::sign("select sign(i32) as sign from tbl1")]
-    #[case::round("select round(i32, 1) as round from tbl1")]
-    #[case::sqrt("select sqrt(i32) as sqrt from tbl1")]
-    #[case::sin("select sin(i32) as sin from tbl1")]
-    #[case::cos("select cos(i32) as cos from tbl1")]
-    #[case::tan("select tan(i32) as tan from tbl1")]
-    #[case::asin("select asin(i32) as asin from tbl1")]
-    #[case::acos("select acos(i32) as acos from tbl1")]
-    #[case::atan("select atan(i32) as atan from tbl1")]
-    #[case::atan2("select atan2(i32, 1) as atan2 from tbl1")]
-    #[case::radians("select radians(i32) as radians from tbl1")]
-    #[case::degrees("select degrees(i32) as degrees from tbl1")]
-    #[case::log2("select log2(i32) as log2 from tbl1")]
-    #[case::log10("select log10(i32) as log10 from tbl1")]
-    #[case::ln("select ln(i32) as ln from tbl1")]
-    #[case::exp("select exp(i32) as exp from tbl1")]
-    #[case::atanh("select atanh(i32) as atanh from tbl1")]
-    #[case::acosh("select acosh(i32) as acosh from tbl1")]
-    #[case::asinh("select asinh(i32) as asinh from tbl1")]
-    #[case::ends_with("select ends_with(utf8, 'a') as ends_with from tbl1")]
-    #[case::starts_with("select starts_with(utf8, 'a') as starts_with from tbl1")]
-    #[case::contains("select contains(utf8, 'a') as contains from tbl1")]
-    #[case::split("select split(utf8, '.') as split from tbl1")]
-    #[case::replace("select regexp_replace(utf8, 'a', 'b') as replace from tbl1")]
-    #[case::length("select length(utf8) as length from tbl1")]
-    #[case::lower("select lower(utf8) as lower from tbl1")]
-    #[case::upper("select upper(utf8) as upper from tbl1")]
-    #[case::lstrip("select lstrip(utf8) as lstrip from tbl1")]
-    #[case::rstrip("select rstrip(utf8) as rstrip from tbl1")]
-    #[case::reverse("select reverse(utf8) as reverse from tbl1")]
-    #[case::capitalize("select capitalize(utf8) as capitalize from tbl1")]
-    #[case::left("select left(utf8, 1) as left from tbl1")]
-    #[case::right("select right(utf8, 1) as right from tbl1")]
-    #[case::find("select find(utf8, 'a') as find from tbl1")]
-    #[case::rpad("select rpad(utf8, 1, 'a') as rpad from tbl1")]
-    #[case::lpad("select lpad(utf8, 1, 'a') as lpad from tbl1")]
-    #[case::repeat("select repeat(utf8, 1) as repeat from tbl1")]
-    #[case::to_date("select to_date(utf8, 'YYYY-MM-DD') as to_date from tbl1")]
-    #[case::like("select utf8 like 'a' as like from tbl1")]
-    #[case::ilike("select utf8 ilike 'a' as ilike from tbl1")]
-    #[case::datestring("select DATE '2021-08-01' as dt from tbl1")]
-    #[case::datetime("select DATETIME '2021-08-01 00:00:00' as dt from tbl1")]
-    #[case::countstar("select COUNT(*) as count from tbl1")]
-    #[case::countstarlower("select COUNT(*) as count from tbl1")]
-    #[case::count("select COUNT(i32) as count from tbl1")]
-    #[case::countcasing("select CoUnT(i32) as count from tbl1")]
-    // #[case::to_datetime("select to_datetime(utf8, 'YYYY-MM-DD') as to_datetime from tbl1")]
-    fn test_compiles_funcs(mut planner: SQLPlanner, #[case] query: &str) -> SQLPlannerResult<()> {
-        let plan = planner.plan_sql(query);
-        assert!(plan.is_ok(), "query: {query}\nerror: {plan:?}");
-
-        Ok(())
-    }
-
     #[rstest]
     fn test_global_agg(mut planner: SQLPlanner, tbl_1: LogicalPlanRef) -> SQLPlannerResult<()> {
         let sql = "select max(i32) from tbl1";
         let plan = planner.plan_sql(sql)?;
 
         let expected = LogicalPlanBuilder::from(tbl_1)
-            .aggregate(vec![col("i32").max()], vec![])?
-            .select(vec![col("i32")])?
+            .alias("tbl1")
+            .aggregate(vec![unresolved_col("i32").max()], vec![])?
+            .select(vec![unresolved_col("i32")])?
             .build();
 
         assert_eq!(plan, expected);
@@ -466,37 +398,101 @@ mod tests {
     }
 
     #[rstest]
-    #[case::basic("select utf8 from tbl1 where i64 > (select max(id) from tbl2 where id = i32)")]
+    #[case::basic(
+        "select utf8 from tbl1 where i64 > (select max(id) from tbl2 where id = i32)",
+        PlanRef::Unqualified
+    )]
     #[case::compound(
-        "select utf8 from tbl1 where i64 > (select max(id) from tbl2 where id = tbl1.i32)"
+        "select utf8 from tbl1 where i64 > (select max(id) from tbl2 where id = tbl1.i32)",
+        PlanRef::Alias("tbl1".into())
     )]
     fn test_correlated_subquery(
         mut planner: SQLPlanner,
         #[case] query: &str,
+        #[case] plan_ref: PlanRef,
         tbl_1: LogicalPlanRef,
         tbl_2: LogicalPlanRef,
     ) -> SQLPlannerResult<()> {
+        use daft_dsl::{Column, ResolvedColumn};
+
         let plan = planner.plan_sql(query)?;
 
-        let outer_col = Arc::new(Expr::OuterReferenceColumn(OuterReferenceColumn {
-            field: Field::new("i32", DataType::Int32),
-            depth: 1,
-        }));
+        let outer_col = Arc::new(Expr::Column(Column::Resolved(ResolvedColumn::OuterRef(
+            Field::new("i32", DataType::Int32),
+            plan_ref,
+        ))));
         let subquery = LogicalPlanBuilder::from(tbl_2)
-            .filter(col("id").eq(outer_col))?
-            .aggregate(vec![col("id").max()], vec![])?
-            .select(vec![col("id")])?
+            .alias("tbl2")
+            .filter(unresolved_col("id").eq(outer_col))?
+            .aggregate(vec![unresolved_col("id").max()], vec![])?
+            .select(vec![unresolved_col("id")])?
             .build();
 
         let subquery = Arc::new(Expr::Subquery(Subquery { plan: subquery }));
 
         let expected = LogicalPlanBuilder::from(tbl_1)
-            .filter(col("i64").gt(subquery))?
-            .select(vec![col("utf8")])?
+            .alias("tbl1")
+            .filter(unresolved_col("i64").gt(subquery))?
+            .select(vec![unresolved_col("utf8")])?
             .build();
 
         assert_eq!(plan, expected);
 
         Ok(())
     }
+
+    #[rstest]
+    fn test_multiple_from_with_join(
+        mut planner: SQLPlanner,
+        tbl_1: LogicalPlanRef,
+        tbl_2: LogicalPlanRef,
+        tbl_3: LogicalPlanRef,
+    ) -> SQLPlannerResult<()> {
+        let sql = "select tbl2.val from tbl1 left join tbl2 on tbl1.utf8 = tbl2.text, (tbl1 as tbl4) right join tbl3 on tbl4.i32 = tbl3.id";
+        let plan = planner.plan_sql(sql)?;
+
+        let first_from = LogicalPlanBuilder::from(tbl_1.clone()).alias("tbl1").join(
+            LogicalPlanBuilder::from(tbl_2).alias("tbl2"),
+            unresolved_col("utf8").eq(unresolved_col("text")).into(),
+            vec![],
+            JoinType::Left,
+            None,
+            JoinOptions::default().prefix("tbl2."),
+        )?;
+
+        let second_from = LogicalPlanBuilder::from(tbl_1)
+            .alias("tbl1")
+            .alias("tbl4")
+            .join(
+                LogicalPlanBuilder::from(tbl_3).alias("tbl3"),
+                unresolved_col("i32").eq(unresolved_col("id")).into(),
+                vec![],
+                JoinType::Right,
+                None,
+                JoinOptions::default().prefix("tbl3."),
+            )?;
+
+        let expected = first_from
+            .cross_join(second_from, JoinOptions::default())?
+            .select(vec![unresolved_col("val")])?
+            .build();
+
+        assert_eq!(plan, expected);
+
+        Ok(())
+    }
+
+    #[rstest]
+    #[case::basic("select tbl1.test from tbl1 as tbl2")]
+    #[case::subquery("select tbl1.test from (select * from tbl1) as tbl2")]
+    fn test_subquery_alias_bad_scope(
+        mut planner: SQLPlanner,
+        #[case] query: &str,
+    ) -> SQLPlannerResult<()> {
+        let result = planner.plan_sql(query);
+
+        assert!(result.is_err_and(|e| { matches!(e, PlannerError::ColumnNotFound { .. }) }));
+
+        Ok(())
+    }
 }
