@@ -7,8 +7,8 @@ use std::{
 
 use common_error::{DaftError, DaftResult};
 use daft_core::prelude::*;
-use daft_dsl::{Expr, ExprRef};
-use daft_table::Table;
+use daft_dsl::{Column, Expr, ExprRef, ResolvedColumn};
+use daft_recordbatch::RecordBatch;
 use indexmap::{IndexMap, IndexSet};
 
 use crate::column_stats::ColumnRangeStatistics;
@@ -28,7 +28,7 @@ impl Hash for TableStatistics {
 }
 
 impl TableStatistics {
-    pub fn from_stats_table(table: &Table) -> DaftResult<Self> {
+    pub fn from_stats_table(table: &RecordBatch) -> DaftResult<Self> {
         // Assumed format is each column having 2 rows:
         // - row 0: Minimum value for the column.
         // - row 1: Maximum value for the column.
@@ -36,21 +36,19 @@ impl TableStatistics {
             return Err(DaftError::ValueError(format!("Expected stats table to have 2 rows, with min and max values for each column, but got {} rows: {}", table.len(), table)));
         }
         let mut columns = IndexMap::with_capacity(table.num_columns());
-        for name in table.column_names() {
-            let col = table.get_column(&name).unwrap();
+        for col in table.columns() {
             let stats = ColumnRangeStatistics::new(Some(col.slice(0, 1)?), Some(col.slice(1, 2)?))?;
-            columns.insert(name, stats);
+            columns.insert(col.name().to_string(), stats);
         }
         Ok(Self { columns })
     }
 
     #[must_use]
-    pub fn from_table(table: &Table) -> Self {
+    pub fn from_table(table: &RecordBatch) -> Self {
         let mut columns = IndexMap::with_capacity(table.num_columns());
-        for name in table.column_names() {
-            let col = table.get_column(&name).unwrap();
+        for col in table.columns() {
             let stats = ColumnRangeStatistics::from_series(col);
-            columns.insert(name, stats);
+            columns.insert(col.name().to_string(), stats);
         }
         Self { columns }
     }
@@ -85,11 +83,11 @@ impl TableStatistics {
             .map(|e| self.eval_expression(e))
             .collect::<crate::Result<Vec<_>>>()?;
 
-        let new_col_stats = result_cols
-            .into_iter()
-            .zip(expected_schema.fields.keys())
-            .map(|(c, f)| (f.clone(), c))
-            .collect::<IndexMap<_, _>>();
+        let new_col_stats = expected_schema
+            .field_names()
+            .map(ToString::to_string)
+            .zip(result_cols)
+            .collect();
 
         Ok(Self {
             columns: new_col_stats,
@@ -101,7 +99,7 @@ impl TableStatistics {
 
         if let Some(schema) = schema {
             // if schema provided, use it
-            for field in schema.fields.values() {
+            for field in schema.fields() {
                 let name = field.name.as_str();
                 let elem_size = if let Some(stats) = self.columns.get(name) {
                     // first try to use column stats
@@ -132,7 +130,7 @@ impl TableStatistics {
     pub fn eval_expression(&self, expr: &Expr) -> crate::Result<ColumnRangeStatistics> {
         match expr {
             Expr::Alias(col, _) => self.eval_expression(col.as_ref()),
-            Expr::Column(col_name) => {
+            Expr::Column(Column::Resolved(ResolvedColumn::Basic(col_name))) => {
                 let col = self.columns.get(col_name.as_ref());
                 let Some(col) = col else {
                     return Err(crate::Error::DaftCoreCompute {
@@ -176,19 +174,19 @@ impl TableStatistics {
         fill_map: Option<&HashMap<&str, ExprRef>>,
     ) -> crate::Result<Self> {
         let mut columns = IndexMap::new();
-        for (field_name, field) in &schema.fields {
-            let crs = match self.columns.get(field_name) {
+        for field in schema.as_ref() {
+            let crs = match self.columns.get(&field.name) {
                 Some(column_stat) => column_stat
                     .cast(&field.dtype)
                     .unwrap_or(ColumnRangeStatistics::Missing),
                 None => fill_map
                     .as_ref()
-                    .and_then(|m| m.get(field_name.as_str()))
+                    .and_then(|m| m.get(field.name.as_str()))
                     .map(|e| self.eval_expression(e))
                     .transpose()?
                     .unwrap_or(ColumnRangeStatistics::Missing),
             };
-            columns.insert(field_name.clone(), crs);
+            columns.insert(field.name.clone(), crs);
         }
         Ok(Self { columns })
     }
@@ -201,8 +199,8 @@ impl Display for TableStatistics {
             .iter()
             .map(|(s, c)| c.combined_series().unwrap().rename(s))
             .collect::<Vec<_>>();
-        let tbl_schema = Schema::new(columns.iter().map(|s| s.field().clone()).collect()).unwrap();
-        let tab = Table::new_with_size(tbl_schema, columns, 2).unwrap();
+        let tbl_schema = Schema::new(columns.iter().map(|s| s.field().clone()));
+        let tab = RecordBatch::new_with_size(tbl_schema, columns, 2).unwrap();
         write!(f, "{tab}")
     }
 }
@@ -210,39 +208,40 @@ impl Display for TableStatistics {
 #[cfg(test)]
 mod test {
     use daft_core::prelude::*;
-    use daft_dsl::{col, lit};
-    use daft_table::Table;
+    use daft_dsl::{lit, resolved_col};
+    use daft_recordbatch::RecordBatch;
 
     use super::TableStatistics;
     use crate::column_stats::TruthValue;
 
     #[test]
     fn test_equal() -> crate::Result<()> {
-        let table = Table::from_nonempty_columns(vec![
-            Int64Array::from(("a", vec![1, 2, 3, 4])).into_series()
-        ])
-        .unwrap();
+        let table =
+            RecordBatch::from_nonempty_columns(vec![
+                Int64Array::from(("a", vec![1, 2, 3, 4])).into_series()
+            ])
+            .unwrap();
         let table_stats = TableStatistics::from_table(&table);
 
         // False case
-        let expr = col("a").eq(lit(0));
+        let expr = resolved_col("a").eq(lit(0));
         let result = table_stats.eval_expression(&expr)?;
         assert_eq!(result.to_truth_value(), TruthValue::False);
 
         // Maybe case
-        let expr = col("a").eq(lit(3));
+        let expr = resolved_col("a").eq(lit(3));
         let result = table_stats.eval_expression(&expr)?;
         assert_eq!(result.to_truth_value(), TruthValue::Maybe);
 
         // True case
         let table =
-            Table::from_nonempty_columns(
-                vec![Int64Array::from(("a", vec![0, 0, 0])).into_series()],
-            )
+            RecordBatch::from_nonempty_columns(vec![
+                Int64Array::from(("a", vec![0, 0, 0])).into_series()
+            ])
             .unwrap();
         let table_stats = TableStatistics::from_table(&table);
 
-        let expr = col("a").eq(lit(0));
+        let expr = resolved_col("a").eq(lit(0));
         let result = table_stats.eval_expression(&expr)?;
         assert_eq!(result.to_truth_value(), TruthValue::True);
 
