@@ -1,4 +1,5 @@
 #![feature(let_chains)]
+#![allow(clippy::useless_conversion)]
 
 #[cfg(not(target_env = "msvc"))]
 use tikv_jemallocator::Jemalloc;
@@ -46,12 +47,12 @@ fn should_enable_chrome_trace() -> bool {
 
 #[cfg(feature = "python")]
 pub mod pylib {
-    use common_tracing::init_tracing;
-    use lazy_static::lazy_static;
+    use std::sync::LazyLock;
+
+    use common_tracing::{init_opentelemetry_providers, init_tracing};
     use pyo3::prelude::*;
-    lazy_static! {
-        static ref LOG_RESET_HANDLE: pyo3_log::ResetHandle = pyo3_log::init();
-    }
+
+    static LOG_RESET_HANDLE: LazyLock<pyo3_log::ResetHandle> = LazyLock::new(pyo3_log::init);
 
     #[pyfunction]
     pub fn version() -> &'static str {
@@ -71,7 +72,7 @@ pub mod pylib {
     #[pyfunction]
     pub fn refresh_logger(py: Python) -> PyResult<()> {
         use log::LevelFilter;
-        let logging = py.import_bound(pyo3::intern!(py, "logging"))?;
+        let logging = py.import(pyo3::intern!(py, "logging"))?;
         let python_log_level = logging
             .getattr(pyo3::intern!(py, "getLogger"))?
             .call0()?
@@ -93,40 +94,47 @@ pub mod pylib {
         Ok(())
     }
 
+    #[pyfunction]
+    pub fn set_compute_runtime_num_worker_threads(num_threads: usize) -> PyResult<()> {
+        common_runtime::set_compute_runtime_num_worker_threads(num_threads)?;
+        Ok(())
+    }
+
     #[pymodule]
     fn daft(py: Python, m: &Bound<PyModule>) -> PyResult<()> {
         refresh_logger(py)?;
         init_tracing(crate::should_enable_chrome_trace());
+        init_opentelemetry_providers();
 
         common_daft_config::register_modules(m)?;
         common_system_info::register_modules(m)?;
         common_resource_request::register_modules(m)?;
         common_file_formats::python::register_modules(m)?;
         common_scan_info::register_modules(m)?;
+        daft_catalog::register_modules(m)?;
+        daft_connect::register_modules(m)?;
+        daft_context::register_modules(m)?;
         daft_core::register_modules(m)?;
         daft_core::python::register_modules(m)?;
-        daft_local_execution::register_modules(m)?;
-        daft_dsl::register_modules(m)?;
-        daft_table::register_modules(m)?;
-        daft_io::register_modules(m)?;
-        daft_parquet::register_modules(m)?;
         daft_csv::register_modules(m)?;
+        daft_distributed::register_modules(m)?;
+        daft_dsl::register_modules(m)?;
+        daft_functions::register_modules(m)?;
+        daft_io::register_modules(m)?;
         daft_json::register_modules(m)?;
+        daft_local_execution::register_modules(m)?;
+        daft_local_plan::register_modules(m)?;
         daft_logical_plan::register_modules(m)?;
+        daft_parquet::register_modules(m)?;
         daft_micropartition::register_modules(m)?;
+        daft_recordbatch::register_modules(m)?;
         daft_scan::register_modules(m)?;
         daft_scheduler::register_modules(m)?;
+        daft_session::register_modules(m)?;
         daft_sql::register_modules(m)?;
-        daft_functions::register_modules(m)?;
-        daft_functions_json::register_modules(m)?;
-        daft_connect::register_modules(m)?;
-
-        // Register catalog module
-        let catalog_module = daft_catalog::python::register_modules(m)?;
-        daft_catalog_python_catalog::python::register_modules(&catalog_module)?;
-
+        daft_shuffles::python::register_modules(m)?;
         // Register testing module
-        let testing_module = PyModule::new_bound(m.py(), "testing")?;
+        let testing_module = PyModule::new(m.py(), "testing")?;
         m.add_submodule(&testing_module)?;
         daft_scan::python::register_testing_modules(&testing_module)?;
 
@@ -134,7 +142,20 @@ pub mod pylib {
         m.add_wrapped(wrap_pyfunction!(build_type))?;
         m.add_wrapped(wrap_pyfunction!(refresh_logger))?;
         m.add_wrapped(wrap_pyfunction!(get_max_log_level))?;
-        daft_image::python::register_modules(m)?;
+        m.add_wrapped(wrap_pyfunction!(set_compute_runtime_num_worker_threads))?;
+
+        daft_dashboard::register_modules(m)?;
+        daft_cli::register_modules(m)?;
+
+        // We need to do this here because it's the only point in the rust codebase that we have access to all crates.
+        let mut functions_registry = daft_dsl::functions::FUNCTION_REGISTRY
+            .write()
+            .expect("Failed to acquire write lock on function registry");
+        functions_registry.register::<daft_functions::numeric::NumericFunctions>();
+        functions_registry.register::<daft_functions::float::FloatFunctions>();
+        functions_registry.register::<daft_image::functions::ImageFunctions>();
+        functions_registry.register::<daft_functions_json::JsonFunctions>();
+
         Ok(())
     }
 }
