@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 import uuid
-from typing import Literal
+from typing import Any, Literal
 
 import pandas as pd
 import pyarrow as pa
@@ -9,7 +9,7 @@ import pytest
 
 import daft
 import daft.context
-from daft.table import MicroPartition
+from daft.recordbatch import MicroPartition
 
 # import all conftest
 from tests.integration.io.conftest import *
@@ -88,6 +88,33 @@ def join_strategy(request):
             yield "sort_merge"
 
 
+@pytest.fixture(scope="function")
+def make_spark_df(spark_session):
+    def _make_spark_df(data: dict[str, Any]):
+        fields = [name for name in data]
+        rows = list(zip(*[data[name] for name in fields]))
+        return spark_session.createDataFrame(rows, fields)
+
+    yield _make_spark_df
+
+
+@pytest.fixture(scope="function")
+def assert_spark_equals(spark_session):
+    def _assert_spark_dfs_eq(df1, df2):
+        if isinstance(df1, daft.DataFrame):
+            df1 = df1.to_pandas()
+        else:
+            df1 = df1.toPandas()
+        if isinstance(df2, daft.DataFrame):
+            df2 = df2.to_pandas()
+        else:
+            df2 = df2.toPandas()
+
+        assert df1.equals(df2)
+
+    yield _assert_spark_dfs_eq
+
+
 @pytest.fixture(scope="function")
 def make_df(data_source, tmp_path) -> daft.Dataframe:
     """Makes a dataframe when provided with data."""
@@ -118,11 +145,11 @@ def make_df(data_source, tmp_path) -> daft.Dataframe:
             import pyarrow.parquet as papq
 
             name = str(uuid.uuid4())
-            daft_table = MicroPartition.from_arrow(pa_table)
+            daft_recordbatch = MicroPartition.from_arrow(pa_table)
             partitioned_tables = (
-                daft_table.partition_by_random(repartition, 0)
+                daft_recordbatch.partition_by_random(repartition, 0)
                 if len(repartition_columns) == 0
-                else daft_table.partition_by_hash([daft.col(c) for c in repartition_columns], repartition)
+                else daft_recordbatch.partition_by_hash([daft.col(c) for c in repartition_columns], repartition)
             )
             for i, tbl in enumerate(partitioned_tables):
                 tmp_file = tmp_path / (name + f"-{i}")
@@ -184,6 +211,18 @@ def assert_df_equals(
             raise
 
 
+def check_answer(df: daft.DataFrame, expected_answer: dict[str, Any], is_sorted: bool = False):
+    daft_df = df.to_pandas()
+    expected_df = daft.from_pydict(expected_answer).to_pandas()
+    # when this is an empty result, no need to check data types.
+    check_dtype = not expected_df.empty
+    if is_sorted:
+        assert_df_equals(daft_df, expected_df, assert_ordering=True, check_dtype=check_dtype)
+    else:
+        sort_keys = df.column_names
+        assert_df_equals(daft_df, expected_df, sort_key=sort_keys, assert_ordering=False, check_dtype=check_dtype)
+
+
 @pytest.fixture(
     scope="function",
     params=[1, None] if get_tests_daft_runner_name() == "native" else [None],
